#$Revision: 1.10 $
import getopt, sys, copy
from MatchCollection import *
from MatchGenerator import *
from HitCollection import *
from HitGenerator import *
from Filter import *
import os
import preproc
from composite import *

class ChargeSieve:

	# Constructor takes optional DtaSelectParams object as input
	def __init__(self, params=None):
		self.data = {}
		self.data["working dir"]="."
		self.dtaHitGen = None
		self.pseudoGen = None
		self.spectrumCollection = None 

	def __getitem__(self, key):
		return self.data[key]

	def __setitem__(self, key, item):
		self.data[key] = item		


	def calcAreaPlatt(self, classifier, testData):

		threshholds = [ 0.0, 0.01, 0.02, 0.03, 0.04, 0.05, 0.1, 0.15, 0.2, 0.25, \
			0.3, 0.35, 0.4, 0.45, 0.5, 0.55, 0.65, 0.7,0.75,0.8, \
			0.85,0.9,0.95,0.96,0.97,0.98,0.99,1.00]   
		baseTP = baseTPprot = 0
		area = 0.0
		print "\t".join(["FILTER:","TruePosPept","TruePosPeptRate","FalsePosPept",\
			"TruePosProt", "TruePosProtRate", "FalsePosProt", \
			"Threshold","NumOrig", "NumKept", "PercentRemoved", "ClassThreshold"])

		cfilter  = CstateFilterPlatt()
		cfilter["classifier"] = classifier

		for t in threshholds:
			print >>sys.stderr, "Filtering with threshold = ", t 
			if t == 0.5:
				cfilter["threshold2"] = cfilter["threshold3"] = t
			elif t > 0.5:
				cfilter["threshold2"] = 0.0
				cfilter["threshold3"] = t
			elif t < 0.5:
				cfilter["threshold2"] = t
				cfilter["threshold3"] = 1.0 

			filteredSpecColl = cfilter.filterSpectra(self.spectrumCollection,\
					specDataSet=testData)
			pseudoMatch = self.pseudoGen.generateMatches(filteredSpecColl)
			pseudoMatch.outputToFilename("".join(["pseudo.",str(t),".sqt"]))
			hitCollection  = self.dtaHitGen.generateHits(pseudoMatch)

			tp = hitCollection.calcTPmatches()
			tpprot = hitCollection.calcTP()
			if t == 0.0:
				baseTP = tp
				baseTPprot = tpprot

			print "\t".join( map (str, [  \
					"FILTER:", \
					tp, float(tp)/ baseTP, hitCollection.calcFPmatches(), \
					tpprot, float(tpprot)/ baseTPprot, hitCollection.calcFP(), t, \
					self.spectrumCollection["number spectra"],\
					filteredSpecColl["number spectra"], \
					1.0-(filteredSpecColl["number spectra"]/ \
						self.spectrumCollection["number spectra"]), \
					t])) 
		
		return 0.0
			
	def calcSinglePlatt(self, classifier, testData):

		baseTP = baseTPprot = 0
		print "\t".join(["FILTER:","TruePosPept","TruePosPeptRate","FalsePosPept",\
			"TruePosProt", "TruePosProtRate", "FalsePosProt", \
			"Threshold","NumOrig", "NumKept", "PercentRemoved", "ClassThreshold"])

		cfilter  = CstateFilterPlatt()
		cfilter["classifier"] = classifier
		
		# don't filter anythin
		cfilter["threshold2"] = -0.1
		cfilter["threshold3"] = 1.1
		filteredSpecColl = cfilter.filterSpectra(self.spectrumCollection,\
				specDataSet=testData)
		pseudoMatch = self.pseudoGen.generateMatches(filteredSpecColl)
		pseudoMatch.outputToFilename("".join(["pseudo.","0.0",".sqt"]))
		hitCollection  = self.dtaHitGen.generateHits(pseudoMatch)

		baseTP     = hitCollection.calcTPmatches()
		baseTPprot = hitCollection.calcTP()

		cfilter["threshold2"] = 0.05
		cfilter["threshold3"] = 0.90

		filteredSpecColl = cfilter.filterSpectra(self.spectrumCollection,\
				specDataSet=testData)
		pseudoMatch = self.pseudoGen.generateMatches(filteredSpecColl)
		pseudoMatch.outputToFilename("".join(["pseudo.","0.05,0.90",".sqt"]))
		hitCollection  = self.dtaHitGen.generateHits(pseudoMatch)

		tp = hitCollection.calcTPmatches()
		tpprot = hitCollection.calcTP()

		print "\t".join( map (str, [  \
				"FILTER:", \
				tp, float(tp)/ baseTP, hitCollection.calcFPmatches(), \
				tpprot, float(tpprot)/ baseTPprot, hitCollection.calcFP(), "0.05,0.90", \
				self.spectrumCollection["number spectra"],\
				filteredSpecColl["number spectra"], \
				1.0-(filteredSpecColl["number spectra"]/ \
					self.spectrumCollection["number spectra"]), \
				cfilter["threshold2"]  ])) 
		
		return 0.0

	def calcArea(self, classifier, testData):

		threshholds = [ 0.0, 0.05, 0.1, 0.15, 0.2, 0.25, \
			0.3, 0.35, 0.4, 0.45, 0.5 ]   
		baseTP = baseTPprot = 0
		area = 0.0
		print "\t".join(["FILTER:","TruePosPept","TruePosPeptRate","FalsePosPept",\
			"TruePosProt", "TruePosProtRate", "FalsePosProt", \
			"Threshold","NumOrig", "NumKept", "PercentRemoved", "ClassThreshold"])

		cfilter  = CstateFilter()
		cfilter["classifier"] = classifier

		for t in threshholds:
			print >>sys.stderr, "Filtering with threshold = ", t 
			cfilter["threshold2"] = cfilter["threshold3"] = t
			filteredSpecColl = cfilter.filterSpectra(self.spectrumCollection,\
					specDataSet=testData)
			pseudoMatch = self.pseudoGen.generateMatches(filteredSpecColl)
			pseudoMatch.outputToFilename("".join(["pseudo.",str(t),".sqt"]))
			hitCollection  = self.dtaHitGen.generateHits(pseudoMatch)

			tp = hitCollection.calcTPmatches()
			tpprot = hitCollection.calcTP()
			if t == 0.0:
				baseTP = tp
				baseTPprot = tpprot
			else:
				area  = area  + 0.1 * float(hitCollection.calcTPmatches()) / baseTP

			print "\t".join( map (str, [  \
					"FILTER:", \
					tp, float(tp)/ baseTP, hitCollection.calcFPmatches(), \
					tpprot, float(tpprot)/ baseTPprot, hitCollection.calcFP(), t, \
					self.spectrumCollection["number spectra"],\
					filteredSpecColl["number spectra"], \
					1 - t, cfilter["threshold"]  ])) 
		
		return area
			

#-------
# Main
#-------
if __name__ == '__main__':
	if len(sys.argv) < 3: 
		print """Usage: python %s 
	-r <cvector-train-file> 
	-s <sqt-test-filename> 
	-m <ms2-test-filename> 
	-t <cvector-test-file>
	[-k gaussian|polynomial]
	[-w gaussian width]
	[-d polynomial degree]
	[-c c value]
""" % (sys.argv[0])
		sys.exit(-1)

	optlist,args = getopt.getopt(sys.argv[1:],'r:s:m:t:k:w:d:c:')

	sqt =  "SQT.sqt"
	ms2 = "MS2"
	train = "CVEC-TRAIN"
	test = "CVEC-TEST"
	kernel = 'polynomial'
	width = 10 
	cVal = 10
	deg = 3
	
	for option_name,option_value in optlist:
		if option_name == '-r':
			train=option_value
		if option_name == '-s':
			sqt=option_value
		if option_name == '-m':
			ms2=option_value
		if option_name == '-t':
			test=option_value
		if option_name == '-k':
			kernel=option_value
		if option_name == '-w':
			width=float(option_value)
		if option_name == '-d':
			deg=int(option_value)
		if option_name == '-c':
			cVal=float(option_value)

	c = ChargeSieve()
	c.dtaHitGen = DtaSelectHitGenerator()
	c.dtaHitGen["params"] = DtaSelectParams()
	
	print >>sys.stderr, "Parsing .sqt file %s." % sqt
	matchCollection = SqtMatchCollection()
 	matchCollection.parseFromFilename(sqt)

	c.pseudoGen = PseudoSequestMatchGenerator(matchCollection)

	print >>sys.stderr, "Parsing .ms2 file %s." % ms2
	spectrumCollection = Ms2SpectrumCollection(ms2)
	spectrumCollection.allSpectra()

	mcfilt = Filter()
	c.spectrumCollection \
		= mcfilt.filterSpectra(spectrumCollection, keep = 'multi-charge')

	print >>sys.stderr, "Parsing training file %s." % train
 	DataSet = datafunc.SparseCDataSet
	trainData = DataSet(train, labelsColumn = 1) 

	#--
	rs  = preproc.Rescale()
	rs.train(trainData)
	# trainData.normalize(1)

	print >>sys.stderr, "Parsing testing file %s." % test
	testData = DataSet(test, idColumn = 0 , labelsColumn = 1) 
	#--
	rs.test(testData)
	#--
	# testData.normalize(1)

	
	if kernel in ('Polynomial','polynomial','poly'):
		print >>sys.stderr, "Using polynomial kernel of degree %.3f." % deg
		trainData.attachKernel('poly', degree = deg)
	elif kernel in ('Gaussian','gaussian','rbf'):
		print >>sys.stderr, "Using Gaussian kernel of width %.3f." % width
		trainData.attachKernel('gaussian', gamma = width)

	# train SVM out here 
	s = svm.SVM(C=cVal)
	p = Platt(s)
	p.train(trainData)  


	# p =  Platt(s)
	# p.train(trainData) 
	area = c.calcSinglePlatt(p, testData)  

	
	print "KERNEL = %s, C = %.3f, DEG = %i, WIDTH = %i, AREA = %.3f" % \
		(kernel, cVal, deg, width, area)

