#---------------------------------
import os
import random
import sys
import re
import time
import cPickle
import generatePeptides
import hmmParams
import tempfile
import popen2

#---------------------------------
VERBOSE = True

#---------------------------------
class Runner:

  #---------------------------------
  def run(self):
    cmd = self.constructCmdLine()
    if VERBOSE == True:
      print >>sys.stderr,  "Running %s" % cmd
    popper = popen2.Popen3(cmd, True) # True means capture STDERR
    self.stdout = popper.fromchild.read()
    popper.fromchild.close()
    self.stderr = popper.childerr.read()
    popper.childerr.close()
    if VERBOSE == True:
      for line in self.stderr:
        print >>sys.stderr, line,
    popper.wait()

#---------------------------------
class NormalizePfileRunner(Runner):

  #---------------------------------
  def __init__(self, trainParams=True, inputPfile=None, outputPfile=None, statisticsFile="foo"):
    self.trainParams = trainParams
    self.inputPfile = inputPfile
    self.outputPfile = outputPfile
    self.statisticsFile = statisticsFile

  #---------------------------------
  def run(self):
    startTime = time.time()
    Runner.run(self)
    endTime = time.time()
    if VERBOSE == True:
      print >>sys.stderr, "Running pfile_create took around: %.3f" \
        % (endTime - startTime)

  #---------------------------------
  def constructCmdLine(self): 

    # GAUSSIAN2 add pfile_cat or pfile_gaussian_normalize
    if self.trainParams == True:
      statisticsPrefix = "-oh"
    else:
      statisticsPrefix = "-ih"
    cmdLine =  "pfile_gaussian -oswap -iswap -i %s -h 500 -o %s %s %s; "  \
          % (self.inputPfile, self.outputPfile, statisticsPrefix, self.statisticsFile)

    return cmdLine

#---------------------------------
class CreatePfileRunner(Runner):

  #---------------------------------
  def __init__(self, parameterFile=None, ints=9, floats=3, numFiles=18, \
      outputDir="foo", ionFilesType="single"):
    self.parameter_file = parameterFile
    self.outputDir = outputDir
    self.numInts = ints
    self.numFloats = floats
    self.numFiles = numFiles
    self.ionFilesType = ionFilesType

  #---------------------------------
  def run(self):
    startTime = time.time()
    Runner.run(self)
    endTime = time.time()
    if VERBOSE == True:
      print >>sys.stderr, "Running pfile_create took around: %.3f" \
        % (endTime - startTime)

  #---------------------------------
  def constructCmdLine(self): 

    cmdLine = ""
    for filePrefix in range(self.numFiles):
      cmdLine +=  \
        "pfile_create -oswap -i %s/%s-%s.prepfile -o %s/%s-%s.pfile -b -l %i -f %i; "  \
        % (self.outputDir, filePrefix, self.ionFilesType, self.outputDir, filePrefix, \
        self.ionFilesType, self.numInts, self.numFloats)

    return cmdLine

#---------------------------------
class CreateIonFilesRunner(Runner):

  #---------------------------------
  def __init__(self, spectrum, peptides=None, parameterFile=None, \
      outputDir="foo", startingSentenceIdx=0, ionFilesType="single"):
    self.parameter_file = parameterFile
    self.peptides = peptides
    self.spectrum = spectrum
    self.outputDir = outputDir
    self.startingSentenceIdx = startingSentenceIdx
    self.ionFilesType = ionFilesType

  #---------------------------------
  def run(self):
    self.setup()
    startTime = time.time()
    Runner.run(self)
    endTime = time.time()
    if VERBOSE == True:
      print >>sys.stderr, "Running create_psm_files took around: %.3f" \
        % (endTime - startTime)
    os.unlink(self.peptideFile)
    os.unlink(self.spectrumFile)

  #---------------------------------
  def constructCmdLine(self): 

    # special suffix prepfile
    cmdLine = "create_psm_files %s %s %s %s %s --starting-sentence-idx %i; " \
      % (self.peptideFile, self.spectrum["first scan"], self.spectrumFile, \
          self.outputDir, self.ionFilesType, self.startingSentenceIdx) 
    return cmdLine

  #---------------------------------
  def setup(self):
    peptideFile = "peptide-%i.txt" % int(random.random() * 10000)
    fh = open(peptideFile, "w")
    for peptide in self.peptides:
      fh.write(peptide + "\n")
    fh.close()

    spectrumFile = "spectrum-%i.ms2" % int(random.random() * 10000)
    fh = open(spectrumFile, "w")
    fh.write(str(self.spectrum))
    fh.close()
    self.peptideFile = peptideFile
    self.spectrumFile = spectrumFile

#---------------------------------
class GetSpectrumRunner(Runner):

  #---------------------------------
  def __init__(self, scan, ms2_spectrum_filename, output_file_name):
    self.scan = scan
    self.ms2_spectrum_filename = ms2_spectrum_filename
    self.output_file_name = output_file_name

  #---------------------------------
  def run(self):
    startTime = time.time()
    Runner.run(self)
    endTime = time.time()
    if VERBOSE == True:
      print >>sys.stderr, "Running search_spectrum took around: %.3f" \
        % (endTime - startTime)

  #---------------------------------
  def constructCmdLine(self): 
    myDict = self.__dict__
    cmdLine = "crux-get-ms2-spectrum --verbosity 0 "
    cmdLine += " %s" % self.scan
    cmdLine += " %s" % self.ms2_spectrum_filename
    cmdLine += " %s" % self.output_file_name
    return cmdLine


#---------------------------------
class SearchSpectraRunner(Runner):

  #---------------------------------
  def __init__(self, ms2_spectrum_filename, protein_input_filename, \
      parameterFile="crux.params"):
    self.score_type  = 'xcorr'
    self.output_mode = "all"
    self.number_decoy_set = "0"
    self.spectrum_charge = "2"
    self.sqt_output_file = "temp.sqt"
    self.parameter_file = parameterFile
    self.protein_input_filename = protein_input_filename
    self.ms2_spectrum_filename = ms2_spectrum_filename
    self.verbosity = "0"

  #---------------------------------
  def run(self):
    startTime = time.time()
    Runner.run(self)
    endTime = time.time()
    if VERBOSE == True:
      print >>sys.stderr, "Running search_spectrum took around: %.3f" \
        % (endTime - startTime)

  #---------------------------------
  def constructCmdLine(self): 
    myDict = self.__dict__
    cmdLine = "crux-search-for-matches"
    for key in myDict.keys():
      if key in ['protein_input_filename', 'ms2_spectrum_filename']: continue
      option = re.sub('_', '-', key)
      if myDict[key] == True or myDict[key] == False:
        if myDict[key] == True: 
          cmdLine += " --%s " % option
        elif myDict[key] == False:
          pass
        continue

      cmdLine += " --%s %s " % (option, myDict[key])

    cmdLine += " %s" % self.ms2_spectrum_filename
    cmdLine += " %s" % self.protein_input_filename 
    return cmdLine

  #---------------------------------
  def getPeptides(self, maxPeptides=hmmParams.MAX_PEPS):
    peptides = {}
    sqt_output_file = open(self.sqt_output_file)
    for line in sqt_output_file:
      if line.startswith('#'): continue
      if not line.startswith('M'): continue
      line = line.rstrip('\n')
      fields = line.split()
      if len(fields) < 4: continue
      # TODO so that we are evaluating the actual peptide
      # peptide = re.sub('I','L',fields[6])
      peptide = fields[9].split(".")[1]
      peptides[peptide] = 1
      if len(peptides) >= maxPeptides: break
    sqt_output_file.close()
    return peptides.keys()

#---------------------------------
class GeneratePeptideRunner(Runner):
  def __init__(self, protein_input_filename):
    self.output_sequence = 'T'
    self.min_mass = 200.0
    self.max_mass = 7200.0
    self.min_length = 6
    self.max_length = 40
    self.cleavages = 'tryptic'
    # self.cleavages = 'all'
    # self.missed_cleavages  = 'T'
    self.sort = 'none'
    self.parameter_file = "%s/models/crux.params" % os.getenv("RIPTIDE_HOME")
    self.isotopic_mass = 'average'
    self.verbosity  = 0
    self.use_index =  'F' 
    self.protein_input_filename = protein_input_filename
    self.redirect = "crux-generate-peptides.redirect"


  #---------------------------------
  def run(self):
    startTime = time.time()
    cmdline = self.constructCmdLine()
    if VERBOSE == True:
      print >>sys.stderr, "Cmd = %s" % cmdline
    os.system(cmdline)
    endTime = time.time()
    if VERBOSE == True:
      print >>sys.stderr, "Running crux-generate-peptides took around: %.3f" \
        % (endTime - startTime)

  #---------------------------------
  def constructCmdLine(self): 
    myDict = self.__dict__
    cmdLine = "crux-generate-peptides"
    for key in myDict.keys():
      if key in  ['protein_input_filename', 'redirect']: continue
      option = re.sub('_', '-', key)
      if myDict[key] == True or myDict[key] == False:
        if myDict[key] == True: 
          cmdLine += " --%s " % option
        elif myDict[key] == False:
          pass
        continue

      cmdLine += " --%s %s " % (option, myDict[key])

    return cmdLine + " %s > %s " % (self.protein_input_filename, self.redirect)

  #---------------------------------
  def getPeptides(self):
    peptides = {}
    fh = open(self.redirect)
    for line in fh:
      if line.startswith('#'): continue
      line = line.rstrip('\n')
      fields = line.split()
      if len(fields) < 5: continue
      # TODO so that we are evaluating the actual peptide
      # peptide = re.sub('I','L',fields[4])
      peptide = fields[4]
      peptides[peptide] = 1
    fh.close()
    return peptides.keys()

#---------------------------------
if __name__ == '__main__':
  # KEEP following
#  if len(sys.argv) < 3:
#    print >>sys.stderr, "Usage: %s <pickledpsm> <fasta-file>" % sys.argv[0]
#    sys.exit()
#
#  import MatchCollection
#  psm = cPickle.load(open(sys.argv[1]))
#  fastaFile = sys.argv[2]
#  
#  suid2peps = {}
#  for match in psm["matches"]:
#    spectrum = match.spectrum
#    runner = GeneratePeptideRunner(fastaFile)
#    generatePeptides.setRanges(spectrum, runner)
#    runner.run()
#    peps = runner.getPeptides()
#    suid2peps[spectrum.uid()] = len(peps)
#  
#  output = os.path.basename(sys.argv[1]).split('.')[0] + "," + \
#      os.path.basename(sys.argv[2]).split('.')[0]
#  cPickle.dump(suid2peps, open(output + ".pickledpeps", "w")) 

  import SpectrumCollection
  runner = GetSpectrumRunner("1", "foo.ms2", "foo-out.ms2")
  runner.run()
  spectrumColl = SpectrumCollection.Ms2SpectrumCollection("foo-out.ms2")
  spectrum = spectrumColl.allSpectra()[0]
  print spectrum.__str__()
