#!/usr/bin/python
#$Revision: 1.21 $
import getopt, sys, copy, os
from MatchCollection import *
from HitCollection import *
from SpectrumCollection import *
from Utils import *
import datafunc
import svm
import multi
import assess
import myio
import ker
import classifiers
from composite import *
FALSE=0
TRUE=1

def empty_copy(object):
	class Empty: pass
	newcopy = Empty()
	newcopy.__class__ = object.__class__
	newcopy.__init__()
	return newcopy

#---------
# Filter should have
#  - SVMparams object
#  - Cstate threshold params
# and have a method to set them within the Filter
# thus we need a wrapper for the charge-vector generatio code and its parameters
# 
# then the pseudo-code for the first incarnation would be:
# cstateFilter = CstateFilter()
# cmat = calcCvector(SpectrumCollection)
# svmParams = SVMparams()  # default at first
# add them to the Filter
# cstateFilter["charge matrix"]=cmat
# cstateFilter["svm params"]=svmparms
# run the svm 
# should be some data structure that contains confidence levels for each
# svm. then just apply different thresholds to these confidences.
# first make it symmetrical
# iterate throught different stringencies of thresholds.
# should probably calculate percent eliminated rather than explicit threshold
# at first??
# for threshold in cstateparmsCollection:
#   SpectrumCollection filtspecs = applyFilter(SpectrumCollection)
#   sqtMatches = PseudoSequestMatchGenerator.generateMatches(filtspecs) 
#   hits = DtaSelectHitGenerator(sqtMatches)
#   hits.calcTP() 
#   output the True Positives, False Positives,
#   the threshold and % of spectrum eliminated for each charge
#---------

def dta2scan(patternID):
	a = patternID.split(".")
	return depad(a[1])

class Filter:
	"Class that filters spectra."
	# To be implemented in the sub-classes
	def filterSpectra(self, specColl, keep = 'multi-charge'):
		filteredSpectra = SpectrumCollection()
		if keep=="multi-charge":
			# iterate over set of spectra
			for s in specColl.allSpectra():
				if int(s["charge"]) != int(1):
					filteredSpectra.appendSpectrum(s)
		else:
			print >>sys.stderr, "Functionality not supported."

		return filteredSpectra

#-- 

class TwoToThreeFilter:

	def __init__(self):
		self.data = {}
		self.data["threshold2"] = 0
		self.data["threshold3"] = 0

	def __getitem__(self, key):
		return self.data[key]

	def __setitem__(self, key, item):
		self.data[key] = item		

	def __str__(self):
		pass


	def filterSpectra(self, specColl, specDataSet=None):
		"Filters a SpectrumCollection specColl using the built-in heuristic." 

		patternID = specDataSet.labels.patternID
		for i in range(len(patternID)):
			pattern = specDataSet.getPattern(i)
			# if pattern[0] > blah and pattern[1] > blah
			s = specColl.getSpectrum
		return filteredSpectra


class CstateFilterPlatt:
	"Contains the information necessary to filter by charge-state."

	def __init__(self):
		self.data = {}
		self.data["threshold2"] = 0
		self.data["threshold3"] = 0
		self.data["classifier"]=""

	def __getitem__(self, key):
		return self.data[key]

	def __setitem__(self, key, item):
		self.data[key] = item		

	def __str__(self):
		pass

	def filterSpectra(self, specColl, specDataSet):
		"Filters a SpectrumCollection specColl using the Platt classifier."

		patternID = specDataSet.labels.patternID

		scanID = []
		if re.compile(".dta$").search(patternID[0], 1):
			scanID = map(dta2scan, patternID)
		else:
			scanID = map(depad, patternID) 
		
		print >>sys.stderr, "Regenerating function values"
		# generate the function values for this data set anew.
		funcDict = {}
		for idx in range(len(specDataSet)):
			funcDict[int(scanID[idx])] = self["classifier"].decisionFunc(specDataSet, idx)
		
		print >>sys.stderr, "Keeping spectra in dataset only" 
		# only keep those spectra that are in the dataSet (i.e. in the funcDict). 
		# this is a hack for CV.
		spectra = filter(lambda x: funcDict.has_key(int(x["first scan"])), specColl.allSpectra())

		filteredSpectra = empty_copy(specColl)
		filteredSpectra["headerLines"] = specColl["headerLines"]
		filteredSpectra.addFilterHistory("charge_czar")
		
		elim_count=0
		
		for key in funcDict.keys(): # FIX
			print >>sys.stderr, key, "\t", funcDict[key] # FIX

		for idx in range(len(spectra)): 
			# get the spectrum from the collection
			s = spectra[idx]
			f = float(funcDict[ int(s["first scan"]) ])
			# print "F=" + str(f)

			if int(s["charge"]) == 1:
				filteredSpectra.appendSpectrum(s)
			elif f < self["threshold2"] and int(s["charge"]) == 2:
				filteredSpectra.appendSpectrum(s)
			elif self["threshold2"] <= f and f <= self["threshold3"]:
				filteredSpectra.appendSpectrum(s)
			elif self["threshold3"] < f and int(s["charge"]) == 3:
				filteredSpectra.appendSpectrum(s)
			else:
				# we are eliminating this spectrum
				elim_count+=1

		print >>sys.stderr, "Done filtering. Eliminated %i of %i spectra" % \
			(elim_count, len(spectra))
		return filteredSpectra



class CstateFilter:
	"Contains the information necessary to filter by charge-state."

	def __init__(self):
		self.data = {}
		self.data["threshold2"] = 0
		self.data["threshold3"] = 0
		self.data["classifier"]=""
		self.data["threshold"] = 0

	def __getitem__(self, key):
		return self.data[key]

	def __setitem__(self, key, item):
		self.data[key] = item		

	def __str__(self):
		pass

	def filterSpectra(self, specColl, specDataSet):
		"Filters a SpectrumCollection specColl using the trained svm classifer."

		patternID = specDataSet.labels.patternID

		scanID = []
		if re.compile(".dta$").search(patternID[0], 1):
			scanID = map(dta2scan, patternID)
		else:
			scanID = map(depad, patternID) 
		
		print >>sys.stderr, "Regenerating function values"
		# generate the function values for this data set anew.
		funcDict = {}
		for idx in range(len(specDataSet)):
			funcDict[scanID[idx]] = self["classifier"].decisionFunc(specDataSet, idx)

		print >>sys.stderr, "Keeping spectra in dataset only" 
		# only keep those spectra that are in the dataSet (i.e. in the funcDict). 
		# this is a hack for CV.
		spectra = filter(lambda x: funcDict.has_key(x["first scan"]), specColl.allSpectra())

		print >>sys.stderr, "Sorting spectra in dataset only" 

		# sort the spectra by the function absolute values to set threshold
		spectra.sort(lambda x,y: \
			cmp( \
				abs(funcDict[x["first scan"]]), \
				abs(funcDict[y["first scan"]]) 	\
			) \
		)
		
		# need to set the threshold value. FIX forced symmetric threshold.
		if (self["threshold2"] == 0.0):
			thresh_idx = len(spectra) - 1
		else: 
			thresh_idx =  int((1 - 2.0 * self["threshold2"]) * len(spectra))
		thresh = abs(funcDict[spectra[thresh_idx]["first scan"]])
		print >>sys.stderr, "Thresh idx = %i, threshold2 = %.3f, thresh = %.3f" % \
			(thresh_idx, self["threshold2"], thresh)

		self["threshold"] = thresh

		# in the +2only, ambiguous, or +3 only ranges
		filteredSpectra = SpectrumCollection()
		elim_count=0

		for idx in range(len(spectra)): 
			# get the spectrum from the collection
			s = spectra[idx]
			if int(s["charge"]) == 1:
				filteredSpectra.appendSpectrum(s)
			elif funcDict[s["first scan"]] < - thresh and int(s["charge"]) == 2:
				filteredSpectra.appendSpectrum(s)
			elif - thresh <= funcDict[s["first scan"]] and \
				funcDict[s["first scan"]] <= thresh:
				filteredSpectra.appendSpectrum(s)
			elif thresh < funcDict[s["first scan"]] and int(s["charge"]) == 3:
				filteredSpectra.appendSpectrum(s)
			else:
				elim_count+=1
				# we are eliminating this spectrum

		print >>sys.stderr, "Done filtering. Eliminated %i of %i spectra" % \
			(elim_count, len(spectra))
		return filteredSpectra


#-------
# Main
#-------
if __name__ == '__main__':
	if len(sys.argv) != 4:
		print "Usage: %s <train-filename> <test-filename> <fakeSpecCollection>" % (sys.argv[0])
		sys.exit(1)

 	cfilter = CstateFilterPlatt()	
	DataSet = datafunc.SparseCDataSet
	trainData = DataSet(sys.argv[1], labelsColumn = 1) 
	trainData.normalize(1)
	s = svm.SVM()
	p = Platt2(s)
	p.train(trainData)
	
	cfilter["classifier"] = p
	cfilter["threshold2"] = 0.05
	cfilter["threshold3"] = 0.90
	testData = DataSet(sys.argv[2], idColumn = 0 , labelsColumn = 1) # first column is ids

	testSpecColl = Ms2SpectrumCollection(sys.argv[3])
	testSpecColl.allSpectra() # FIX goofy

	filteredSpecColl = cfilter.filterSpectra(testSpecColl, specDataSet=testData)
	print "lenfilt = ", len(filteredSpecColl.allSpectra())
	print "lenunfilt = ", len(testSpecColl.allSpectra())
	print "percent filtered = ", 100 * ( 1.0 - 
		len(filteredSpectraColl.allSpectra()) / len(testSpecColl.allSpectra()))


# results = svm.stratifiedCV(data)
# then, the decision functions in r will
# tbe the cross-validated training outputs,
# i.e. the results of training on N-1 splits
# and then applying both classifiers (svm and
# probability post-processing) to the remaining
# Nth split (At least I think the decision functions
# will be in r; they will also be in d).


