import sys, os, os.path, cPickle
import Spectrum
import HmmData, Mass
import re, math
from hmmParams import *

# 2010-04
OUTPUT_AUX = False
MAX_HMMTRAIN = 1000000
ABS_MIN_LEGIT = -100000
#------------------------------------------------------------------------------
def readEMoutput(inputFile):

  segProbs   = []
  frameProbs = []

  minSegProb =  999999.
  maxSegProb = -999999.
  sumSegProb = 0.

  minFrmProb =  999999.
  maxFrmProb = -999999.
  sumFrmProb = 0.

  numProb = 0

  aFile = file(inputFile)
  for aLine in aFile:
    if aLine.startswith ( 'Segment' ):
      i1 = 8
      i2 = aLine.find(' ',8) - 1
      segNumber = int(aLine[i1:i2])

      i1 = aLine.find(')) = ') + 4
      i2 = aLine.find(',', i1)
      currProb = float(aLine[i1:i2])
      segProbs.append(currProb)

      if ( minSegProb > currProb ): minSegProb = currProb
      if ( maxSegProb < currProb ): maxSegProb = currProb
      sumSegProb = sumSegProb + currProb

      i1 = aLine.find('per frame =') + 11
      i2 = aLine.find(',', i1)
      currProb = float(aLine[i1:i2])
      frameProbs.append(currProb)

      if ( minFrmProb > currProb ): minFrmProb = currProb
      if ( maxFrmProb < currProb ): maxFrmProb = currProb
      sumFrmProb = sumFrmProb + currProb

      numProb = numProb + 1

  aFile.close()

  avgSegProb = sumSegProb / float(numProb)

  avgFrmProb = sumFrmProb / float(numProb)

  numBins = 53
  minProb = -200.
  deltaProb = 4.

  segBins = [0] * numBins
  for ii in range(len(segProbs)):
     if ( segProbs[ii] < 990. ):
       currProb = segProbs[ii]
       ibin = int ( (currProb - minProb)/deltaProb )  +  1
       if ( ibin < 0 ): ibin = 0
       if ( ibin > (numBins-1) ): ibin = (numBins-1)
       segBins[ibin] = segBins[ibin] + 1

  for ibin in range(len(segBins)):
    if ( segBins[ibin] > 0 ):
      binProb = (float(ibin-1) + 0.5) * deltaProb  +  minProb

  return segProbs

MODEL_DIR = "%s/models/" % os.getenv("RIPTIDE_HOME")


#---------------------------------------------------------------------------
# returns means, vars, weights
#---------------------------------------------------------------------------
def parse_gaussian_mixture(paramFile):
  fh = open(paramFile)
  MEAN_PREFIX = "mean"
  VAR_PREFIX  = "cvar"
  WEIGHT_PREFIX = "gmw"
  means = []; vars = []; weights = [];
  for line in fh:
    line = line.rstrip('\n')
    if line.startswith(MEAN_PREFIX):
      fields = line.split()
      if len(fields) == 3:
        means.append(float(fields[2]))
    elif line.startswith(VAR_PREFIX):
      fields = line.split()
      vars.append(float(fields[2]))
    elif line.startswith(WEIGHT_PREFIX):
      fields = line.split()
      weights = map(float, fields[2:])
  fh.close()
  return means, vars, weights

#---------------------------------------------------------------------------
def clusterify(file):
  return re.sub('/atm/','/n/', file)

#---------------------------------------------------------------------------
def constructTrainCmd(inputFiles, model, paramFile):
  directory = MODEL_DIR + "model-%s/params" % model.name
  paramFilePrefix = os.path.basename(paramFile).split('.')[0]
  if model.format == "pfile":
    gmtkInputFilename = inputFiles[0]
  else:
    gmtkInputFilename = "gmtk.testlist"
    fh = open(gmtkInputFilename, "w")
    for file in inputFiles:
      fh.write(file)
    fh.close()

  # print >>sys.stderr, "Running GMTK"
  outputFile = os.path.abspath("trainGMTK-%s-%s.out" % \
      (model.name, paramFilePrefix))
  outputFile = clusterify(outputFile)
  gmtkCommand = """gmtkEMtrainNew -strFile %s/model_train.str -of1 %s -nf1 %i\
  -ni1 %i -fmt1 %s -inputMasterFile %s/model_train.master.EM -lldp 0.001 \
  -dirichletPriors T -maxEmIters 100 -outputTrainableParameters %s/%s \
  -mcvr 1e+999 \
  -mcsr 1e+999 \
  -objsNotToTrain %s/nonTrainable.list -verbosity 15 >& %s""" %  \
  (directory, gmtkInputFilename, model.numFloats, model.numInts, \
      model.format, directory, directory, paramFile, directory, outputFile)

  # try taking out the var floor
  # -varFloor 1e-02 \
  # no vanishing because of Dirichlet priors

  # print >>sys.stderr, gmtkCommand
  #pvec = os.popen(gmtkCommand)
  #ret = pvec.close()
  # print >>sys.stderr, "Finished running gmtkEMtrainNew"
  #if ret != None: raise Exception, "Problems running GMTK command: %s. Returned %i" % \
  #    (gmtkCommand, ret)
  return gmtkCommand

#---------------------------------------------------------------------------
def trainGMTK(inputFiles, model, paramFile):
  gmtkCommand = constructTrainCmd(inputFiles, model, paramFile)

    # print >>sys.stderr, gmtkCommand
  pvec = os.popen(gmtkCommand)
  ret = pvec.close()
  # print >>sys.stderr, "Finished running gmtkEMtrainNew"
  if ret != None: raise Exception, \
    "Problems running GMTK command: %s. Returned %i" % (gmtkCommand, ret)

  # Note coupled to constructTrainCmd above!
  fullParamFileName = \
      MODEL_DIR + "model-%s/params/%s" % (model.name, paramFile)
  return fullParamFileName

#---------------------------------------------------------------------------
global COUNTER
COUNTER = 0

#---------------------------------------------------------------------------
def constructTestCmd(inputFiles, model, paramFile):
  directory = MODEL_DIR + "model-%s/params" % model.name
  paramFilePrefix = os.path.basename(paramFile).split('.')[0]
  global COUNTER
  COUNTER += 1
  idx = COUNTER
  gmtkInputFilename = inputFiles[0]

  outputFile = os.path.abspath("%i-%s-%s.testout" \
      % (idx, model.name, paramFilePrefix))
  outputFile = clusterify(outputFile)
  gmtkCommand = """gmtkJT -strFile %s/model_test.str \
		-of1 %s -nf1 %s -ni1 %s -fmt1 %s \
		-inputMasterFile %s/model_test.master.JT \
		-inputTrainableParameters %s/%s \
		-verbosity 15 >& %s""" % (directory, gmtkInputFilename, model.numFloats,
        model.numInts, model.format, directory, directory, paramFile, \
         outputFile )

  return gmtkCommand, outputFile

#---------------------------------------------------------------------------
def testGMTK(inputFiles, model, paramFile):
  """Runs junction tree on the GMTK model with paramFile on
  inputFiles"""

  gmtkCommand, outputFile = constructTestCmd(inputFiles, model, paramFile)
  # print >>sys.stderr, gmtkCommand
  pvec = os.popen(gmtkCommand)
  ret = pvec.close()
  # print >>sys.stderr, "Finished running gmtkJT"
  if ret != None: 
    fh = open(outputFile)
    print >>sys.stderr, fh.read()
    fh.close()
    raise Exception, "Problems running GMTK command: %s. Returned %i" % \
        (gmtkCommand, ret)

  seg2prob = readEMoutput(outputFile)
  return seg2prob

#---------------------------------------------------------------------------
def testGMTKaccumulator(inputFiles, model, paramFile, kernel=KERNEL):
  directory = MODEL_DIR + "model-%s/params" % model.name

  seg2vector = []
  writeLogVals = GMTK_WRITE_LOG_VALS

  gmtkInputFilename = "gmtkInput.list"
  if model.format == "pfile":
    gmtkInputFilename = inputFiles[0]
  else:
    gmtkInputFilename = "gmtk.testlist"
    fh = open(gmtkInputFilename, "w")
    for file in inputFiles:
      fh.write(file)
    fh.close()

  # print >>sys.stderr, "Running GMTK"


    # TODO make this a construct test command rather than inspecting the
    # model. Acutally, we should put the construction of commands in the 
    # GMTKmodel object rather than here. But this will work for now! :)


  useFisherKernel = 'F'
  if kernel == 'fisher':
    useFisherKernel = 'T'

  outputFile = "trainGMTK.out"
  print >>sys.stderr, "Running gmtkKernel"
  gmtkCommand = """gmtkKernel -strFile %s/model_train.str -of1 %s -nf1 %s \
      -ni1 %s -fmt1 %s -inputMasterFile %s/model_test.master.JT \
  -inputTrainableParameters %s/%s -storeFeat %s -annotate F \
  -writeLogVals %s -objsNotToUtilize %s/nonTrainable.list \
  -verbosity 15 -fisherKernel %s > testGMTKaccumulator.out""" %  \
  (directory, gmtkInputFilename, model.numFloats, model.numInts, model.format, directory, \
   directory, paramFile, outputFile, writeLogVals, directory, useFisherKernel)

  print gmtkCommand
  pvec = os.popen(gmtkCommand)
  ret = pvec.close()

  if ret != None: 
    raise Exception, "Problems running GMTK command: %s. Returned %i" % \
        (gmtkCommand, ret)
  
  fh = open(outputFile); 
  for line in fh:
    vector = line.rstrip('\n')
    seg2vector.append(vector)
  fh.close()
  return seg2vector

#---------------------------------------------------------------------------
class IonFisherVectorCollection:
  """Object for holding a collection of PSMs and their various accumulator kernel
  vectors.
  """
  #-------------------------------------------------------------------------
  def __init__(self, model=None):
    self.d = {}
    self.model = model

  #-------------------------------------------------------------------------
  def __str__(self):
    string = ""
    for suid in self.d.keys():
      for pep in self.d[suid].keys():
        for ionType in self.d[suid][pep].keys():
          vector = self.d[suid][pep][ionType]
          string += "\t".join(map(str, [suid, pep, ionType, vector])) + "\n"

    return string
 
  #-------------------------------------------------------------------------
  def outputSvmString(self, fh, positive = True):
    if positive == True:
      positivityString = "1"
    else:
      positivityString = "-1"
    string = ""
    for suid in self.d.keys():
      for pep in self.d[suid].keys():
        longVector = ""
        for ionType in self.model.fileTypes:
          if ionType == 'p': continue  # FIX
          if not self.d[suid][pep].has_key(ionType): 
            raise SystemExit, "BUG - File types should be consistent!"
          vector = self.d[suid][pep][ionType]
          vector =  re.sub(" +", ",", vector)
          vector = re.sub("0.00000000000000000e\+00", "0", vector)
          longVector += "%s," % vector 
        outSuid = re.sub(",","-", suid)
        string = \
          ("%s.%s,%s,%s" % (outSuid, pep, positivityString, longVector)) 
        string = string.rstrip(',')
        string += "\n"
        fh.write(string)

  #-------------------------------------------------------------------------
  def svmString(self, positive = True):
    if positive == True:
      positivityString = "1"
    else:
      positivityString = "-1"
    string = ""
    for suid in self.d.keys():
      for pep in self.d[suid].keys():
        longVector = ""
        for ionType in self.model.fileTypes:
          if ionType == 'p': continue  # FIX
          if not self.d[suid][pep].has_key(ionType): 
            raise SystemExit, "BUG - File types should be consistent!"
          vector = self.d[suid][pep][ionType]
          vector =  re.sub(" +", ",", vector)
          vector = re.sub("0.00000000000000000e\+00", "0", vector)
          longVector += "%s," % vector 
        outSuid = re.sub(",","-", suid)
        string += \
          ("%s-%s,%s,%s" % (outSuid, pep, positivityString, longVector)) 
        string = string.rstrip(',')
        string += "\n"
    return string

  #-------------------------------------------------------------------------
  def setVector(self, suid, pep, ionType, vector):
    suid = str(suid); pep = str(pep); ionType = str(ionType); 
    vector = str(vector)
    if not self.d.has_key(suid): 
      self.d[suid] = {}
    if not self.d[suid].has_key(pep): 
      self.d[suid][pep] = {}
    self.d[suid][pep][ionType] = vector

  #-------------------------------------------------------------------------
  def getVector(self, suid, pep, ionType):
    suid = str(suid); pep = str(pep); ionType = str(ionType); 
    return self.d[suid][pep][ionType]

  #-------------------------------------------------------------------------
  def populate(self, psmCollection, ionType, vectors, positive = True):
    matches = psmCollection["matches"]

    for idx in range(len(matches)):
      match = matches[idx]
      suid = match.spectrum.uid()
      pep  = match.peptide
      # For CAMSI
      # locus = match["locus"]
      # pep += "-%s" % locus
      pep = pep.rstrip("\n")
      # print probs[idx]
      self.setVector(suid, pep, ionType, vectors[idx])

#---------------------------------------------------------------------------
class IonScoreCollection:
  """Object for holding a collection of PSMs and their various ion
  scores. 
  """
  #-------------------------------------------------------------------------
  def __init__(self, model=None):
    self.LOR      = 0
    self.POSITIVE = 1
    self.NEGATIVE = 2
    self.d = {}
    self.aux = {}
    self.lorsCalculated = False
    self.model = model

  #-------------------------------------------------------------------------
  def __str__(self):
    string = ""
    for suid in self.d.keys():
      for pep in self.d[suid].keys():
        for ionType in self.d[suid][pep].keys():
          scores = self.d[suid][pep][ionType]
          string += "\t".join(map(str, [suid, pep, ionType, scores[self.LOR], \
              scores[self.POSITIVE], scores[self.NEGATIVE] ]) ) + "\n"

    return string
 
  #-------------------------------------------------------------------------
  # TODO consolidate IonScoreCollection and IonFisherVectorCollection
  def outputSvmString(self, fh, positive = True):
    if positive == True:
      positivityString = "1"
    else:
      positivityString = "-1"
    for suid in self.d.keys():
      for pep in self.d[suid].keys():
        lors = []
        for ionType in self.model.fileTypes:
          if ionType == 'p': continue  # FIX
          if not self.d[suid][pep].has_key(ionType): 
            raise SystemExit, "BUG - File types should be consistent!"
          scores = self.d[suid][pep][ionType]
          lors.append(scores[self.LOR]) 
          lors.append(scores[self.POSITIVE]) 
          lors.append(scores[self.NEGATIVE]) 

        lors.append(int(len(pep))) 
        if (OUTPUT_AUX):
          lors.append(self.aux[suid]["logTIC"])
          lors.append(self.aux[suid]["numPeaks"])
          lors.append(abs(Mass.peptide_mass(pep, z=1) - self.aux[suid]["mass"]))

        outSuid = re.sub(",","-", suid)
        string = ("%s.%s,%s," % (outSuid, pep, positivityString)) + \
            ",".join(map(str,lors)) + "\n"
        fh.write(string)

  def svmString(self, positive = True):
    if positive == True:
      positivityString = "1"
    else:
      positivityString = "-1"
    string = ""
    for suid in self.d.keys():
      for pep in self.d[suid].keys():
        lors = []
        for ionType in self.model.fileTypes:
          if ionType == 'p': continue  # FIX
          if not self.d[suid][pep].has_key(ionType): 
            raise SystemExit, "BUG - File types should be consistent!"
          scores = self.d[suid][pep][ionType]
          lors.append(scores[self.LOR]) 
          lors.append(scores[self.POSITIVE]) 
          lors.append(scores[self.NEGATIVE]) 

        lors.append(int(len(pep))) 
        # 2010-04
        if (OUTPUT_AUX):
          lors.append(self.aux[suid]["logTIC"])
          lors.append(self.aux[suid]["numPeaks"])
          lors.append(abs(Mass.peptide_mass(pep, z=1) - self.aux[suid]["mass"]))

        outSuid = re.sub(",","-", suid)
        string += ("%s-%s,%s," % (outSuid, pep, positivityString)) + \
            ",".join(map(str,lors)) + "\n"

    return string

  #-------------------------------------------------------------------------
  def setScore(self, suid, pep, ionType, scoretype, score):
    suid = str(suid); pep = str(pep); ionType = str(ionType); 
    scoretype = int(scoretype); score = float(score);
    if not scoretype in [ self.LOR, self.POSITIVE, self.NEGATIVE ]:
      raise Exception, "Invalid scoretype %s" % str(scoretype)
    if not self.d.has_key(suid): 
      self.d[suid] = {}
    if not self.d[suid].has_key(pep): 
      self.d[suid][pep] = {}
    if not self.d[suid][pep].has_key(ionType): 
      self.d[suid][pep][ionType] = [ None, None, None ]

    self.d[suid][pep][ionType][scoretype] = score

  #-------------------------------------------------------------------------
  def setAux(self, suid, type, value):
    suid = str(suid); type = str(type);
    if not self.aux.has_key(suid): 
      self.aux[suid] = {}
    self.aux[suid][type] = value

  #-------------------------------------------------------------------------
  def getScore(self, suid, pep, ionType, scoretype):
    suid = str(suid); pep = str(pep); ionType = str(ionType); 
    scoretype = int(scoretype); 
    return self.d[suid][pep][ionType][scoretype]

  #-------------------------------------------------------------------------
  def lors(self):
    if self.lorsCalculated == True: return True
    for suid in self.d.keys():
      for pep in self.d[suid].keys():
        for ionType in self.d[suid][pep].keys():
          self.d[suid][pep][ionType][self.LOR] = \
          self.d[suid][pep][ionType][self.POSITIVE] - \
          self.d[suid][pep][ionType][self.NEGATIVE]
            
  #-------------------------------------------------------------------------
  def populateAux(self, psmCollection):
    matches = psmCollection["matches"]
    for idx in range(len(matches)):
      spectrum = matches[idx].spectrum
      spectrum.parsePeaks()
      suid = spectrum.uid()
      if (spectrum["total energy"] <= 0.0):
        self.setAux(suid, "logTIC", -10000000000.0)
      else:
        self.setAux(suid, "logTIC", math.log(spectrum["total energy"]))
      self.setAux(suid, "numPeaks", len(spectrum["peaks"]))
      self.setAux(suid, "mass", float(spectrum["assumed m"]))

  #-------------------------------------------------------------------------
  def populate(self, psmCollection, ionType, probs, positive = True):
    if positive == True: scoretype = self.POSITIVE
    else: scoretype = self.NEGATIVE
    matches = psmCollection["matches"]

    for idx in range(len(matches)):
      match = matches[idx]
      suid = match.spectrum.uid()
      pep  = match.peptide
      # For CAMSI
      # locus = match["locus"]
      # pep += "-%s" % locus
      pep = pep.rstrip("\n")
      # print probs[idx]
      self.setScore(suid, pep, ionType, scoretype, probs[idx])

    self.populateAux(psmCollection)

#---------------------------------------------------------------------------
def paramString(paramPrefix, ionType, positive = True):
  if positive == True: positivityString = "pos"
  else: positivityString = "neg"
  return "%s,%s,%s.params" % (paramPrefix, ionType, positivityString)


#---------------------------------------------------------------------------
global colsToRemoveDict
colsToRemoveDict = {}
colsToRemove = [2, 31, 32, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 47, 50,
  51, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 66, 78, 81, 87, 90, 93, 96,
  99, 102, 107, 112, 117, 122, 142, 160, 176, 177, 181, 182, 183, 184, 190, 
  191, 192, 193, 198, 199, 200, 201, 202, 208, 209, 210, 211, 217, 218, 219, 
  220, 226, 227, 231, 232, 233, 234, 240, 241, 242, 243, 248, 249, 250, 251, 
  252, 258, 259, 260, 261, 267, 268, 269, 270, 275, 276, 283, 284] 

for i in colsToRemove:
  # put in -2 because these vectors are numbers only, whereas the indices 
  # above were derived from vectors with a character and a class label
  colsToRemoveDict[i-2] = 1

#---------------------------------------------------------------------------
def fixFisherVectors(vectors):
  newVectors = []
  global colsToRemoveDict
  for vector in vectors:
    fields = vector.split(" ")
    newFields = []
    for idx in range(len(fields)):
      if not colsToRemoveDict.has_key(idx):
        newFields.append(fields[idx])
    newVector = ' '.join(newFields)
    newVectors.append(newVector)
  return newVectors

#---------------------------------------------------------------------------
def sionFisherKernel(psmCollection, model, paramPrefix, gmtkFiles=None):
  """
  calculate and return a IonFisherVectorCollection that contains the
  accumulator vectors the matches in psmCollection under the pos and neg trained
  versions of model for each ion type. paramPrefix is the prefix to the
  various parameter files for each ion type in the model params dir. 
  """

  assert(len(psmCollection["matches"])!=0, "Zero length matches")
  # instantiate our output --- the collection of scores foreach PSM and ionType
  vectors = IonFisherVectorCollection(model=model)

  # create the GMTK files if necessary
  positiveGmtkFiles = {}
  negativeGmtkFiles = {}
  if gmtkFiles is None:
    prefix = os.path.basename(psmCollection["file"]).split('.')[0] + "," + paramPrefix
    outDir = os.path.abspath(prefix) + ".hmmtraindirs"
    if not os.path.isdir(outDir): os.mkdir(outDir)
    # GAUSSIAN add training file parameter (use paramPrefix)
    model.writeIonFilesC(psmCollection, outDir, positiveGmtkFiles, positive=True)
    model.writeIonFilesC(psmCollection, outDir, negativeGmtkFiles, positive=False)
  else:
    print >>sys.stderr, "Sion lor called with GMTK files, no longer supported"
    sys.exit(1)
   
  # iterate through each ion type
  print model.fileTypes
  for ionType in model.fileTypes: 

    if ionType == 'p': continue  # FIX

    posParamFile = paramString(paramPrefix, ionType, positive = True)
    negParamFile = paramString(paramPrefix, ionType, positive = False)

    #--- normal kernel operations
    if KERNEL != "both":
      posVectors =  \
        testGMTKaccumulator(positiveGmtkFiles[ionType], model, posParamFile)
      negVectors =  \
        testGMTKaccumulator(negativeGmtkFiles[ionType], model, negParamFile)

      # if KERNEL == "fisher":
      #   posVectors = fixFisherVectors(posVectors)
      #   negVectors = fixFisherVectors(negVectors)

    #--- KERNEL == "both", special handling
    else:

      posVectors = testGMTKaccumulator(positiveGmtkFiles[ionType], 
          model, posParamFile, kernel="fisher")
      negVectors = testGMTKaccumulator(negativeGmtkFiles[ionType], 
          model, negParamFile, kernel="fisher")
      # posVectors = fixFisherVectors(posVectors)
      # negVectors = fixFisherVectors(negVectors)

      posVectorsAccum = testGMTKaccumulator(positiveGmtkFiles[ionType], 
          model, posParamFile, kernel="accumulator")
      negVectorsAccum = testGMTKaccumulator(negativeGmtkFiles[ionType], 
          model, negParamFile, kernel="accumulator")
      
      posVectors = map(lambda x: x[0] + x[1], zip(posVectorsAccum, posVectors))
      negVectors = map(lambda x: x[0] + x[1], zip(negVectorsAccum, negVectors))
      # print len(negVectors[0].split(' '))

    vectors.populate(psmCollection, ionType, posVectors, positive = True)
    vectors.populate(psmCollection, ionType, negVectors, positive = False)


  # return the IonFisherVectorCollection
  return vectors


#---------------------------------------------------------------------------
def constructTestCmds(psmCollection, model, paramPrefix, posGmtkFiles=None, negGmtkFiles=None):

  assert(len(psmCollection["matches"])!=0, "Zero matches")
  commands = []; files = { 'POS': {}, 'NEG': {} };

  # iterate through each ion type
  for ionType in model.fileTypes:
    if ionType == 'p': continue  # FIX

    # run testGMTK for the pos model ...
    paramFile = paramString(paramPrefix, ionType, positive = True)
    gmtkCommand, outputFile = \
        constructTestCmd(posGmtkFiles[ionType], model, paramFile)

    commands.append(gmtkCommand + "\n")
    files['POS'][ionType] = outputFile

    # ... and the neg model
    paramFile = paramString(paramPrefix, ionType, positive = False)
    gmtkCommand, outputFile = \
        constructTestCmd(negGmtkFiles[ionType], model, paramFile)

    commands.append(gmtkCommand + "\n")
    files['NEG'][ionType] = outputFile

  return commands, files
  
#-----------------------------------------------------------------------------
def calculateScores(psmCollection, files, model):

  # instantiate our output --- the collection of scores foreach PSM and ionType
  scores = IonScoreCollection(model=model)

  # iterate through each ion type
  for ionType in model.fileTypes:
    if ionType == 'p': continue  # FIX

    # put the pos probs into our IonScoreCollection
    posProbs = readEMoutput(files['POS'][ionType])
    scores.populate(psmCollection, ionType, posProbs, positive = True)

    # put the neg probs into our IonScoreCollection
    negProbs = readEMoutput(files['NEG'][ionType])
    scores.populate(psmCollection, ionType, negProbs, positive = False)

  # calculate our lors
  scores.lors()

  # return the IonScoreCollection
  return scores



#---------------------------------------------------------------------------
# FIX resolve inconsistencies
#---------------------------------------------------------------------------
def sionLor(psmCollection, model, paramPrefix, gmtkFiles=None):
  """
  calculate and return a IonScoreCollection that contains LORs, pos, and
  neg scores for the matches in psmCollection under the pos and neg trained
  versions of model for each ion type. paramPrefix is the prefix to the
  various parameter files for each ion type in the model params dir. 
  will create gmtkFiles from scratch, or optionally use the files in
  gmtkFiles, which *must* be in the same order as the matches in
  psmCollection
  """
  assert(len(psmCollection["matches"])!=0, "Zero matches")
  # instantiate our output --- the collection of scores foreach PSM and ionType
  scores = IonScoreCollection(model=model)

  # create the GMTK files if necessary
  print >>sys.stderr, "Writing ion files"
  positiveGmtkFiles = {}
  negativeGmtkFiles = {}
  if gmtkFiles is None:
    gmtkFiles = {}
    prefix = os.path.basename(psmCollection["file"]).split('.')[0] + "," + paramPrefix
    outDir = os.path.abspath(prefix) + ".hmmtraindirs"
    if not os.path.isdir(outDir): os.mkdir(outDir)
    # GAUSSIAN add training file parameter (use paramPrefix)
    model.writeIonFilesC(psmCollection, outDir, positiveGmtkFiles, positive=True)
    model.writeIonFilesC(psmCollection, outDir, negativeGmtkFiles, positive=False)
  else:
    print >>sys.stderr, "Sion lor called with GMTK files, no longer supported"
    sys.exit(1)
    

  # iterate through each ion type
  print >>sys.stderr, "Testing GMTK"
  for ionType in model.fileTypes:

    if ionType == 'p': continue  # FIX

    # run testGMTK for the pos model ...
    paramFile = paramString(paramPrefix, ionType, positive = True)
    print >>sys.stderr, "Testing GMTK positive model for ion type %s" % ionType
    posProbs = testGMTK(positiveGmtkFiles[ionType], model, paramFile)

    # put the pos probs into our IonScoreCollection
    scores.populate(psmCollection, ionType, posProbs, positive = True)

    # ... and the neg model
    paramFile = paramString(paramPrefix, ionType, positive = False)
    print >>sys.stderr, "Testing GMTK negative model for ion type %s" % ionType
    negProbs = testGMTK(negativeGmtkFiles[ionType], model, paramFile)

    # put the neg probs into our IonScoreCollection
    scores.populate(psmCollection, ionType, negProbs, positive = False)

  # calculate our lors
  scores.lors()

  # return the IonScoreCollection
  return scores
