#--------------------------------------
import os
import Spectrum
import Crux

from HmmData import *
from HmmOrder import *
from hmmParams import *
NUMBER_NOISE_PEAKS = 40
DEFAULT_MODEL_DIR = "%s/models" % os.getenv("RIPTIDE_HOME")

#--------------------------------------
class GMTKmodel:

  #--------------------------------------
  def __init__(self, name, rootDir=DEFAULT_MODEL_DIR, paramPrefix="default-prefix"):
    self.name = name
    self.format = 'ascii'
    self.numFloats = 0
    self.numInts   = 0
    self.dir = '%s/model-%s/' % (rootDir, name)
    self.paramPrefix = paramPrefix
    self.paramDir = '%s/params/' % self.dir

  #--------------------------------------
  def isBinary(self):
    if self.format == 'ascii': return False
    else: return True
    
  #--------------------------------------
  def setBinary(self, bool=True):
    if bool == True:
      self.format = 'binary'
    else:
      self.format = 'ascii'

  #--------------------------------------
  def getFormat(self):
    if self.format == 'ascii': return 'w'
    elif self.format == 'binary': return 'wb'
    else: raise SystemExit, "Strange format %s" % self.format

  #--------------------------------------
  def createDir(self, rootDir, match):
    dirName = rootDir + os.sep + str(match.spectrum.uid())
    if not os.path.isdir(dirName): os.mkdir(dirName)
    subDirName = dirName + os.sep + "%s.hmmtraindir" % match.peptide
    if not os.path.isdir(subDirName): os.mkdir(subDirName)
    return subDirName

#--------------------------------------
class GMTKmodelGaussianMixture:

  #--------------------------------------
  def __init__(self, name, **args):
    self.name = name
    self.format = 'ascii'
    self.numFloats = 1
    self.numInts   = 0

#--------------------------------------
class GMTKmodelIon(GMTKmodel):

  #--------------------------------------
  def __init__(self, name, **args):
    GMTKmodel.__init__(self, name, **args)
    self.fileTypes = []

  #--------------------------------------
  def prepareToWrite(self, match, rootDir):
    dirName = self.createDir(rootDir, match)
    theoretical = prepareSpectrum(match)
    peptideInformation = peptideInfo(theoretical.peptide, theoretical["charge"])
    return dirName, theoretical, peptideInformation

  #--------------------------------------
  def fileName(self, dirName, fileType):
    file = dirName + "/" + fileType + ".hmmtrain"
    return file  

#  #--------------------------------------
#  def nameIonFiles(self, match, rootDir, gmtkFiles):
#    """<match> is the PSM that will be output to an appropriately named
#    directory in <dir>
#    appends to the filelist in gmtkFiles dict (key=type, value=fileList)"""
#
#    dirName = self.createDir(rootDir, match)
#
#    for fileType in self.fileTypes:
#      # create the file for this ion type
#      outputFile = self.fileName(dirName, fileType)
#      if gmtkFiles.has_key(fileType): gmtkFiles[fileType].append(outputFile)
#      else: gmtkFiles[fileType] = [ outputFile ]

  #--------------------------------------
  # GAUSSIAN2 add an optional training file parameter if we need to cat the
  # test and training files together
  def writeIonFilesC(self, psmCollection, rootDir, gmtkFiles, positive=True): 

    # if the files already exist, just return
    outputFiles = {}
    self.nameIonFiles(rootDir, outputFiles)
    for key in outputFiles.keys():
      if os.path.isfile(outputFiles[key][0]):
        self.nameIonFiles(rootDir, gmtkFiles)
        print >>sys.stderr, "Returning, as output files seem to exist"
        return
      
    uidToPeptide = {}
    uidToSpectrumObject = {}

    for match in psmCollection["matches"]:
      uid = match.spectrum.uid()
      if uidToPeptide.has_key(uid):
        uidToPeptide[uid].append(match.peptide)
      else:
        uidToPeptide[uid] = [ match.peptide ]
        uidToSpectrumObject[uid] = match.spectrum

    for fileType in self.fileTypes:
      gmtkFiles[fileType] = []

    startingSentenceIdx = 0
    for uid in uidToPeptide.keys():
      peptides = uidToPeptide[uid]
      spectrum = uidToSpectrumObject[uid]
      runner = Crux.CreateIonFilesRunner(spectrum, peptides, \
          outputDir=rootDir, startingSentenceIdx=startingSentenceIdx,\
          ionFilesType=self.ionFilesType) 
      runner.run()
      startingSentenceIdx += len(peptides)
  
    pfileRunner = Crux.CreatePfileRunner(numFiles=len(self.fileTypes), \
      outputDir=rootDir, ints=self.numInts, floats=self.numFloats, \
      ionFilesType=self.ionFilesType)

    pfileRunner.run()
    
    # GAUSSIAN2 if we are testing, we might need to cat our test files
    # to our training files after converting to pfiles but before Gaussian
    # normalizing. Training files should be in a directory 
    # 1) cat each testing file to the (previously catted) negative and positive 
    # 2) cat both the negative and positive, and somehow save parameters
    # 3) do everything separately, but again somehow save parameters

    self.nameIonFiles(rootDir, gmtkFiles)

  #--------------------------------------
  # tightly coupled to CreatePfileRunner
  def nameIonFiles(self, rootDir, gmtkFiles, positive=True): 
   
    for idx in range(len(self.fileTypes)):
      type = self.fileTypes[idx]
      gmtkFiles[type] = ["%s/%i-%s.pfile" % \
          (os.path.abspath(rootDir), idx, self.ionFilesType)]

#--------------------------------------
class GMTKmodelSion(GMTKmodelIon):

  #--------------------------------------
  def __init__(self, name, charge=2, **args):
    GMTKmodelIon.__init__(self, name, **args)
    self.numFloats = 3
    self.numInts   = 9
    self.fileTypes = []
    self.ionFilesType = "single"
    self.descriptiveString = self.ionFilesType + "," + self.paramPrefix
    self.format = "pfile"

    # NOTE keep in synch with scorer.c single_ion_constraints

    for ion in ['b','y','a']:
      for charge in ["","+2"]:
        for neutral in ["", "-nh3", "-h2o"]:
          fileType = "%s%s%s" % (ion, charge, neutral)
          self.fileTypes.append(fileType)

#    if int(charge)==1:
#      charge = ""
#      for ion in ['b','y','a']:
#        for neutral in ["", "-nh3", "-h2o"]:
#          fileType = "%s%s%s" % (ion, charge, neutral)
#          self.fileTypes.append(fileType)

  #--------------------------------------
  def writeIonFiles(self, psmCollection, rootDir, gmtkFiles):

    for match in psmCollection["matches"]:
      dirName, theoretical, peptideInformation = \
          self.prepareToWrite(match, rootDir)

      for fileType in self.fileTypes:
        # create the file for this ion type
        outputFile = self.fileName(dirName, fileType)
        if gmtkFiles.has_key(fileType): gmtkFiles[fileType].append(outputFile)
        else: gmtkFiles[fileType] = [ outputFile ]
        fh = open(outputFile, self.getFormat() )
        # fileType in this case is an ion type
        peaks = theoretical.getPeaksOfType(fileType) 
        # output the peaks
        for peak in peaks: 
          fh.write(sionHmmString(peak, peptideInfo=peptideInformation, \
            binary=self.isBinary()))
        fh.close()


#--------------------------------------
class GMTKmodelSionFast(GMTKmodelSion): 
#--------------------------------------
  def __init__(self, name, charge=2, **args):
    GMTKmodelSion.__init__(self, name, **args)
    if KERNEL == "accumulator" or KERNEL == 'fisher':
      self.fileTypes = ['y']
    # 2010-04 changed to multiple ion types
#     self.fileTypes = ['b','y'] # ACCUM
#    self.fileTypes = ['b','y','a'] # ACCUM
    self.fileTypes = []
    for ion in ['b','y','a']:
      for charge in ["","+2"]:
        for neutral in ["", "-nh3", "-h2o"]:
          fileType = "%s%s%s" % (ion, charge, neutral)
          self.fileTypes.append(fileType)
#    print self.fileTypes
#
    self.numFloats = 3
    self.numInts   = 9
    self.format = "pfile"

  #--------------------------------------
  def writeIonFiles(self, psmCollection, rootDir, gmtkFiles):
    self.writeIonFilesC(psmCollection, rootDir, gmtkFiles)

#--------------------------------------
class GMTKmodelSionGaussian(GMTKmodelSionFast):

  #--------------------------------------
  # set train to False if we are going to use old parameters files that have already been trained
  def __init__(self, name, charge=2, training=True, **args):
    GMTKmodelSion.__init__(self, name, **args)
    self.trainNormParams = training
   
  #--------------------------------------
  def writeIonFilesC(self, psmCollection, rootDir, gmtkFiles, positive=True): 
    if positive == True: positivityString = "pos"
    else: positivityString = "neg"
    GMTKmodelSion.writeIonFilesC(self, psmCollection, rootDir, gmtkFiles)
    for fileIdx in range(len(self.fileTypes)):
      inputPfile = "%s/%i-%s.pfile" % (rootDir, fileIdx, self.ionFilesType) 
      paramString = "%s,%s,%s" % (self.paramPrefix, self.fileTypes[fileIdx], positivityString)
      outputPfile = "%s/%s.normpfile" % (rootDir, paramString)
      statisticsFile = "%s/%s.stats" % (self.paramDir, paramString) 
      runner = Crux.NormalizePfileRunner(trainParams=self.trainNormParams, \
        inputPfile=inputPfile, outputPfile=outputPfile, statisticsFile=statisticsFile)
      runner.run()
    self.nameIonFiles(rootDir, gmtkFiles, positive=positive)

  #--------------------------------------
  # note tightly couple to NormalizePfileRunner
  # GAUSSIAN negative or positive in name of normpfile
  def nameIonFiles(self, rootDir, gmtkFiles, positive=True):
    if positive == True: positivityString = "pos"
    else: positivityString = "neg"
    for fileIdx in range(len(self.fileTypes)):
      type = self.fileTypes[fileIdx]
      paramString = "%s,%s,%s" % (self.paramPrefix, self.fileTypes[fileIdx], positivityString)
      outputPfile = "%s/%s.normpfile" % (rootDir, paramString)
      gmtkFiles[type] = [outputPfile]

#--------------------------------------
class GMTKmodelMultiTwoD(GMTKmodelIon):

  #--------------------------------------
  def __init__(self, name, charge=2, **args):
    GMTKmodelIon.__init__(self, name, **args)

    # all pairs, no +2 pairs
    self.fileTypes = [ "b,y", "b,a", "y,a", "b,b-h2o", "b,b-nh3", "b,b+2", \
      "b-h2o,b-h2o+2", "b-nh3,b-nh3+2", "y,y-h2o", "y,y-nh3", "y,y+2", \
      "y-h2o,y-h2o+2", "y-nh3,y-nh3+2", "a,a+2", "b+2,y+2" ]

    if KERNEL == "accumulator" or KERNEL == 'fisher':
      self.fileTypes = ['b,y']

    # 2010-04
    # self.fileTypes = ['b,y'] # ACCUM

    self.ionFilesType = "paired"
    self.descriptiveString = self.ionFilesType + "," + self.paramPrefix

    # 2010-04
    # if charge==1: 
    #   self.fileTypes = filter(lambda x: not '+2' in x, self.fileTypes)

    self.format = "pfile"
    self.numFloats = 6
    self.numInts   = 12

  #--------------------------------------------------------------------------
  def twoPeakString(self, bPeak, firstPeak, secondPeak):
    # we analyze everything from the perspective of the b-ion

    outputDict = HmmOrder.createOutputDict('m2D')
    setDefaults('m2D', outputDict)

    addGeneralInfo(outputDict, bPeak)
    assert(firstPeak.hasRankI and secondPeak.hasRankI)

    addPeakPairInfo(outputDict, firstPeak, secondPeak)

    returnString = ""
    if DEBUG == False: 
      returnString = HmmOrder.formatOutput(outputDict, 'm2D')
    else: 
      tuples = map(lambda x: str( (x, outputDict[x]) ) ,  HmmOrder.outputOrder)
      returnString = "\n".join( map(str, tuples) )  + "\n\n"
    return returnString

  #--------------------------------------
  def writeIonFiles(self, psmCollection, rootDir, gmtkFiles): 

    for match in psmCollection["matches"]:
      dirName, theoretical, peptideInformation = \
          self.prepareToWrite(match, rootDir)

      for fileType in self.fileTypes:
        # create the file for this ion type
        outputFile = self.fileName(dirName, fileType)
        if gmtkFiles.has_key(fileType): gmtkFiles[fileType].append(outputFile)
        else: gmtkFiles[fileType] = [ outputFile ]

        fh = open(outputFile, self.getFormat() )

        # noise analysis is a little special
        if fileType.startswith('n'):
          noisePeaks = theoretical.getNoisePeaks()
          noisePeaks.sort(lambda x,y: cmp(y.i, x.i))
          noisePeaks = noisePeaks[0:NUMBER_NOISE_PEAKS]
          for peak in noisePeaks:
            peak.partnerPeak = peak # hack to get twoPeakString to work properly
            fh.write(self.twoPeakString(peak, peak, peak))
        else:
          for peak in theoretical["richPeaks"]:
            # everything will be from the perspective of the b-ion
            if peak.type != Spectrum.B_KEY: continue
            # using the two peaks, output the line
            firstPeakType, secondPeakType = fileType.split(',')
            firstPeak  = peak.getPeakOfType(firstPeakType)
            secondPeak = peak.getPeakOfType(secondPeakType)
            fh.write(self.twoPeakString(peak, firstPeak, secondPeak))

        fh.close()

  
#--------------------------------------
class GMTKmodelTwoD(GMTKmodelMultiTwoD):

  #--------------------------------------
  def __init__(self, name, **args):
    GMTKmodelMultiTwoD.__init__(self, name, **args)
    self.fileTypes = [ 'b,y' ]

#--------------------------------------
def gmtkModelFactory(name, charge=2, training=True, paramPrefix="foo"):

  if (name.startswith('singleIon')):
    if (GAUSSIAN_NORM == False):
      print >>sys.stderr, "Creating fast sion"
      model = GMTKmodelSionFast(name, charge=charge, paramPrefix=paramPrefix)
    else:
      print >>sys.stderr, "Creating normed fast sion"
      model = GMTKmodelSionGaussian(name, charge=charge, training=training, paramPrefix=paramPrefix)
  elif '2D' == name:
    print >>sys.stderr, "Creating 2D"
    model = GMTKmodelTwoD(name, paramPrefix=paramPrefix)
  elif 'm2D' == name:
    print >>sys.stderr, "Creating multi-2D"
    model = GMTKmodelMultiTwoD(name, charge=charge, paramPrefix=paramPrefix)
  elif 'gmix' == name:
    print >>sys.stderr, "Creating Gaussian mixture"
    model = GMTKmodelGaussianMixture(name)
  else:
    raise SystemExit, "Could not find model name %s" % name
  return model

#--------------------------------------
if __name__ == '__main__':
  if len(sys.argv) < 4:
    raise SystemExit, "Usage: python GMTKmodel.py <model-name> <psm-file> <output-dir>"
  import cPickle
  paramPrefix = "mini"
  model = gmtkModelFactory(sys.argv[1], training=True, paramPrefix=paramPrefix)
  psmCollection = cPickle.load(open(sys.argv[2]))
  dir = sys.argv[3]
  gmtkFiles = {}
  print "Positive"
  model.writeIonFilesC(psmCollection, dir, gmtkFiles, positive=True)
  print gmtkFiles
  print "Negative"
  model.writeIonFilesC(psmCollection, dir, gmtkFiles, positive=False)
  print gmtkFiles

#  for fileType in gmtkFiles.keys():
#    fh = open("%s.gmtkinput" % fileType, "w")
#    for file in gmtkFiles[fileType]:
#      fh.write(file + "\n");
#    fh.close()
#

