import sys, tempfile, os, re
import socket
try:
  import pylab
except Exception, e:
  pass
import Fit, GMTK, GMTKmodel
import math
#xs, ys = Fit.make_gaussian_xy(0.0, 1.0, -6.0, 6.0)
#newxs, newys = Fit.make_gaussian_xy(-2.0, 1.0, -6.0, 6.0)
#ys = [ ys[i] + newys[i] for i in range(len(newys))]
#newxs, newys = Fit.make_gaussian_xy(2.0, 1.0, -6.0, 6.0)
#ys = [ ys[i] + newys[i] for i in range(len(newys))]
#
#bigys = [ int(i * 200) for i in ys ]
###--------------------------
# pylab.plot(xs, bigys)
# pylab.hold(True)


#--------------------------
# START here
#--------------------------
class GaussianMixture2D:

  #--------------------------
  def __init__(self):

    # array of doubles, x and y means
    self.means = []

    # array of double, x, y variances
    self.vars = []

    # array of x, y covariances
    self.covars = []

    # array component weights
    self.weights = []
    
    # array component weights
    self.bs = []

  #--------------------------
  def __str__(self):
    string = ""
    for idx in range(len(self.means)):
      values = [ self.means[idx][0], self.means[idx][1], self.vars[idx][0],\
          self.covars[idx], self.vars[idx][1], self.weights[idx] ]
      string += "\t".join( map( lambda x: "%.4f" % x, values)) + "\n"
      
    return string
 

  #--------------------------
  def parse(self, fileName):
    fh = open(fileName)
    dlinkLineCounter = 0 
    for line in fh:
      line.rstrip('\n')
      if line.startswith("gmw_yb"):
        self.weights = map(float, line.split()[2:])
      elif line.startswith("mean_yb"):
        self.means.append( tuple(map(float, line.split()[2:])) ) 
      elif line.startswith("cvar_yb"):
        self.vars.append( tuple(map(float, line.split()[2:])) ) 
      elif line.startswith("dlink_yb"):
        dlinkLineCounter = 1
      elif dlinkLineCounter > 0:
        dlinkLineCounter += 1
        if dlinkLineCounter == 4:
          # TODO make sure this is right
          # self.covars.append( - float(line.split()[1]) / 1000)
          self.covars.append(0.0)
          # self.bs.append( float(line.split()[1]) )
          # self.bs.append( -float(line.split()[1]) )
          dlinkLineCounter = 0
      elif line.startswith("% Components"):
        break
      else:
        pass
    fh.close()
  
  #--------------------------
  def calcCovars(self):
    for idx in range(len(self.bs)):
      self.covars.append( self.bs[idx] * self.vars[idx][1])

  #--------------------------
  def plot(self, title="foo", xtitle="x", ytitle="y"):
    self.calcCovars()
    matlabTemp = "matlab.m"
    fh = open(matlabTemp, "w")
    command = "contgauss([ "
    for idx in range(len(self.means)):
      if len(self.covars) != 0:
        command += "%.5f %.5f %.5f %.5f %.5f %.5f; " % (self.means[idx][0], \
          self.means[idx][1], self.vars[idx][0], self.covars[idx], \
          self.vars[idx][1], self.weights[idx])
      else:
        command += "%.5f %.5f %.5f %.5f %.5f %.5f; " % (self.means[idx][0], \
          self.means[idx][1], self.vars[idx][0], 0.0, \
          self.vars[idx][1], self.weights[idx])

    command += "], '%s', '%s', '%s');" % (title, xtitle, ytitle)
    fh.write(command)
    fh.close()
    shellCommand = "matlab -nosplash -nojvm -r 'run matlab; quit; ' " 
    print "Return %i" % os.system(shellCommand)

#---------------------------
class GaussianMixture:

  #--------------------------
  def __init__(self):
    self.means = []
    self.vars = []
    self.weights = []

  #--------------------------
  # input: a list of scores
  # output: the means, vars and weights of the (three) gaussian mixture fit to 
  # them by GMTK
  #--------------------------
  def learn(self, scores):
    # output the scores to an input file
    tmpTuple = tempfile.mkstemp()
    tmpFileFd, tmpFileName = tmpTuple[0], tmpTuple[1]
    tmpFile = os.fdopen(tmpFileFd, "w")
    for score in scores: 
      tmpFile.write("%s\n" % score)
    tmpFile.close()

    # run GMTK on the input file, writing to param file uniqueName
    host = socket.gethostname().split('.')[0]
    uniqueName = host + "-" + os.path.basename(tmpFileName)
    gmix = GMTKmodel.gmtkModelFactory("gmix")
    paramFile = GMTK.trainGMTK([tmpFileName], gmix, uniqueName)
    
    # parse the resulting param file
    # paramFile = re.sub("~", "/homes/", paramFile)
    self.means, self.vars, self.weights = GMTK.parse_gaussian_mixture(paramFile)

  #--------------------------
  # input: a p-value
  # output: the -log(p-value)
  #--------------------------
  def logp(self, score):
    return -math.log(self.pvalue(score)) 

  #--------------------------
  # input: a score for which to calculate the p-value
  # output: the p-value
  #--------------------------
  def pvalue(self, score):
  
    cmd = "echo '"
    for idx in range(len(self.means)):
      zscore = - (score - self.means[idx])/ pow(self.vars[idx], 0.5) 
      cmd += "pnorm(%.6f); " % zscore
    cmd += "' | R --no-save --slave"
    
    # TODO redo command, maybe with scipy
    stdin, stdouterr = os.popen4(cmd)
    stdin.close()
    pvalues = []
    for line in stdouterr:
      pvalues.append( float(line.split()[1]) )
    stdouterr.close()
    pvalue = 0.0
    for idx in range(len(pvalues)):
      pvalue += pvalues[idx] * self.weights[idx]
    return pvalue


  def plot(self, scores):
    # put the bounds on the plot well away from the distributions
    minimum = min(scores)
    maximum = max(scores)

    xs, ys = Fit.make_gaussians_xy(self.means, self.vars, self.weights, \
        minimum, maximum)

    # try:
    #   pylab.plot(xs, ys)
    #   pylab.hold(True)
    #  pylab.hist(scores, bins=20, normed=True, facecolor='w')
    #except Exception, e:
    #  print >>sys.stderr, "Trouble with pylab %s" % e

#--------------------------
# input: a list of (sorted) svm scores, 
#   which (should) fit a 3-component Gaussian mixture
# output: a plot of the Gaussian fit
#--------------------------
def plotScoreFit(scores):

  # instantiate a GaussianMixture
  gmix = GaussianMixture()
  # learn the Gaussian parameters (optionally exclude the top one)
  scores.sort()
  scoresToLearn = scores[:-1]
  gmix.learn(scoresToLearn)
  print "Means: " + str(gmix.means)
  print "Vars: " + str(gmix.vars)
  print "Weights: " + str(gmix.weights)

#--------------------------
# input: a list of (sorted) svm scores, 
#   which (should) fit a 3-component Gaussian mixture
# output: the top score transformed to a p-values
#--------------------------
def transformTopScore(scores):

  # instantiate a GaussianMixture
  # learn the Gaussian parameters (optionally exclude the top one)
  gmix = GaussianMixture()
  scores.sort()
  topScore = scores[-1]
  scoresToLearn = scores[:-1]
  gmix.learn(scoresToLearn)
  logp = gmix.logp(topScore)
  return logp

#-------------------------------
if __name__ == '__main__':
  if len(sys.argv) < 1:
    raise SystemExit, "Usage: python %s" % sys.argv[0]
   
  gmix = GaussianMixture()
  scores = [0]*5 + [1] * 3 + [-1] *3 + [2] + [-2]
  plotScoreFit(scores)

#  scoreFile = sys.argv[1]
#  scores = []
#  fh = open(scoreFile)
#  for line in fh: scores.append(float(line.rstrip('\n').split()[1]))
#  fh.close()
#  pvalue = transformTopScore(scores)
#  
#  print "-log(P) = %.3f" % pvalue
#
#  plotScoreFit(scores)
#  # try:
  #   pylab.savefig(scoreFile + ".eps")
  #   pylab.savefig(scoreFile + ".png")
  # except Exception, e:
  #  print >>sys.stderr, "Trouble with pylab %s" % e



  #means, vars, weights = GMTK.parse_gaussian_mixture(paramFile)
  #xs, sumys = Fit.make_gaussians_xy(means, vars, weights, -6.0, 6.0)
  #pylab.plot(xs, sumys)
  #pylab.show()

