#!/usr/bin/python
#$Revision: 1.18 $
import getopt, sys, copy
from MatchCollection import *
from MatchGenerator import *
from HitCollection import *
import os
import time

G_dtaSelectExecutable = "DTASelect"
G_dtaOutputFileName = "DTASelect-filter.txt"

class HitGenerator:
	"""Class that contains information on how to generate a HitCollection from 
		 - a MatchCollection
		 - a ProteinCollection"""
	# To be implemented in the sub-classes
	def generateHits(self):
		pass


#-- 
# TODO -Create a Params parent object ??
class DtaSelectParams:
	"Contains the parameter necessary to run DtaSelect."

	def __init__(self):
		self.data = {}
		# set default params
		# sequest.params file necessary (or object)
		self.data["sequest params"] = "sequest.params"
		if not os.access(self.data["sequest params"], os.F_OK):
			print >>sys.stderr, \
				"ERROR: Need a sequest.params file for DtaSelectParams instantiation."
			sys.exit(-1)
		# TODO check if dta_select executable is in path or throw exception.

	def __getitem__(self, key):
		return self.data[key]

	def __setitem__(self, key, item):
		self.data[key] = item		

	def __str__(self):
		# TODO give a real parameter list
		return "-1 0.30 -2 0.30 -3 0.30 -d 0.1 -i 0.1 -o -Smn 7 -p 1 -y 2 -a false"


class DtaSelectHitGenerator(HitGenerator):
	"Creates a DtaSelectHitCollection"

	# Constructor takes optional DtaSelectParams object as input
	def __init__(self, params=None):
		self.data = {}
		self.data["params"]=DtaSelectParams()
		self.data["working dir"]="."

	def __getitem__(self, key):
		return self.data[key]

	def __setitem__(self, key, item):
		self.data[key] = item		

	def generateHits(self, matchCollection):
		"Creates a DtaSelectHitCollection object from the matchCollection."
		# check if matchCollection is an actual MatchCollection
		# if it is an SqtMatchCollection, then just run dtaSelect
		if isinstance(matchCollection, SqtMatchCollection):
			pass	
		# if its not, then convert to an SqtMatchCollection
		elif isinstance(matchCollection, MatchCollection):
			pass
			# TODO sqt = SqtMatchCollection(matchCollection) 		
		else:
			print >>sys.stderr, \
				"ERROR: generateHits passed a non-MatchCollection object." 
			sys.exit(-1)

		sqtAbsPath = os.path.abspath(matchCollection["file"].name)
		sqtParamsAbsPath = os.path.abspath(self.data["params"]["sequest params"])

		# construct the command line. 
		# FIX Probably a better way to redirect output
		cmdLine = " ".join([ G_dtaSelectExecutable, str(self.data["params"]), \
			" > ", "DTASelect.stdout"]) 

		# create and cd to working directory
		dirName = str(time.time())
		cwd = os.getcwd()
		if os.mkdir(dirName):
			print >>sys.stderr, "ERROR: Failed to make directory: ", dirName
			sys.exit(-1)
		
		if os.chdir(dirName):
			print >>sys.stderr, "ERROR: Could not change to dir" + dirName
			sys.exit(-1)
		
		# link input files into directoy 
		if os.symlink(os.path.abspath(sqtAbsPath), 		\
									os.path.basename(sqtAbsPath)) or \
			 os.symlink(os.path.abspath(sqtParamsAbsPath),		\
									os.path.basename(sqtParamsAbsPath)):	
			print >>sys.stderr, "ERROR: Could not link .sqt or sequest.params files."
			sys.exit(-1)

		print >>sys.stderr, "RUNNING:", cmdLine

		retcode = os.system(cmdLine)
		if retcode != 0:
			print >>sys.stderr, "dirName=", dirName
			print >>sys.stderr, "sqtAbsPath=", sqtAbsPath
			print >>sys.stderr, "ERROR: DTASelect was terminated by signal", - retcode
			sys.exit(-1)
		else:
			print >>sys.stderr, "SUCCESS:", retcode

		if not os.access(G_dtaOutputFileName, os.F_OK):
			# TODO better would be to parse the STDOUT of DTASelect. but not today.
			print >>sys.stderr, "ERROR: DTASelect did not generate an output file."
			sys.exit(-1)
			
		dta = DtaSelectHitCollection(os.path.abspath(G_dtaOutputFileName))

		os.chdir(cwd)
		return dta

#-------
# Main
#-------
if __name__ == '__main__':
	if len(sys.argv) < 2:
		print "Usage: %s <sqt-filename>" % (sys.argv[0])
		sys.exit(1)

	dtaHitGen = DtaSelectHitGenerator()
	dtaHitGen["params"] = DtaSelectParams()
	
	for i in range(1,len(sys.argv)):
		print >>sys.stderr, "Parsing SQT file."
		matchCollection = SqtMatchCollection()
		matchCollection.parseFromFilename(sys.argv[i])
	
		print >>sys.stderr, "Running DTASelect."
		dtaHitCollection = dtaHitGen.generateHits(matchCollection)

		print sys.argv[i], "\t", dtaHitCollection.calcTP()
		

	

	
