#-------------------
# $Version$
#-------------------

import cPickle, re, sys, os
import Spectrum
import SpectrumCollection
import MatchCollection
import Mass
import HmmOrder
try:
  import Numeric
except:
  print >>sys.stderr, "Could not import Numeric."

DEBUG = False

MAX_RAW_INT = 20
MISS_RAW_INT = 0

MAX_RANK_INT = 20
MISS_RANK_INT = 0

MISS_RAW_FLOAT = 0.0
MISS_RANK_FLOAT = 0.0

NOISE_SUM_MAX = 100

MZ_INT_OFFSET = 0.10
MZ_INT_PER_PREC_MZ = 5
MZ_INT_MIN = 0
MZ_INT_MAX = 10

ESTIMATED_TIC_TO_BASE_PEAK_RATIO = 25.0

#--------------------------------------------------------------------------
def setDefaults(format, o):

  if format == 'SION':

    o["fNmzRatio"] = 0.0
    o["fCmzRatio"] = 0.0 

    o["fRaw"]  = MISS_RAW_FLOAT
    o["fRank1"] = MISS_RANK_FLOAT

    o["iProtonMobilityBit"] = 0

    o["iNbasicCount"] = 0
    o["iCbasicCount"] = 0
    o["iPossible1"]    = 0 
    o["iObservable1"]  = 0 
    o["iState1"]       = 0 

  elif format == '2D':

    o["fNmzRatio"] = 0.0
    o["fCmzRatio"] = 0.0 

    o["fBintRaw"]  = MISS_RAW_FLOAT
    o["fBintRank"] = MISS_RANK_FLOAT

    o["fYintRaw"]  = MISS_RAW_FLOAT
    o["fYintRank"] = MISS_RANK_FLOAT

    o["fBh2oRaw"]  = MISS_RAW_FLOAT
    o["fBh2oRank"] = MISS_RANK_FLOAT

    o["fYh2oRaw"]  = MISS_RAW_FLOAT
    o["fYh2oRank"] = MISS_RANK_FLOAT

    o["fBnh3Raw"]  = MISS_RAW_FLOAT
    o["fBnh3Rank"] = MISS_RANK_FLOAT

    o["fYnh3Raw"]  = MISS_RAW_FLOAT
    o["fYnh3Rank"] = MISS_RANK_FLOAT

    o["fBcoRaw"]   = MISS_RAW_FLOAT
    o["fBcoRank"]  = MISS_RANK_FLOAT

    o["iTotalNoiseFraction"]   = 0.0
    o["iMaxNoiseRank"]        = MISS_RAW_INT 

    # b ion 
    o["iBpossible"]       = 0 
    o["iBstate"]          = 0 

    # y ion 
    o["iYpossible"]       = 0 
    o["iYstate"]          = 0 

    # h2o loss
    o["iBh2oPossible"]    = 0
    o["iBh2oState"]       = 0

    o["iYh2oPossible"]    = 0
    o["iYh2oState"]       = 0

    # nh3 loss
    o["iBnh3Possible"]    = 0
    o["iBnh3State"]       = 0

    o["iYnh3Possible"]    = 0
    o["iYnh3State"]       = 0

    # co loss
    o["iBcoPossible"]     = 0
    o["iBcoState"]        = 0

  elif format == 'm2D':

    o["fNmzRatio"]= 0.0
    o["fCmzRatio"]= 0.0

    o["fFirstIntRaw"]= MISS_RAW_FLOAT
    o["fSecondIntRaw"]= MISS_RAW_FLOAT

    o["iNmzRatio"]=0 
    o["iCmzRatio"]=0

    o["iNpeptIdx"]=0
    o["iCpeptIdx"]=0 

    o["iNaaId"]=0    
    o["iCaaId"]=0   

    o["iFirstPossible"]=0
    o["iFirstState"]=0    

    o["iSecondPossible"]=0
    o["iSecondState"]=0


#--------------------------------------------------------------------------
def mz2int(mz, peptideMz):
  return int((mz/ peptideMz + MZ_INT_OFFSET) * MZ_INT_PER_PREC_MZ)

#--------------------------------------------------------------------------
def addGeneralSionInfo(o, peak, peptideInfo=None):

  o["iNpeptIdx"] = peak.nidx
  o["iCpeptIdx"] = peak.cidx

  if (peptideInfo != None):
    o["iProtonMobilityBit"]   = peptideInfo["iProtonMobilityBit"]

  peptideMass = Mass.peptide_mass(peak.spectrum.peptide,  \
		  z=peak.spectrum["charge"])
  peptideMz = peptideMass / peak.spectrum["charge"]

  # TODO at some point, MzRatio should be renamed to MassRatio
  o["fMzRatio"] = peak.mz / peptideMass
  o["iMzRatio"] = mz2int(peak.mz, peptideMass)

  if o["iMzRatio"] < MZ_INT_MIN : o["iMzRatio"] = MZ_INT_MIN
  if o["iMzRatio"] > MZ_INT_MAX : o["iMzRatio"] = MZ_INT_MAX

  o["iNaaId"]    = Mass.aaId(peak.nterm)
  o["iCaaId"]    = Mass.aaId(peak.cterm)

#--------------------------------------------------------------------------
def addGeneralInfo(o, b, peptideInfo=None, spectrumInfo=None):
  y = b.partnerPeak

  o["iNpeptIdx"] = len(b.frag)
  o["iCpeptIdx"] = len(y.frag)

  if (spectrumInfo != None):
    o["iTotalNoiseFraction"]        = spectrumInfo["iTotalNoiseFraction"]
    o["iMaxNoiseRank"]        = spectrumInfo["iMaxNoiseRank"]

  if (peptideInfo != None):
    o["iProtonMobilityBit"]   = peptideInfo["iProtonMobilityBit"]

  # the precursor peak does not need, and should not have,
  # the rest of this information
  if y.type == 'p': return

  peptideMass = b.m + y.m 
  # FIX CHARGE +2 assumed!
  peptideMz = peptideMass / 2.0

  assert(b.z == 1 and y.z == 1)
  o["fNmzRatio"] = b.m / peptideMz
  o["iNmzRatio"] = mz2int(b.m, peptideMz)
  if o["iNmzRatio"] < MZ_INT_MIN : o["iNmzRatio"] = MZ_INT_MIN
  if o["iNmzRatio"] > MZ_INT_MAX : o["iNmzRatio"] = MZ_INT_MAX

  o["fCmzRatio"]     = y.m / peptideMz
  o["iCmzRatio"]   = mz2int(y.m, peptideMz)
  if o["iCmzRatio"] < MZ_INT_MIN : o["iCmzRatio"] = MZ_INT_MIN
  if o["iCmzRatio"] > MZ_INT_MAX : o["iCmzRatio"] = MZ_INT_MAX

  o["iNaaId"]    = Mass.aaId(b.nterm)
  o["iCaaId"]    = Mass.aaId(b.cterm)
  # o["iNaaClass"] = Mass.aaClass(b.nterm)
  # o["iCaaClass"] = Mass.aaClass(b.cterm)
  # o["iNaaRank"]  = Mass.aaRank(b.nterm)
  # o["iCaaRank"]  = Mass.aaRank(b.cterm)
 
  o["iNbasicCount"]  = Mass.countBasic(b.frag)
  o["iCbasicCount"]  = Mass.countBasic(y.frag)


#--------------------------------------------------------------------------
def addMaxSumInfo(o, b):

  o["fYintRawSum"] =  \
    o["fYintRaw"] +  o["fYh2oRaw"] + o["fYnh3Raw"]
  o["fBintRawSum"] =  \
    o["fBintRaw"] +  o["fBh2oRaw"] + o["fBnh3Raw"] + o["fBcoRaw"]

  o["fYintRankMax"] =  \
    max(o["fYintRank"],  o["fYh2oRank"], o["fYnh3Rank"])
  o["fBintRankMax"] =  \
    max(o["fBintRank"],  o["fBh2oRank"], o["fBnh3Rank"], o["fBcoRank"])

#--------------------------------------------------------------------------
def addByInfo(o, b):

  if b.observable == True:
    o["iBpossible"]   = 1
    if b.observed == True:
      o["iBstate"]      = 1
      o["fBintRaw"]     = b.i
      o["fBintRank"]    = b.rankI

  y = b.partnerPeak
  if y.observable == True:
    o["iYpossible"]   = 1
    if y.observed == True:
      o["iYstate"]      = 1
      o["fYintRaw"]     = y.i
      o["fYintRank"]    = y.rankI

#--------------------------------------------------------------------------
def addPeakPairInfo(o, first, second):

  if first.possible == True:
    o["iFirstPossible"] = first.numberPossible
    if first.observable == True:
      o["iFirstObservable"] = 1
      if first.observed == True:
        o["iFirstState"]    = 1
        o["fFirstRaw"]      = first.i
        o["fFirstRank"]     = first.rankI

  if second.possible == True:
    o["iSecondPossible"] = second.numberPossible
    if second.observable == True:
      o["iSecondObservable"] = 1
      if second.observed == True:
        o["iSecondState"]    = 1
        o["fSecondRaw"]      = second.i
        o["fSecondRank"]     = second.rankI

#--------------------------------------------------------------------------
def addSionInfo(o, peak):
  if peak.possible == True:
    o["iPossible1"]   = peak.numberPossible
    if peak.observable == True:
      o["iObservable1"] = 1
      if peak.observed == True:
        o["iState1"]    = 1
        o["fRaw"]      = peak.i
        o["fRank1"]     = peak.rankI

#--------------------------------------------------------------------------
def doH2Omax(o, b):
  o["iBstate"]   = max(o["iBstate"],   o["iBh2oState"])
  o["fBintRank"] = max(o["fBintRank"], o["fBh2oRank"])

  o["iYstate"]   = max(o["iYstate"],   o["iYh2oState"])
  o["fYintRank"] = max(o["fYintRank"], o["fYh2oRank"])

#--------------------------------------------------------------------------
def doNeutralMax(o, b):

  o["iBstate"]   = max(o["iBstate"],  o["iBh2oState"], o["iBnh3State"],
      o["iBcoState"])

  o["fBintRank"] = max(o["fBintRank"], o["fBh2oRank"], o["fBnh3Rank"],
      o["fBcoRank"])

  o["iYstate"]   = max(o["iYstate"],  o["iYh2oState"], o["iYnh3State"])
  o["fYintRank"] = max(o["fYintRank"], o["fYh2oRank"], o["fYnh3Rank"])


#--------------------------------------------------------------------------
# FIX FIX FIX ions so that possible, observable, and observed are consistent
#--------------------------------------------------------------------------
def addH2Oloss(o, b):

  assert b.spectrum["hasNeutrals"] == True
  loss = b.canLoseH2O()
  if loss > 0:
    h2o = b.childPeaks[Spectrum.H2O_CHILDPEAK_KEY] 
    if h2o.observable == True:
      o["iBh2oPossible"] = loss
      if h2o.observed == True:
        o["iBh2oState"]   = 1
        o["fBh2oRaw"]     = h2o.i
        o["fBh2oRank"]    = h2o.rankI

  y = b.partnerPeak

  loss = y.canLoseH2O()
  if loss > 0:
    h2o = y.childPeaks[Spectrum.H2O_CHILDPEAK_KEY] 
    if h2o.observable == True:
      o["iYh2oPossible"] = loss
      if h2o.observed == True:
        o["iYh2oState"]   = 1
        o["fYh2oRaw"]     = h2o.i
        o["fYh2oRank"]    = h2o.rankI

#-----------------------------------------------------------------------------
def addNH3loss(o, b):

  assert b.spectrum["hasNeutrals"] == True

  loss = b.canLoseNH3()
  if loss > 0:
    nh3 = b.childPeaks[Spectrum.NH3_CHILDPEAK_KEY]
    if nh3.observable == True:
      o["iBnh3Possible"] = loss
      if nh3.observed == True:
        o["iBnh3State"]   = 1
        o["fBnh3Raw"]     = nh3.i
        o["fBnh3Rank"]    = nh3.rankI

  y = b.partnerPeak
  loss = y.canLoseNH3()
  if loss > 0:
    nh3 = y.childPeaks[Spectrum.NH3_CHILDPEAK_KEY]
    if nh3.observable == True:
      o["iYnh3Possible"] = loss
      if nh3.observed == True:
        o["iYnh3State"]   = 1
        o["fYnh3Raw"]     = nh3.i
        o["fYnh3Rank"]    = nh3.rankI

#----------------------------------------------------------------------------
def addCOloss(o, b):
  assert b.spectrum["hasNeutrals"] == True
  loss = 1
  co = b.childPeaks[Spectrum.A_CHILDPEAK_KEY]
  if co.observable == True :
    o["iBcoPossible"] = loss
    if co.observed == True:
      o["iBcoState"]   = 1
      o["fBcoRaw"]     = co.i
      o["fBcoRank"]    = co.rankI

#----------------------------------------------------------------------------
def sionHmmString(richPeak, peptideInfo=None, binary=False):
  # must have been given relativeRankRaw
  assert(richPeak.hasRankI)

  outputDict = HmmOrder.createOutputDict('SION')
  setDefaults('SION', outputDict)
  addGeneralSionInfo(outputDict, richPeak, peptideInfo)
  addSionInfo(outputDict, richPeak)

  returnString = ""
  if DEBUG == False: 
    if binary == True:
      returnString = HmmOrder.formatBinaryOutput(outputDict)
    else:
      returnString = HmmOrder.formatOutput(outputDict)
  else: 
    tuples = map(lambda x: str( (x, outputDict[x]) ) ,  HmmOrder.sionOutputOrder)
    returnString = "\n".join( map(str, tuples) )  + "\n\n"
  return returnString



#--------------------------------------------------------------------------
def twoDstring(richpeak, spectrumInfo=None, peptideInfo=None):
  # we analyze everything from the perspective of the b-ion
  if richpeak.type != 'b': return '' 

  b = richpeak
  y = richpeak.partnerPeak
  
  # must have been given relativeRankRaw
  assert(b.hasRankI and y.hasRankI)

  outputDict = HmmOrder.createOutputDict('2D')
  setDefaults('2D', outputDict)
  addGeneralInfo(outputDict, b, peptideInfo)
  addByInfo(outputDict, b)
  addH2Oloss(outputDict, b)
  addNH3loss(outputDict, b)
  addCOloss(outputDict, b)
  addMaxSumInfo(outputDict, b)

  returnString = ""
  if DEBUG == False: 
    returnString = HmmOrder.formatOutput(outputDict, '2D')
  else: 
    tuples = map(lambda x: str( (x, outputDict[x]) ) ,  HmmOrder.outputOrder)
    returnString = "\n".join( map(str, tuples) )  + "\n\n"
  return returnString

#--------------------------------------------------------------------------
def analyzeNoisePeaks(theoretical, spectrumInfo): 
  maxNoiseRank        = -1
  noisePeaks = theoretical.getNoisePeaks()
  totalNoiseSum = 0.0
  for peak in noisePeaks:
    totalNoiseSum += peak.i
    if peak.rankI > maxNoiseRank: 
      maxNoiseRank = peak.rankI

  spectrumInfo["iTotalNoiseFraction"] = \
      int(NOISE_SUM_MAX * float(totalNoiseSum) / theoretical["total energy"])
  spectrumInfo["iMaxNoiseRank"] = \
      int(maxNoiseRank * MAX_RANK_INT) 

#--------------------------------------------------------------------------
def peptideInfo(peptide, z):
  dict = {}
  mobile = z - Mass.countBasic(peptide)
  if mobile > 0:
    dict["iProtonMobilityBit"] = 1
  else:
    dict["iProtonMobilityBit"] = 0
  return dict

#--------------------------------------
# match is a peptide spectrum match
# hmmTrainData is a filehandle
#--------------------------------------
def writePepHmmData(match, hmmTrainData, card=None):

  global MAX_RANK_INT
  global MAX_RAW_INT

  observed = match.spectrum

  if card != None:
    MAX_RANK_INT = card
    MAX_RAW_INT = card

  theoretical = Spectrum.RichSpectrum()
  theoretical.addPeptideIons(match.peptide, neutrals=True, multiCharge=True)
  theoretical.addNearestIntensitiesFromSpectrum(observed, doubleCounting=False)
  peptideInformation = peptideInfo(theoretical.peptide, theoretical["charge"])

  for peak in theoretical["richPeaks"]:
    hmmTrainData.write(hmmString(peak, peptideInfo=peptideInformation))

#--------------------------------------
def prepareSpectrum(match):
  observed = match.spectrum
  richObserved = Spectrum.RichSpectrum()
  richObserved.addSpectrumInfo(observed)
  richObserved.addPeaksDirectly(observed)
  richObserved.sumNormalizeIntensity()
  richObserved.rankRelativizeIntensity()
  theoretical = Spectrum.RichSpectrum()
  theoretical.addPeptideIons(\
    match.peptide, neutrals=True, multiCharge=True, z=match.charge)
  theoretical.addNearestIntensitiesFromSpectrum(\
      richObserved, doubleCounting=False)
  return theoretical

#-------
# Main
#-------
if __name__ == '__main__':

  if len(sys.argv) < 2:
    raise SystemExit, "Usage: %s <psm-file>" % (sys.argv[0])

  import cPickle
  from Spectrum import RichSpectrum
  r = RichSpectrum()
  try: import MatchCollection
  except: pass
  psm = cPickle.load(open(sys.argv[1]))
  psm1 = psm["matches"][0]
  psm1.spectrum.parsePeaks()
  from GMTKmodel import *
  model = gmtkModelFactory('model-2D')
  model.writeIonFiles(psm, "foo", {}) 


#--------------------------------------
#def writePepHmmData(match, hmmTrainData):
#  observed = match.spectrum
#  # observed.rankRelativizeRaw()
#
#  # use model to determine the "theoretical" ion spectra
#  theoretical = Spectrum.RichSpectrum()
#  theoretical.addPeptideIons(match.peptide, neutrals=True)
#  # theoretical.addSpectrumInfo(observed)
#  # theoretical.addNearestIntensitiesFromSpectrum(observed, doubleCounting=True)
#  theoretical.addNearestIntensitiesFromSpectrum(observed, doubleCounting=False)
#  # theoretical.rankRelativizeRaw()
#
#  spectrumInfo = {}
#  # analyzeNoisePeaks(theoretical, spectrumInfo) # FIXFAST
#
#  for peak in theoretical["richPeaks"]:
#    hmmTrainData.write(hmmString(peak, spectrumInfo))
#
