#-------------------
# $Version$
#-------------------

import pickle, re, sys
import Spectrum
import SpectrumCollection
import MatchCollection
import Mass
import HmmOrder
try:
  import Numeric
except:
  print >>sys.stderr, "Could not import Numeric."

DEBUG = False
MISSING_PEAK_OFFSET = 0.00
MISSING_PEAK_RELATIVE_RANK_INTENSITY = 0.0
PEAK_RELATIVE_RANK_INTEGER = 100
MISSING_PEAK_RELATIVE_RANK_INTEGER = 0
PEAK_RELATIVE_RANK_REDUCED_INTEGER = 20
MISSING_PEAK_RELATIVE_RANK_REDUCED_INTEGER = 0
NOISE_SUM_MAX = 100

MZ_INT_OFFSET = 0.10
MZ_INT_PER_PREC_MZ = 5
MZ_INT_MIN = 0
MZ_INT_MAX = 10


#--------------------------------------------------------------------------
def setDefaults(o):

  # noise peaks

  o["iTotalNoiseFraction"]   = 0.0
  o["iMaxNoiseRank"]        = MISSING_PEAK_RELATIVE_RANK_INTENSITY 
  o["iMaxNoiseRankAdj"]     = MISSING_PEAK_RELATIVE_RANK_INTENSITY 
  o["iMaxNoiseRedRank"]     = MISSING_PEAK_RELATIVE_RANK_REDUCED_INTEGER
  o["iMaxNoiseRedRankAdj"]  = MISSING_PEAK_RELATIVE_RANK_REDUCED_INTEGER

  # b ion 
  o["iBstate"]       = 0 
  o["fBrank"]        = MISSING_PEAK_RELATIVE_RANK_INTENSITY 
  o["fBrankAdj"]     = MISSING_PEAK_RELATIVE_RANK_INTENSITY 
  o["iBrank"]        = MISSING_PEAK_RELATIVE_RANK_INTEGER
  o["iBredRank"]     = MISSING_PEAK_RELATIVE_RANK_REDUCED_INTEGER
  o["iBrankAdj"]     = MISSING_PEAK_RELATIVE_RANK_INTEGER
  o["iBredRankAdj"]  = MISSING_PEAK_RELATIVE_RANK_REDUCED_INTEGER

  # y ion 
  o["iYstate"]       = 0 
  o["fYrank"]        = MISSING_PEAK_RELATIVE_RANK_INTENSITY 
  o["fYrankAdj"]     = MISSING_PEAK_RELATIVE_RANK_INTENSITY 
  o["iYrank"]        = MISSING_PEAK_RELATIVE_RANK_INTEGER
  o["iYredRank"]     = MISSING_PEAK_RELATIVE_RANK_REDUCED_INTEGER
  o["iYrankAdj"]     = MISSING_PEAK_RELATIVE_RANK_INTEGER
  o["iYredRankAdj"]  = MISSING_PEAK_RELATIVE_RANK_REDUCED_INTEGER

  # h2o loss
  o["iBh2oPossible"] = 0
  o["iBh2oState"]    = 0
  o["fBh2oRank"]     =  MISSING_PEAK_RELATIVE_RANK_INTENSITY
  o["fBh2oRankAdj"]  =  MISSING_PEAK_RELATIVE_RANK_INTENSITY
  o["iBh2oRank"]       = MISSING_PEAK_RELATIVE_RANK_INTEGER
  o["iBh2oRedRank"]    = MISSING_PEAK_RELATIVE_RANK_REDUCED_INTEGER
  o["iBh2oRankAdj"]    = MISSING_PEAK_RELATIVE_RANK_INTEGER
  o["iBh2oRedRankAdj"] = MISSING_PEAK_RELATIVE_RANK_REDUCED_INTEGER

  o["iYh2oPossible"] = 0
  o["iYh2oState"]    = 0
  o["fYh2oRank"]     =  MISSING_PEAK_RELATIVE_RANK_INTENSITY
  o["fYh2oRankAdj"]  =  MISSING_PEAK_RELATIVE_RANK_INTENSITY
  o["iYh2oRank"]       = MISSING_PEAK_RELATIVE_RANK_INTEGER
  o["iYh2oRedRank"]    = MISSING_PEAK_RELATIVE_RANK_REDUCED_INTEGER
  o["iYh2oRankAdj"]    = MISSING_PEAK_RELATIVE_RANK_INTEGER
  o["iYh2oRedRankAdj"] = MISSING_PEAK_RELATIVE_RANK_REDUCED_INTEGER

  # nh3 loss
  o["iBnh3Possible"] = 0
  o["iBnh3State"]    = 0
  o["fBnh3Rank"]     =  MISSING_PEAK_RELATIVE_RANK_INTENSITY
  o["fBnh3RankAdj"]  =  MISSING_PEAK_RELATIVE_RANK_INTENSITY
  o["iBnh3Rank"]       = MISSING_PEAK_RELATIVE_RANK_INTEGER
  o["iBnh3RedRank"]    = MISSING_PEAK_RELATIVE_RANK_REDUCED_INTEGER
  o["iBnh3RankAdj"]    = MISSING_PEAK_RELATIVE_RANK_INTEGER
  o["iBnh3RedRankAdj"] = MISSING_PEAK_RELATIVE_RANK_REDUCED_INTEGER

  o["iYnh3Possible"] = 0
  o["iYnh3State"]    = 0
  o["fYnh3Rank"]     =  MISSING_PEAK_RELATIVE_RANK_INTENSITY
  o["fYnh3RankAdj"]  =  MISSING_PEAK_RELATIVE_RANK_INTENSITY
  o["iYnh3Rank"]       = MISSING_PEAK_RELATIVE_RANK_INTEGER
  o["iYnh3RedRank"]    = MISSING_PEAK_RELATIVE_RANK_REDUCED_INTEGER
  o["iYnh3RankAdj"]    = MISSING_PEAK_RELATIVE_RANK_INTEGER 
  o["iYnh3RedRankAdj"] = MISSING_PEAK_RELATIVE_RANK_REDUCED_INTEGER

  # co loss
  o["iBcoPossible"] = 0
  o["iBcoState"]    = 0
  o["fBcoRank"]     =  MISSING_PEAK_RELATIVE_RANK_INTENSITY
  o["fBcoRankAdj"]  =  MISSING_PEAK_RELATIVE_RANK_INTENSITY
  o["iBcoRank"]       = MISSING_PEAK_RELATIVE_RANK_INTEGER
  o["iBcoRedRank"]    = MISSING_PEAK_RELATIVE_RANK_REDUCED_INTEGER
  o["iBcoRankAdj"]    = MISSING_PEAK_RELATIVE_RANK_INTEGER
  o["iBcoRedRankAdj"] = MISSING_PEAK_RELATIVE_RANK_REDUCED_INTEGER


#--------------------------------------------------------------------------
def addGeneralInfo(o, b, spectrumInfo):
  y = b.partnerPeak

  o["iNpeptIdx"] = len(b.frag)
  o["iCpeptIdx"] = len(y.frag)

  if len(spectrumInfo.keys()) > 0:
    o["iTotalNoiseFraction"]        = spectrumInfo["iTotalNoiseFraction"]
    o["iMaxNoiseRank"]        = spectrumInfo["iMaxNoiseRank"]
    o["iMaxNoiseRankAdj"]     = spectrumInfo["iMaxNoiseRankAdj"]
    o["iMaxNoiseRedRank"]     = spectrumInfo["iMaxNoiseRedRank"]
    o["iMaxNoiseRedRankAdj"]  = spectrumInfo["iMaxNoiseRedRankAdj"]

  # the precursor peak does not need, and should not have,
  # the rest of this information
  if y.type == 'p': return

  peptideMass = b.m + y.m 
  # FIX CHARGE +2 assumed!
  peptideMz = peptideMass / 2.0

  assert(b.z == 1 and y.z == 1)
  o["fNmzRatio"] = b.m / peptideMz
  o["iNmzRatio"] = int((b.m / peptideMz + MZ_INT_OFFSET) * MZ_INT_PER_PREC_MZ)
  if o["iNmzRatio"] < MZ_INT_MIN : o["iNmzRatio"] = MZ_INT_MIN
  if o["iNmzRatio"] > MZ_INT_MAX : o["iNmzRatio"] = MZ_INT_MAX

  o["fCmzRatio"]     = y.m / peptideMz
  o["iCmzRatio"]   = int((y.m / peptideMz + MZ_INT_OFFSET) * MZ_INT_PER_PREC_MZ)
  if o["iCmzRatio"] < MZ_INT_MIN : o["iCmzRatio"] = MZ_INT_MIN
  if o["iCmzRatio"] > MZ_INT_MAX : o["iCmzRatio"] = MZ_INT_MAX

  o["iNaaId"]    = Mass.aaId(b.nterm)
  o["iCaaId"]    = Mass.aaId(b.cterm)
  o["iNaaClass"] = Mass.aaClass(b.nterm)
  o["iCaaClass"] = Mass.aaClass(b.cterm)
  o["iNaaRank"]  = Mass.aaRank(b.nterm)
  o["iCaaRank"]  = Mass.aaRank(b.cterm)
 
  o["iNbasicCount"]  = Mass.countBasic(b.frag)
  o["iCbasicCount"]  = Mass.countBasic(y.frag)
  
  #
#--------------------------------------------------------------------------
def addByInfo(o, b):

  precursorMz = b.spectrum["precursor mz"]
  if b.observed == True:
    o["iBstate"]   = 1
    o["fBrank"]    = b.rankI
    o["iBrank"]    = int(b.rankI * PEAK_RELATIVE_RANK_INTEGER)
    o["iBredRank"] = int(b.rankI* PEAK_RELATIVE_RANK_REDUCED_INTEGER)

    adj = b.adj 
    o["fBrankAdj"]     = adj
    o["iBrankAdj"]     = int(adj * PEAK_RELATIVE_RANK_INTEGER)
    o["iBredRankAdj"]  = int(adj * PEAK_RELATIVE_RANK_REDUCED_INTEGER)

  y = b.partnerPeak
  if y.observed == True:
    o["iYstate"]   = 1
    o["fYrank"]    = y.rankI
    o["iYrank"]    = int(y.rankI* PEAK_RELATIVE_RANK_INTEGER)
    o["iYredRank"] = int(y.rankI* PEAK_RELATIVE_RANK_REDUCED_INTEGER)

    adj = y.adj
    o["fYrankAdj"]     = adj
    o["iYrankAdj"]     = int(adj * PEAK_RELATIVE_RANK_INTEGER)
    o["iYredRankAdj"]  = int(adj * PEAK_RELATIVE_RANK_REDUCED_INTEGER)

#--------------------------------------------------------------------------
def addH2Oloss(o, b):

  assert b.spectrum["hasNeutrals"] == True
  precursorMz = b.spectrum["precursor mz"]
  loss = b.canLoseH2O()
  if loss > 0:
    h2o = b.childPeaks['H2O'] # TODO really should be a fixed type
    o["iBh2oPossible"] = loss
    if h2o.observed == True:
      o["iBh2oState"]    = 1
      o["fBh2oRank"]     = h2o.rankI
      o["iBh2oRank"]     = int(h2o.rankI* PEAK_RELATIVE_RANK_INTEGER) 
      o["iBh2oRedRank"]  = int(h2o.rankI* PEAK_RELATIVE_RANK_REDUCED_INTEGER)

      adj = h2o.adj

      o["fBh2oRankAdj"]    =  adj
      o["iBh2oRankAdj"]    = int(adj * PEAK_RELATIVE_RANK_INTEGER)
      o["iBh2oRedRankAdj"] = int(adj * PEAK_RELATIVE_RANK_REDUCED_INTEGER)

  y = b.partnerPeak

  loss = y.canLoseH2O()
  if loss > 0:
    h2o = y.childPeaks['H2O'] # TODO really should be a fixed type
    o["iYh2oPossible"] = loss
    if h2o.observed == True:
      o["iYh2oState"]    = 1
      o["fYh2oRank"]     = h2o.rankI
      o["iYh2oRank"]     = int(h2o.rankI* PEAK_RELATIVE_RANK_INTEGER) 
      o["iYh2oRedRank"]  = int(h2o.rankI* PEAK_RELATIVE_RANK_REDUCED_INTEGER)

      adj = h2o.adj

      o["fYh2oRankAdj"]    =  adj
      o["iYh2oRankAdj"]    = int(adj * PEAK_RELATIVE_RANK_INTEGER)
      o["iYh2oRedRankAdj"] = int(adj * PEAK_RELATIVE_RANK_REDUCED_INTEGER)
#---------------------------------------------------------------------------------
def addNH3loss(o, b):

  assert b.spectrum["hasNeutrals"] == True
  precursorMz = b.spectrum["precursor mz"]
  loss = b.canLoseNH3()
  if loss > 0:
    nh3 = b.childPeaks['NH3'] # TODO really should be a fixed type
    o["iBnh3Possible"] = loss
    if nh3.observed == True:
      o["iBnh3State"]    = 1
      o["fBnh3Rank"]     = nh3.rankI
      o["iBnh3Rank"]     = int(nh3.rankI* PEAK_RELATIVE_RANK_INTEGER) 
      o["iBnh3RedRank"]  = int(nh3.rankI* PEAK_RELATIVE_RANK_REDUCED_INTEGER)
      adj = nh3.adj
      o["fBnh3RankAdj"]    =  adj
      o["iBnh3RankAdj"]    = int(adj * PEAK_RELATIVE_RANK_INTEGER)
      o["iBnh3RedRankAdj"] = int(adj * PEAK_RELATIVE_RANK_REDUCED_INTEGER)

  y = b.partnerPeak

  loss = y.canLoseNH3()
  if loss > 0:
    nh3 = y.childPeaks['NH3'] # TODO really should be a fixed type
    o["iYnh3Possible"] = loss
    if nh3.observed == True:
      o["iYnh3State"]    = 1
      o["fYnh3Rank"]     = nh3.rankI
      o["iYnh3Rank"]     = int(nh3.rankI* PEAK_RELATIVE_RANK_INTEGER) 
      o["iYnh3RedRank"]  = int(nh3.rankI* PEAK_RELATIVE_RANK_REDUCED_INTEGER)
      adj = nh3.adj
      o["fYnh3RankAdj"]    =  adj
      o["iYnh3RankAdj"]    = int(adj * PEAK_RELATIVE_RANK_INTEGER)
      o["iYnh3RedRankAdj"] = int(adj * PEAK_RELATIVE_RANK_REDUCED_INTEGER)
#----------------------------------------------------------------------------
def addCOloss(o, b):
  assert b.spectrum["hasNeutrals"] == True
  precursorMz = b.spectrum["precursor mz"]
  loss = 1
  co = b.childPeaks['a'] # TODO really should be a fixed type
  o["iBcoPossible"] = loss
  if co.observed == True:
    o["iBcoState"]    = 1
    o["fBcoRank"]     = co.rankI
    o["iBcoRank"]     = int(co.rankI* PEAK_RELATIVE_RANK_INTEGER) 
    o["iBcoRedRank"]  = int(co.rankI* PEAK_RELATIVE_RANK_REDUCED_INTEGER)

    adj = co.adj

    o["fBcoRankAdj"]    =  adj
    o["iBcoRankAdj"]    = int(adj * PEAK_RELATIVE_RANK_INTEGER)
    o["iBcoRedRankAdj"] = int(adj * PEAK_RELATIVE_RANK_REDUCED_INTEGER)



#--------------------------------------------------------------------------
def hmmString(richpeak, spectrumInfo):
  # we analyze everything from the perspective of the b-ion
  if richpeak.type != 'b': return '' 

  b = richpeak
  y = richpeak.partnerPeak
  
  # must have been given relativeRankIntensity
  assert(b.hasRankI and y.hasRankI)

  outputDict = HmmOrder.createOutputDict()
  setDefaults(outputDict)
  addGeneralInfo(outputDict, b, spectrumInfo)
  addByInfo(outputDict, b)
  addH2Oloss(outputDict, b)
  addNH3loss(outputDict, b)
  addCOloss(outputDict, b)
  if DEBUG == False: return HmmOrder.formatOutput(outputDict)
  else: return str("\n".join( map(str,outputDict.items()) ) + "\n\n")

#--------------------------------------------------------------------------
def analyzeNoisePeaks(theoretical, spectrumInfo): 
  maxNoiseRank        = -1
  maxNoiseRankAdj     = -1
  noisePeaks = theoretical.getNoisePeaks()
  totalNoiseSum = 0.0
  for peak in noisePeaks:
    totalNoiseSum += peak.i
    if peak.rankI > maxNoiseRank: 
      maxNoiseRank = peak.rankI
    if peak.adj > maxNoiseRankAdj:
      maxNoiseRankAdj = peak.adj

  if maxNoiseRank == -1 or maxNoiseRankAdj == -1: 
    return
  spectrumInfo["iTotalNoiseFraction"] = \
      int(NOISE_SUM_MAX * float(totalNoiseSum) / theoretical["total energy"])
  spectrumInfo["iMaxNoiseRank"] = \
      int(maxNoiseRank * PEAK_RELATIVE_RANK_INTEGER) 
  spectrumInfo["iMaxNoiseRankAdj"] = \
      int(maxNoiseRankAdj * PEAK_RELATIVE_RANK_INTEGER)
  spectrumInfo["iMaxNoiseRedRank"] = \
      int(maxNoiseRank * PEAK_RELATIVE_RANK_REDUCED_INTEGER) 
  spectrumInfo["iMaxNoiseRedRankAdj"] = \
      int(maxNoiseRankAdj * PEAK_RELATIVE_RANK_REDUCED_INTEGER)

#--------------------------------------
# match is a peptide spectrum match
# hmmTrainData is a filehandle
#--------------------------------------
def writePepHmmData(match, hmmTrainData):
  observed = match.spectrum
  # observed.rankRelativizeIntensity()

  # use model to determine the "theoretical" ion spectra
  theoretical = Spectrum.RichSpectrum()
  theoretical.addPeptideIons(match.peptide, neutrals=True)
  # theoretical.addSpectrumInfo(observed)
  # theoretical.addNearestIntensitiesFromSpectrum(observed, doubleCounting=True)
  theoretical.addNearestIntensitiesFromSpectrum(observed, doubleCounting=False)
  # theoretical.rankRelativizeIntensity()

  spectrumInfo = {}
  # analyzeNoisePeaks(theoretical, spectrumInfo) # FIXFAST

  for peak in theoretical["richPeaks"]:
    hmmTrainData.write(hmmString(peak, spectrumInfo))

