#------------
# $Revision: 1.37 $
#------------
import struct

outputOrders = {}

outputOrders['SION'] = [

#---------------------------
# N below is the peptide length.
#---------------------------

#-----------
# FLOATS
#-----------
"fMzRatio", # [0,10] An integer representation of fNmzRatio

"fRaw",  
"fRank1",  

#-----------
# INTS 
#-----------
"iProtonMobilityBit", # [0,1] A bit representation of proton mobility
"iMzRatio", # [0,10] An integer representation of fNmzRatio

"iNpeptIdx", # [0,N] Index of the amide-bond cleavage position.
"iCpeptIdx", # as above, but from the C-term end

"iNaaId",    # [0,19] Amino-acid N-term to cleavage position
"iCaaId",    # as above, but C-term to cleavage position

"iPossible1", # [0,1] Could we conceivably detect the b-ion?
"iObservable1", # [0,1] Could we conceivably detect the b-ion?
"iState1"    # [0,1] Was a peak observed within +/-2m/z of predicted p peak?
]

outputOrders['m2D'] = [

#---------------------------
# N below is the peptide length.
#---------------------------
# All intRanks are lowest = low peak int, highest = high peak intRaw
#---------------------------

#-----------
# FLOATS
#-----------
"fNmzRatio", # [0,10] An integer representation of fNmzRatio
"fCmzRatio", # as above, but for fCmzRatio

"fFirstRaw",
"fSecondRaw",  # [0,20] (missing = 0) an int rep for intRaw

"fFirstRank",
"fSecondRank",  # [0,20] (missing = 0) an int rep for intRaw

#-----------
# INTS 
#-----------
"iNmzRatio", # [0,10] An integer representation of fNmzRatio
"iCmzRatio", # as above, but for fCmzRatio

"iNpeptIdx", # [0,N] Index of the amide-bond cleavage position.
"iCpeptIdx", # as above, but from the C-term end

# amino info
"iNaaId",    # [0,19] Amino-acid N-term to cleavage position
"iCaaId",    # as above, but C-term to cleavage position

"iFirstPossible", # [0,N] Could we conceivably detect the b-ion?
"iFirstObservable", # [0,1] Could we conceivably detect the b-ion?
"iFirstState",    # [0,1] Was a peak observed within +/-2m/z of predicted peak?

"iSecondPossible", 
"iSecondObservable", # [0,1] Could we conceivably detect the b-ion?
"iSecondState"

]


outputOrders['2D'] = [

#---------------------------
# N below is the peptide length.
#---------------------------
# All intRanks are lowest = low peak int, highest = high peak intRaw
#---------------------------

#-----------
# FLOATS
#-----------
"fNmzRatio", # [0,10] An integer representation of fNmzRatio
"fCmzRatio", # as above, but for fCmzRatio

# raw
"fYintRawSum",  # NEW
"fBintRawSum",  # NEW

"fYh2oRaw",
"fYnh3Raw",
"fYintRaw",

"fBintRaw",  # [0,20] (missing = 0) an int rep for intRaw
"fBh2oRaw",  # as iBintRaw but for H2O
"fBnh3Raw",
"fBcoRaw",

# rank
"fYintRankMax",  # NEW
"fBintRankMax",  # NEW

"fYh2oRank",
"fYnh3Rank",
"fYintRank",

"fBintRank",  # [0,20] (missing = 0) an int rep for intRank
"fBh2oRank",  # as iBintRank but for H2O
"fBnh3Rank",
"fBcoRank",


#-----------
# INTS 
#-----------

#-----------
# general ints
#-----------
# FIX later we can add this back in, but the original 2D did not have it
# "iProtonMobilityBit", # [0,1] A bit representation of proton mobility 
"iNmzRatio", # [0,10] An integer representation of fNmzRatio
"iCmzRatio", # as above, but for fCmzRatio

"iNpeptIdx", # [0,N] Index of the amide-bond cleavage position.
"iCpeptIdx", # as above, but from the C-term end

# amino info
"iNaaId",    # [0,19] Amino-acid N-term to cleavage position
"iCaaId",    # as above, but C-term to cleavage position

"iNaaClass", # - Will be AA class. Not yet used
"iCaaClass", # as above

"iNaaRank",  # - Will be AA intRank. Not yet used.
"iCaaRank",

"iNbasicCount", # [0,N] Total number of K,R residues on the N-term fragment 
"iCbasicCount", # as above, but for C-term fragment

"iTotalNoiseFraction", # [0,100] integer representing total percentage of unaccounted for peaks

"iMaxNoiseRank",        # [0,100] Rank relative peak int of the maximum noise peak
# "iNumberNoisePeaks",   # [0,3000] Total number of noise peaks in spectrum

#-----------
# +1 ions
#-----------
"iBpossible", # [0,N] Could we conceivably detect the b-ion?
"iBstate",    # [0,1] Was a peak observed within +/-2m/z of predicted p peak?

"iYpossible", 
"iYstate",

# h2o loss
"iBh2oPossible",  # [0,N] number of possible H2O losses 
"iBh2oState",     # [0,1] Was this peak observed within the m/z tolerance?

"iYh2oPossible",  
"iYh2oState",

# nh3 loss
"iBnh3Possible",  
"iBnh3State",  

"iYnh3Possible",  
"iYnh3State",

# co loss
"iBcoPossible",  
"iBcoState",  

]

NULL_FLOAT = -0.1
NULL_INT   = 0 # FIX return this to -1 later
# return dict with keys as hmmOrder, values as NULL_FLOAT or NULL_INT
# BINARY cache this, with defaults
def createOutputDict(order='SION'):
  outputDict = {}
  for val in outputOrders[order]:
    if val.startswith("i"): outputDict[val] = NULL_INT
    elif val.startswith("f"): outputDict[val] = NULL_FLOAT
    else: raise SystemExit, "Weird val %s" % val
  return outputDict

# return dict with keys as hmmOrder, values as NULL_FLOAT or NULL_INT
def createOutputDictOrder(order='SION'):
  outputDict = {}
  idx = 0
  for val in outputOrders[order]:
    outputDict[val] = idx
    idx += 1
  return outputDict

# BINARY make this faster
def createBinaryFormatString(order='SION'):
  formatString = ""
  for item in outputOrders[order]:
    if item.startswith("f"):   formatString += "f "
    elif item.startswith("i"): formatString += "i "
  return formatString

def createFormatString(order='SION'):
  formatString = ""
  for item in outputOrders[order]:
    if item.startswith("f"):
      formatString += "%9.4f "
    elif item.startswith("i"):
      formatString += "%3i "
  formatString += "\n"
  return formatString

# return a formatted string, input is an outputDict
# BINARY cache format, try to get the flat list pack thing to work
def formatBinaryOutput(outputDict, order='SION'):
  # force bigendian with ">"
  string = ""
  for key in outputOrders[order]:
    if key.startswith("f"): 
      string += struct.pack(">f", outputDict[key])
    elif key.startswith("i"): 
      string += struct.pack(">i", outputDict[key])
  return string

# return a formatted string, input is an outputDict
def formatOutput(outputDict, order='SION'):
  formatString = createFormatString(order=order)
  outputList = []
  for key in outputOrders[order]:
    outputList.append(outputDict[key])

  return formatString % tuple(outputList)

def commonParamsString(order='SION'):

  string = """
#define CONST_Dirichlet  1.0
#define CARD_BINARY      2  // this is used for all binary variables
#define CARD_B2          4  // 2 x 2 = 4
#define CARD_aaId       20  // # of different amino acids
#define CARD_iMzRatio   21  // # of bins for the float mz-ratio 
#define CARD_iPeptIdx   40  // maximum peptide length
#define CARD_pepRegion   5  // these are the pepHMM-derived 'regions'
#define CARD_intRank    21  // initially floats were binned in [0,100]
#define CARD_intRaw     21  // initially floats were binned in [0,100]
#define CARD_mix         3
#define CARD_possCount  40
"""

  for idx in range(len(outputOrders[order])):
    string += "#define LOC_" + outputOrders[order][idx] + " " + str(idx) + "\n"
  return string

if __name__=='__main__':
  print commonParamsString(order='m2D')



