#$Revision: 1.2 $
import Numeric
import misc
import datafunc
import sys
import getopt
import preproc
import svm

__docformat__ = "restructuredtext en"


usage = """USAGE: pyml-learning-curve.py [options] <data> <labels>
  Produces ROC scores as a function of training set size.

  Options:
    -num-folds <int>
    -num-reps <int>
    -c-values <int>,<int>,...
    -negatives-only
    -sizes <int>,<int>,... (default is 16, 32, 64, ..., max)
"""

def learningCurve(classifier, data, **args) :

    """
    :Returns:

    - a list of result objects (if the number of repetitions per size is 1)
      or a list of dictionaries that provide the average of the various accuracy
      measures (rocArea, successRate etc.) for each dataset size.
    - 
    
    :Parameters:

    - `classifier` - a classifier object
    - `data` - a dataset object
    
    :Keywords:

    - `sizes` - a list of dataset sizes.  default: [16, 32, 64,..., max]
    - `classLabel` - compute the learning curve with respect to a particular
      class
    - `numFolds` - number of CV folds to use [default: 5]
    - `numReps` - number of samples for each dataset size [default: 1]
    """

    if 'sizes' in args :
        sizes = args['sizes']
    else :
        sizes = []
        size = 32
        while size < len(data) :
            sizes.append(size)
            size*=2
        sizes.append(len(data))
    numFolds = 5
    if 'numFolds' in args :
        numFolds = args['numFolds']
    numReps = 1
    if 'numReps' in args :
        numReps = args['numReps']
    classLabel = None
    if 'classLabel' in args :
        classLabel = args['classLabel']

    results = []
    print sizes

    for i in range(len(sizes)) :
        res = []
        for rep in range(numReps) :
            sample = datafunc.sample(data, sizes[i], stratified = True)
            res.append(classifier.stratifiedCV(sample))
        if numReps == 1 :
            results.extend(res)
        else :
            results.append(averageResults(res))

    return results, sizes

def averageResults(resultsList) :

    attributes = ['rocArea', 'rocNarea', 'successRate', 'balancedSuccessRate']
    results = {}

    for attribute in attributes :
        results[attribute] = Numeric.average(misc.extractAttribute(resultsList,
                                                                   attribute))

    return results


if __name__ == '__main__':

	if len(sys.argv) < 1: 
		print """Usage: python %s 
	-r <cvector-train-file> 
	[-k gaussian|polynomial]
	[-w gaussian width]
	[-d polynomial degree]
""" % (sys.argv[0])
		sys.exit(-1)

	optlist,args = getopt.getopt(sys.argv[1:],'r:k:w:d:')

	sqt =  ms2 = train = test = ""
	kernel = 'polynomial'
	width  = cVal = 0.010
	deg = 1
	
	for option_name,option_value in optlist:
		if option_name == '-r':
			train=option_value
		if option_name == '-k':
			kernel=option_value
		if option_name == '-w':
			width=float(option_value)
		if option_name == '-d':
			deg=int(option_value)
		if option_name == '-c':
			cVal=float(option_value)

	
	print >>sys.stderr, "Parsing training file %s." % train
	DataSet = datafunc.SparseCDataSet
	trainData = DataSet(train, labelsColumn = 1) 
	rs = preproc.Rescale()
	rs.train(trainData)

	if kernel in ('Polynomial','polynomial','poly'):
		print >>sys.stderr, "Using polynomial kernel of degree %.3f." % deg
		trainData.attachKernel('poly', degree = deg) 
	elif kernel in ('Gaussian','gaussian','rbf'):
		print >>sys.stderr, "Using Gaussian kernel of width %.3f." % width
		trainData.attachKernel('gaussian', gamma = width)

	print >>sys.stderr, "Training svm with C = %.3f." % cVal
	s=svm.SVM(C=cVal)

	results = learningCurve(s, trainData, numReps=3)
	i=0

	for r in range(len(results[0])):
		i = i + 1
		print "%i th rocArea = %.3f" % ( i , results[0][r]["rocArea"])
	

