#!/usr/bin/python
#$Revision: 1.11 $
import getopt, sys, copy, os
from MatchCollection import *
from SpectrumCollection import *
from HitGenerator import * # FIX 
# from IPython.Shell import IPShellEmbed

class MatchGenerator:
	"Class that generate a MatchCollection from a SpectrumCollection"
	# To be implemented in the sub-classes
	def generateMatches(self):
		pass


# TODO Maybe create a params parent object??
class PseudoSequestMatchGenerator(MatchGenerator):
	"Using cached SqtMatchCollection will generate a 'faked' SqtMatchCollection."

	def __init__(self, sqtMatchCollection):
		self.data = {}
		if not isinstance(sqtMatchCollection, SqtMatchCollection):
			print >>sys.stderr, "ERROR: PseudoSequestMatchGenerator \
				must be passed SqtMatchCollection instance as input."
		self.data["SqtMatchCollection"] = sqtMatchCollection 

	def __getitem__(self, key):
		return self.data[key]

	def __setitem__(self, key, item):
		self.data[key] = item		

	def __str__(self):
		pass

	def generateMatches(self, spectrumCollection):

		# TODO possibly put in something to check if the ms2 and 
		# the sqt files are from the same run??
		matchCollection = SqtMatchCollection()  
		matchCollection["headerLines"] = self["SqtMatchCollection"]["headerLines"]
		matchCollection["fileName"] = "pseudo.sqt" 
		#FIX  # self["SqtMatchCollection"]["file"]  ".sqt"
		for match in self["SqtMatchCollection"].allMatches():
			# FIX make this a spectrum in the match
			s = Ms2Spectrum(firstScan=match["firstScan"], charge=int(match["charge"]))
			# FIX if no matches we have a problem
			if spectrumCollection.hasSpectrum(s):
				matchCollection.appendMatch(match)
			else:
				pass
				# print "Does not have", str(match)
				# print "From spectrum", str(s) 

		return matchCollection

#-------
# Main
#-------
if __name__ == '__main__':
	if len(sys.argv) < 4:
		print "Usage: %s <sqt-filename> <ms2-filename> <sqt-output-file>" % (sys.argv[0])
		sys.exit(1)

	dtaHitGen = DtaSelectHitGenerator()
	dtaHitGen["params"] = DtaSelectParams()

	matchCollection = SqtMatchCollection()
	matchCollection.parseFromFilename(sys.argv[1])
	realHitCollection = dtaHitGen.generateHits(matchCollection)

	pseudoGen = PseudoSequestMatchGenerator(matchCollection)

	btp = realHitCollection.calcTP()
	btpm = realHitCollection.calcTPmatches()
	bfp = realHitCollection.calcFP()
	bfpm = realHitCollection.calcFPmatches()

	spectrumCollection = Ms2SpectrumCollection(sys.argv[2])
	spectrumCollection.allSpectra()
	pseudoMatch = pseudoGen.generateMatches(spectrumCollection)

	pseudoMatch.outputToFilename(sys.argv[3])

	# sys.exit(-1)

	pseudoHitCollection = dtaHitGen.generateHits(pseudoMatch)

	atp = pseudoHitCollection.calcTP()
	atpm = pseudoHitCollection.calcTPmatches()
	afp = pseudoHitCollection.calcFP()
	afpm = pseudoHitCollection.calcFPmatches()

	print "before TP = ", btp 
	print "after  TP  = ", atp
	print "% TP preserved =", float(atp)/btp * 100.0
	
	print "before FP = ", bfp
	print "after  FP  = ", afp
	if bfp != 0.0:
		print "% FP preserved =", float(afp)/bfp * 100.0

	print "before TP matches = ", btpm
	print "after  TP  matches = ", atpm
	print "% TP preserved matches =", float(atpm)/btpm * 100.0
	
	print "before FP matches = ", bfpm
	print "after  FP matches = ", afpm
	if bfpm != 0.0:
		print "% FP preserved matches =", float(afpm)/bfpm * 100.0


