#$Revision: 1.6 $
import modelSelection 
import datafunc
import svm
import sys
import ker
import preproc
#---
# Main
#-------
if __name__ == '__main__':
	if len(sys.argv) < 3: 
		print """Usage: python %s 
	-r <cvector-train-file> 
	-s <sqt-test-filename> 
	-m <ms2-test-filename> 
	-t <cvector-test-file>
	[-k gaussian|polynomial]
	[-w gaussian width]
	[-d polynomial degree]
	[-c c value]
""" % (sys.argv[0])
		sys.exit(-1)

	optlist,args = getopt.getopt(sys.argv[1:],'r:s:m:t:k:w:d:c:')

	sqt =  "SQT.sqt"
	ms2 = "MS2"
	train = "CVEC-TRAIN"
	test = "CVEC-TEST"
	kernel = 'polynomial'
	width = 10 
	cVal = 10
	deg = 3
	
	for option_name,option_value in optlist:
		if option_name == '-r':
			train=option_value
		if option_name == '-s':
			sqt=option_value
		if option_name == '-m':
			ms2=option_value
		if option_name == '-t':
			test=option_value
		if option_name == '-k':
			kernel=option_value
		if option_name == '-w':
			width=float(option_value)
		if option_name == '-d':
			deg=int(option_value)
		if option_name == '-c':
			cVal=float(option_value)

	c = ChargeSieve()
	c.dtaHitGen = DtaSelectHitGenerator()
	c.dtaHitGen["params"] = DtaSelectParams()
	
	print >>sys.stderr, "Parsing .sqt file %s." % sqt
	matchCollection = SqtMatchCollection()
 	matchCollection.parseFromFilename(sqt)

	c.pseudoGen = PseudoSequestMatchGenerator(matchCollection)

	print >>sys.stderr, "Parsing .ms2 file %s." % ms2
	spectrumCollection = Ms2SpectrumCollection(ms2)
	spectrumCollection.allSpectra()

	mcfilt = Filter()
	c.spectrumCollection \
		= mcfilt.filterSpectra(spectrumCollection, keep = 'multi-charge')

	print >>sys.stderr, "Parsing training file %s." % train
 	DataSet = datafunc.SparseCDataSet
	trainData = DataSet(train, labelsColumn = 1) 

	#--
	rs  = preproc.Rescale()
	rs.train(trainData)
	# trainData.normalize(1)

	print >>sys.stderr, "Parsing testing file %s." % test
	testData = DataSet(test, idColumn = 0 , labelsColumn = 1) 
	#--
	rs.test(testData)
	#--
	# testData.normalize(1)

	
	if kernel in ('Polynomial','polynomial','poly'):
		print >>sys.stderr, "Using polynomial kernel of degree %.3f." % deg
		trainData.attachKernel('poly', degree = deg)
	elif kernel in ('Gaussian','gaussian','rbf'):
		print >>sys.stderr, "Using Gaussian kernel of width %.3f." % width
		trainData.attachKernel('gaussian', gamma = width)

	# train SVM out here 
	s = svm.SVM(C=cVal)
	s.train(trainData) 
	area = c.calcArea(s, testData) 
	
	print "KERNEL = %s, C = %.3f, DEG = %i, WIDTH = %i, AREA = %.3f" % \
		(kernel, cVal, deg, width, area)

