#---------------------------------
import os
import sys
import re

from hmmParams import *
#---------------------------------
class Runner:

  #---------------------------------
  def run(self):
    cmd = self.constructCmdLine()
    print >>sys.stderr,  "Running %s" % cmd
    self.stdin, self.stdout, self.stderr = os.popen3(cmd)
    self.stdin.close()

#---------------------------------
class GeneratePeptideRunner(Runner):
  def __init__(self, protein_input_filename):
    self.output_sequence = True
    self.min_mass = 200.0
    self.max_mass = 2400.0
    self.min_length = 6
    self.max_length = 50
    self.cleavages = CLEAVAGES
    self.missed_cleavages = 0
    self.sort = 'none'
    self.isotopic_mass = 'average'
    self.verbosity  = 100
    self.redundancy  = 'redundant'
    self.use_index =  False
    self.protein_input_filename = protein_input_filename

  #---------------------------------
  def constructCmdLine(self): 
    myDict = self.__dict__
    cmdLine = "generate_peptides "
    for key in myDict.keys():
      if key in  'protein_input_filename': continue
      option = re.sub('_', '-', key)
      if myDict[key] == True or myDict[key] == False:
        if myDict[key] == True: cmdLine += " --%s " % option
        elif myDict[key] == True: pass
        continue

      cmdLine += " --%s %s " % (option, myDict[key])

    return cmdLine + " %s" % self.protein_input_filename 

  #---------------------------------
  def getPeptides(self):
    peptides = {}
    for line in self.stdout:
      if line.startswith('#'): continue
      line = line.rstrip('\n')
      fields = line.split()
      peptide = re.sub('I','L',fields[4])
      peptides[peptide] = 1
    return peptides.keys()

#---------------------------------
if __name__ == '__main__':
  if len(sys.argv) < 1:
    print >>sys.stderr, "Usage: %s <protein_input_filename>" % sys.argv[0]
    sys.exit()
  
  runner = GeneratePeptideRunner("test.fasta")
  runner.min_mass = 800.0
  runner.max_mass = 805.0
  runner.run()
  print runner.getPeptides()
  
  
