#--------------------------------------------------------
# $Revision: 1.12 $
#--------------------------------------------------------
import getopt, sys

#--------------------------------------------------------
if __name__ == '__main__':
  if len(sys.argv) < 2: 
    print """Usage: python %s 
  <-r train-file> 
  <-p prefix>
  [-k gaussian|polynomial]
  [-w gaussian width]
  [-d polynomial degree]
  [-m matrix 2 png input]
""" % (sys.argv[0])
    sys.exit(-1)

#--------------------------------------------------------
import pickle, composite, preproc, time, datafunc, svm, assess

#--------------------------------------------------------
# Returns svmFname, rescaleFname 
#--------------------------------------------------------
def main(train=None, prefix=None,kernel='gaussian',width=None,degree=1,cvalue=1):
  DataSet = datafunc.SparseCDataSet
  trainData = DataSet(train, idColumn = 0, labelsColumn = 1) 
  rs = preproc.Rescale()
  rs.train(trainData)

  if kernel in ('Polynomial','polynomial','poly'):
    print >>sys.stderr, "Using polynomial kernel of degree %.3f." % deg
    trainData.attachKernel('poly', degree = deg) 
  elif kernel in ('Gaussian','gaussian','rbf'):
    print >>sys.stderr, "Using Gaussian kernel of width %.3f." % width
    trainData.attachKernel('gaussian', gamma = width)

  print >>sys.stderr, "Training svm with C = %.3f." % cvalue
  s=svm.SVM(C=cvalue)
  # p = Platt(s)
  s.train(trainData, saveSpace=False)  

    # serialize SVM
  svmFname = prefix + '.svm'
  s.save(svmFname)

  # serialize rescale
  rescaleFname = prefix + '.rescale'
  f = open(rescaleFname, "w")
  pickle.dump(rs, f) 
  f.close()

  r = s.stratifiedCV(trainData)
  r.computeStats()

  tp, fp, area = assess.roc(r.Y, r.givenY, r.decisionFunc)

  print "Area = ", area

  return svmFname, rescaleFname
  
#--------------------------------------------------------
if __name__ == '__main__':
  optlist, args = getopt.getopt(sys.argv[1:],'r:t:k:w:d:c:p:m:')
  train = kernel = width = degree = cvalue = prefix = None
  for option_name,option_value in optlist:
    if option_name == '-r': train=option_value
    if option_name == '-k': kernel=option_value
    if option_name == '-w': width=float(option_value)
    if option_name == '-d': degree=int(option_value)
    if option_name == '-c': cvalue=float(option_value)
    if option_name == '-p': prefix=option_value

  svmFname, rescaleFname = main(train=train, prefix=prefix, kernel=kernel, \
      width=width, degree=degree, cvalue=cvalue)

#---------------------------
#if matrix2png != None:
#    dataFields = []
#    data = open(train)
#    for line in data:
#      line = line.rstrip()
#      dataFields.append(line.split(','))
#    data.close()
#    outputLines = []
#
#    for idx in range(len(trainData)):
#      outputFields = []
#      for field in dataFields[idx]:
#        outputFields.append(field)
#      outputFields[0] = outputFields[0].split('.')[1]
#      outputFields.append(s.decisionFunc(trainData, idx))
#      outputLines.append(outputFields)
#
#    outputLines.sort(lambda x,y: cmp(float(x[-1]), float(y[-1])))
#
#    fh = open(matrix2png, "w")
#    header = "svm label b y a b-h2o y-h2o b-nh3 y-nh3 logtic numions disc".split(" ")
#    fh.write("\t".join(header) + "\n")
#    for idx in range(len(outputLines)):
#      fh.write("\t".join(map(str, outputLines[idx])) + "\n")
#    fh.close()
#---------------------------
