#---------------------------------------------------------------------------
# File: spFilter.py
#---------------------------------------------------------------------------
# Author: $Author: aklammer $
#---------------------------------------------------------------------------
# Revision: $Revision: 1.12 $
#---------------------------------------------------------------------------
# Inputs:
# 
#  - a <collection-file> which contains the spectra 
#  - the output of generatePeptides.py, <prefix>.peps, which maps spectra 
#    to candidate peptides:
#
#     SPECTRUM_ID \t HMM_CAND_FILE \n
#     - SPECTRUM_ID is the identifier of the spectrum
#     - HMM_CAND_FILE is the file containing the candidate peptides
#---------------------------------------------------------------------------
# Outputs:
#
#  - another <prefix>,sp.peps file, mapping spectra to candidate peptides
#---------------------------------------------------------------------------
import os, os.path, sys, cPickle
from Mass import mono_mass
import MatchCollection
from hmmTestSpectrum import parsePeps
#try:
#  from pylab import *
#  import psyco
#  psyco.full()
#except:
#  print >>sys.stderr, "Could not import or use psyco or pylab!"

from hmmParams import *

#-------------------------------------------------------------------------------
def mzToInt(spectrum, equalizeFlanking=False):
  peaks = spectrum["peaks"]
  binned = {}
  for peak in peaks:
    mz  = int(peak[0]+0.5)
    i   = peak[1]
    if binned.has_key(mz): binned[mz] += i
    else: binned[mz] = i

  
  if equalizeFlanking == True:
    newInts = {}
    binkeys = binned.keys()
    for peakMz in binkeys:
      if binned.has_key(peakMz+1): 
        if binned[peakMz+1] < binned[peakMz]: 
          newInts[peakMz+1] = binned[peakMz]
      else:
        newInts[peakMz+1] = binned[peakMz]

      if binned.has_key(peakMz-1): 
        if binned[peakMz-1] < binned[peakMz]: 
          newInts[peakMz-1] = binned[peakMz]
      else:
        newInts[peakMz-1] = binned[peakMz]
    for peakMz in newInts: binned[peakMz] = newInts[peakMz] 

  # FIX number of peaks matched within +/- 1?
  mzs = binned.keys()
  mzs.sort(lambda x,y: cmp(binned[y],binned[x]))
  topPeaks = {}
  normDenom = binned[mzs[0]] / 100.0
  for mz in mzs[0:MAX_PEAKS]:
    topPeaks[mz] = binned[mz] / normDenom
  return topPeaks

#-------------------------------------------------------------------------------
def calcIntIons(pep, z=1):
  bs = []
  ys = []

  bIonMass = mono_mass('H_plus')
  for idx in range(len(pep)-1):
    bIonMass += mono_mass(pep[idx])
    bs.append(int(bIonMass+0.5))

  yIonMass = mono_mass('H_plus') + mono_mass('H2O')
  for idx in range(len(pep)-1, 0, -1):
    yIonMass += mono_mass(pep[idx])
    ys.append(int(yIonMass+0.5))

  return bs, ys

#-------------------------------------------------------------------------------
def spScore(mzIntDict, bs, ys):

  matchedIonSum = 0.0
  numberMatchedIons = 0
  consecutiveIonCount = 0.0
  immoniumIonScore = 0.0 # we are ignoring this for now
  numberPredictedIons  = len(bs) + len(ys)

  sawLastB = False
  for b in bs:
    if mzIntDict.has_key(b): 
      matchedIonSum += mzIntDict[b]
      numberMatchedIons += 1
      if sawLastB == True: consecutiveIonCount += 1
      sawLastB = True
    else:
      sawLastB = False
   
  sawLastY = False
  for y in ys:
    if mzIntDict.has_key(y): 
      matchedIonSum += mzIntDict[y]
      numberMatchedIons += 1
      if sawLastY == True: consecutiveIonCount += 1
      sawLastY = True
    else:
      sawLastY = False

  Sp = matchedIonSum * numberMatchedIons * (1 + consecutiveIonCount * BETA) *\
      (1 + immoniumIonScore * RHO) / float(numberPredictedIons)

  if OUTPUT_SCORES: 
    print ("matchedIonSum = %.3f, numberMatchedIons = %i, " + \
    "consecutiveIonCount = %i, numberPredictedIons = %i, Sp = %.3f") % \
    (matchedIonSum, numberMatchedIons, consecutiveIonCount, \
    numberPredictedIons, Sp)

  return Sp

#-------------------------------------------------------------------------------
def spFilter(spectrum, peps):

  mzIntDict = mzToInt(spectrum) 
  pep2score = {}
  pepCount = 0
  for pep in peps:
    pepCount += 1
    if pepCount % PEPS_PER_COMMENT == 0 : 
      print >>sys.stderr, "Reached peptide %i" % pepCount
    bs, ys = calcIntIons(pep)
    if OUTPUT_SCORES: print pep
    pep2score[pep] = spScore(mzIntDict, bs, ys)

  peps = pep2score.keys()
  peps.sort(lambda x,y: cmp(pep2score[y], pep2score[x]))
  if MAX_PEPS >= len(peps):
    filteredPeps = peps
  else:
    filteredPeps = peps[0:MAX_PEPS]
  return filteredPeps

#-------------------------------------------------------------------------------
def scorePeps(spectrum, peps, scoreFunc=spScore):
  mzIntDict = mzToInt(spectrum) 
  pep2score = {}
  peps = parsePeps(candFile)
  for pep in peps:
    bs, ys = calcIntIons(pep)
    pep2score[pep] = scoreFunc(mzIntDict, bs, ys)
  return pep2score

#-------------------------------------------------------------------------------
def scorePepsFlank(spectrum, peps):
  mzIntDict = mzToInt(spectrum, equalizeFlanking = True) 
  pep2score = {}
  peps = parsePeps(candFile)
  for pep in peps:
    bs, ys = calcIntIons(pep)
    pep2score[pep] = spScore(mzIntDict, bs, ys)
  return pep2score

#-------------------------------------------------------------------------------
def spFilterFromFiles(spectrum, candFile, spCandFile):
  peps = parsePeps(candFile)

  filteredPeps = spFilter(spectrum, peps)

  fh = open(spCandFile, 'w')
  for pep in filteredPeps: fh.write(pep + "\n")
  fh.close()

#-------------------------------------------------------------------------------
if __name__ == "__main__":
  if len(sys.argv) < 4:
    print >>sys.stderr, """
Usage: %s <collection-file> <peps-file> 
    <collection-file> - a pickledpsm file
    <peps> - The .peps file.
    <score> - The score to use

    output probs and zscores for each spectrum to <peps>-sp.peps file
  """ % os.path.basename(sys.argv[0])
    sys.exit(1)

  collectionFile = sys.argv[1]
  pepsFile = sys.argv[2]

  print >>sys.stderr, "Reading spectrum collection %s" % collectionFile
  coll = cPickle.load(open(collectionFile))

  # parse the .peps file
  pepsFh = open(pepsFile)
  rank = []
  outFile = ""
  for line in pepsFh:
    line = line.rstrip('\n')

    suid, candFile = line.split()
    peps = parsePeps(candFile)

    spectrum = coll.getSpectrum(suid)
    peptide = ""
    for match in coll["matches"]:
      if match.spectrum.uid() == str(suid):
        peptide = match.peptide

    if spectrum == None: continue
    spectrum.parsePeaks()

    print >>sys.stderr, "Analyzing spectrum %s" % suid
    if sys.argv[3] == 'sp':
      pep2score = scorePeps(spectrum, peps, spScore)
      outFile = "sp1.png"
    elif sys.argv[3] == 'sp-flank':
      pep2score = scorePepsFlank(spectrum, peps)
      outFile = "sp-flank.png"
    else:
      raise SystemExit, "Unknown score %s" % sys.argv[3]
    
    peps = pep2score.keys()
    peps.sort(lambda x,y: cmp(pep2score[y], pep2score[x]))

    for idx in range(len(peps)):
      if peps[idx] == peptide: rank.append(idx) 

  rank.sort(lambda x,y: cmp(y,x))
  cumdist =  [[],[]]
  for idx in range(len(rank)):
    cumdist[0].append(rank[idx])
    cumdist[1].append(1.0 - float(idx)/len(rank))

  axis([0,5000,0.8,1.0])
  plot(cumdist[0], cumdist[1])
  savefig(outFile)
  show()
    

  pepsFh.close()
