#!/usr/bin/python
#------------------
# $Revision: 1.117 $
#------------------
import getopt,sys,re,math,os,os.path
from Utils import *
from Mass import avg_mass, tol_mass, mono_mass, peptide_mass
import Mass
try: import Numeric
except: print >>sys.stderr, "Could not import Numeric!"

# ION_TYPES = [ 'b','y','a','b-h2o','y-h2o','b-nh3', 'y-nh3', 'p', 'n', \
ION_TYPES = [ 'b','y','a','b-h2o','y-h2o','b-nh3', 'y-nh3', \
              'b+2','y+2','a+2','b-h2o+2','y-h2o+2','b-nh3+2', 'y-nh3+2']

TYPE2COLOR = {'b':'red', 'y':'red', 'a':'green', 'b-nh3':'red', \
        'y-nh3':'red', 'b-h2o':'magenta', 'y-h2o':'magenta', \
        'b+2':'magenta', 'y+2':'magenta', 'a+2':'magenta', 'b-h2o+2':'magenta',\
        'y-h2o+2':'magenta', 'b-nh3+2':'magenta', 'y-nh3+2':'magenta' }

MAX_MZ = 10000
B_KEY = 'b'
Y_KEY = 'y'
NOISE_KEY = 'n'
A_CHILDPEAK_KEY   = 'a'
H2O_CHILDPEAK_KEY = '-h2o'
NH3_CHILDPEAK_KEY = '-nh3'
PLUS2_CHILDPEAK_KEY = '+2'
PLUS3_CHILDPEAK_KEY = '+3'

DEFAULT_INTENSITY = 1
DEBUG = 1
MIN_PRECURSOR_RATIO = 0.10
BIN_SIZE = 0.20
NUMBER_MZ_BINS = 20
ADJ_FACTORS = \
  [ 0.0014754018253741975,  0.0475812734770069,   0.084433116570503633,  \
    0.19944713534215031,    0.32055543943361081,  0.13274638043172721,    \
    0.11967310823361815,    0.071566942854716278, 0.0218956465003502,    \
    0.00062555533094675749, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 ]
LOG_2 = math.log(2.0)


LOG_2 = math.log(2.0)

#----- SEQUEST normalization constants
PRECURSOR_REMOVAL_WIDTH = 5.0
BIN_PEAK_MAX = 50.0
BIN_NUMBER = 10

#----- spectrum stringify control
PRINT_LABEL = True

#----------------- TODO use an object instead??
# Routines for converting basic types to SpectrumUIDs and vice versa.
#----------------- 
def spectrumUID(spectrumID, spectrumGroupID, charge):
  return (str(int(spectrumID)) + "," +str(spectrumGroupID) + ","+ str(int(charge)))

def uidSpectrumID(spectrumUID):
  if str(spectrumUID) == spectrumUID: return spectrumUID.split(',')[0]
  else: return spectrumUID[0]

def uidSpectrumGroupID(spectrumUID):
  if str(spectrumUID) == spectrumUID: return spectrumUID.split(',')[1]
  else: return spectrumUID[1]

def uidCharge(spectrumUID):
  if str(spectrumUID) == spectrumUID: return spectrumUID.split(',')[2]
  else: return spectrumUID[2]

def uidString(spectrumUID):
  return "%s,%s,%i" % spectrumUID

#--------------------------------------------------------------------------
OBSERVABLE_MZ_MAX = 2400
OBSERVABLE_MZ_MIN = 200

#--------------------------------------------------------------------------
def between(val, min, max): return ((min <= val) and (val <= max))

#--------------------------------------------------------------------------
def observable(mz): return between(mz, OBSERVABLE_MZ_MIN, OBSERVABLE_MZ_MAX)


class Spectrum:
  "Contains information in a single spectrum."

  def __init__(self):
    self.data = {}
    self.data["first scan"]= -1 # the first scan number for this spectrum
    self.data["last scan"]= -1 # the last scan number for this spectrum
    self.data["peaks"] = [] # an array of tuples (float mz, float intensity)
    self.data["num peaks"] = 0 # The count of all peaks
    self.data["assumed m"] = -1 # The M+H+ mass of the spectrum.
    self.data["assumed z"] = self.data["charge"] = 0  # Precursor ion charge
    self.data["min peak mz"] = 0 # The min( all peak m/z)
    self.data["max peak mz"] = 10000 # The max( all peak m/z)
    self.data["total energy"] = -1 # Also known as TIC
    self.data["data source"] = "Python" # Where did the Spectrum come from?
    self.data["precursor mz"] = -1 # The precursor m/z.
    self.data["file source"] = "" # Which file the spectrum came from
    self.data["group id"] = "Nogroupid" # Id allowing spectra to be grouped
    # The following are booleans that indicate whether the spectrum
    # has undergone a particular preprocessing step.
    self.data["sum normalized"] = False 
    self.data["base peak normalized"] = False
    self.data["base peak normalized adjusted"] = False
    self.data["square rooted"] = False 
    self.data["hasRankRelativizedIntensities"] = False
    self.mzToHighPeak = [None] * MAX_MZ

  def __getitem__(self, key):
    return self.data[key]

  def __delitem__(self, key):
    del self.data[key]


  def __setitem__(self, key, item):
    if key == "charge":
      self.data[key] = int(item)
    self.data[key] = item    

  def __str__(self):
    first = "\t".join( map(str, [ 
      self["first scan"],\
      self["last scan"], \
      self["charge"], \
      self["precursor mz"], \
      self["assumed z"], \
      self["assumed m"], \
      self["min peak mz"], \
      self["max peak mz"], \
      self["num peaks"], \
      self["total energy"]
      ])
    ) 
    return "".join([first, peaks])

  def uid(self):
    # TODO implement uid routine in sub classes
    if not self.data.has_key("group id"): self["group id"] = \
        os.path.basename(self["file source"])
    return spectrumUID(self["first scan"] , self["group id"], self["charge"])

  def integerMass(self): 
    return int(self["precursor mz"] * self["charge"] + 0.5)

  def addPeak(self, mz, intensity):
    mz = float(mz)
    intensity = float(intensity)
    self["peaks"].append( (mz, intensity) )
    self["total energy"] += intensity
    self["num peaks"] += 1
    if self["num peaks"] == 1:
      self["min peak mz"] = self["max peak mz"] = mz
      return
    if mz < self["min peak mz"] and intensity > 0.0 : self["min peak mz"] = mz
    if self["max peak mz"] < mz and intensity > 0.0 : self["max peak mz"] = mz

  def sumNormalizeIntensity(self, total_intensity=DEFAULT_INTENSITY, max=1000000000.0):
    sum = 0.0
    for my_peak in self["peaks"]:
      sum += my_peak[1]
    if sum == 0.0: return

    for idx in range(len(self["peaks"])):
      peak = self["peaks"][idx]
      intensity = peak[1]
      intensity /= sum 
      intensity *= total_intensity
      if intensity > max: intensity = max
      self["peaks"][idx] = (peak[0], intensity)

    self["total energy"] = total_intensity
    self["sum normalized"] = True

    # changes spectra!

  def basePeakNormalizeIntensity(self): 
    print >> sys.stderr, "Warning, some fields not updated!" # FIX

    # Check if spectrum has already been normalized
    basePeak = 0.0
    for my_peak in self["peaks"]:
      if my_peak[1] > basePeak: basePeak = my_peak[1]

    for idx in range(len(self["peaks"])):
      peak = self["peaks"][idx]
      intensity = peak[1]
      intensity /= basePeak
      self["peaks"][idx] = (peak[0], intensity)

  def sqrtIntensity(self): 
    print >> sys.stderr, "Warning, some fields not updated!" # FIX
    for idx in range(len(self["peaks"])):
      peak = self["peaks"][idx]
      intensity = peak[1]
      intensity = math.sqrt(intensity)
      self["peaks"][idx] = (peak[0], intensity)

  def sqrtCosineAngle(self, spectrum):
    for idx in range(len(self["peaks"])):
      peak = self["peaks"][idx]
      intensity = peak[1]
      self["peaks"][idx] = (peak[0], math.sqrt(intensity))

    for idx in range(len(spectrum["peaks"])):
      peak = spectrum["peaks"][idx]
      intensity = peak[1]
      self["peaks"][idx] = (peak[0], math.sqrt(intensity))

    return self.cosineAngle(spectrum)
  
  #----------------------
  # Changes spectra!
  # The relative intensity used in Wan et al. (2005) implementation of PepHMM
  # except inverted  (see below)
  #----------------------
  def rankRelativizeIntensity(self):
    peaks = self["peaks"]
    peaks.sort(lambda x,y: cmp(x[1],y[1])) # ascending intensity, so 
    # high peaks have high rank. *not* consistent with PepHMM anymore
    assert(peaks[0][1] <= peaks[-1][1])

    for idx in range(len(peaks)):
      peak = peaks[idx]
      rankI = float(idx + 1)/len(peaks)
      newPeak = (peak[0], rankI)
      peaks[idx] = newPeak

    peaks.sort(lambda x,y: cmp(x[0],y[0])) # resort by m/z
    self["hasRankRelativizedIntensities"] = True


  def cosineAngle(self, spectrum):
    # TODO add peakVectorDict method to Spectrum object
    peak_vector_dict = {}
    for my_peak in self.getPeaks():
      binned_mz = math.floor(my_peak[0] + 0.5)
      if peak_vector_dict.has_key(binned_mz):
        peak_vector_dict[binned_mz] += my_peak[1]
      else:
        peak_vector_dict[binned_mz] =  my_peak[1]

    # FIX assumes Ms2Spectrum
    peak_hat_dict = {}
    for my_peak in spectrum.getPeaks():
      binned_mz = math.floor(my_peak[0] + 0.5)
      if peak_hat_dict.has_key(binned_mz): 
        peak_hat_dict[binned_mz] += my_peak[1]
      else:
        peak_hat_dict[binned_mz] =  my_peak[1]
    
    hathat = dict_dot_product(peak_hat_dict, peak_hat_dict)
    vecvec = dict_dot_product(peak_vector_dict, peak_vector_dict)
    vechat = dict_dot_product(peak_hat_dict, peak_vector_dict)
    denom = hathat * vecvec
    if denom == 0.0: return 0.0
    return (vechat / math.sqrt(denom))
  
  # START HERE
  def numerifyPeaks(self):
    assert(self["peaks"] != None)
    mzs = []
    intensities = []
    for peak in self["peaks"]:
      mzs.append(peak[0])
      intensities.append(peak[1])
    numericMzs = Numeric.array(mzs)
    numericIntensities = Numeric.array(intensities)
    del self["peaks"] #  = None
    self["numericPeaks"] = (numericMzs, numericIntensities)
    # the only reason to run this routine is to reduce size of the object
    if len(self["peakLines"]) > 0: 
      # print >>sys.stderr, "Deleting peakLines"
      del self["peakLines"] # = None

  def deNumerifyPeaks(self):
    assert(self["peaks"] == None and self["numericPeaks"] != None)
    numericMzs = self["numericPeaks"][0]
    numericIntensities = self["numericPeaks"][1]
    self["peaks"] = []
    for idx in range(len(numericMzs)):
      peak = (numericMzs[idx], numericIntensities[idx])
      self["peaks"].append(peak)
    self["numericPeaks"] = None

class DtaSpectrum(Spectrum):
  """Parses a Dta file into a spectrum object. 
  Taken from the czar module. Less than ideal, but such is life."""
  def __init__(self, filename, parsePeaks = False):
    Spectrum.__init__(self) 

    self["data source"] = filename
    self["name"] = os.path.basename(filename)
    
    self["parsePeaks"] = parsePeaks
    self["peakLines"] = []

    fields = self["name"].split('.')
    if len(fields) > 3:
      # long format
      prefix, firstScan, lastScan, charge =  \
          fields[0], fields[1], fields[2], fields[3]
      self["first scan"] = firstScan 
      self["last scan"]  = lastScan
      self["charge"]     = int(charge)
      self["parsePeaks"] = parsePeaks
      self["peakLines"] = []
      self["num peaks"] = 0
      self["peaks"] = []
      self["total energy"] = 0.0
      self["min peak mz"] = 0.0
      self["max peak mz"] = 10000.0
      self["file source"] = prefix
      self["group id"] = prefix
    else:
      # dta filename does not contain meta information
      pass
    
    self.parseDta(filename)

  # TODO possibly put in Spectrum baseclass
  def getPeaks(self):
    if self["parsePeaks"] == False: 
      self.parsePeaks()
    return self["peaks"]


  def parseDta(self, filename):
    dta_file = open(filename, 'r')

    seen_first_line = False
    for line in dta_file:
      # ignore optional character containing lines
      if not line[0].isdigit(): 
        if not seen_first_line: continue
        else: raise SystemExit, "Character line %s embedded in dta file %s!" % (line, filename)
      
      if not seen_first_line:
        self["assumed m"], self["charge"] = line.split()
        seen_first_line = True
        continue

      self["peakLines"].append(line)
      self["num peaks"] = self["num peaks"] + 1
      if self["parsePeaks"]:
        line = line.rstrip(os.linesep)
        mz, intensity = line.split(' ')
        self.addPeak(mz, intensity)
        self["total energy"] = self["total energy"] + float(intensity)
        if mz < self["min peak mz"]: self["min peak mz"] = mz
        if mz > self["max peak mz"]: self["max peak mz"] = mz
    self["charge"] = int(self["charge"])
    self["assumed z"] = self["charge"]

    self["precursor mz"] = (float(self["assumed m"]) \
        + (self["charge"]-1) * Mass.avg_masses['H_plus']) / int(self["charge"])

    dta_file.close()

  def __str__(self):
    firstLine = str(self["assumed m"]) + " " + str(self["assumed z"]) + os.linesep
    lines = ""
    if self["parsePeaks"]:
      lines = "".join(map( \
          lambda x: str(x[0]) + " " + str(x[1]) + os.linesep, self["peaks"]))
    else:
      lines = "".join(self["peakLines"])
    return firstLine + lines

  def output(self, dirName):
    "Output to an appropriately named file in directory dirName"
    
    # TODO add create filename functionality
    file = open(dirName + os.sep + self["name"], 'w')
    file.write(str(self))
    file.close()

  def parsePeaks(self):
    pass

class RichPeak:
  "A peak that can be annotated."

  def __init__(self, mz=None, i=0.0, z=1, m=None):
    if (mz == None and m == None) or (mz != None and m != None):
      raise SystemExit, "RichPeak constructor needs only one of mz or m!"

    # set standard fields
    if mz == None: 
      self.m = m
      self.mz = m / z
    else: 
      self.mz = mz
      self.m = mz * z
    self.z   = z # charge
    self.i   = i # intensity

    self.hasRankI = False 
    self.rankI = 0 # rank relativized intensity. not always present!

    self.hasAdj = False
    self.adj = 0 # adjusted intensity. not always present!
    
    self.mzoffset = 0.0 # difference between the real peak and the annotated peak

    #------------------------------------------------------------------
    # a RichPeak can be possible, observable, and observed.
    # for a "real" peak all three are true. a b-h2o for a b that cannot lose water
    # is an example of a peak that is not possible.
    # a b-h2o for a b that *can* lose water but is outside the m/z window
    # is an example of a peak that is possible, but not observable
    # and a b-h2o for a b that *can* lose water and is inside the m/z window
    # but for which no peak is observed is an example of a peak that is 
    # possible, observable, but not observed
    # The default is to be all three.
    #------------------------------------------------------------------
    self.possible   = True 
    self.numberPossible = 0
    self.observable = True 
    self.observed   = True 

    # RichPeak specific fields
    self.type = '_' # default is unknown
    self.nterm = '_' # default is unknown
    self.cterm = '_' # default is unknown
    self.frag =  ""
    self.partnerPeak = None
    self.parentPeak = None
    self.spectrum = None
    self.childPeaks = {}
    self.nidx = 0
    self.cidx = 0
  
  def __getitem__(self, key):
    if int(key) == 0: return self.mz 
    elif int(key) == 1: return self.i
    else: raise SystemExit, "Strange RichPeak key %s" % str(key)

  def __setitem__(self, key, item):
    if int(key) == 0: self.mz = item
    elif int(key) == 1: self.i = item
    else: raise SystemExit, "Strange RichPeak key %s" % str(key)

  def __setattr__(self, attr, value):
    "Checks if various attributes conform to constraints not covered by Enum."
    self.__dict__[attr] = value
    if attr == "rankI": self.hasRankI = True
  
  def __str__(self):
    type = self.type
    frag = self.frag
    if self.type == '_' or self.type == NOISE_KEY: type  = ""
    if self.type == '_' or self.type == NOISE_KEY: frag  = ""
    if PRINT_LABEL:
      return "%3.2f\t%3.5f\t%s\t%s" % (self.mz, self.i, type, self.color())
    else:
      return "%3.2f\t%3.5f\t%s\t%s" % (self.mz, self.i, "", self.color())
 
  def color(self):
    if TYPE2COLOR.has_key(self.type): return TYPE2COLOR[self.type]
    else: return 'black'
  
  # TODO add set methods for type
  def isBpeak(self): return self.type == 'b'
  def isYpeak(self): return self.type == 'y'
  def isBYpeak(self): return (self.type == 'y') or (self.type == 'b')
  def isApeak(self): return self.type == 'a'
  def isNH3peak(self): 
    return (self.type == 'b-nh3' or self.type == 'y-nh3')
  def isH2Opeak(self): 
    return (self.type == 'b-h2o' or self.type == 'y-h2o')
  def isIsotopePeak(self): return self.type == 'i'

  def getPeakOfType(self, type):
    bPeak = None
    if self.type == B_KEY: bPeak = self
    else: raise SystemExit, "getPeakOfType must get a b-peak, got a %s !" % type
  
    # TODO replace with recursion. However, for now this is fine
    for peak in [ bPeak, bPeak.partnerPeak ]:
      if peak.type == type: return peak
      for childPeak in peak.childPeaks.values():
        if childPeak.type == type: return childPeak
        for grandchildPeak in childPeak.childPeaks.values():
          if grandchildPeak.type == type: return grandchildPeak

    raise SystemExit, "problem getting peak of type %s" % type

  def canLoseH2O(self): 
    count = 0
    for i in self.frag: 
      if i in (Mass.CanLoseH2O): count += 1
    if self.type == 'y' or self.type == 'p' : 
      count += 1  # y can always lose H2O off cterm
    return count

  def canLoseNH3(self): 
    # pretty much everyone can lose an NH3 off their amino terminus
    count = 1
    for i in self.frag: 
      if i in (Mass.CanLoseNH3): count += 1

    return count

def dict_dot_product(dict1, dict2):
  sum = 0.0
  for key in dict1.keys():
    if not dict2.has_key(key): continue
    sum += dict1[key] * dict2[key]
  return sum

class RichSpectrum(Spectrum):
  "A Spectrum object with RichPeak objects (i.e. peaks that can be annotated)."

  def __init__(self):
    Spectrum.__init__(self)

    # The RichSpectrum RichPeak objects are not yet annotated 
    self.data["hasAnnotation"]   = False
    self.data["hasIntensities"]  = False
    self.data["hasNeutrals"]  = False
    self.data["hasMultiCharge"]  = False
    self.data["hasIsotopes"]  = False
    self.data["hasRankRelativizedIntensities"]  = False
    self.data["hasRankRelativizedAdjustedIntensities"]  = False
    self.data["richPeaks"]       = []
    self.data["source spectrum"]  = None
    self.precursor = None
    self.peptide = ""
    self.name = ""
    self.type2peaks = {}
    for ionType in ION_TYPES: self.type2peaks[ionType] = []
    
  def __str__(self):
    #string = str(self["first scan"]) + "." + str(self["last scan"]) \
    #  + "." + str(self["charge"]) + "\n"
    if self.name == "":
      string = ">%s\n" % self.peptide
    else: 
      string = ">%s\n" % self.name
    self["richPeaks"].sort( lambda x,y: cmp(x.mz, y.mz)  )
    return (string + "\n".join(map(str,self["richPeaks"])))

  def __getitem__(self, key):
    return self.data[key]

  def __setitem__(self, key, item):
    self.data[key] = item    

  def getNoisePeaks(self):
    noisePeaks = []
    for peak in self["richPeaks"]:
      if peak.type == NOISE_KEY: 
        noisePeaks.append(peak)
    return noisePeaks

  def getPeaksOfType(self, type):
    if self.type2peaks.has_key(type): 
      return self.type2peaks[type]
    else:
      print >>sys.stderr, "Could not find peak type %s" % type
      return []

  #----------------------
  # Changes spectra!
  # The relative intensity used in Wan et al. (2005) implementation of PepHMM
  # except inverted (see below)
  #----------------------
  def rankRelativizeIntensity(self):
    presentPeaks = filter(lambda x: x.observed == True, self["richPeaks"])
    presentPeaks.sort(lambda x,y: cmp(x[1],y[1])) # ascending order
    # which is different than Ting's method
    assert(presentPeaks[0][1] <= presentPeaks[-1][1])

    for idx in range(len(presentPeaks)):
      peak = presentPeaks[idx]
      rankI = float(idx + 1)/len(presentPeaks)
      presentPeaks[idx].rankI = rankI

    self["hasRankRelativizedIntensities"] = True

  def rankRelativizeLogIntensity(self, max=20):
    if self["hasRankRelativizedIntensities"] == False:
      self.rankRelativizeIntensity()
    presentPeaks = filter(lambda x: x.observed == True, self["richPeaks"])
    for idx in range(len(presentPeaks)):
      rankI = presentPeaks[idx].rankI
      if rankI >= 1.0:
        rankI = 20 # FIX hard coded 20
      else:
        rankI = int( - math.log(1.0 - rankI) / LOG_2 * 2.0 ) 
      if rankI > max: rankI = max
      presentPeaks[idx].rankI = rankI / float(max)

  #----------------------
  # Changes spectra!
  # The relative intensity used in Wan et al. (2005) implementation of PepHMM
  # (but inverted) after Adjusting the peaks for m/z bias. 
  #----------------------
  def rankRelativizeAdjustedIntensity(self):

    peaks = filter(lambda x: x.observed == True, self["richPeaks"])
    precursorMz = self["precursor mz"]
    # FIX change to addAdjustedIntensity
    for idx in range(len(peaks)):
      p  = peaks[idx]
      mzBin = int(((p.mz/precursorMz)-MIN_PRECURSOR_RATIO)/BIN_SIZE)
      assert mzBin < NUMBER_MZ_BINS
      p.adj = p.i / ADJ_FACTORS[mzBin]  
      p.hasAdj = True
    
    peaks.sort(lambda x,y: cmp(x.adj,y.adj)) # ascending intensity
    assert(peaks[0].adj <= peaks[-1].adj)

    for idx in range(len(peaks)):
      p = peaks[idx]
      rankI = float(idx + 1)/len(peaks)
      p.adj = rankI 

    self["hasRankRelativizedAdjustedIntensities"] = True

  # changes spectra!
  def sqrtCosineAngle(self, spectrum):
    for my_peak in self.getPeaks(): my_peak.i = math.sqrt(my_peak.i)
    for idx in range(len(spectrum["peaks"])):
      peak = spectrum["peaks"][idx]
      intensity = peak[1]
      spectrum["peaks"][idx] = (peak[0], math.sqrt(intensity))

    return self.cosineAngle(spectrum)

  def matchedNormTopPeaks(self, spectrum, maxPeaks):

    # sort both sets of peaks by descending intensity
    myPeaks = self.getPeaks()
    myPeaks.sort(lambda x,y: cmp(y.i, x.i))
    hisPeaks = spectrum["peaks"]
    hisPeaks.sort(lambda x,y: cmp(y[1], x[1]))

    # count the number of matching top maxPeaks in each spectrum
    matchingPeaks = 0
    for myIdx in range(len(myPeaks)):
      if myIdx >= maxPeaks: break
      for hisIdx in range(len(hisPeaks)):
        if hisIdx >= maxPeaks: break
        # compare intensities
        mzdel =abs(myPeaks[myIdx].mz - hisPeaks[hisIdx][0])
        if mzdel < tol_mass("peak"):
          matchingPeaks += 1

    return float(matchingPeaks) / maxPeaks
  
  def cosineAngle(self, spectrum):
    # FIX assumes RichSpectrum
    peak_vector_dict = {}
    for my_peak in self.getPeaks():
      binned_mz = math.floor(my_peak.mz + 0.5)
      if peak_vector_dict.has_key(binned_mz):
        peak_vector_dict[binned_mz] += my_peak.i
      else:
        peak_vector_dict[binned_mz] =  my_peak.i

    # FIX assumes Ms2Spectrum
    peak_hat_dict = {}
    for my_peak in spectrum.getPeaks():
      binned_mz = math.floor(my_peak[0] + 0.5)
      if peak_hat_dict.has_key(binned_mz): 
        peak_hat_dict[binned_mz] += my_peak[1]
      else:
        peak_hat_dict[binned_mz] =  my_peak[1]
    
    hathat = dict_dot_product(peak_hat_dict, peak_hat_dict)
    vecvec = dict_dot_product(peak_vector_dict, peak_vector_dict)
    vechat = dict_dot_product(peak_hat_dict, peak_vector_dict)
    denom = hathat * vecvec
    if denom == 0.0: return 0.0
    return (vechat / math.sqrt(denom))
  
  def sortDescendingIntensity(self):
    self["richPeaks"].sort(lambda x,y: cmp(y.i, x.i))

  def getFirstNpeaks(self, n):
    return self["richPeaks"][0:n]

  def addRichPeak(self, peak):
    self["richPeaks"].append(peak)
    mzInt = int(peak.mz)
    if self.mzToHighPeak[mzInt] == None: self.mzToHighPeak[mzInt] = peak
    elif self.mzToHighPeak[mzInt].i > peak.i : self.mzToHighPeak[mzInt] = peak

  def addPeak(self, peak):

    # Update the RichPeak implementation...
    self.addRichPeak(peak)
    # ... but also update tuple peak implementation
    Spectrum.addPeak(self, peak.mz, peak.i)

    # ... and include the peak in the type dict
    if self.type2peaks.has_key(peak.type): 
      self.type2peaks[peak.type].append(peak)
    else:
      self.type2peaks[peak.type] = [ peak ]
    peak.spectrum = self # circular reference. bad, but makes things easier.

  def getPeaks(self): return self["richPeaks"]

  def getBYpeaks(self): 
    return filter(lambda x: x.isBYpeak(), self["richPeaks"])

  def getNH3peaks(self): 
    return filter(lambda x: x.isNH3peak(), self["richPeaks"])

  def getH2Opeaks(self): 
    return filter(lambda x: x.isH2Opeak(), self["richPeaks"])

  def getApeaks(self): 
    return filter(lambda x: x.isApeak(), self["richPeaks"])

  def addSpectrumInfo(self, spectrum):
    # Set standard Spectrum fields 
    self["first scan"]         = spectrum["first scan"]
    self["last scan"]         = spectrum["last scan"]
    self["charge"]             = spectrum["charge"]
    self["precursor mz"]       = spectrum["precursor mz"]
    self["assumed m"]         = spectrum["assumed m"]
    self["assumed z"]         = self["charge"] = spectrum["assumed z"]
    self.data["min peak mz"]   = spectrum["min peak mz"]
    self.data["max peak mz"]   = spectrum["max peak mz"]
    self.data["total energy"] = spectrum["total energy"]
    self["source spectrum"] = spectrum

    # Convert the Spectrum peaks (which are just tuples) 
    # into RichPeak objects (which are capable of annotation)
    # self.data["peaks"] = spectrum["peaks"]
    # for peak_tuple in spectrum["peaks"]:
    #  r = RichPeak(peak_tuple)
    #  self["richPeaks"].append(r)
    #  self["num peaks"] += 1
    # self.data["hasIntensities"]  = True


  def addHighIntensitiesFromSpectrum(self, spectrum):
    # Set standard Spectrum fields 
    self.addSpectrumInfo(spectrum)
    self["richPeaks"].sort( lambda x,y: cmp(x.mz, y.mz)  )
    assert(self["richPeaks"][0][0] <= self["richPeaks"][-1][0])
    assert(spectrum["richPeaks"][0].mz <= spectrum["richPeaks"][-1].mz)

    addedPeaks = {}
    for myPeak in self["richPeaks"]:
      myMzInt = int(myPeak.mz)
      spectrumPeak = spectrum.mzToHighPeak[myMzInt]
      # forget about keeping track of offset
      myPeak.offset = 0.0
      if spectrumPeak == None: 
        myPeak.observed = False
        myPeak.i = 0.0
        continue
      else:
        myPeak.observed = True
        myPeak.i = spectrumPeak.i
        myPeak.adj = spectrumPeak.adj
        myPeak.rankI = spectrumPeak.rankI
        addedPeaks[spectrumPeak.mz] = 1

    # add in all the peaks that were not already added as an annotated ion.
    for peak in spectrum["richPeaks"]: # FIXFAST
      if addedPeaks.has_key(peak.mz): continue
      r = RichPeak(mz=peak.mz, i=peak.i)
      r.rankI = peak.rankI
      r.adj = peak.adj
      r.type = NOISE_KEY
      r.possible = r.observable = r.observed = True
      r.numberPossible = 1
      self.addPeak(r)

    self.data["hasIntensities"]  = True



  # adds Intensities to self from spectrum. Only the closest intensity within tol_mass("pephmm") is included
  def addNearestIntensitiesFromSpectrum(self, spectrum, doubleCounting=False):
    """
    This really kind of an all-purpose PrepareSpectrum for PepHMM routine
    """
    # Set standard Spectrum fields 
    self.addSpectrumInfo(spectrum)

    # rich peaks is almost definitely not sorted by mz
    self["richPeaks"].sort( lambda x,y: cmp(x.mz, y.mz)  )
    assert(self["richPeaks"][0][0] <= self["richPeaks"][-1][0])

    # spectrum should be sorted by mz. 
    # spectrum["peaks"].sort( lambda x,y: cmp(x[0], y[0])  )
    assert(spectrum["richPeaks"][0].mz <= spectrum["richPeaks"][-1].mz)

    # Determine the nearest peak to each of the RichSpectrum peaks, if any
    # peakmz:exist to keep track of which peaks have been added in
    addedPeaks = {}
    nextTooLowIdx = 0

    for my_peak in self["richPeaks"]:
      my_mz, my_i =  my_peak.mz, my_peak.i
      my_near_peaks = [] # will contain tuples of (mz_offset, intensity)
      thisTooLowIdx = nextTooLowIdx
      for peakIdx in range(thisTooLowIdx, len(spectrum["richPeaks"])):
        peak = spectrum["richPeaks"][peakIdx]
        mz, i = peak.mz, peak.i
        mz_offset = mz - my_mz

        if mz_offset < - tol_mass("pephmm"): 
          # print "mzoffset=%f" % mz_offset
          nextTooLowIdx = peakIdx
          continue

        # we've moved beyond any possibility of matching peak 'peak'
        if mz_offset > tol_mass("pephmm"): 
          # print "mzoffset=%f" % mz_offset
          break

        if (abs(mz_offset) <= tol_mass("pephmm")):
          my_near_peaks.append( (mz_offset, peak) ) # strange, but works

      # if no double counting, filter out the peaks already been assigned 
      if doubleCounting==False:
        my_near_peaks = filter(lambda x: not addedPeaks.has_key(x[1].mz), \
          my_near_peaks)

      if len(my_near_peaks) == 0: 
        my_peak.observed = False
        my_peak.offset = 0.0
        my_peak.i = 0.0
        continue

      # sort the nearby peaks in ascending by the absvalue of the mz_offset
      my_near_peaks.sort(lambda x,y: cmp(abs(x[0]), abs(y[0])))
      # set the rich peak intensity to that of the nearest peak
      tuple = my_near_peaks[0]
      mzoffset = tuple[0]
      nearest_peak = tuple[1]
      my_peak.mzoffset = mzoffset
      my_peak.i = nearest_peak.i
      my_peak.adj = nearest_peak.adj
      my_peak.rankI = nearest_peak.rankI
      # set the rich peak mzoffset to that of the nearest peak
      addedPeaks[nearest_peak.mz] = 1 # use the m/z as a unique key

    # add in all the peaks that were not already added as an annotated ion.
    for peak in spectrum["richPeaks"]: # FIXFAST
      if addedPeaks.has_key(peak.mz): continue
      r = RichPeak(mz=peak.mz, i=peak.i)
      r.rankI = peak.rankI
      r.adj = peak.adj
      r.type = NOISE_KEY
      r.possible = r.observable = r.observed = True
      r.numberPossible = 1
      self.addPeak(r)
#
    self.data["hasIntensities"]  = True

  # no fancy stuff, just add the peaks from the spectrum to the RichSpectrum
  def addPeaksDirectly(self, spectrum):
    if spectrum["parsePeaks"] == False: spectrum.parsePeaks()
    for peak in spectrum["peaks"]:
      mz, i = peak[0], peak[1]
      r = RichPeak(mz=peak[0], i=peak[1])
      r.type = '_'
      self.addPeak(r)
    
    self.data["hasIntensities"]  = True

  # adds Intensities to self from spectrum. All intensity within tol_mass is included
  def addIntensitiesFromSpectrum(self, spectrum):
    # Set standard Spectrum fields 
    self.addSpectrumInfo(spectrum)

    # make sure that richPeaks is sorted by mz. spectrum["peaks"] should be already. 
    assert(spectrum["peaks"][0][0] < spectrum["peaks"][-1][0])
    self["richPeaks"].sort( lambda x,y: cmp(x.mz, y.mz)  )

    # Long way for debugging. FIX
    # Step through each spectrum, and determine if the RichSpectrum
    # has any overlapping peaks
    addedPeaks = {}
    for peak in spectrum["peaks"]:
      mz, i = peak[0], peak[1]
      for my_peak in self["richPeaks"]:
        my_mz, my_i =  my_peak.mz, my_peak.i
        if my_peak.mz > mz + tol_mass("peak"): 
          # we've moved beyond any possibility of matching peak 'peak'
          break
        if (my_mz - tol_mass("peak") < mz ) and (mz < my_mz + tol_mass("peak")):
          addedPeaks[mz]  = 1 # use m/z as a unique key
          my_peak.i += i 

    for peak in spectrum["peaks"]:
      if addedPeaks.has_key(peak[0]): continue
      r = RichPeak(mz=peak[0], i=peak[1])
      r.type = '_'
      self.addPeak(r)
    
    self.data["hasIntensities"]  = True

  def addIntensity(self, type=None, intensity=DEFAULT_INTENSITY):
    for my_peak in self["richPeaks"]:
      if type==None or my_peak.type==type:
        my_peak.i = intensity # FIX total ion current should be updated. 

  def sequestNormalizeIntensity(self):

    # remove peaks near the precursor mz
    precursorMz = self["precursor mz"]
    for my_peak in self["richPeaks"]:
      if between(my_peak.mz, precursorMz - PRECURSOR_REMOVAL_WIDTH,  
          precursorMz + PRECURSOR_REMOVAL_WIDTH):
        my_peak.i = 0.0

    # divide the peaks into BIN_NUMBER (10) groups, and sum Normalize each group
    # so that the maximum peak is equal to BIN_PEAK_MAX (50.0)
    observedPeaks = filter(lambda x: x.observed == True and x.i > 0.0, self["richPeaks"])
    self["min peak mz"] = observedPeaks[0].mz
    self["max peak mz"] = observedPeaks[-1].mz
    # print "min = %.3f, max = %.3f" % (self["min peak mz"], self["max peak mz"])
    binWidth = (self["max peak mz"] - self["min peak mz"] + 0.0001)/BIN_NUMBER
    mzToBin = lambda x: int( (x - self["min peak mz"]) / binWidth )
    binMaxIntensity = [ 0 ] * BIN_NUMBER 
    for my_peak in observedPeaks:
      bin = mzToBin(my_peak.mz)
      if my_peak.i > binMaxIntensity[bin]:
        binMaxIntensity[bin] = my_peak.i

    binNormFactor = [ 50.0 / i for i in binMaxIntensity ] 
    for my_peak in observedPeaks:
      bin = mzToBin(my_peak.mz)
      my_peak.i *= binNormFactor[bin]

  def sumNormalizeIntensity(self, total_intensity=DEFAULT_INTENSITY, max=1000000000.0):
    # Check if spectrum has already been normalized
    if self["sum normalized"] == True : return
    sum = 0.0
    for my_peak in self["richPeaks"]:
      sum += my_peak.i 
    if sum == 0.0: return

    for my_peak in self["richPeaks"]:
      my_peak.i /= sum 
      my_peak.i *= total_intensity
      if my_peak.i > max : my_peak.i = max

    self["total energy"] = total_intensity
    self["sum normalized"] = True

  def sqrtIntensity(self): 
    # Check if spectrum has already been normalized
    if self["square rooted"] == True : return
    for my_peak in self["richPeaks"]:
      my_peak.i = math.sqrt(my_peak.i)
    self["square rooted"] = True

  def basePeakNormalizeIntensity(self): 
    # Check if spectrum has already been normalized
    if self["base peak normalized"] == True : return
    basePeak = 0.0
    for my_peak in self["richPeaks"]:
      if my_peak.i > basePeak: basePeak = my_peak.i

    total_intensity = 0.0
    for my_peak in self["richPeaks"]:
      my_peak.i /= basePeak
      total_intensity += my_peak.i

    self["total energy"] = total_intensity
    self["base peak normalized"] = True

  def addAdjustedIntensity(self):
    precursorMz = self["precursor mz"]
    peaks = self["richPeaks"]
    for idx in range(len(peaks)):
      p  = peaks[idx]
      mzBin = int(((p.mz/precursorMz)-MIN_PRECURSOR_RATIO)/BIN_SIZE)
      assert mzBin < NUMBER_MZ_BINS
      p.adj = p.i / ADJ_FACTORS[mzBin]  
      p.hasAdj = True

  def basePeakNormalizeAdjustedIntensity(self): 
    # Check if spectrum has already been normalized
    if self["base peak normalized adjusted"] == True : return
    self.addAdjustedIntensity()

    basePeak = 0.0
    for my_peak in self["richPeaks"]:
      if my_peak.adj > basePeak: basePeak = my_peak.adj

    for my_peak in self["richPeaks"]:
      my_peak.adj /= basePeak

    self["base peak normalized adjusted"] = True
    # FIX no total ion intensity

  def addBYions(self, peptide, z=2):
    bIonMass = mono_mass('H_plus')

    # TODO masses seem to be off by 0.01 compared to UCSF MS-Product
    # calc peptide mass
    peptideMass = peptide_mass(peptide, z=2)

    for idx in range(len(peptide)-1):
      aa = peptide[idx]

      # calc b ion mass
      bIonMass += mono_mass(aa)
      b = RichPeak(m=bIonMass)
      b.type = 'b'
      b.nterm = aa
      b.cterm = peptide[idx+1]
      b.frag = peptide[0:idx+1]
      b.nidx = idx + 1
      b.cidx = len(peptide) - b.nidx
      b.possible = True
      b.numberPossible = 1
  
      # calc y ion mass
      yIonMass = peptideMass - bIonMass
      y = RichPeak(m=yIonMass)
      y.type = 'y'
      y.nterm = b.nterm
      y.cterm = b.cterm
      y.frag = peptide[idx+1:len(peptide)]
      y.nidx = b.nidx
      y.cidx = b.cidx
      y.possible = True
      y.numberPossible = 1

      # add peaks
      b.partnerPeak = y 
      y.partnerPeak = b 
      self.addPeak(b)
      self.addPeak(y)

  def addIsotopes(self):
    newpeaks = []
    for peak in self["richPeaks"]:
      # only add isotopes to +1 peaks
      if peak.z != 1: continue
      isotopePeak = peak.m + Mass.mono_mass("H_plus")
      i = RichPeak(m=isotopePeak, z=1)
      i.parentPeak = peak
      i.type = 'i'
      newpeaks.append(i)
    for peak in newpeaks: self.addPeak(peak)

  def addMultiCharge(self, z=2):
    newpeaks = []
    for peak in self["richPeaks"]:
      if peak.z != 1: continue
      peakMass = peak.m + Mass.mono_mass("H_plus") * z
      c = RichPeak(m=peakMass, z=z)
      c.parentPeak = peak
      # FIX add copy auxiliary fields
      c.type = peak.type + "+" + str(z)
      c.nidx = peak.nidx
      c.cidx = peak.cidx
      c.cterm = peak.cterm
      c.nterm = peak.nterm
      c.possible = peak.possible
      c.observable = peak.observable
      c.observed = peak.observed
      c.numberPossible = peak.numberPossible
      newpeaks.append(c)
      if z==2: peak.childPeaks[PLUS2_CHILDPEAK_KEY] = c
      elif z==3: peak.childPeaks[PLUS3_CHILDPEAK_KEY] = c
      else: raise SystemExit, "Strange charge %i" % z 
    for peak in newpeaks: self.addPeak(peak)


  def addAions(self):
    newpeaks = []
    for peak in self["richPeaks"]:
      if not peak.isBpeak(): continue
      aIonMass = peak.m - Mass.mono_mass("CO")
      a = RichPeak(m=aIonMass, z=peak.z)
      a.parentPeak = peak
      a.nidx = peak.nidx
      a.cidx = peak.cidx
      a.cterm = peak.cterm
      a.nterm = peak.nterm
      a.type = 'a'

      a.possible = True
      a.numberPossible = 1

      if not observable(a.mz): a.observable = False

      newpeaks.append(a)
      peak.childPeaks[A_CHILDPEAK_KEY] = a

    for peak in newpeaks: self.addPeak(peak)

  def addNH3loss(self):
    newpeaks = []
    for peak in self["richPeaks"]:

      if not (peak.type == 'b' or peak.type == 'y'): continue

      ionMass = peak.m - Mass.mono_mass("NH3")
      nh3 = RichPeak(m=ionMass, z=peak.z)
      nh3.parentPeak = peak
      nh3.nidx = peak.nidx
      nh3.cidx = peak.cidx
      nh3.cterm = peak.cterm
      nh3.nterm = peak.nterm

      if peak.type == 'b': nh3.type = 'b-nh3'
      elif peak.type == 'y': nh3.type = 'y-nh3'
      
      if peak.canLoseNH3() == 0: 
        nh3.possible = False
      else:
        nh3.possible = True
        nh3.numberPossible = peak.canLoseNH3()
      if not observable(nh3.mz): nh3.observable = False

      newpeaks.append(nh3)
      peak.childPeaks[NH3_CHILDPEAK_KEY] = nh3

    for peak in newpeaks: self.addPeak(peak)

  def addH2Oloss(self):
    newpeaks = []
    for peak in self["richPeaks"]:
      if not (peak.type == 'b' or peak.type == 'y'): continue
      ionMass = peak.m - Mass.mono_mass("H2O")
      h2o = RichPeak(m=ionMass, z=peak.z)
      h2o.parentPeak = peak
      h2o.nidx = peak.nidx
      h2o.cidx = peak.cidx
      h2o.cterm = peak.cterm
      h2o.nterm = peak.nterm

      if peak.type == 'b': h2o.type = 'b-h2o'
      elif peak.type == 'y': h2o.type = 'y-h2o'

      if peak.canLoseH2O() == 0: 
        h2o.possible = False
      else:
        h2o.possible = True
        h2o.numberPossible = peak.canLoseH2O()
      if not observable(h2o.mz): h2o.observable = False

      newpeaks.append(h2o)
      peak.childPeaks[H2O_CHILDPEAK_KEY] = h2o

    for peak in newpeaks: self.addPeak(peak)

  #--------------
  # TODO one day make sure we use avg and mono masses consistently
  #--------------
  def addPeptideIons(self, peptide, z=2, neutrals=False, multiCharge=False,\
      isotopes=False):
    self["charge"] = int(z)
    check_peptide(peptide)
    self.peptide = peptide
    self["assumed m"] = float(Mass.peptide_mass(peptide, z=1))
    self["precursor mz"] = (float(self["assumed m"]) + \
        (self["charge"]-1) * Mass.avg_masses['H_plus'])  / int(self["charge"])
    self.addBYions(peptide)  
    if neutrals==True:
      self.data["hasNeutrals"]  = True
      self.addAions()
      self.addH2Oloss()
      self.addNH3loss()

    if multiCharge==True:
      self.data["hasMultiCharge"]  = True
      if int(z)==1: pass
      elif int(z)==2: 
        self.addMultiCharge(2)      
      elif int(z)==3: 
        self.addMultiCharge(2)      

    if isotopes==True: 
      self.data["hasIsotopes"]  = True
      self.addIsotopes()

    self.data["hasAnnotation"]  = True

#--------------------------------- 
# Checks for non-standard AAs in peptide
#--------------------------------- 
def check_peptide(peptide):
  for aa_idx in range(len(peptide)):
    aa = peptide[aa_idx]  
    try:
      m = mono_mass(aa)
    except:
      raise SystemExit, "Non-standard aa %s in peptide %s" % (aa, peptide)




class OldMs2Spectrum(Spectrum):
  "Contains the information in an old style MS-MS spectrum."

  def __init__(self, ms2_spectrum = None):
    Spectrum.__init__(self)

    # Set standard Spectrum fields 
    self.data["first scan"] = ms2_spectrum["first scan"]
    self.data["last scan"] = ms2_spectrum["last scan"]
    self.data["charge"] = ms2_spectrum["charge"]

    # OldMs2Spectrum specific fields
    self.data["parsePeaks"] = False
    self.data["peakLines"] = []

    for i in range(len(ms2_spectrum["peakLines"])):
      dot_count = 0
      seq = ms2_spectrum["peakLines"][i]
      last_dot_idx = -1 
      for j in range(len(seq)):
        if seq[j] == '.':
          dot_count = dot_count + 1
        if dot_count == 2:
          last_dot_idx = j
          break
      self.data["peakLines"].append(\
        ms2_spectrum["peakLines"][i][0:last_dot_idx] + "\n")
  
  def __str__(self):
    string = ":" + str(self["first scan"]) + "." + str(self["last scan"]) \
    + "." + str(self["charge"]) + "\n"
    return (string + "".join(self["peakLines"]))

class Ms2Spectrum(Spectrum):
  "Contains information in a single MS-MS spectrum."

  def __init__(self, firstScan=-1, charge=-1, precursor_mz=-1, filesource="None"):
    Spectrum.__init__(self)

    # Set standard Spectrum fields
    self.data["data source"] = "Python"
    self.data["first scan"]= firstScan # TODO make consistent
    self.data["assumed z"] = self.data["charge"] = int(charge)
    self.data["precursor mz"] = precursor_mz

    # Create Ms2Spectrum specific fields
    self.data["comment"] = "No comment"
    self.data["peakLines"] = []
    self.data["parsePeaks"] = False
    self.data["comment lines"] = []
    self.data["D line"] = ""
    self.data["suppress comments"] = False
    self.data["file source"] = "None"

  def addRichSpectrumInfo(self, rich, firstscan=0):
    rich["richPeaks"].sort(lambda x,y: cmp(x.mz, y.mz))
    for peak in rich["richPeaks"]:
      self.addPeak(peak.mz, peak.i) # this is a tuple
    self["first scan"] = firstscan
    self["last scan"] = firstscan + 2
    self["assumed z"] = int(rich["charge"])
    self["assumed m"] = float(rich["assumed m"])
    self["precursor mz"] = (float(self["assumed m"]) + \
        (self["charge"]-1) * Mass.avg_masses['H_plus'])  / int(self["charge"])

    self.data["parsePeaks"] = True

  def __getitem__(self, key):
    return self.data[key]

  def __setitem__(self, key, item):
    if key == "charge": 
      self.data["assumed z"] = int(item)
    if key == "assumed z":
      self.data["charge"] = int(item)
    self.data[key] = item    
  
  def __str__(self):
    first = "\t".join( map(str, [ 
      "S", \
      int(self["first scan"]),\
      int(self["last scan"]), \
      self["precursor mz"] \
      ]))
      
    second = "\t".join( map(str, [
      "Z", \
      self["assumed z"], \
      (self["precursor mz"] - Mass.avg_mass("H_plus")) * self["assumed z"] + \
      Mass.avg_mass("H_plus")  \
      ]))
    
    third = ""
    if self["suppress comments"] == False:
      third = "".join( self["comment lines"] )
      if self.data.has_key("D line"):
        third += self["D line"]
  
    if self["parsePeaks"] == False:
      return "\n".join([first, second, third +  \
        "".join(self.data["peakLines"])])
    else:
      peaks = "\n".join( map(lambda x: \
      " ".join([str(x[0]),str(x[1])]), self["peaks"]) )
      return "\n".join([first, second, third + peaks]) + "\n"
  
  #---------------------------------------
  # Converts a DtaSpectrum object to an Ms2Spectrum object.
  #---------------------------------------
  def fromDtaSpectrum(self, dta):
    
    copyKeys = [ "first scan", "last scan", "peaks", "num peaks", \
    "assumed m", "assumed z", "charge", "min peak mz", "max peak mz", \
    "total energy", "data source", "precursor mz", "file source", \
    "retention time", "sum normalized", "base peak normalized", \
    "base peak normalized adjusted", "square rooted",  \
    "hasRankRelativizedIntensities", "group id" ]

    #--- copy over the keys
    for key in copyKeys: self[key] = dta[key]

    self["suppress comments"] == True
    self["comment lines"] = ""
  
    self["parsePeaks"] = True
    self["peakLines"] = []

  def getPeaks(self):
    if self["parsePeaks"] == False: 
      self.parsePeaks()
    return self["peaks"]

  def parsePeaks(self):
    if self["parsePeaks"] == True: return
    for line in self["peakLines"]:
      fields  = line.rstrip('\n').split()
      if len(fields) != 2: continue
      mz, i = fields[0], fields[1]
      self.addPeak(mz, i)
    self["parsePeaks"] = True

  def mergeString(self, spectrum):
    if self["first scan"] != spectrum["first scan"]:
      print >>sys.stderr, "Attempt to merge different spectra!\n"
      sys.exit(-1)

    if self["assumed z"] < spectrum["assumed z"]:
      two = self
      three = spectrum
    else:
      two = spectrum
      three = self

    first = "\t".join( map(str, [ 
      "S", \
      self["first scan"],\
      self["last scan"], \
      self["precursor mz"] \
      ]))
      
    second = "\t".join( map(str, [
      "Z", \
      two["assumed z"], \
      two["assumed m"] \
      # (two["precursor mz"] - 1.0 ) * two["assumed z"] + 1  \
      ]))

    if self["suppress comments"] == False:
      if two.data.has_key("D line"):
        if two["D line"] != "":
          second += "\n" + two["D line"]

    third = "\t".join( map(str, [
      "Z", \
      three["assumed z"], \
      three["assumed m"]  \
    ]))

    if self["suppress comments"] == False:
      if three.data.has_key("D line"):
        if three["D line"] != "":
          third += "\n" + three["D line"]
      fourth = "".join( self["comment lines"] )

    if self["parsePeaks"] == False:
      return "\n".join([first, second, third, fourth + \
        "".join(self.data["peakLines"])])
    else:
      peaks = "\n".join( map(lambda x: \
      " ".join([str(x[0]),str(x[1])]), self["peaks"]) )
      return "\n".join([first, second, third, fourth + peaks])

  def isSameAs(self, spectrum): # TODO find real equals routine
    return (self.data["first scan"] == spectrum.data["first scan"]) and  \
      (int(self.data["charge"]) == int(spectrum.data["charge"]))  
      # TODO do we need precursor mass here as well.

#-------
# Main
#-------
if __name__ == '__main__':

  if len(sys.argv) < 2:
    raise SystemExit, "Usage: %s <psm-file> <num-objs>" % (sys.argv[0])

  import pickle
  import copy
  import time
  r = RichSpectrum()
  try: import MatchCollection
  except: pass
  psm = pickle.load(open(sys.argv[1]))

  for match in psm["matches"]:
    match.spectrum.parsePeaks()
    match.spectrum.numerifyPeaks()

  # START HERE
  psmArray = []
  numobjs = int(sys.argv[2])
  for idx in range(numobjs):
    psmArray.append(copy.deepcopy(psm))

  # print >>sys.stderr, "Going to sleep"
  # while(True): pass
  # time.sleep(300)

  
  pickle.dump(psm, open(sys.argv[1] + ".new", "w"))


#  psm1 = psm["matches"][0]
#  r.addPeptideIons(psm1.peptide, neutrals=True)
#  r2 = RichSpectrum()
#  psm1.spectrum.parsePeaks()
#  r2.addPeaksDirectly(psm1.spectrum)
#  r.addNearestIntensitiesFromSpectrum(r2)
#  r.basePeakNormalizeIntensity() 
#  print r.type2peaks
#  # print "Missing peaks ", len(filter(lambda x: x.observed == False, r["richPeaks"]))
  # print "Zero peaks ", len(filter(lambda x: x.i == 0.0, r["richPeaks"]))
  # print "Non-Zero peaks ", len(filter(lambda x: x.i != 0.0, r["richPeaks"]))
  # print "Noise peaks ", len(filter(lambda x: x.type == '_', r["richPeaks"]))
  # print "Non-noise peaks ", len(filter(lambda x: x.type != '_', r["richPeaks"]))
  # print "Annotated peaks ", len(filter(lambda x: x.type != '_' and\
  #   x.i != 0.0 , r["richPeaks"]))
  # print "Total peaks ", len(r["richPeaks"])
  





