#$Revision: 1.5 $
import getopt, sys, copy
from MatchCollection import *
from MatchGenerator import *
from HitCollection import *
from HitGenerator import *
from Filter import *
from composite import *
import os
import preproc
# from IPython.Shell import IPShellEmbed 


class TestPlatt:

	# Constructor takes optional DtaSelectParams object as input
	def __init__(self, params=None):
		self.data = {}
		self.trainData = None


	def __getitem__(self, key):
		return self.data[key]

	def __setitem__(self, key, item):
		self.data[key] = item		


	def calcResultSet(self, kernel = 'polynomial', cVal = 1, width=1, deg=2):
		if kernel in ('Polynomial','polynomial','poly'):
			print >>sys.stderr, "Using polynomial kernel of degree %.3f." % deg
			self.trainData.attachKernel('poly', degree = deg) 
		elif kernel in ('Gaussian','gaussian','rbf'):
			print >>sys.stderr, "Using Gaussian kernel of width %.3f." % width
			self.trainData.attachKernel('gaussian', gamma = width)

		print >>sys.stderr, "Training svm with C = %.3f." % cVal
		s=svm.SVM(C=cVal)

		p = Platt(s) 
		
		r = p.stratifiedCV(self.trainData, seed = 5)
		r.computeStats()

		# rsvm = s.stratifiedCV(self.trainData, seed = 5)
		# rsvm.computeStats()

		return r
			

#-------
# Main
#-------
if __name__ == '__main__':
	if len(sys.argv) < 2: 
		print """Usage: python %s 
	-r <cvector-train-file> 
	-t <cvector-test-file>
	[-k gaussian|polynomial]
	[-w gaussian width]
	[-d polynomial degree]
""" % (sys.argv[0])
		sys.exit(-1)

	optlist,args = getopt.getopt(sys.argv[1:],'r:t:k:w:d:c:')

	train = "CVEC-TRAIN-ALL"
	test = ""
	kernel = 'gaussian'
	width  = 0.010
	cVal = 100
	deg = 2
	
	for option_name,option_value in optlist:

		if option_name == '-s':
			sqt=option_value
		if option_name == '-m':
			ms2=option_value
		if option_name == '-r':
			train=option_value
		if option_name == '-t':
			test=option_value
		if option_name == '-k':
			kernel=option_value
		if option_name == '-w':
			width=float(option_value)
		if option_name == '-d':
			deg=int(option_value)
		if option_name == '-c':
			cVal=float(option_value)

	c = TestPlatt()
	
	print >>sys.stderr, "Parsing training file %s." % train
	DataSet = datafunc.SparseCDataSet
	c.trainData = DataSet(train, labelsColumn = 1) 
	rs = preproc.Rescale()
	rs.train(c.trainData)

	results = c.calcResultSet(cVal=cVal,deg=deg,width=width,kernel=kernel) 
	
	# sorting givenL by the decisionFunc values in a roundabout way
	# probably not necessary if givenL is 1:1 with decisionFunc
	ints = range(len(results.givenL))
	ints.sort( lambda x,y : \
			cmp( results.decisionFunc[x], results.decisionFunc[y] ) )

	results.decisionFunc.sort()
	sorted_L = [ results.givenL[i] for i in ints ]
	
	number = 10 
	
	# Aaron attempts to allocate arrays of size number
	partition_number 	= [ 0 for i in range(number) ]
	positive_sum 			= [ 0 for i in range(number) ]
	prob_sum 					= [ 0 for i in range(number) ]

	# validation for Platt. 
	for i in range(len(results.decisionFunc)):
		idx = int(i * number / float(len(results.decisionFunc) + 1) ) 
		partition_number[idx] =  partition_number[idx] + 1
		prob_sum[idx] = prob_sum[idx] + results.decisionFunc[i]
		# print results.decisionFunc[i]
		# print sorted_L[i]
		if int(sorted_L[i]) > 0:
			positive_sum[idx] =  positive_sum[idx] + 1

	# Empirical probability vs. observed probability.
	file = open("output", "w")
	for i in range(len(positive_sum)):
		if partition_number[i] == 0 : break 
		file.write("".join([`positive_sum[i] / float(partition_number[i])`, "\t", \
			`prob_sum[i] / float(partition_number[i])`, "\n"]))
	

	# FIX why are we getting negative values
	file.close()

