#$Revision: 1.1 $

#Author: Barbara Frewen
#Project: quality-czar
#Date: 1/31/06
#Function: take a set of labeled feauter vectors.
#          train and test an svm on them and 
#          return the ROC score
import getopt, sys, copy
from MatchCollection import *
from MatchGenerator import *
from HitCollection import *
from HitGenerator import *
from Filter import *
import os
import preproc
import assess

class TestSvm:

	# Constructor takes optional DtaSelectParams object as input
	def __init__(self, params=None):
		self.data = {}
		self.trainData = None

	def __getitem__(self, key):
		return self.data[key]

	def __setitem__(self, key, item):
		self.data[key] = item		


	def printRoc(self, file, kernel = 'polynomial', cVal = 1, width=1, deg=2):
		if kernel in ('Polynomial','polynomial','poly'):
			print >>sys.stderr, "Using polynomial kernel of degree %.3f." % deg
			self.trainData.attachKernel('poly', degree = deg) 
		elif kernel in ('Gaussian','gaussian','rbf'):
			print >>sys.stderr, "Using Gaussian kernel of width %.3f." % width
			self.trainData.attachKernel('gaussian', gamma = width)
			
		print >>sys.stderr, "Training svm with C = %.3f." % cVal
		s=svm.SVM(C=cVal)
		
		r = s.loo(self.trainData)
		r.computeStats()
	
		tp, fp, area = assess.roc(r.Y, r.givenY, r.decisionFunc)
		
#		print "Area = ", area
		print >> sys.stderr, "Area = %s" % area
		for i in range(len(tp)):	
			file.write("".join([str(fp[i]), "\t", str(tp[i]), "\n"]))
			

#-------
# Main
#-------
if __name__ == '__main__':
	if len(sys.argv) < 3: 
		print """Usage: python %s 
	-r <training data> 
	[-k gaussian|polynomial] default polynomial
	[-w gaussian width]      default 1
	[-d polynomial degree]   default 1
	[-c error value]         default 1
	[-o output file name]    default file.out
""" % (sys.argv[0])
		sys.exit(-1)

#TODO: add an option for the type of cross validation
	optlist,args = getopt.getopt(sys.argv[1:],'r:k:w:d:c:o:')

	train = test = ""
	kernel = 'polynomial'
	width  = cVal = 1
	deg = 1
	outfile_name = "file.out"
	
	for option_name,option_value in optlist:
		if option_name == '-r':
			train=option_value
		if option_name == '-k':
			kernel=option_value
		if option_name == '-w':
			width=float(option_value)
		if option_name == '-d':
			deg=int(option_value)
		if option_name == '-c':
			cVal=float(option_value)
		if option_name == '-o':
			outfile_name=option_value

	if kernel in ('Polynomial','polynomial','poly'):
		kernel = 'poly'
	elif kernel in ('Gaussian','gaussian','rbf'):
		kernel = 'gaussian'
	else:
		print >> sys.stderr, "Unknown kernel %s" % kernel
		sys.exit(-1)

#	c = TestSvm()
	
	print >>sys.stderr, "Parsing training file %s." % train
	DataSet = datafunc.SparseCDataSet
#	c.trainData = DataSet(train, labelsColumn = 1) 
	trainingData = DataSet(train, labelsColumn = 1) 
	rs = preproc.Rescale()
#	rs.train(c.trainData)
	rs.train(trainingData)
	
	print >> sys.stderr, "Using %s kernel of degree %i" % (kernel, deg)
	trainingData.attachKernel(kernel, degree = deg)

	print >> sys.stderr, "Training SVM with C = %.3f" % cVal
	trainingSVM = svm.SVM(C=cVal)

	print >> sys.stderr, "Testing with stratified cross validation"
	trainingResults = trainingSVM.stratifiedCV(trainingData)
	trainingResults.computeStats()
	tp, fp, area = assess.roc(trainingResults.Y, trainingResults.givenY, trainingResults.decisionFunc)

	print >> sys.stderr, "Area = %s" % area
	
        print >> sys.stderr, "Printing results (scores and labels) to %s" % outfile_name
	file = open(outfile_name, "w")
	for i in range(len(tp)):
		file.write("".join(["1\t", str(tp[i]), "\n"]))
	for i in range(len(fp)):
		file.write("".join(["-1\t", str(fp[i]), "\n"]))
	#c.printRoc(file, cVal=cVal,deg=deg,width=width,kernel=kernel) 
	file.close()
