# $Revision: 1.5 $
import getopt, sys, copy
from MatchCollection import *
from MatchGenerator import *
from HitCollection import *
from HitGenerator import *
from Filter import *
import os
import preproc

class TestSvm:

  # Constructor takes optional DtaSelectParams object as input
  def __init__(self, params=None):
    self.data = {}
    self.trainData = None
    self.testData = None
    self.svm = None

  def __getitem__(self, key):
    return self.data[key]

  def __setitem__(self, key, item):
    self.data[key] = item    


  def calcRoc(self, kernel = 'polynomial', cVal = 1, width=1, deg=2):
    if kernel in ('Polynomial','polynomial','poly'):
      print >>sys.stderr, "Using polynomial kernel of degree %.3f." % deg
      self.trainData.attachKernel('poly', degree = deg) 
      self.testData.attachKernel('poly', degree = deg) 
    elif kernel in ('Gaussian','gaussian','rbf'):
      print >>sys.stderr, "Using Gaussian kernel of width %.3f." % width
      self.trainData.attachKernel('gaussian', gamma = width)
      self.testData.attachKernel('gaussian', gamma = width)
      
    print >>sys.stderr, "Training svm with C = %.3f." % cVal
    s=svm.SVM(C=cVal)
    s.train(self.trainData)
    r = s.test(self.testData)
    # r = s.stratifiedCV(self.trainData)
    r.computeStats()
    c.svm = s
    return r
      

#-------
# Main
#-------
if __name__ == '__main__':
  if len(sys.argv) < 3: 
    print """Usage: python %s 
  -r <cvector-train-file> 
  -t <cvector-test-file>
  [-k gaussian|polynomial]
  [-w gaussian width]
  [-d polynomial degree]
""" % (sys.argv[0])
    sys.exit(-1)

  optlist,args = getopt.getopt(sys.argv[1:],'r:t:k:w:d:c:')

  kernel = 'gaussian'
  width  = cVal = 1
  deg = 2
  
  for option_name,option_value in optlist:
    if option_name == '-r':
      train=option_value
    if option_name == '-t':
      test=option_value
    if option_name == '-k':
      kernel=option_value
    if option_name == '-w':
      width=float(option_value)
    if option_name == '-d':
      deg=int(option_value)
    if option_name == '-c':
      cVal=float(option_value)

  c = TestSvm()
  
  print >>sys.stderr, "Parsing training file %s." % train
  DataSet = datafunc.SparseCDataSet
  c.trainData = DataSet(train, labelsColumn = 1) 
  rs = preproc.Rescale()
  rs.train(c.trainData)

  print >>sys.stderr, "Parsing testing file %s." % test
  c.testData = DataSet(test, labelsColumn = 1)
  rs.test(c.testData)
  results = c.calcRoc(cVal=cVal,deg=deg,width=width,kernel=kernel) 
  outputFh = open(test + ".discriminant", "w")
  for idx in range(len(c.testData)):
    label = c.testData.labels.patternID[idx]
    print >>outputFh, "%s\t%s\n" \
      % (label, float(c.svm.decisionFunc(c.testData, idx))),
  outputFh.close()
  print results.getROC()
