#---------------------------------------------------------------------------
# File: bestLor.py
#---------------------------------------------------------------------------
# Author: $Author: aklammer $
#---------------------------------------------------------------------------
# Revision: $Revision: 1.7 $
#---------------------------------------------------------------------------
# Inputs:
# 
#  - two directories, one for the positive model's evaluation of 
#    candidate peptides, one for negative model. Each directory contains:
#  
#  - the outputs of hmmTestSpectrum.py, file <prefix>.score, 
#    containing the Z scores and scores for each peptide-spectrum match:
# 
#     spectrum id \t peptide \t score \t zscore \n
#---------------------------------------------------------------------------
# Output:
#  - the best Lor for
# 
#---------------------------------------------------------------------------
import os, os.path, sys, random, cPickle

USE_PROB = True

def outputLor(prefix , neg, pos):
  fhOut = open(prefix + ".bestlor", 'w')
  for suid in pos.keys():
    peplor = []
    if not neg.has_key(suid): 
      # print >>sys.stderr, "What crap are you feeding me? " + suid
      continue
    for pep in pos[suid].keys():
      if not neg[suid].has_key(pep): 
        # print >>sys.stderr, "What crap are you feeding me? " + suid + " " + pep
        continue
      peplor.append( (pep, pos[suid][pep] - neg[suid][pep]) )
    peplor.sort(lambda x,y: cmp(y[1], x[1]))
    fhOut.write(suid + "\t" + peplor[0][0] + "\t" + `peplor[0][1]` + "\n")
  fhOut.close()

def parseFiles(negativeFiles, positiveFiles):
  suid2pos = {}
  suid2posProb = {}
  suid2negProb = {}
  for filename in positiveFiles:
    fh = open(filename)
    pep2score = {}
    pep2prob = {}
    suid = ""
    for line in fh:
      line = line.rstrip('\n')
      suid, peptide, prob, zscore = line.split()
      pep2score[peptide] = float(zscore)
      pep2prob[peptide] = float(prob)

    suid2posProb[suid] = pep2prob
    suid2pos[suid] = pep2score
    fh.close()

  suid2neg = {}
  for filename in negativeFiles:
    fh = open(filename)
    pep2score = {}
    pep2prob = {}
    suid = ""
    for line in fh:
      line = line.rstrip('\n')
      suid, peptide, prob, zscore = line.split()
      zscore = float(zscore)
      if (zscore < -100): 
        zscore = 100 # FIX!
      pep2score[peptide] = float(zscore)
      pep2prob[peptide] = float(prob)

    suid2negProb[suid] = pep2prob
    suid2neg[suid] = pep2score
    fh.close()

  neg = pos = None
  if USE_PROB == True: 
    neg = suid2negProb
    pos = suid2posProb
  else:
    neg = suid2neg
    pos = suid2pos

  return neg, pos

#------------------------------------------------------------------------------
if __name__ == "__main__":
  if len(sys.argv) < 3:
    print >>sys.stderr, """
Usage: %s <positive-file-list> <negative-file-list> <output-prefix>.bestlor
  """ % os.path.basename(sys.argv[0])
    sys.exit(1)

  positiveFiles = []
  fh  = open(sys.argv[1])
  for line in fh:
    line = line.rstrip('\n')
    positiveFiles.append(line)
  fh.close()

  negativeFiles = []
  fh  = open(sys.argv[2])
  for line in fh:
    line = line.rstrip('\n')
    negativeFiles.append(line)
  fh.close()

  zscorePrefix = sys.argv[-1]
  
  neg, pos = parseFiles(negativeFiles, positiveFiles)
  outputLor(zscorePrefix, neg, pos)

