#!/usr/bin/python2.4
#------------
# $Revision: 1.21 $
#------------
import getopt, sys, re, os, os.path, math
try:
  import Numeric
except:
  print >>sys.stderr, "Warning: Could not import Numeric in compileAaStats."
from Spectrum import *
from SpectrumCollection import *

SIGNED_RANK = "signed-rank"
MIN_NON_ZERO = 1

def parsePsms(filename):
  psmfile = open(filename, 'r')
  psms = [] # array of dicts
  for line in psmfile:
    line.rstrip('\n')
    fields = line.split()
    psm = {}
    psm["ALGORITHM"] = fields[0]
    psm["FILE"] = fields[1]
    psm["SCAN"] = fields[2]
    psm["PEPTIDE"] = fields[3]
    psm["CHARGE"] = fields[4]
    psm["SCORE"] = fields[5]
    psms.append(psm)
  return psms

def init_counts_dict(counts_dict):
  for each in list(Mass.CanonicalAminos):
    each = str(each)
    if not counts_dict.has_key(each): counts_dict[each] = {}
    for everyother in list(Mass.Aminos):
      everyother = str(everyother)
      if not counts_dict[each].has_key(everyother): 
        counts_dict[each][everyother] = {'GT' : 0, 'LT' : 0, 'EQ' : 0, 'DI' \
            : [], 'PR' : [], 'P' : 0, 'AVG_PROB' : 0 }

def initAverageProbs(average_dict):
  for each in list(Mass.CanonicalAminos):
    each = str(each)
    average_dict[each] = []

def intensities_equal(first_i, second_i):
  noise = 0.0
  if first_i >= second_i:
    noise = math.sqrt(first_i)
  else :
    noise = math.sqrt(second_i)
  if (first_i - noise <= second_i) and (second_i <= first_i + noise):
    return True
  else:
    return False

def count_average_psm(richspec, average_dict):
  for eachpeak in richspec["richPeaks"]:
    if (eachpeak.mz < richspec["min peak mz"]) or  \
        (eachpeak.mz > richspec["max peak mz"]):
      continue

    average_dict[eachpeak.cterm].append(eachpeak.i)

def count_psm(richspec, nterm_counts_dict, cterm_counts_dict):
  # only examine the b-ions. y-ions are accessed through partnerPeak field
  for eachpeak in richspec["richPeaks"]:
    if not eachpeak.isBpeak(): continue
    for everyotherpeak in richspec["richPeaks"]:
      if not everyotherpeak.isBpeak(): continue
      # if we are outside the detected range, continue
      if (eachpeak.mz < richspec["min peak mz"]) or  \
          (eachpeak.mz > richspec["max peak mz"]):
        continue
      if (everyotherpeak.mz < richspec["min peak mz"]) or  \
        (everyotherpeak.mz  > richspec["max peak mz"]): 
        continue

      if eachpeak.i == 0 and everyotherpeak.i == 0: continue

      eachPeakIntensity = eachpeak.i + eachpeak.partnerPeak.i
      everyOtherPeakIntensity = everyotherpeak.i + everyotherpeak.partnerPeak.i
      idiff = eachPeakIntensity - everyOtherPeakIntensity 
      iprob = eachPeakIntensity/(eachPeakIntensity + everyOtherPeakIntensity) 
      # nterm_counts_dict keeps track of cleavages nterm to specific aas
      # hence, we populate with the aas cterm to the cleavage site.
      # and the opposite for cterm_counts_dict.
      nterm_counts_dict[eachpeak.cterm][everyotherpeak.cterm]['DI'].append(idiff)
      cterm_counts_dict[eachpeak.nterm][everyotherpeak.nterm]['DI'].append(idiff)
      nterm_counts_dict[eachpeak.cterm][everyotherpeak.cterm]['PR'].append(iprob)
      cterm_counts_dict[eachpeak.nterm][everyotherpeak.nterm]['PR'].append(iprob)
      if intensities_equal(eachpeak.i, everyotherpeak.i):
        nterm_counts_dict[eachpeak.cterm][everyotherpeak.cterm]['EQ']+=1
        cterm_counts_dict[eachpeak.nterm][everyotherpeak.nterm]['EQ']+=1
      elif idiff > 0:
        nterm_counts_dict[eachpeak.cterm][everyotherpeak.cterm]['GT']+=1
        cterm_counts_dict[eachpeak.nterm][everyotherpeak.nterm]['GT']+=1
      else:
        nterm_counts_dict[eachpeak.cterm][everyotherpeak.cterm]['LT']+=1
        cterm_counts_dict[eachpeak.nterm][everyotherpeak.nterm]['LT']+=1


def pval(counts_dict):
  sorted_keys = counts_dict.keys()
  sorted_keys.sort()
  bonferroni = float(len(sorted_keys) * len(sorted_keys) / 2)
  for key1 in sorted_keys:
    for key2 in sorted_keys:
      if key1 == key2 : continue
      counts = counts_dict[key1][key2]

      non_zero_len = len(filter(lambda x: x != 0, counts['DI']))

      if non_zero_len < MIN_NON_ZERO:
        print >>sys.stderr, "Low counts in %s %s field = %i" % \
            (key1, key2, non_zero_len)
        counts['P'] = 0 # TODO 'n/a'
        continue

      p = os.popen2(SIGNED_RANK)
      for diff in counts['DI']:
        print >>p[0], diff
      p[0].close()
      output = p[1].read().split()
      output[4].rstrip('\n')
      counts['P'] = bonferroni * float(output[4].replace('p-value=',''))


SMALL = 0.00001
def prob(counts_dict):
  sorted_keys = counts_dict.keys()
  sorted_keys.sort()
  for key1 in sorted_keys:
    for key2 in sorted_keys:
      counts = counts_dict[key1][key2]
      prob_len = len(counts['PR'])
      if prob_len < MIN_NON_ZERO:
        print >>sys.stderr, "Low counts in %s %s field = %i" % \
            (key1, key2, prob_len)
        counts['AVG_PROB'] =  0.5 # TODO 'n/a'
        continue
      average = Numeric.average(counts['PR'])
      # TODO ensures non-zero probabilities
      if average < SMALL: average = SMALL
      elif average > 1 - SMALL: average = 1 - SMALL
      counts['AVG_PROB'] = average

def print_counts(counts_dict, label="", mode="counts", labels=True):
  sorted_keys = counts_dict.keys()
  sorted_keys.sort()

  output = open(label, 'w')

  if labels == True:
    output.write(label + '\t')
    output.write('\t'.join(sorted_keys) + "\n")
  for key1 in sorted_keys:
    if labels == True:
      output.write(key1)
    for key2 in sorted_keys:
      counts = counts_dict[key1][key2]
      if mode == "counts":
        output.write('\t' + '/'.join(
          map(str,[counts['GT'], counts['EQ'], counts['LT']])))
      elif mode == "countbias":
        output.write('\t' + str(counts['GT'] - counts['LT']))
      elif mode == "fracbias":
        sum = counts['GT'] + counts['LT'] + counts['EQ']
        if sum == 0:
          output.write('\t0')
        else:
          output.write('\t' + str(float(counts['GT'] - counts['LT'])/\
            (counts['GT'] + counts['LT'] + counts['EQ'])))
      elif mode == "pval":
        # From Table B.2 in Zar (1999)
        output.write( '\t' + str(counts['P']))
      elif mode == "prob":
        output.write( '\t' + str(counts['AVG_PROB']))
      else:
        output.write('\t' + '/'.join(
          map(str,[counts['GT'], counts['EQ'], counts['LT']])))
    output.write('\n')
  output.close()

# soon to be obsolete
def aa_counts(specColl, psms, mode="counts"):

  print >>sys.stderr, "Reading in ms2 file."
  ms2 =  os.path.basename(specColl["filename"]).replace('.ms2','')  

  nterm_counts_dict = {} 
  cterm_counts_dict = {} 
  average_dict = {} 
  init_counts_dict(nterm_counts_dict)
  init_counts_dict(cterm_counts_dict)

  print >>sys.stderr, "Analyzing %i psms." % len(psms)
  for psm in psms:
    if psm["FILE"] != ms2: continue
    richspec = RichSpectrum()
    richspec.addPeptideIons(psm["PEPTIDE"])
    print >>sys.stderr, "Possible broken method call below"
    spectrum = specColl.getSpectrum(spectrumUID(psm["SCAN"], psm["FILE"], psm["CHARGE"]))
    richspec.addIntensitiesFromSpectrum(spectrum)
    count_psm(richspec, nterm_counts_dict, cterm_counts_dict)

  if mode == "all" or mode == "pval":
    pval(nterm_counts_dict)
    pval(cterm_counts_dict)

  if mode == "all" or mode == "prob":
    prob(nterm_counts_dict)
    prob(cterm_counts_dict)
  
  return nterm_counts_dict, cterm_counts_dict

# similar to aa_counts above, but takes a psmMatchCollection instead
# returns an nterm_counts_dict and cterm_counts_dict
def calcStats(psmMatchCollection, mode="counts"):

  nterm_counts_dict = {} 
  cterm_counts_dict = {} 
  average_dict = {} 
  init_counts_dict(nterm_counts_dict)
  init_counts_dict(cterm_counts_dict)

  matches = psmMatchCollection["matches"]
  print >>sys.stderr, "Analyzing %i psms." % len(matches)
  for match in matches:
    richspec = RichSpectrum()
    richspec.addPeptideIons(match.peptide)
    richspec.addIntensitiesFromSpectrum(match.spectrum)
    count_psm(richspec, nterm_counts_dict, cterm_counts_dict)

  if mode == "all" or mode == "pval":
    pval(nterm_counts_dict)
    pval(cterm_counts_dict)

  if mode == "all" or mode == "prob":
    prob(nterm_counts_dict)
    prob(cterm_counts_dict)
  
  return nterm_counts_dict, cterm_counts_dict

# returns an ntermProbsDict and ctermProbsDict
def calcAverageProbs(psmMatchCollection):

  ntermProbs = {}
  ctermProbs = {} 
  initAverageProbs(ntermProbs)
  initAverageProbs(ctermProbs)

  matches = psmMatchCollection["matches"]
  print >>sys.stderr, "Analyzing %i psms." % len(matches)
  for match in matches:
    richspec = RichSpectrum()
    richspec.addPeptideIons(match.peptide)
    richspec.addIntensitiesFromSpectrum(match.spectrum)
    richspec.sumNormalizeIntensity()

    for eachpeak in richspec["richPeaks"]:
      if not eachpeak.isBpeak(): continue
      eachPeakIntensity = eachpeak.i + eachpeak.partnerPeak.i
      ntermProbs[eachpeak.cterm].append(eachPeakIntensity)
      ctermProbs[eachpeak.nterm].append(eachPeakIntensity)

  for key in ntermProbs.keys():
    ntermProbs[key] = Numeric.average(ntermProbs[key])
    ctermProbs[key] = Numeric.average(ctermProbs[key])

  return ntermProbs, ctermProbs


#-------
# Main
#-------
if __name__ == '__main__':

  if len(sys.argv) < 2:
    raise SystemExit, "Usage: %s <ms2-filename> <psm-filename> [mode]" % (sys.argv[0])

  specColl = Ms2SpectrumCollection(sys.argv[1], parsePeaks=True)
  specColl.allSpectra()
  psms = parsePsms(sys.argv[2])
  mode = "counts"
  if len(sys.argv) > 3: mode = sys.argv[3]


  psms_prefix = sys.argv[2].replace(".psm", "-")

  #if mode == "all" or mode == "avg":
  #  aa_avg_counts(specColl, psms)

  nterm_counts_dict, cterm_counts_dict = aa_counts(specColl, psms, mode)

  if mode != "all":
    print ""
    print_counts(nterm_counts_dict, psms_prefix + "N-term", mode=mode)
    print ""
    print_counts(cterm_counts_dict, psms_prefix + "C-term", mode=mode)
    print ""
  else :
    print ""
    print_counts(nterm_counts_dict, psms_prefix + "N-counts", mode="counts")
    print ""
    print_counts(nterm_counts_dict, psms_prefix + "N-bias", mode="fracbias")
    print ""
    print_counts(nterm_counts_dict, psms_prefix + "N-pval", mode="pval")
    print ""
    print_counts(nterm_counts_dict, psms_prefix + "N-prob", mode="prob")
    print ""

    print_counts(cterm_counts_dict, psms_prefix + "C-counts", mode="counts")
    print ""
    print_counts(cterm_counts_dict, psms_prefix + "C-bias", mode="fracbias")
    print ""
    print_counts(cterm_counts_dict, psms_prefix + "C-pval", mode="pval")
    print ""
    print_counts(cterm_counts_dict, psms_prefix + "C-prob", mode="prob")
    print ""







