#---------------------------------------------------------------------------
# File: createCandidatePsm.py
#---------------------------------------------------------------------------
# Author: $Author: aklammer $
#---------------------------------------------------------------------------
# Revision: $Revision: 1.3 $
#---------------------------------------------------------------------------
# Inputs:
# 
#  - a <collection-file> which contains the spectra 
#  - the output of generatePeptides.py, <prefix>.peps, which maps spectra 
#    to candidate peptides:
#
#     SPECTRUM_ID \t HMM_CAND_FILE \n
#     - SPECTRUM_ID is the identifier of the spectrum
#     - HMM_CAND_FILE is the file containing the candidate peptides
#---------------------------------------------------------------------------
# Outputs:
#
#  - a new pickledpsm file
# 
#     spectrum id \t peptide \t prob \n
# 
#---------------------------------------------------------------------------
import os, os.path, sys, cPickle, tempfile, random
import Mass
import HmmData
import math
import Numeric
import MatchCollection, Match

#------------------
# random peptides
# top-ranked peptides
#------------------

if __name__ == "__main__":
  if len(sys.argv) < 3:
    print >>sys.stderr, """
Usage: %s <collection-file> <peps-file> <GMTK-model-name> <GMTK-params-file>
    <collection-file> - a pickledpsm file
    <peps> - The .peps file.
    <pep-fraction> - The fraction of candidate peptides to put in matchCollection.

    output probs and zscores for each spectrum to <colleciton-prefix>.scoredir
  """ % os.path.basename(sys.argv[0])
    sys.exit(1)

  collectionFile = sys.argv[1]
  pepsFile = sys.argv[2]
  pepFraction = int(sys.argv[3])

  # parse the .peps file
  suid2cand = {}
  pepsFh = open(pepsFile)
  for line in pepsFh:
    line = line.rstrip('\n')
    suid, candFile = line.split()
    suid2cand[suid] = candFile 
  pepsFh.close()

  # parse the spectrum collection
  coll = cPickle.load(open(collectionFile))

  # create the new match collection
  newColl = MatchCollection.PsmMatchCollection()

  for match in coll["matches"]:
    suid = str(match.spectrum.uid())
    peps = []
    pepsFile = open(suid2cand[suid])
    for line in pepsFile:
      line = line.rstrip('\n')
      peps.append(line)
    pepsFile.close() 
    peps.sort(lambda x,y: cmp(random.random(), random.random()))
    pepCount = 0
    for pep in peps:
      pepCount += 1
      if pep != match.peptide and (pepCount % pepFraction) == 0: 
        newMatch = Match.Match()
        newMatch.spectrum = match.spectrum
        newMatch.peptide = pep
        newColl.appendMatch(newMatch)

  newMatchCollectionName = os.path.basename(collectionFile).split('.')[0] + ",cand.pickledpsm"
  cPickle.dump(newColl, open(newMatchCollectionName, 'w'))








