import sys, os, Model

if len(sys.argv) < 2 :
	raise SystemExit, """Usage: python %s <control-file> """ % (sys.argv[0])
elif not sys.argv[1].endswith(".ctl"):
	raise SystemExit, """Usage: python %s <control-file> """ % (sys.argv[0])

legend_dict = Model.model2legend

def legendify(x):
	if legend_dict.has_key(x): return "'" + legend_dict[x] + "'"
	else: return "'" + x + "'"

# Parse the input control file
inFile = open(sys.argv[1], 'r')
simtuples = []
allsims   = {}
for line in inFile:
	line = line.rstrip('\n')
	sims = line.split()
	if len(sims) == 2:
		sim1, sim2 = line.split()
		simtuples.append((sim1, sim2))
		allsims[sim1] = 1
		allsims[sim2] = 1
	elif len(sims) == 1: 
		allsims[sims[0]] = 1
	else:
		raise SystemExit, "Too many fields in line %s" % line
inFile.close()

# Create the matlab script file
fileNamePrefix = sys.argv[1].replace('.ctl', '')
matlabFilename = fileNamePrefix + '.m'
matlabfile = open(matlabFilename, 'w')
figures = []
for simtuple in simtuples:
	vs_figure_name = "%s-vs-%s.eps" % ( simtuple[0], simtuple[1] )
	figures.append( (vs_figure_name, fileNamePrefix + ".vs") )
	vs_cmd = "plot_vs('%s', '%s', '%s')\n" %  \
			( simtuple[0] , simtuple[1], vs_figure_name )
	matlabfile.write(vs_cmd)

	diff_figure_name = "%s-diff-%s.eps" % ( simtuple[0], simtuple[1] )
	figures.append( (diff_figure_name, fileNamePrefix + ".diff") )
	diff_cmd = "plot_diff('%s', '%s', '%s')\n" %  \
			( simtuple[0] , simtuple[1], diff_figure_name )
	matlabfile.write(diff_cmd)

everysim = allsims.keys()
everysim.sort()
sim_vector_string = ','.join( map(lambda x: "'" + x + "'", everysim))

everySimPrefix = [ i.split('.')[0] for i in everysim ] 

sim_legend_string = ','.join(map(legendify, everySimPrefix))

dist_figure_name = "%s-dist.eps" % fileNamePrefix
figures.append( (dist_figure_name, fileNamePrefix + ".dist") )
dist_cmd = "plot_dist(strvcat(%s), '%s', strvcat(%s))\n" % \
	(sim_vector_string, dist_figure_name, sim_legend_string) 
matlabfile.write(dist_cmd)
matlabfile.close()

# Run matlab
cmd = "matlab -nosplash -nojvm -r 'run %s; quit;'" % fileNamePrefix
os.system(cmd)

# Output figures and description
figsfile = open(fileNamePrefix + ".figs", 'w')
for figuretuple in figures:
	figsfile.write(figuretuple[0] + "\t" + figuretuple[1] + "\n")
figsfile.close()
