#!/usr/bin/python
#------------------
# $Revision: 1.2 $
#------------------
import sys
MASS_TOL = 3
SCAN_TOLERANCE = 8 

#-------
# Usage
#-------
if __name__ == '__main__':

  if len(sys.argv) < 2:
    raise SystemExit, "Usage: %s <ms2-file> <hardklor-reliable-file>" % (sys.argv[0])

import SpectrumCollection
import os


#-------
# HardKlor reliable peptides
#-------
class HardKlorPeptideCollection:

  def __init__(self):
    self.hits = []
    self.mzToHit = {}
    self.fieldNames = ['FILE NAME', 'FIRST SCAN', 'LAST SCAN', 'NUMBER SCANS', \
    'CHARGE', 'MONOISOTOPIC MASS', 'HIGHEST INTENSITY', 'INTENSITY SUM',\
    'FIRST RETENTION TIME', 'LAST RETENTION TIME', \
    'HIGHEST INTENSITY RETENTION TIME', 'BEST CORRELATION']

  def parseHit(self, line):
    line = line.rstrip('\n')
    fields = line.split()
    hit = {}
    for idx in range(len(fields)):
      hit[self.fieldNames[idx]] = fields[idx]
    return hit

  def addHit(self, hit):
    self.hits.append(hit)
    chargedMz = (float(hit['MONOISOTOPIC MASS']) + int(hit['CHARGE'])) / float(hit['CHARGE'])
    hit['PRECURSOR MZ'] = chargedMz
    integerMz = int(chargedMz + 0.5)

    if self.mzToHit.has_key(integerMz):
      self.mzToHit[integerMz].append(hit)
    else:
      self.mzToHit[integerMz] = [ hit ]

  def parseFromFile(self, filename):
    fh = open(filename)
    for line in fh:
      hit = self.parseHit(line)
      self.addHit(hit)
    fh.close()

  def getNearbyHits(self, precursorMz, scan):
    scan = int(scan)
    intPrec = int(float(precursorMz) + 0.5)
    hits = []
    for mass in range(intPrec - MASS_TOL, intPrec + MASS_TOL + 1):
      if not self.mzToHit.has_key(mass): continue
      for hit in self.mzToHit[mass]: hits.append(hit)
    
    return filter(lambda x: int(x['FIRST SCAN']) - SCAN_TOLERANCE <= scan \
        and scan <= int(x['LAST SCAN']) + SCAN_TOLERANCE, hits)

#-------
# Main
#-------
if __name__ == '__main__':

  #----- read in the spectrum collection
  ms2coll = SpectrumCollection.Ms2SpectrumCollection(sys.argv[1])

  #----- read in the reliable hardklor hits
  pepColl = HardKlorPeptideCollection()
  pepColl.parseFromFile(sys.argv[2])

  #----- open output files
  prefix = os.path.basename(sys.argv[1]).split('.')[0]
  nopepFh = open(prefix + "-nopep.ms2", "w")
  onepepFh = open(prefix + "-onepep.ms2", "w")
  multipepFh = open(prefix + "-multipep.ms2", "w")
  for fh in [nopepFh, onepepFh, multipepFh]:
    print >>fh, ''.join(ms2coll["headerLines"]),

  #----- for each spectrum in the spectrum collection
  for spectrum in ms2coll.allSpectra():
    precursorMz = spectrum["precursor mz"]
    scan = int(spectrum["first scan"])
    charge = int(spectrum["charge"])

    #----- determine if there are any overlapping peptides by precursor mass
    #----- and scan number 
    hits = pepColl.getNearbyHits(precursorMz, scan)

    #----- use some slop (TODO get from Mike's code)

    #----- if there are no peptides 
    if len(hits) == 0:
      #-----  output to the no-peptide file, continue
      print >>nopepFh, str(spectrum),
    #----- elif there is more than one peptide 
    elif len(hits) > 1:
      #----- output this spectrum to the possible-co-elution file
      print >>multipepFh, str(spectrum),
    #----- make sure the charge state is the right one. if not continue
    elif charge != int(hits[0]["CHARGE"]):
      continue
    #----- we've got a winner
    else:
      #----- change the precursor mass to that calculated by Hardklor
      spectrum["precursor mz"] = float(hits[0]["PRECURSOR MZ"])
      #-----  	output this spectrum to the single peptide file
      print >>onepepFh, str(spectrum),

  #----- clean up
  nopepFh.close()
  onepepFh.close()
  multipepFh.close()
