import os
import Spectrum
import Dataset
import Fit
import GaussianMixture
import cPickle
import math
# TRANSFORM = True
TRANSFORM = False
CENSOR_FRACTION = 0.00

#--- returns scanGroup2score
def getTopScores(dirName, correctForNumberPeptides=False, numberPeptideDict=None):
  topScoreLines = []
  files = os.listdir(dirName)
  # print >>sys.stderr, files
  scanGroup2score = {}
  for file in files:
    filePath = dirName + "/" + file
    if not filePath.endswith(".scores"): continue
    suid = file.split('.')[0]
    scan = Spectrum.uidSpectrumID(suid)
    group = Spectrum.uidSpectrumGroupID(suid)
    charge = Spectrum.uidCharge(suid)

    maxScore = -1000000000000000
    scoreFh = open(filePath)
    for line in scoreFh:
      peptide, score = line.split()
      score = float(score)
      if score > maxScore: 
        if correctForNumberPeptides == True:
          correctedScore = score - math.log(numberPeptideDict[suid])
          scanGroup2score[(scan, group)] = (peptide, correctedScore)
        else:
          scanGroup2score[(scan, group)] = (peptide, score)
        maxScore = score
    scoreFh.close()

  return scanGroup2score


def outputScores(fh, scores, real):
  for charge in scores.keys():
    for scanGroup in scores[charge].keys():
      scan, group = scanGroup[0], scanGroup[1]
      peptide, score = scores[charge][scanGroup][0], scores[charge][scanGroup][1]
      if real == True: type = "real"
      else: type = "rand"
      string = '\t'.join(map(str, ['riptide', group, scan, type, peptide, charge,\
          score, '0.0'])) + "\n"
      fh.write(string)


import sys
if len(sys.argv) < 3 :
  raise SystemExit, "Usage: %s <forward-dir> <random-dir> <param-prefix> [forward-peptide-dict] [random-peptide-dict]\n" % sys.argv[0]

forwardPrefix  = sys.argv[1]
randomPrefix   = sys.argv[2]
paramPrefix    = sys.argv[3]
forwardPeptideDict = randomPeptideDict = None
corrected = ""
correctForNumberPeptides = False
if len(sys.argv) > 4:
  forwardPeptideDict = cPickle.load(open(sys.argv[4]))
  randomPeptideDict = cPickle.load(open(sys.argv[5]))
  corrected = "-corrected"
  correctForNumberPeptides = True

forwardCharge2score = { }
randomCharge2score  = { }
for charge in [1,2,3]:
  forwardCharge2score[charge] = getTopScores("%s+%i,%s.scoredir" \
      % (forwardPrefix, charge, paramPrefix), \
      correctForNumberPeptides=correctForNumberPeptides, \
      numberPeptideDict=forwardPeptideDict)
  randomCharge2score[charge]  = getTopScores("%s+%i,%s.scoredir"  \
      % (randomPrefix, charge, paramPrefix), \
      correctForNumberPeptides=correctForNumberPeptides, \
      numberPeptideDict=randomPeptideDict) 

for scores in [forwardCharge2score, randomCharge2score]:
  for scanGroup in scores[2].keys():
    if scores[3].has_key(scanGroup):
      score3 = scores[3][scanGroup][1] 
      score2 = scores[2][scanGroup][1] 
      if score3 > score2: scores[2].pop(scanGroup)
      elif score3 < score2: scores[3].pop(scanGroup)
      else: print >>sys.stderr, "Ambiguous score %s, who knew?!" % scanGroup

fh = open(forwardPrefix + corrected + ".psm", "w")
outputScores(fh, forwardCharge2score, real=True)
outputScores(fh, randomCharge2score,  real=False)
fh.close()
  

