import os
import Spectrum
import Dataset
import Fit
import GaussianMixture
# TRANSFORM = True
TRANSFORM = False
CENSOR_FRACTION = 0.00

def topZscore(scores, censor=0.0):
  scores.sort()
  mean, stddev = Fit.fit_gaussian(scores)
  zscores = map(lambda x: Fit.compute_gaussian_zscore(x, mean, stddev), scores)
  return zscores[-1]

def topEvdPvalue(scores, censor=0.0):
  scores.sort()
  threshold = scores[ int(len(scores) * censor) ]
  if scores[10] == scores[50]: 
    raise Exception, "Problem with consecutive identical values!"
  minScore = scores[0]
  maxScore = scores[-1]
  range = maxScore - minScore
  if range <= 0.0000001: raise Exception, "Problem with small range!"
  scores = [ (i - minScore)/range for i in scores ] 
  guess = 1
  mu, lamda = Fit.censored_fit_evd(scores, threshold, guess, 0.001)
  return Fit.compute_evd_pvalue(scores[-1], mu, lamda)

def getTopScore(dirName, type='real', scoreMethod='top'):
  topScoreLines = []
  files = os.listdir(dirName)
  # print >>sys.stderr, files
  for file in files:
    filePath = dirName + "/" + file
    # if not filePath.endswith("svminput.fixed.positive.scores"): continue
    # if not filePath.endswith("svminput.fixed.scores"): continue
    if not filePath.endswith(".scores"): continue
    # print >>sys.stderr, "%s" % file
    suid = file.split('.')[0]
    
    scan = Spectrum.uidSpectrumID(suid)
    group = Spectrum.uidSpectrumGroupID(suid)
    charge = Spectrum.uidCharge(suid)

    maxPeptide = ""
    scoreFh = open(filePath)
    maxScore = -1000000000000000
    allscores = []
    for line in scoreFh:
      peptide, score = line.split()
      score = float(score)
      allscores.append(score)
      if score > maxScore: 
        maxScore = score
        maxPeptide = peptide
    scoreFh.close()

    try:
      if scoreMethod == 'zscore':
        score = topZscore(allscores)
      elif scoreMethod == 'evd':
        score = topEvdPvalue(allscores, censor=CENSOR_FRACTION)
      elif scoreMethod == 'top':
        # print >>sys.stderr, "In top"
        score = maxScore
      elif scoreMethod == 'gmix':
        score = GaussianMixture.transformTopScore(allscores)
    except Exception, e:
      print >>sys.stderr, "Problem calculating score for %s: %s" % (file, e)
      continue
    # TODO. maxPeptide does not reflect normalization!
    topScoreLines.append(
        '\t'.join(map(str, ['riptide', group, scan, type, maxPeptide, charge,\
          score, '0.0'])))
  return topScoreLines

def getTopScoreLines(dirName, type='real'):
  topScoreLines = []
  files = os.listdir(dirName)
  for file in files:
    filePath = dirName + "/" + file
    if not filePath.endswith(".scores"): continue
    suid = file.split('.')[0]
    
    scan = Spectrum.uidSpectrumID(suid)
    group = Spectrum.uidSpectrumGroupID(suid)
    charge = Spectrum.uidCharge(suid)

    scoreFh = open(filePath)
    maxScore = -1000000000000000
    allscores = []
    for line in scoreFh:
      peptide, score = line.split()
      score = float(score)
      allscores.append(score)
      if score > maxScore: 
        maxScore = score
        maxPeptide = peptide
    scoreFh.close()
    allscores.sort()
    medianScore = allscores[int(len(allscores)/2.0)]
    transform = maxScore - medianScore
    if TRANSFORM:
      maxScore = transform
    topScoreLines.append(
        '\t'.join(map(str, ['riptide', group, scan, type, maxPeptide, charge,\
          maxScore , '0.0'])))
  return topScoreLines

import sys
if len(sys.argv) <3:
  raise SystemExit, "Usage: %s <forward-dir> <random-dir> [gaussian?] \n" % sys.argv[0]

forwardDir = sys.argv[1]
randomDir = sys.argv[2]
if len(sys.argv) > 3: 
  useGaussian = True
else: 
  useGaussian = False

if useGaussian == True:
  fh = open(forwardDir + "-gaussian.psm", "w")
else:
  fh = open(forwardDir + ".psm", "w")

if useGaussian == True:
  forwardScores = getTopScore(forwardDir, type='real', scoreMethod='gmix')
  randomScores = getTopScore(randomDir, type='rand', scoreMethod='gmix')
else:
  forwardScores = getTopScore(forwardDir, type='real', scoreMethod='top')
  randomScores = getTopScore(randomDir, type='rand', scoreMethod='top')


#----------
# make sure we've got the same ids in both forward and random
#----------
# scan2score = {}
# for score in forwardScores:
#   scan2score[score.split('\t')[2]] = score
# 
# newRandom = []
# newForward = []
# for score in randomScores:
#   scan = score.split('\t')[2]
#   if scan2score.has_key(scan):
#     newRandom.append(score)
#     newForward.append(scan2score[scan])
# 
# forwardScores = newForward
# randomScores  = newRandom
# 
for score in map(str, forwardScores): fh.write(score + "\n")
for score in map(str, randomScores): fh.write(score + "\n")
fh.close()
  

