#---------------------------------------------------------------------------
# File: hmmAccuracy.py
#---------------------------------------------------------------------------
# Author: $Author: aklammer $
#---------------------------------------------------------------------------
# Revision: $Revision: 1.13 $
#---------------------------------------------------------------------------
# Inputs:
# 
#  - the output of hmmTest.py, file <prefix>.zscore, 
#    containing the Z scores for each peptide-spectrum match:
# 
#     spectrum id \t peptide \t zscore \n
# 
#  - <positives>.txt to compare with
# 
#---------------------------------------------------------------------------
import os, os.path, sys, random, cPickle, re 
import Mass
import HmmData
import math
import Numeric
import Spectrum

#------------------------------------------------------------------------------
def dtaIdToSuid(dtaId):
  groupId, firstScan, lastScan, charge = dtaId.split('/')[1].split('.')
  suid = ",".join([ firstScan, groupId, charge])
  return suid

#------------------------------------------------------------------------------
def annotSpectrum(match = None, outFile = None):
  match.spectrum.parsePeaks()

  r = Spectrum.RichSpectrum()
  r.addPeptideIons(match.peptide, neutrals=True)

  r2 = Spectrum.RichSpectrum()
  r2.name = match.spectrum.uid() + "-" + "raw"
  r2.addPeaksDirectly(match.spectrum)

  r.addNearestIntensitiesFromSpectrum(r2)

  r.basePeakNormalizeIntensity()
  r["richPeaks"].append(Spectrum.RichPeak(mz=2000, i=0))
  r["richPeaks"].insert(0,Spectrum.RichPeak(mz=-500, i=0))
  r.name = match.spectrum.uid() + "-" + match.peptide
  outFile.write(str(r) + "\n")

#------------------------------------------------------------------------------
if __name__ == "__main__":
  if len(sys.argv) < 3:
    print >>sys.stderr, """
Usage: %s <prefix> <positives>
    <best-file> - The <prefix>.best file.
    <positives> - A .txt filename (for OMICS data) or .psm filename (for in
    house data), which contains the list of positive PSMs
    [pickledpsm] - The pickledpsm file for extracting spectra.
  """ % os.path.basename(sys.argv[0])
    sys.exit(1)

  best = sys.argv[1]
  positivesFile = sys.argv[2]

  psm = None
  if len(sys.argv) >= 4:
    psmFile = sys.argv[3]
    psm = cPickle.load(open(psmFile))

  fh = open(best)
  suid2peptide = {}
  for line in fh:
    line = line.rstrip()
    suid, peptide, zscore = line.split()
    suid2peptide[suid] = peptide
  fh.close()

  suid2positivePeptide = {}
  if positivesFile.endswith(".txt"):
    fh = open(positivesFile)
    for line in fh:
      line = line.rstrip()
      fields = line.split()
      dtaId, peptide = fields[0], fields[3]
      peptide = peptide.split('.')[1]
      suid = dtaIdToSuid(dtaId)
      suid2positivePeptide[suid] = peptide
    fh.close()
  elif positivesFile.endswith(".psm"):
    fh = open(positivesFile)
    for line in fh:
      line = line.rstrip()
      fields = line.split()
      groupId = fields[1]
      scan = fields[2]
      peptide = fields[3]
      charge = fields[4]
      suid = Spectrum.spectrumUID(scan, groupId, charge)
      suid2positivePeptide[suid] = peptide
    fh.close()

  # print suid2positivePeptide
  # print suid2peptide
  rightCount = 0
  totalCount = 0
  problems = []
  if psm != None:
    output = str(''.join(psmFile.split('.')[0:-1])) + "-bad.annot"
    badOutFile = open(output, 'w')
    output = str(''.join(psmFile.split('.')[0:-1])) + "-good.annot"
    goodOutFile = open(output, 'w')
  seenPeptides = {}
  for suid in suid2peptide.keys():
    truePeptide = re.sub('I','L', suid2positivePeptide[suid])
    testPeptide = re.sub('I','L', suid2peptide[suid])

    #if seenPeptides.has_key(truePeptide): continue # FIX
    #seenPeptides[truePeptide] = True # FIX
    totalCount+=1

    if truePeptide == testPeptide:
      rightCount+=1
      if psm != None:
        annotSpectrum(match = psm.getMatch(suid), outFile = goodOutFile)
    else:
      if psm != None:
        match = psm.getMatch(suid)
        annotSpectrum(match = psm.getMatch(suid), outFile = badOutFile)
      print "%s\t%s\t%s" % \
          (suid, suid2positivePeptide[suid], suid2peptide[suid])

  print "Accuracy = %i/%i = %.3f" % ( rightCount, totalCount, \
      float(rightCount)/totalCount )
 
  if psm != None:
    badOutFile.close()
    goodOutFile.close()




