#---------------------------------------------------------------------------
# File: hmmAccuracy.py
#---------------------------------------------------------------------------
# Author: $Author: aklammer $
#---------------------------------------------------------------------------
# Revision: $Revision: 1.7 $
#---------------------------------------------------------------------------
# Inputs:
# 
#  - the output of hmmTest.py, file <prefix>.zscore, 
#    containing the Z scores for each peptide-spectrum match:
# 
#     spectrum id \t peptide \t zscore \n
# 
#  - <positives>.txt to compare with
# 
#---------------------------------------------------------------------------
import os, os.path, sys, random, cPickle, re 
import Mass
import HmmData
import math
try:
  import Numeric
  from pylab import *
except:
  print >>sys.stderr, "Could not import Numeric or pylab!"
import Spectrum
import bestLor

#------------------------------------------------------------------------------
def dtaIdToSuid(dtaId):
  groupId, firstScan, lastScan, charge = dtaId.split('/')[1].split('.')
  suid = ",".join([ firstScan, groupId, charge])
  return suid

#------------------------------------------------------------------------------
def annotSpectrum(match = None, outFile = None):
  match.spectrum.parsePeaks()

  r = Spectrum.RichSpectrum()
  r.addPeptideIons(match.peptide, neutrals=True)

  r2 = Spectrum.RichSpectrum()
  r2.name = match.spectrum.uid() + "-" + "raw"
  r2.addPeaksDirectly(match.spectrum)

  r.addNearestIntensitiesFromSpectrum(r2)

  r.basePeakNormalizeIntensity()
  r["richPeaks"].append(Spectrum.RichPeak(mz=2000, i=0))
  r["richPeaks"].insert(0,Spectrum.RichPeak(mz=-500, i=0))
  r.name = match.spectrum.uid() + "-" + match.peptide
  outFile.write(str(r) + "\n")

DO_ANNOT = False
#------------------------------------------------------------------------------
if __name__ == "__main__":
  if len(sys.argv) < 3:
    print >>sys.stderr, """
Usage: %s <prefix> <positives>
    <positive-dir>
    <negative-dir>
    <pickledpsm> - The pickledpsm file for extracting spectra.
  """ % os.path.basename(sys.argv[0])
    sys.exit(1)

  positiveDir = sys.argv[1]
  negativeDir = sys.argv[2]
  psmFile = sys.argv[3]

  negativeFnames = map(lambda x: os.path.abspath(negativeDir + "/" + x), filter( lambda x: x.endswith(".scores"), os.listdir(positiveDir) ) )
  positiveFnames = map(lambda x: os.path.abspath(positiveDir + "/" + x), filter( lambda x: x.endswith(".scores"), os.listdir(positiveDir) ) ) 

  neg, pos = bestLor.parseFiles(negativeFnames, positiveFnames)
  psm = cPickle.load(open(psmFile))

  lor = {}
  for suid in neg.keys():
    for peptide in neg[suid].keys():
      if not lor.has_key(suid): 
        lor[suid] = {}
      if not pos[suid].has_key(peptide): continue
      lor[suid][peptide] = pos[suid][peptide] - neg[suid][peptide]

  print "\n\n", positiveDir

  trueCount = 0
  totalCount = 0
  uniqCount = 0
  uniqTrueCount = 0
  seenPeptide = {}
  for suid in lor.keys():
    totalCount += 1
    truePeptide = psm.getMatch(suid).peptide
    peptides = lor[suid].keys()
    peptides.sort(lambda x,y: cmp(lor[suid][y],lor[suid][x]))
    topPeptide = peptides[0]

    truePeptide = re.sub('I','L', truePeptide)
    topPeptide  = re.sub('I','L', topPeptide)

    if topPeptide == truePeptide:
      trueCount += 1 
    else:
      print "%s\t%s\t%s" % (suid, truePeptide, topPeptide)

    if seenPeptide.has_key(truePeptide): continue
    seenPeptide[truePeptide] = 1 
    uniqCount += 1
    if topPeptide == truePeptide:
      uniqTrueCount += 1 
  

  print "Redundant Accuracy = %i/%i = %.3f" % ( trueCount, totalCount, \
      float(trueCount)/totalCount )

  print "Unique Accuracy = %i/%i = %.3f" % ( uniqTrueCount, uniqCount, \
      float(uniqTrueCount)/uniqCount)


  print positiveDir, "\n\n"

  sys.exit(0)

#------------------------------
  


  posLors = []
  negLors = []
  topNegLors = []
  for suid in lor.keys():
    thisSuidNegLors = []
    match = psm.getMatch(suid)
    truePeptide = match.peptide
    for peptide in lor[suid].keys():
      if peptide == truePeptide:
        posLors.append(lor[suid][peptide])
      else:
        negLors.append(lor[suid][peptide])
        thisSuidNegLors.append(lor[suid][peptide])
    thisSuidNegLors.sort()
    topNegLors.append(thisSuidNegLors[-1])

  negLors = filter(lambda x: x > -40, negLors)
  negHist = hist(negLors, bins = 20, normed = True, hold = False) 
  posHist = hist(posLors, bins = 20, normed = True, hold = False) 
  hold(False)
  plot(negHist[1], negHist[0], label="Neg")
  hold(True)
  xlabel("LOR")
  ylabel("Prob")
  plot(posHist[1], posHist[0], label="Pos")
  legend()
  savefig("all-prob.png")
  show()

  hold(False)
  negLors = filter(lambda x: x > 20, negLors)
  negHist = hist(negLors, bins = 20,  hold = False) 
  posHist = hist(posLors, bins = 20,  hold = False) 
  plot(negHist[1], negHist[0], label="Neg")
  hold(True)
  xlabel("LOR")
  ylabel("Freq")
  plot(posHist[1], posHist[0], label="Pos")
  legend()
  savefig("all-freq.png")
  show()

  hold(False)
  negHist = hist(topNegLors, bins = 20,  hold = False) 
  posHist = hist(posLors, bins = 20,  hold = False) 
  plot(negHist[1], negHist[0], label="Neg")
  hold(True)
  xlabel("LOR")
  ylabel("Freq")
  plot(posHist[1], posHist[0], label="Pos")
  legend()
  savefig("topneg-freq.png")
  show()

