#---------------------------------------------------------------------------
# File: hmmAccuracy.py
#---------------------------------------------------------------------------
# Author: $Author: aklammer $
#---------------------------------------------------------------------------
# Revision: $Revision: 1.15 $
#---------------------------------------------------------------------------
# Inputs:
# 
#  - the output of hmmTest.py, file <prefix>.zscore, 
#    containing the Z scores for each peptide-spectrum match:
# 
#     spectrum id \t peptide \t zscore \n
# 
#  - <positives>.txt to compare with
# 
#---------------------------------------------------------------------------
import os, os.path, sys, random, cPickle, re 
import Mass
import HmmData
import math
try:
  import Numeric
  from pylab import *
except:
  print >>sys.stderr, "Could not import Numeric or pylab!"
import Spectrum
import bestLor

#------------------------------------------------------------------------------
def dtaIdToSuid(dtaId):
  groupId, firstScan, lastScan, charge = dtaId.split('/')[1].split('.')
  suid = ",".join([ firstScan, groupId, charge])
  return suid

#------------------------------------------------------------------------------
def annotSpectrum(match = None, outFile = None):
  match.spectrum.parsePeaks()

  r = Spectrum.RichSpectrum()
  r.addPeptideIons(match.peptide, neutrals=True)

  r2 = Spectrum.RichSpectrum()
  r2.name = match.spectrum.uid() + "-" + "raw"
  r2.addPeaksDirectly(match.spectrum)

  r.addNearestIntensitiesFromSpectrum(r2)

  r.basePeakNormalizeIntensity()
  r["richPeaks"].append(Spectrum.RichPeak(mz=2000, i=0))
  r["richPeaks"].insert(0,Spectrum.RichPeak(mz=-500, i=0))
  r.name = match.spectrum.uid() + "-" + match.peptide
  outFile.write(str(r) + "\n")

#------------------------------------------------------------------------------
def fixFile(test, labels):
  newTest = test + ".fixed"
  outFh = open(newTest,"w")
  fh = open(test)
  for line in fh:
    label = line.split('\t')[0]
    labels.append(label)
    line = re.sub(",","-", line)
    line = re.sub(":","-", line)
    line = re.sub("\t",",",line) 
    line = re.sub(",1,",",",line) 
    outFh.write(line)
  fh.close()
  outFh.close()
  return newTest


DO_ANNOT = False
# FIELDS = range(2,29)
FIELDS = range(2)
#------------------------------------------------------------------------------
if __name__ == "__main__":
  if len(sys.argv) < 3:
    print >>sys.stderr, """
Usage: %s 
  <positive-dir>
  <psm-file>
  """ % os.path.basename(sys.argv[0])
    sys.exit(1)

  positiveDir = sys.argv[1]
  psmFile = sys.argv[2]
  
  positiveFnames = map(lambda x: os.path.abspath(positiveDir + "/" + x), filter( lambda x: x.endswith(".scores"), os.listdir(positiveDir) ) ) 

  psm = cPickle.load(open(psmFile))

  for field in FIELDS:

    topPeps = {}

    for test in positiveFnames:
      suid = ""
      pep2score = {}
      fh = open(test)
      for line in fh:
        line = line.rstrip()
        fields = line.split(',')
        suid, peptide = fields[0].split('.')
        score = float(fields[field])
        pep2score[peptide] = score
      fh.close()
      
      peps = pep2score.keys()
      peps.sort(lambda x,y: cmp(pep2score[y],pep2score[x]))
      topPeps[suid] = peps[0]

    # get the topPeptide
    trueCount = 0
    uniqCount = 0
    uniqTrueCount = 0
    seenPeptide = {}
    totalCount = 0
    for match in psm["matches"]:
      suid = match.spectrum.uid()
      truePeptide = psm.getMatch(suid).peptide
      # truePeptide = re.sub('I','L', truePeptide)

      suidLabel = re.sub(",","-",suid)
      if not topPeps.has_key(suidLabel): 
        print >>sys.stderr, "Skipping %s" % suid
        continue
      # topPeptide  = re.sub('I','L', topPeps[suidLabel])
      topPeptide  = topPeps[suidLabel]
      totalCount += 1

      if topPeptide == truePeptide:
        trueCount += 1 
      else:
        print "Field %i %s\t%s\t%s" % (field, suid, truePeptide, topPeptide)

      if seenPeptide.has_key(truePeptide): continue
      seenPeptide[truePeptide] = 1 
      uniqCount += 1
      if topPeptide == truePeptide:
        uniqTrueCount += 1 
    

    print "Field %i Redundant Accuracy = %i/%i = %.3f" % ( field, trueCount, totalCount, \
        float(trueCount)/totalCount )

    print "Field %i Unique Accuracy = %i/%i = %.3f" % ( field, uniqTrueCount, uniqCount, \
        float(uniqTrueCount)/uniqCount)
#
##------------------------------
#  
#
#
#  posLors = []
#  negLors = []
#  topNegLors = []
#  for suid in lor.keys():
#    thisSuidNegLors = []
#    match = psm.getMatch(suid)
#    truePeptide = match.peptide
#    for peptide in lor[suid].keys():
#      if peptide == truePeptide:
#        posLors.append(lor[suid][peptide])
#      else:
#        negLors.append(lor[suid][peptide])
#        thisSuidNegLors.append(lor[suid][peptide])
#    thisSuidNegLors.sort()
#    topNegLors.append(thisSuidNegLors[-1])
#
#  #cPickle.dump(negLors, open("neg", "w"))
#  #cPickle.dump(posLors, open("pos", "w"))
#
#  negLors = filter(lambda x: x > -40, negLors)
#  negHist = hist(negLors, bins = 20, normed = True, hold = False) 
#  posHist = hist(posLors, bins = 20, normed = True, hold = False) 
#  hold(False)
#  plot(negHist[1], negHist[0], label="Neg")
#  hold(True)
#  xlabel("LOR")
#  ylabel("Prob")
#  plot(posHist[1], posHist[0], label="Pos")
#  legend()
#  savefig("all-prob.png")
#  show()
#
#  hold(False)
#  negLors = filter(lambda x: x > 20, negLors)
#  negHist = hist(negLors, bins = 20,  hold = False) 
#  posHist = hist(posLors, bins = 20,  hold = False) 
#  plot(negHist[1], negHist[0], label="Neg")
#  hold(True)
#  xlabel("LOR")
#  ylabel("Freq")
#  plot(posHist[1], posHist[0], label="Pos")
#  legend()
#  savefig("all-freq.png")
#  show()
#
#  hold(False)
#  negHist = hist(topNegLors, bins = 20,  hold = False) 
#  posHist = hist(posLors, bins = 20,  hold = False) 
#  plot(negHist[1], negHist[0], label="Neg")
#  hold(True)
#  xlabel("LOR")
#  ylabel("Freq")
#  plot(posHist[1], posHist[0], label="Pos")
#  legend()
#  savefig("topneg-freq.png")
#  show()
#
