#---------------------------------------------------------------------------
# File: hmmAccuracy.py
#---------------------------------------------------------------------------
# Author: $Author: aklammer $
#---------------------------------------------------------------------------
# Revision: $Revision: 1.28 $
#---------------------------------------------------------------------------
# Inputs:
# 
#  - the output of hmmTest.py, file <prefix>.zscore, 
#    containing the Z scores for each peptide-spectrum match:
# 
#     spectrum id \t peptide \t zscore \n
# 
#  - <positives>.txt to compare with
# 
#---------------------------------------------------------------------------
import os, sys
#------------------------------------------------------------------------------
if __name__ == "__main__":
  if len(sys.argv) < 3:
    print >>sys.stderr, """
Usage: %s 
  <svm-prefix>
  <svm-input-file>
  """ % os.path.basename(sys.argv[0])
    sys.exit(1)

#------------------------------------------------------------------------------
TOP_PEPS = 1
#------------------------------------------------------------------------------
from hmmParams import *
import os.path, random, cPickle, re 
import Mass
import HmmData
import math
try: import Numeric; from pylab import *;
except: print >>sys.stderr, "Could not import Numeric or pylab!"
import Spectrum
import bestLor

#------------------------------------------------------------------------------
import preproc
import assess
import datafunc
import svm

#------------------------------------------------------------------------------
def fixFile(test, labels):
  newTest = test + ".fixed"
  outFh = open(newTest,"w")
  fh = open(test)
  for line in fh:
    label = line.split(',')[0]
    labels.append(label)
    line = re.sub(",1,",",",line) 
    line = re.sub("0000000000000000", "0", line)
    outFh.write(line)
  fh.close()
  outFh.close()
  # os.unlink(test)
  return newTest

#------------------------------------------------------------------------
def outputStats(outputFh, psm, topPeps, suid2tuple):
  trueCount = 0
  uniqCount = 0
  uniqTrueCount = 0
  totalCount = 0
  seenPeptide = {}
  for match in psm["matches"]:

    suid = match.spectrum.uid()
    suidFix = re.sub(',','-',suid)

    if not topPeps.has_key(suidFix): 
      print >>sys.stderr, "Skip %s" % suid 
      continue  # FIX!

    totalCount += 1
    truePeptide = psm.getMatch(suid).peptide
    truePeptide = re.sub('I','L', truePeptide)
    topPeptide  = re.sub('I','L', topPeps[suidFix])

    if topPeptide == truePeptide:
      trueCount += 1 
    else:
      print >>outputFh, "SVM %s\t%s\t%s" % (suid, truePeptide, topPeptide)
    if suid2tuple.has_key(suidFix):
      print >>outputFh, "DISC\t%s\t%.4f\t%s" % (suidFix, suid2tuple[suidFix][0], topPeptide) # the top peptide
      print >>outputFh, "TOPPEPS\t%s\t%.4f\t%.4f" % (suidFix, suid2tuple[suidFix][0], suid2tuple[suidFix][1])
    else:
      print >>sys.stderr, "Missing correct peptide for %s in lookup" % suidFix

    if seenPeptide.has_key(truePeptide): continue
    seenPeptide[truePeptide] = 1 
    uniqCount += 1
    if topPeptide == truePeptide:
      uniqTrueCount += 1 
  
  print >>outputFh, "SVM Redundant Accuracy = %i/%i = %.3f" % ( trueCount, totalCount, \
     float(trueCount)/totalCount )

  print >>outputFh, "SVM Unique Accuracy = %i/%i = %.3f" % ( uniqTrueCount, uniqCount, \
     float(uniqTrueCount)/uniqCount)

  return [] # TODO will be accuracyResults

#------------------------------------------------------------------------------
# disc accuracy
#------------------------------------------------------------------------------
def discAccuracy(outputFh, psm, scoreDir, peps=None):
  trueCount = 0
  uniqCount = 0
  uniqTrueCount = 0
  totalCount = 0
  seenPeptide = {}

  scoreFiles = filter(lambda x: x.endswith(".scores"), os.listdir(scoreDir))
  suid2peps = {}
  suid2toppep = {}
  for file in scoreFiles:
    fh = open(file)
    suid = file.split('.')[0]
    suid2peps[suid] = []
    for line in fh:
      pep, score = line.rstrip('\n').split()
      suid2peps[suid].append( (pep,float(score)) )
    fh.close()

    suid2peps[suid].sort(lambda x,y: cmp(y[1], x[1]))
    suid2toppep[suid] = suid2peps[suid][0]

  for match in psm["matches"]:

    suid = match.spectrum.uid()

    if not suid2toppep.has_key(suid): 
      #print >>sys.stderr, "SKIPPING %s" % suid 
      continue  

    if peps != None:
      pep = re.sub('I','L', match.peptide)
      if not peps.has_key(pep):
        print >>sys.stderr, "SKIPPING %s" % suid 
        continue  

    #---- 
    totalCount += 1
    seenTrue = False
    for idx in range(TOP_PEPS):
      topPeptide = suid2peps[suid][idx][0]
      truePeptide = psm.getMatch(suid).peptide
      truePeptide = re.sub('I','L', truePeptide)
      topPeptide  = re.sub('I','L', topPeptide)

      truePeptideDisc = 0.0
      for pep in suid2peps[suid]:
        pepseq = re.sub('I', 'L', pep[0])
        if pepseq == truePeptide: 
          truePeptideDisc = pep[1]
          break

      if topPeptide == truePeptide: seenTrue = True

      if seenTrue == False and idx == 0:
        print "%s\t%s\t%.2f\t%s\t%.2f\n" % (suid, topPeptide, \
          suid2toppep[suid][1], truePeptide, truePeptideDisc)

    if seenPeptide.has_key(truePeptide): continue
    seenPeptide[truePeptide] = 1 
    uniqCount += 1
    if seenTrue == True: uniqTrueCount += 1 
  
  print >>outputFh, "SVM Redundant Accuracy = %i/%i = %.3f" % ( trueCount, totalCount, \
     float(trueCount)/totalCount )

  print >>outputFh, "SVM Unique Accuracy = %i/%i = %.3f" % ( uniqTrueCount, uniqCount, \
     float(uniqTrueCount)/uniqCount)

#-------------------------------------------------------------------------
def parsePeps(candFile):
  peps = {}
  fh = open(candFile)
  for line in fh:
    peptide = line.rstrip('\n').split()[0]
    peptide = re.sub('I','L', peptide)
    peps[peptide] = 1
  fh.close()
  return peps

#------------------------------------------------------------------------------
# single SVM analysis
#------------------------------------------------------------------------------
def singleSvmAccuracy(svmInputFile=None, svmFilePrefix=None):
 
  svmFname = svmFilePrefix + ".svm"
  rescaleFname = svmFilePrefix + ".rescale"

  rs = cPickle.load(open(rescaleFname))
  s = svm.loadSVM(svmFname)

  labels = []
  newInputFile = fixFile(svmInputFile, labels)

  DataSet = datafunc.SparseCDataSet
  try:
    testData = DataSet(newInputFile, idColumn = 0)
  except:
    print >>sys.stderr, "Failed to parse test data set %s " \
        % (newInputFile)
    return None

  rs.test(testData)

  pep2score = {}
  for idx in range(len(testData)):
    pep = labels[idx].split('.')[1] 
    pep2score[pep]=float(s.decisionFunc(testData, idx)) 
  
  return pep2score

#------------------------------------------------------------------------------
if __name__ == '__main__':
  svmFilePrefix = sys.argv[1]
  svmInputFile = sys.argv[2]
  pep2score = singleSvmAccuracy(svmInputFile=svmInputFile, svmFilePrefix=svmFilePrefix)
  for pep in pep2score.keys():
    print pep, "\t", pep2score[pep]
