#---------------------------------------------------------------------------
# File: hmmAccuracy.py
#---------------------------------------------------------------------------
# Author: $Author: aklammer $
#---------------------------------------------------------------------------
# Revision: $Revision: 1.1 $
#---------------------------------------------------------------------------
# Inputs:
# 
#  - the output of hmmTest.py, file <prefix>.zscore, 
#    containing the Z scores for each peptide-spectrum match:
# 
#     spectrum id \t peptide \t zscore \n
# 
#  - <positives>.txt to compare with
# 
#---------------------------------------------------------------------------
import os, os.path, sys, random, pickle, re 
import Mass
import HmmData
import math
try:
  import Numeric
  from pylab import *
except:
  print >>sys.stderr, "Could not import Numeric or pylab!"
import Spectrum
import bestLor

# PyML
import preproc
import assess
import datafunc
import svm

#------------------------------------------------------------------------------
def dtaIdToSuid(dtaId):
  groupId, firstScan, lastScan, charge = dtaId.split('/')[1].split('.')
  suid = ",".join([ firstScan, groupId, charge])
  return suid

#------------------------------------------------------------------------------
def annotSpectrum(match = None, outFile = None):
  match.spectrum.parsePeaks()

  r = Spectrum.RichSpectrum()
  r.addPeptideIons(match.peptide, neutrals=True)

  r2 = Spectrum.RichSpectrum()
  r2.name = match.spectrum.uid() + "-" + "raw"
  r2.addPeaksDirectly(match.spectrum)

  r.addNearestIntensitiesFromSpectrum(r2)

  r.basePeakNormalizeIntensity()
  r["richPeaks"].append(Spectrum.RichPeak(mz=2000, i=0))
  r["richPeaks"].insert(0,Spectrum.RichPeak(mz=-500, i=0))
  r.name = match.spectrum.uid() + "-" + match.peptide
  outFile.write(str(r) + "\n")

#------------------------------------------------------------------------------
def fixFile(test, correctPeptide):
  newTest = test + ".nopositive"
  outFh = open(newTest,"w")
  fh = open(test)
  for line in fh:
    label = line.split(',')[0]
    peptide = label.split('.')[1]
    if correctPeptide == peptide: continue
    line = line.rstrip('\n')
    line = re.sub(",1,",",",line) 
    fields = line.split(',')
    for field_idx in range(1,len(fields)-3):
      fields[field_idx] = str(float(fields[field_idx])/float(fields[-1]))
    line = ','.join(fields)
    line += '\n'
    outFh.write(line)
  fh.close()
  outFh.close()
  return newTest

#------------------------------------------------------------------------------
DO_ANNOT = False
#------------------------------------------------------------------------------
if __name__ == "__main__":
  if len(sys.argv) < 3:
    print >>sys.stderr, """
Usage: %s 
  <positive-dir>
  <psm-file>
  """ % os.path.basename(sys.argv[0])
    sys.exit(1)

  kernel = "polynomial"
  degree = 1

  positiveDir = sys.argv[1]
  psmFile = sys.argv[2]

  positiveFnames = map(lambda x: os.path.abspath(positiveDir + "/" + x), filter( lambda x: x.endswith(".scores"), os.listdir(positiveDir) ) ) 

  # positiveFnames = filter( lambda x: os.path.basename(x).startswith("99"),positiveFnames)

  psm = pickle.load(open(psmFile))

  topPeps = {}
  suid2tuple = {}
  for test in positiveFnames:
    # newTest = test
    suid = os.path.basename(test).split('.')[0]
    match = psm.getMatch(suid)
    newTest = fixFile(test, match.peptide)

