#---------------------------------------------------------------------------
# File: hmmTestPrep.py
#---------------------------------------------------------------------------
# Author: $Author: aklammer $
#---------------------------------------------------------------------------
# Revision: $Revision: 1.3 $
#---------------------------------------------------------------------------
# Inputs:
#   - the output of generatePeptides, <prefix>.peps, which 
#     maps spectra to peptide files:
#     
#     spectrum id \t null peps file \t cand peps file \n
#
#   - a cPickled SpectrumCollection with spectra 
#---------------------------------------------------------------------------
# Outputs:
#  - a <prefix>.hmm directory, with two directories for each spectra in 
#    <prefix>.peps :
# 
#      - a SPECTRUM_ID.hmmnull directory, containing .hmmtrain files for 
#        the null distribution of peptides for the spectrum SPECTRUM_ID
#      - a SPECTRUM_ID.hmmcand directory, containing .hmmtrain files for
#        the candidate peptides for the spectrum SPECTRUM_ID
#    
#  - an output file <prefix>.hmmdirs mapping the spectra to hmmdir
#    directories:
# 
#     spectrum id \t null hmm test dir \t cand hmm test dir \n
#---------------------------------------------------------------------------

import os, os.path, sys, random, cPickle
import Mass
import HmmData

if len(sys.argv) < 2:
  print >>sys.stderr, """
Usage: %s <prefix> <pickledpsm>
    <prefix> - A prefix to the <prefix>.peps file.
    <pickledpsm> - A pickled PsmCollection
""" % os.path.basename(sys.argv[0])
  sys.exit(1)

def parsePeps(file):
  """
  returns spectrumid:([nullpeps],[candpeps])
  """
  fh = open(file + ".peps")
  idx = 0
  spec2peps = {}
  for line in fh:
    idx += 1
    if idx % 100 == 0: print >>sys.stderr, "Reached spectrum %i" % idx
    line = line.rstrip('\n')
    spectrumID, nullFname, candFname = line.split()
    if spec2peps.has_key(spectrumID): raise SystemExit, "Dup spectrum"
    spec2peps[spectrumID] = ( [], [] )
    
    fhNull = open(nullFname)
    for lineNull in fhNull:
      lineNull = lineNull.rstrip('\n')
      spec2peps[spectrumID][0].append(lineNull)
    fhNull.close()

    fhCand = open(candFname)
    for lineCand in fhCand:
      lineCand = lineCand.rstrip('\n')
      spec2peps[spectrumID][1].append(lineCand)
    fhCand.close()

  fh.close()
  return spec2peps

def createTestFile(match, dirName):
  hmmTrainDataFilename = dirName + "/" + "%s-%s.hmmtrain" % \
    (match.spectrum.uid(), match.peptide)
  hmmTrainData = open(hmmTrainDataFilename, 'w')
  HmmData.writePepHmmData(match, hmmTrainData)
  hmmTrainData.close()

#-----------------------------------------
# MAIN
#-----------------------------------------
filePrefix = sys.argv[1]
# maps spectrumid:(nullfile,candfile)
suidToPeps = parsePeps(filePrefix)
psmColl  = cPickle.load(open(sys.argv[2]))
idx = 0

hmmDirsFile = open(filePrefix + ".hmmdirs", 'w')
outDir = filePrefix + ".hmm"
if not os.path.isdir(outDir): os.mkdir(outDir)
os.chdir(outDir)

print >>sys.stderr, "Creating null and cand dirs for %i matches" % len(psmColl["matches"])
for match in psmColl["matches"]:
  idx+=1
  if idx % 10 == 0: print >>sys.stderr, "Reached idx %i" % idx
  if not Mass.canonicalOnly(match.peptide): 
    print >>sys.stderr, "Skipping non-canonical aa at idx %i" % idx
    continue
  if match.spectrum["parsePeaks"] == False: match.spectrum.parsePeaks()
  match.spectrum.rankRelativizeIntensity()

  suid = match.spectrum.uid()
  # create null dir
  if not suidToPeps.has_key(suid): 
    print >>sys.stderr, "Do not have %s" % `suid`
    continue
  nullDirName = suid + ".hmmnull"
  if not os.path.isdir(nullDirName): os.mkdir(nullDirName)

  print >>sys.stderr, "Processing %s" % `suid`
  # populate null dir
  for nullPep in suidToPeps[suid][0]:
    match.peptide = nullPep
    if not Mass.canonicalOnly(match.peptide): 
      print >>sys.stderr, "Skipping non-canonical aa at idx %i" % idx
      continue
    createTestFile(match, nullDirName) 

  # create cand dir
  candDirName = suid + ".hmmcand"
  if not os.path.isdir(candDirName): os.mkdir(candDirName)
  for candPep in suidToPeps[suid][1]:
    match.peptide = candPep
    if not Mass.canonicalOnly(match.peptide): 
      print >>sys.stderr, "Skipping non-canonical aa at idx %i" % idx
      continue
    createTestFile(match, candDirName) 

  nullDirAbsPath = os.getcwd() + "/" + os.path.basename(nullDirName)
  candDirAbsPath = os.getcwd() + "/" + os.path.basename(candDirName)
  hmmDirsFile.write("%s\t%s\t%s\n" % (str(suid), nullDirAbsPath, candDirAbsPath))

hmmDirsFile.close()
