#---------------------------------------------------------------------------
# File: hmmTestSpectraSion.py
#---------------------------------------------------------------------------
# Author: $Author: aklammer $
#---------------------------------------------------------------------------
# Revision: $Revision: 1.38 $
#---------------------------------------------------------------------------
# Inputs:
# 
#  - a <collection-file> which contains the spectra 
#  - the output of generatePeptides.py, <prefix>.peps, which maps spectra 
#    to candidate peptides:
#
#     SPECTRUM_ID \t HMM_CAND_FILE \n
#     - SPECTRUM_ID is the identifier of the spectrum
#     - HMM_CAND_FILE is the file containing the candidate peptides
#  - a trained GMTK model specifier  
#  - a trained GMTK model params file prefix specifier  
#---------------------------------------------------------------------------
# Outputs:
#
#  - a directory <collection>,<GMTK-model>,<GMTK-params>.scoredir, containing the output files of 
#    hmmTestSpectrum.py run on each spectrum in the <collection-prefix>.psm
#    peptide-spectrum match:
# 
#     spectrum id \t peptide \t prob \n
# 
#---------------------------------------------------------------------------
import os, sys

#---------------------------------------------------------------------------
if __name__ == "__main__":
  if len(sys.argv) < 4:
    print >>sys.stderr, """
Usage: %s <collection-file> <indexed-fasta-file> <GMTK-model-name> <GMTK-params-file> <svm-prefix> <null?>
    <collection-file> - a pickledpsm file
    <indexed-fasta-file> - The indexed fasta file.
    <GMTK-model-name> - A GMTK model name, to use to test GMTK
    <GMTK-params-file> - A GMTK params fname, to use to test GMTK
    <svm-prefix> - The prefix to an .svm and .rescale object.
    <null> - (T|F) Is this a null run?

    output the scores for each spectrum to <colleciton-prefix>.scoredir
  """ % os.path.basename(sys.argv[0])
    sys.exit(1)

#---------------------------------------------------------------------------
from hmmParams import *
import os.path, cPickle, tempfile
import Mass
import HmmData
import math
import Numeric
import MatchCollection
import generatePeptides
import GMTKmodel 

#---------------------------------------------------------------------------
# Returns directory with ".scores" files in it
#---------------------------------------------------------------------------
def main(collectionFile=None, fastaFname=None, model=None, gmtkParms=None, kernel='sion', svmPrefix='prefix', null='F'):

  if null not in ['T', 'F']: raise SystemExit, "Strange null %s" % null

  # parse the spectrum collection
  coll = cPickle.load(open(collectionFile))

  runDir = ','.join(map(lambda x: os.path.basename(x).split('.')[0], 
      [ fastaFname, model.name, gmtkParms ])) + '.scoredir'

  if not os.path.isdir(runDir):  os.mkdir(runDir)

  cmdLines = []

  positiveSuids = []
  if os.path.isdir(runDir) and REDO == False:
    if null == 'F':
      positiveFnames = map(lambda x: os.path.abspath(runDir+ "/" + x), \
        filter( lambda x: x.endswith(".scores"), os.listdir(runDir) ) ) 
    else:
      positiveFnames = map(lambda x: os.path.abspath(runDir+ "/" + x), \
        filter( lambda x: x.endswith(".nullscores"), os.listdir(runDir) ) ) 
    positiveFnames = filter(lambda x: os.path.getsize(x) > 0, positiveFnames)
    positiveSuids = map(lambda x: os.path.basename(x).split('.')[0], positiveFnames)

  for idx in range(len(coll["matches"])):

    
    match = coll["matches"][idx]
    suid = str(match.spectrum.uid())
    if suid in positiveSuids: continue

    # create the small match collection, with only one match
    smallMatchCollection = MatchCollection.PsmMatchCollection()
    smallMatchCollection["matches"] = [ match ]
    smallCollectionFile = runDir + "/" + suid + ".pickledpsm"

    cPickle.dump(smallMatchCollection,open(smallCollectionFile, 'w'))

    # create the cmdlines
    cmdLine = "space_req %s >& /dev/null; python %s %s %s %s %s %s %s %s; " % \
        ( S_DIR, HMM_TEST_SPECTRUM, suid, \
          os.path.abspath(smallCollectionFile), fastaFname, \
          model.name, gmtkParms, os.path.abspath(svmPrefix), null) 
    cmdLines.append(cmdLine)

  # stop if there aren't any commands
  if len(cmdLines) == 0: 
    print >>sys.stderr, "No commands, so no migrate"
    return runDir

  # output to migrate file
  tmpTuple = tempfile.mkstemp()
  tmpFileFd, tmpFile = tmpTuple[0], tmpTuple[1]
  migrateFh = os.fdopen(tmpFileFd, "w")

  count = lineCount = 0
  for cmdLine in cmdLines: 
    count += 1
    if (count % NUM_JOBS_IN_CLUSTER == 1): 
      lineCount += 1
      if lineCount < SLEEP_MAX: 
        sleepTime = lineCount * SLEEP_TIME
        migrateFh.write("sleep %i;" % sleepTime)

    migrateFh.write(cmdLine)

    if (count % NUM_JOBS_IN_CLUSTER == 0): migrateFh.write("\n")

  if (count % NUM_JOBS_IN_CLUSTER != 0): migrateFh.write("\n")
  migrateFh.close()

  cwd = os.getcwd()
  # create and cd to rundir
  os.chdir(runDir)

  # sys.exit(1)
  # run migrate
  migrateCmdLine = "migrate %s --restart -f %s" % (MIGRATE_OPTIONS, tmpFile)

  print >>sys.stderr, "Running %s" % migrateCmdLine
  pvec = os.popen(migrateCmdLine)
  ret = pvec.close()
  print >>sys.stderr, "Finished migrate"
  os.chdir(cwd)
  return runDir

#---------------------------------------------------------------------------
if __name__ == "__main__":
  model = GMTKmodel.gmtkModelFactory(sys.argv[3])
  main(collectionFile=sys.argv[1], fastaFname=sys.argv[2], \
    model=model, gmtkParms=sys.argv[4], svmPrefix=sys.argv[5], null=sys.argv[6])



