#!/usr/bin/python
#-----------------
# $Revision: 1.2 $
#-----------------
import sys
import os
import Spectrum
import SpectrumCollection

#-------
# Main
#-------
if __name__ == '__main__':

  usage = """Usage: %s <ms2-file> <peptide>  >  <annot-file>
    The <ms2-file> should contain a single spectrum. 
    The <peptide> will be mapped to the spectrum.
    Output is to STDOUT, in the .annot format (input to spectacle.pl)
  """ % sys.argv[0]

  if len(sys.argv) < 3: 
    raise SystemExit, usage

  ms2File = sys.argv[1]
  peptide = sys.argv[2]

  print >>sys.stderr, "Loading spectrum file %s" % sys.argv[1]
  spectrumCollection = SpectrumCollection.Ms2SpectrumCollection(ms2File)

  spectrum = spectrumCollection["spectra"][0]
  prefix = os.path.basename(ms2File).split('.')[0]

  richObserved = Spectrum.RichSpectrum() 
  richObserved.addSpectrumInfo(spectrum)
  richObserved.addPeaksDirectly(spectrum)

  print  >>sys.stderr, "Outputting spectra for %s" % prefix
  annotatedObserved = Spectrum.RichSpectrum()
  annotatedObserved.addPeptideIons(\
    peptide, neutrals=True, multiCharge=True)

  annotatedObserved.addSpectrumInfo(richObserved)
  annotatedObserved.addNearestIntensitiesFromSpectrum(richObserved)
  annotatedObserved.sumNormalizeIntensity()

  annotatedObserved.name = "%s" % prefix 
  print annotatedObserved,

  print >>sys.stderr, "Color scheme is:\n%s" % '\n'.join( \
      map(lambda x: "  %s = %s" % (x, Spectrum.TYPE2COLOR[x]), \
      Spectrum.TYPE2COLOR.keys()))
