# $Revision: 1.9 $
import sys
from Bio import Fasta

max_mass = 10000.0
min_mass = 400.0

amino_masses = {}
amino_masses['A'] = 71.0788
amino_masses['B'] = 114.5962
amino_masses['C'] = 103.1388
amino_masses['D'] = 115.0886
amino_masses['E'] = 129.1155
amino_masses['F'] = 147.1766
amino_masses['G'] = 57.0519
amino_masses['H'] = 137.1411
amino_masses['I'] = 113.1594
amino_masses['K'] = 128.1741
amino_masses['L'] = 113.1594
amino_masses['M'] = 131.1926
amino_masses['N'] = 114.1038
amino_masses['O'] = 114.1472
amino_masses['P'] = 97.1167
amino_masses['Q'] = 128.1307
amino_masses['R'] = 156.1875
amino_masses['S'] = 87.0782
amino_masses['T'] = 101.1051
amino_masses['U'] = 150.0388
amino_masses['V'] = 99.1326
amino_masses['W'] = 186.2132
amino_masses['X'] = 113.1594
amino_masses['Y'] = 163.1760
amino_masses['Z'] = 128.6231

amino_indices = {}
amino_indices['A'] = 0
amino_indices['B'] = 1
amino_indices['C'] = 2
amino_indices['D'] = 3
amino_indices['E'] = 4
amino_indices['F'] = 5
amino_indices['G'] = 6
amino_indices['H'] = 7
amino_indices['I'] = 8
amino_indices['K'] = 9
amino_indices['L'] = 10
amino_indices['M'] = 11
amino_indices['N'] = 12
amino_indices['O'] = 13
amino_indices['P'] = 14
amino_indices['Q'] = 15
amino_indices['R'] = 16
amino_indices['S'] = 17
amino_indices['T'] = 18
amino_indices['U'] = 19
amino_indices['V'] = 20
amino_indices['W'] = 21
amino_indices['X'] = 22
amino_indices['Y'] = 23
amino_indices['Z'] = 24

def note_composition(tryptic_seq, compositions):
  composition = [ 0 ] * len(amino_indices.keys())
  for idx in range(len(tryptic_seq)):
    amino_idx = amino_indices[tryptic_seq[idx]]
    composition[amino_idx] += 1
  tup = tuple(composition)
  if compositions.has_key(tup):
    compositions[tup] += 1
  else:
    compositions[tup] = 1
  #print composition

def calc_mass(sequence):
  mass = 0.0
  for i in range(len(sequence)):
    mass = mass + amino_masses[sequence[i]]
  return mass

def distribution(masses, distribution = None):
  if distribution == None:
    distribution = {}
  # print masses
  for i in range(len(masses)):
    mass = int(masses[i] + 0.5)
    if distribution.has_key(mass):
      distribution[mass] = distribution[mass] + 1
    else:
      distribution[mass] = 1
  return distribution
    
parser = Fasta.RecordParser()
usage = "Usage: python masses.py <tryptic|non-tryptic> <filename>"
if len(sys.argv) != 3:
  print >>sys.stderr, usage
  sys.exit(1)
tryptic_only = sys.argv[1]
if tryptic_only != "tryptic" and tryptic_only != "non-tryptic":
  print >>sys.stderr, usage
  sys.exit(1)
file  = open(sys.argv[2])
iterator = Fasta.Iterator(file, parser)
cur_record = iterator.next()
seq_list = []
protein_count = 0
masses = []
dist = {}
seq_count = 0 
record_count = 0
compositions = {}
while cur_record != None:
  record_count = record_count + 1  
  if record_count % 100 == 0:
    print >>sys.stderr, "Reached sequence %i" % record_count
  protein_count = protein_count + 1
  seq = cur_record.sequence
  tryptic_seq = []
  for i in range(len(seq)):
    if tryptic_only == "tryptic":
      if (seq[i] == 'K' or seq[i] == 'R'):
        if i < len(seq)-1:
          if seq[i+1] != 'P':
            tryptic_seq.append(seq[i])
            mass = calc_mass(tryptic_seq)
            if mass < max_mass and mass > min_mass:
              masses.append(mass)
              note_composition(tryptic_seq, compositions)
            tryptic_seq = []
        else:
          tryptic_seq.append(seq[i])
          mass = calc_mass(tryptic_seq)
          if mass < max_mass and mass > min_mass:
            masses.append(mass)
          note_composition(tryptic_seq, compositions)
          tryptic_seq = []
      else:
        tryptic_seq.append(seq[i])
      dist = distribution(masses, dist)
      seq_count = len(masses) + seq_count
      masses = []
    elif tryptic_only == "non-tryptic":
      mass = 0.0
      composition_tracker = [ 0 ] * len(amino_indices.keys())
      for j in range(i,len(seq)):
        # print seq[i:j+1]
        mass = mass + amino_masses[seq[j]]
        amino_idx = amino_indices[seq[j]]
        composition_tracker[amino_idx] += 1

        if mass > max_mass:
          break
        if mass < min_mass:
          continue

        tup = tuple(composition_tracker)
        print tup
        #if compositions.has_key(tup):
        #  compositions[tup] += 1
        #else:
        #  compositions[tup] = 1

        masses.append(mass)
      dist = distribution(masses, dist)
      seq_count = len(masses) + seq_count
      masses = []
      
  
  cur_record = iterator.next()

print "Compositions:%i" % len(compositions.keys())

# print masses
sum = 0.0
weighted_sum = 0.0
for i in dist.keys():
  sum = sum + dist[i]
  weighted_sum = weighted_sum + dist[i] * dist[i]

avg = sum / len(dist)

print "Filename\tProteins\tPeptides\tAverage\tExpected"
print sys.argv[2], "\t", protein_count, "\t",seq_count, \
  "\t", avg, "\t", weighted_sum / sum 
  

