#--------------------------------------------------------
# $Revision: 1.4 $
#--------------------------------------------------------
import getopt, sys

#--------------------------------------------------------
if __name__ == '__main__':
  if len(sys.argv) < 2: 
    print """Usage: python %s 
  <-r train-file> 
  [-k gaussian|polynomial]
  [-w gaussian width]
  [-d polynomial degree]
  [-m matrix 2 png input]
""" % (sys.argv[0])
    sys.exit(-1)

#--------------------------------------------------------
import cPickle, composite, preproc, time, datafunc, svm, assess
from pylab import *

#--------------------------------------------------------
# Returns svmFname, rescaleFname 
#--------------------------------------------------------
def main(train=None, kernel='polynomial',width=None,degree=1):
  DataSet = datafunc.SparseCDataSet
  trainData = DataSet(train, idColumn = 0, labelsColumn = 1) 

  rs = preproc.Rescale()
  rs.train(trainData)

  if kernel in ('Polynomial','polynomial','poly'):
    print >>sys.stderr, "Using polynomial kernel of degree %.3f." % degree
    trainData.attachKernel('poly', degree = degree) 
  elif kernel in ('Gaussian','gaussian','rbf'):
    print >>sys.stderr, "Using Gaussian kernel of width %.3f." % width
    trainData.attachKernel('gaussian', gamma = width)

  kernel = trainData.kernel
  positives  = trainData.labels.classes[1]
  negatives  = trainData.labels.classes[0]

  posVSpos = []
  for first_idx in range(len(positives)):
    first_dataset_idx = positives[first_idx]
    for second_idx in range(first_idx,len(positives)):
      second_dataset_idx = positives[second_idx]
      #print >>sys.stderr, "Evaluating %i vs. %i" % \
      #    (first_dataset_idx, second_dataset_idx)
      posVSpos.append(kernel.eval(trainData, first_dataset_idx, second_dataset_idx))
    
  print "POS vs POS" 
  print posVSpos
  
  negVSpos = []
  for first_idx in range(len(negatives)):
    first_dataset_idx = negatives[first_idx]
    for second_idx in range(first_idx,len(positives)):
      second_dataset_idx = positives[second_idx]
      #print >>sys.stderr, "Evaluating %i vs. %i" % \
      #    (first_dataset_idx, second_dataset_idx)
      negVSpos.append(kernel.eval(trainData, first_dataset_idx, second_dataset_idx))

  print "NEG vs POS" 
  print negVSpos
    
  negVSneg = []
  for first_idx in range(len(negatives)):
    first_dataset_idx = negatives[first_idx]
    for second_idx in range(first_idx,len(negatives)):
      second_dataset_idx = negatives[second_idx]
      #print >>sys.stderr, "Evaluating %i vs. %i" % \
      #    (first_dataset_idx, second_dataset_idx)
      negVSneg.append(kernel.eval(trainData, first_dataset_idx, second_dataset_idx))
    
  print "NEG vs NEG" 
  print negVSneg

  subplot(221)
  hist(posVSpos, bins=30)
  title('Pos vs. Pos')

  subplot(222)
  hist(negVSpos, bins=30)
  title('Neg vs. Pos')

  subplot(223)
  hist(negVSneg, bins=30)
  title('Neg vs. Neg')
  savefig(train + '.eps')
  


#--------------------------------------------------------
if __name__ == '__main__':
  optlist, args = getopt.getopt(sys.argv[1:],'r:t:k:w:d:c:p:m:')
  train = kernel = width = degree = None
  for option_name,option_value in optlist:
    if option_name == '-r': train=option_value
    if option_name == '-k': kernel=option_value
    if option_name == '-w': width=float(option_value)
    if option_name == '-d': degree=int(option_value)

  main(train=train, kernel=kernel, width=width, degree=degree)

#---------------------------
#if matrix2png != None:
#    dataFields = []
#    data = open(train)
#    for line in data:
#      line = line.rstrip()
#      dataFields.append(line.split(','))
#    data.close()
#    outputLines = []
#
#    for idx in range(len(trainData)):
#      outputFields = []
#      for field in dataFields[idx]:
#        outputFields.append(field)
#      outputFields[0] = outputFields[0].split('.')[1]
#      outputFields.append(s.decisionFunc(trainData, idx))
#      outputLines.append(outputFields)
#
#    outputLines.sort(lambda x,y: cmp(float(x[-1]), float(y[-1])))
#
#    fh = open(matrix2png, "w")
#    header = "svm label b y a b-h2o y-h2o b-nh3 y-nh3 logtic numions disc".split(" ")
#    fh.write("\t".join(header) + "\n")
#    for idx in range(len(outputLines)):
#      fh.write("\t".join(map(str, outputLines[idx])) + "\n")
#    fh.close()
#---------------------------
