#---------------------------------------------------------------------------
# File: hmmAccuracy.py
#---------------------------------------------------------------------------
# Author: $Author: aklammer $
#---------------------------------------------------------------------------
# Revision: $Revision: 1.6 $
#---------------------------------------------------------------------------
# Inputs:
# 
#  - the output of hmmTest.py, file <prefix>.zscore, 
#    containing the Z scores for each peptide-spectrum match:
# 
#     spectrum id \t peptide \t zscore \n
# 
#  - <positives>.txt to compare with
# 
#---------------------------------------------------------------------------
import os, os.path, sys, random, cPickle, re 
import Mass
import HmmData
import math
try:
  import Numeric
  from pylab import *
except:
  print >>sys.stderr, "Could not import Numeric or pylab!"
import Spectrum
import bestLor
import GMTKmodel
from hmmParams import *

#------------------------------------------------------------------------------
def dtaIdToSuid(dtaId):
  groupId, firstScan, lastScan, charge = dtaId.split('/')[1].split('.')
  suid = ",".join([ firstScan, groupId, charge])
  return suid

#------------------------------------------------------------------------------
def annotSpectrum(match = None, outFile = None):
  match.spectrum.parsePeaks()

  r = Spectrum.RichSpectrum()
  r.addPeptideIons(match.peptide, neutrals=True)

  r2 = Spectrum.RichSpectrum()
  r2.name = match.spectrum.uid() + "-" + "raw"
  r2.addPeaksDirectly(match.spectrum)

  r.addNearestIntensitiesFromSpectrum(r2)

  r.basePeakNormalizeIntensity()
  r["richPeaks"].append(Spectrum.RichPeak(mz=2000, i=0))
  r["richPeaks"].insert(0,Spectrum.RichPeak(mz=-500, i=0))
  r.name = match.spectrum.uid() + "-" + match.peptide
  outFile.write(str(r) + "\n")

#------------------------------------------------------------------------------
def fixFile(test, labels):
  newTest = test + ".fixed"
  outFh = open(newTest,"w")
  fh = open(test)
  for line in fh:
    label = line.split('\t')[0]
    labels.append(label)
    line = re.sub(",","-", line)
    line = re.sub(":","-", line)
    line = re.sub("\t",",",line) 
    line = re.sub(",1,",",",line) 
    outFh.write(line)
  fh.close()
  outFh.close()
  return newTest

def getAccuracy(field, positiveFnames, psm):
  topPeps = {}

  for test in positiveFnames:
    suid = ""
    pep2score = {}
    fh = open(test)
    for line in fh:
      line = line.rstrip()
      fields = line.split(',')
      suid, peptide = fields[0].split('.')
      score = float(fields[field])
      pep2score[peptide] = score
    fh.close()
    
    peps = pep2score.keys()
    peps.sort(lambda x,y: cmp(pep2score[y],pep2score[x]))
    topPeps[suid] = peps[0]

  # get the topPeptide
  trueCount = 0
  uniqCount = 0
  uniqTrueCount = 0
  seenPeptide = {}
  totalCount = 0
  for match in psm["matches"]:
    suid = match.spectrum.uid()
    truePeptide = psm.getMatch(suid).peptide
    truePeptide = re.sub('I','L', truePeptide)

    suidLabel = re.sub(",","-",suid)
    if not topPeps.has_key(suidLabel): 
      print >>sys.stderr, "Skipping %s" % suid
      continue
    topPeptide  = re.sub('I','L', topPeps[suidLabel])
    topPeptide  = topPeps[suidLabel]
    totalCount += 1

    if topPeptide == truePeptide:
      trueCount += 1 
    else:
      pass
      # print "Field %i %s\t%s\t%s" % (field, suid, truePeptide, topPeptide)

    if seenPeptide.has_key(truePeptide): continue
    seenPeptide[truePeptide] = 1 
    uniqCount += 1
    if topPeptide == truePeptide:
      uniqTrueCount += 1 

  #print "Field %i Redundant Accuracy = %i/%i = %.3f" % ( field, trueCount, totalCount, \
  #    float(trueCount)/totalCount )

  #print "Field %i Unique Accuracy = %i/%i = %.3f" % ( field, uniqTrueCount, uniqCount, \
  #    float(uniqTrueCount)/uniqCount)
  accuracy = float(uniqTrueCount)/uniqCount
  return accuracy


#------------------------------------------------------------------------------
def makePlot(file):
  fh = open(file)
  labels = []
  heights = []
  for line in fh:
    label, height = line.rstrip('\n').split()
    labels.append(label)
    heights.append(float(height))
  fh.close()

  lefts =  [ i - 0.325 for i in range(1,len(labels)+1) ] 
  bar(lefts, heights, width=0.75)
  prefix = os.path.basename(file).split('.')[0]
  title(prefix)
  labels = [ re.sub('h2o','o', re.sub('nh3','n', i)) for i in labels ]
  labels = [ re.sub('\+','', i) for i in labels ]
  for idx in range(len(labels)):
    tups = labels[idx].split(',')
    if len(tups) > 1:
      if tups[0] == tups[1]: 
        labels[idx] = tups[0]
  xticks(range(len(labels)+1), [''] + labels, rotation=90.0, size=7)
  savefig(prefix + ".png")
  savefig(prefix + ".eps")
  show()

DO_ANNOT = False
#------------------------------------------------------------------------------
if __name__ == "__main__":
  if len(sys.argv) < 3:
    print >>sys.stderr, """
Usage: %s 
  <positive-dir>
  <psm-file>
  <model-name>
  """ % os.path.basename(sys.argv[0])
    sys.exit(1)

  positiveDir = sys.argv[1]
  psmFile = sys.argv[2]
  modelName = sys.argv[3]

  prefix = os.path.basename(positiveDir).split('.')[0]
  outputFile = prefix + ".accuracy"
  if not looksGood(outputFile): 
    model = GMTKmodel.gmtkModelFactory(modelName)
    FIELDS = []
    for idx in range(len(model.fileTypes)): FIELDS.append(idx * 3 + 2)
    # for idx in range(18): FIELDS.append((idx + 1) * 3) # FIX!!
    
    positiveFnames = map(lambda x: os.path.abspath(positiveDir + "/" + x), \
        filter( lambda x: x.endswith(".svminput"), os.listdir(positiveDir) ) ) 

    positiveFnames = filter(lambda x: os.path.getsize(x) > 0, positiveFnames)

    psm = cPickle.load(open(psmFile))
    accuracies = [0.0] * len(FIELDS)
    for fieldIdx in range(len(FIELDS)):
      field = FIELDS[fieldIdx]
      accuracies[fieldIdx] = getAccuracy(field, positiveFnames, psm)
  
    fh = open(outputFile, "w")
    for idx in range(len(accuracies)):
      fh.write("%s\t%.3f\n" % (model.fileTypes[idx], accuracies[idx]))
    fh.close()

  makePlot(outputFile)


