#!/usr/bin/python
from cPickle import *
import sys, re, os

def createBarMatlab(clusterLabels, barTypeLabels, values, outputFile):
	"""clusterLabels = [] of clusterLabels
	barTypeLabels = [] of barTypeLabels
	values = [ [] [] .. [] ], the outer [] the same length as labels
	outputFile = "" where the matlab output goes
	"""

	assert len(clusterLabels) == len(values)
	barString = "bar([ "
	for cluster in range(len(clusterLabels)):
		barString+="["
		for value in values[cluster]:
			barString+=str(float(value))
			barString+=","
		barString+="];"
	barString+="])"
	
	labelString = "set(gca,'Xtick',1:%i)\n" % len(clusterLabels)
	labelString += "set(gca,'XTickLabel',{"
	for idx in range(len(clusterLabels)):
		labelString+="'%s';" % clusterLabels[idx]
	labelString+="})"

	legendString = "legend("
	legendArray = []
	for barType in barTypeLabels:
		legendArray.append("'%s'" % str(barType))
	legendString+=','.join(legendArray)	
	legendString+=")"

	printString = "print('-depsc', '%s')" % outputFile

	tmpFilePrefix = "temp"
	tmpFile = tmpFilePrefix + ".m"

	file = open(tmpFile, 'w')
	file.write(barString + '\n')
	file.write(legendString + '\n')
	file.write(labelString + '\n')
	file.write(printString + '\n')
	file.close()

	matlabCmd = "matlab -nosplash -nojvm -r 'run %s; quit;'" % tmpFilePrefix
	os.system(matlabCmd)

if len(sys.argv) < 3:
	raise SystemExit, "Usage: %s <model1> <model2>" % sys.argv[0]


model1 = load(open(sys.argv[1]))
model2 = load(open(sys.argv[2]))

# FIX do keys and values always return in the same order
m1c = [ i / sum(model1.ctermProbDict.values()) \
		for i in model1.ctermProbDict.values()]
m1n = [ i / sum(model1.ntermProbDict.values()) \
		for i in model1.ntermProbDict.values()]
m2c = [ i / sum(model2.ctermProbDict.values()) \
		for i in model2.ctermProbDict.values()]
m2n = [ i / sum(model2.ntermProbDict.values()) \
		for i in model2.ntermProbDict.values()]

ctermEps = "%s-vs-%s-CTERM.eps" % (model1.shortname, model2.shortname) 
createBarMatlab(model1.ctermProbDict.keys(), \
		[ model1.shortname, model2.shortname ], \
		zip(m1c, m2c), ctermEps)

ntermEps = "%s-vs-%s-NTERM.eps" % (model1.shortname, model2.shortname) 
createBarMatlab(model1.ntermProbDict.keys(), \
		[ model1.shortname, model2.shortname ], \
		zip(m1n, m2n), ntermEps)\
