#---------------------------------------------------------------------------
# File: hmmAccuracy.py
#---------------------------------------------------------------------------
# Author: $Author: aklammer $
#---------------------------------------------------------------------------
# Revision: $Revision: 1.2 $
#---------------------------------------------------------------------------
# Inputs:
# 
#  - the output of hmmTest.py, file <prefix>.zscore, 
#    containing the Z scores for each peptide-spectrum match:
# 
#     spectrum id \t peptide \t zscore \n
# 
#  - <positives>.txt to compare with
# 
#---------------------------------------------------------------------------
import os, os.path, sys, random, cPickle, re 
import Mass
import HmmData
import math
try:
  import Numeric
  from pylab import *
except:
  print >>sys.stderr, "Could not import Numeric or pylab!"
import Spectrum
import bestLor
import GMTKmodel
from hmmParams import *
from assess import *

#------------------------------------------------------------------------------
def makePlot(file):
  fh = open(file)
  labels = []
  heights = []
  for line in fh:
    label, height = line.rstrip('\n').split()
    labels.append(label)
    heights.append(float(height))
  fh.close()

  lefts =  [ i - 0.325 for i in range(1,len(labels)+1) ] 
  bar(lefts, heights, width=0.75)
  prefix = os.path.basename(file).split('.')[0]
  title(prefix)
  labels = [ re.sub('h2o','o', re.sub('nh3','n', i)) for i in labels ]
  labels = [ re.sub('\+','', i) for i in labels ]
  for idx in range(len(labels)):
    tups = labels[idx].split(',')
    if len(tups) > 1:
      if tups[0] == tups[1]: 
        labels[idx] = tups[0]
  xticks(range(len(labels)+1), [''] + labels, rotation=90.0, size=7)
  ylim(0.50,1.00)
  savefig(prefix + ".png")
  savefig(prefix + ".eps")
  show()

#------------------------------------------------------------------------------
def parseSvminput(file):
  names = []
  labels = []
  data = []
  fh = open(file)
  for line in fh:
    line = line.rstrip('\n')
    a = line.split(',')
    names.append(a.pop(0))
    labels.append(int(a.pop(0)))
    a = map(float, a)
    data.append(a)
  fh.close()
  return names, labels, data

#------------------------------------------------------------------------------
def getRoc(field, names, labels, data):
  fieldData = []
  for row in data:
    fieldData.append(row[field])

  rocVal = roc(None, labels, fieldData, n=None, targetClass=1, normalize=True)
  return rocVal[2]

#------------------------------------------------------------------------------
if __name__ == "__main__":
  if len(sys.argv) < 3:
    print >>sys.stderr, """
Usage: %s 
  <svminput>
  <model-name>
  """ % os.path.basename(sys.argv[0])
    sys.exit(1)

  svmInput = sys.argv[1]
  modelName = sys.argv[2]

  names, labels, data = parseSvminput(svmInput)

  prefix = os.path.basename(svmInput).split('.')[0]
  outputFile = prefix + ".accuracy"
  if not looksGood(outputFile): 
    model = GMTKmodel.gmtkModelFactory(modelName)
    FIELDS = []
    for idx in range(len(model.fileTypes)): 
      FIELDS.append(idx * 3)
    
    rocs = [0.0] * len(FIELDS)
    for fieldIdx in range(len(FIELDS)):
      field = FIELDS[fieldIdx]
      rocs[fieldIdx] = getRoc(field, names, labels, data)
  
    fh = open(outputFile, "w")
    for idx in range(len(rocs)):
      fh.write("%s\t%.3f\n" % (model.fileTypes[idx], rocs[idx]))
    fh.close()

  makePlot(outputFile)


