#!/usr/bin/python
#-----------------
# $Revision: 1.5 $
#-----------------
import cPickle, re, sys
import os
import compile_aa_stats  
import Spectrum
import SpectrumCollection
import MatchCollection
import Model

#-------
# Main
#-------
if __name__ == '__main__':

  usage = """Usage: %s <pickled-psm-filename> <suid-filename>
    For each spectrum suid in the <scan-num-file>, retrieve a spectrum
    from pickled-psm, and output three spectra for it.
      - The original spectrum, with predicted ions colored. 
      - The SEQUEST-normalized spectrum, with predicted ions colored. 
      - The SEQUEST-theoretical spectrum, with predicted ions colored.
  """

  # TODO add different similarity score calculations
  if len(sys.argv) < 3:
    raise SystemExit, usage % (sys.argv[0])

  if not sys.argv[1].endswith(".pickledpsm"):
    print >>sys.stderr, "pickled-psm-filename must end in pickledpsm!"
    raise SystemExit, usage % (sys.argv[0])

  # Input PSMs
  print >>sys.stderr, "Loading PSM file %s" % sys.argv[1]
  psmFile = open(sys.argv[1])
  psmColl = cPickle.load(psmFile)
  psmFile.close()

  # Input suids
  suids = {}
  suidFilename = sys.argv[2]
  suidFile = open(suidFilename)
  for line in suidFile: line = line.rstrip('\n'); suids[line]=1;
  suidFile.close()
  sequestModel = Model.SequestModel()

  # Calculate similarity scores
  outFile = open(os.path.basename(suidFilename).split('.')[0] + ".annot", 'w')
  for match in psmColl["matches"]:

    uid = match.spectrum.uid()
    if not suids.has_key(uid): continue

    richObserved = Spectrum.RichSpectrum() 
    richObserved.addSpectrumInfo(match.spectrum)
    richObserved.addPeaksDirectly(match.spectrum)

    print  >>sys.stderr, "Outputting spectra for %s" % uid
    annotatedObserved = Spectrum.RichSpectrum()
    annotatedObserved.addPeptideIons(\
        match.peptide, neutrals=True, multiCharge=True)
    annotatedObserved.addSpectrumInfo(richObserved)
    annotatedObserved.addNearestIntensitiesFromSpectrum(richObserved)
    annotatedObserved.sumNormalizeIntensity()

    annotatedObserved.name = "%s-observed" % uid
    annotatedObserved["richPeaks"].append(Spectrum.RichPeak(0.0,0.0))
    annotatedObserved["richPeaks"].append(Spectrum.RichPeak(2000.0,0.0))
    print >>outFile, annotatedObserved

    annotatedObserved.name = "%s-normalized" % uid
    annotatedObserved.sequestNormalizeIntensity()
    print >>outFile, annotatedObserved

    # use model to determine the "theoretical" ion spectra
    sequestTheoretical = Spectrum.RichSpectrum()
    sequestTheoretical.addPeptideIons(match.peptide, neutrals=True)
    sequestTheoretical.addSpectrumInfo(match.spectrum)
    # make sure the model is the SEQUEST model
    sequestModel.addIntensityInfo(sequestTheoretical)
    sequestTheoretical.sumNormalizeIntensity()

    sequestTheoretical.name = "%s-sequest" % uid
    sequestTheoretical["richPeaks"].append(Spectrum.RichPeak(0.0,0.0))
    sequestTheoretical["richPeaks"].append(Spectrum.RichPeak(2000.0,0.0))
    print >>outFile, sequestTheoretical 
    print >>sys.stderr, "Finished outputting spectra for %s" % uid

  outFile.close()

