#!/usr/bin/python
#-----------------
# $Revision: 1.2 $
#-----------------
import cPickle, re, sys
import compile_aa_stats  
import Spectrum
import SpectrumCollection
import MatchCollection
import Model

#-------
# Main
#-------
if __name__ == '__main__':

	usage = """Usage: %s <pickled-psm-filename> <pickled-model-filename> <scan-num>
	For each spectrum in the <pickled-psm-filename>, predict a spectrum
	using the model in <pickled-model-filename> and calculate a similarity
	score. Output this score in <prefix1>-<model-shortname>.sim where
	<prefix> is the prefix of the <pickled-psm-filename>."""

	# TODO add different similarity score calculations
	if len(sys.argv) < 3:
		raise SystemExit, usage % (sys.argv[0])

	if not sys.argv[1].endswith(".pickledpsm"):
		print >>sys.stderr, "pickled-psm-filename must end in pickledpsm!"
		raise SystemExit, usage % (sys.argv[0])

	# Input PSMs
	print >>sys.stderr, "Loading PSM file %s" % sys.argv[1]
	psmFile = open(sys.argv[1])
	psmColl = cPickle.load(psmFile)
	psmFile.close()

	# Input Model
	print >>sys.stderr, "Loading model file %s" % sys.argv[2]
	modelFile = open(sys.argv[2])
	model = cPickle.load(modelFile)
	modelFile.close()

	# Calculate similarity scores
	idx = 0
	scores = []
	for match in psmColl["matches"]:
		idx+=1
		if idx/10.0 == idx/10: print >>sys.stderr, "Reached idx %i" % idx
	
		observed = match.spectrum
		if not int(observed["first scan"]) == int(sys.argv[3]): continue


		observed.sumNormalizeIntensity()

		outFile = open("obs." + sys.argv[1] + "." + sys.argv[2] + "." + sys.argv[3] + ".ms2", 'w')
		print >>outFile, observed
		outFile.close()
	
		# use model to determine the "theoretical" ion spectra
		theoretical = Spectrum.RichSpectrum()
		theoretical.addPeptideIons(match.peptide)
		theoretical.addSpectrumInfo(match.spectrum)
		model.addIntensityInfo(theoretical)
		theoretical.sumNormalizeIntensity()

		outFile = open("theo." + sys.argv[1] + "." + sys.argv[2] + "." + sys.argv[3] + ".ms2", 'w')
		print >>outFile, theoretical 
		outFile.close()

		s = theoretical.cosineAngle(observed)
		scores.append(s)

	# Output
	outFileName = sys.argv[2].split(".")[0] + ".sim"
	print >>sys.stderr, "Outputting similarity scores to %s"  % outFileName
	outFile = open(outFileName, 'w')
	for score in scores: outFile.write(str(score) + "\n")
	outFile.close()
