#-------------------------------------------------------------------------------
# predictMs2.py
#-------------------------------------------------------------------------------
# Given a file in the following format:
# 
#   <peptide>\t<charge>\n
# 
# and a cPickled Model, create an Ms2 file with spectra predicted by the Model
# for each peptide at the specified charge state.
#-------------------------------------------------------------------------------
import Spectrum
import SpectrumCollection
import Model
import sys, cPickle, os.path

#-------
# Main
#-------
if __name__ == '__main__':

  if len(sys.argv) < 4: raise SystemExit, \
        """
Usage: %s <pepz-filename> <model> <ms2-output> [blibsearch-ssl]

   <pepz-filename>  - A file containing peptide-charge pairs for which 
                      to predict spectra (format: peptide\\tcharge\\n)
   <model>          - A cPickled Model object
   <ms2-output>     - The .ms2 output file containing predicted spectra
   [blibsearch-ssl] - A file mapping spectra to peptides (for Bibliospec)
        """ %\
            os.path.basename(sys.argv[0])

  searchFile = ""
  if len(sys.argv) > 4 != None: searchFile = sys.argv[4]
  if searchFile != "": 
    ssl = open(searchFile, 'w')
    ssl.write("file\tscan\tcharge\tsequence\tmodifications\n")

  pepZ = open(sys.argv[1])
  m = cPickle.load(open(sys.argv[2])) 
  specColl = SpectrumCollection.Ms2SpectrumCollection()
  specColl["filename"] = sys.argv[3]
  idx = 0
  for line in pepZ:
    idx += 1
    if idx % 100 == 0: print >>sys.stderr, "Reached pepz %i" % idx
    line.rstrip()
    peptide, z = line.split()
    r = Spectrum.RichSpectrum()
    r.addPeptideIons(peptide, z=z, neutrals=False)
    m.addIntensityInfo(r)
    ms2 = Spectrum.Ms2Spectrum()
    ms2.addRichSpectrumInfo(r, firstscan=idx)
    ms2.sumNormalizeIntensity(1)
    specColl.appendSpectrum(ms2)
    if searchFile != "":
      ssl.write("%s\t%i\t%s\t%s\t%s\n" % (sys.argv[3], idx, z, peptide, "0"))

  if searchFile != "": ssl.close()

  out = open(sys.argv[3], 'w') 
  out.write(str(specColl))
  out.close()

