#!/usr/bin/python
#-----------------
# $Revision: 1.23 $
#-----------------
import cPickle, re, sys, os
import Spectrum
import SpectrumCollection
import MatchCollection
from HmmData import *

#-------
# Main
#-------
if __name__ == '__main__':

  usage = """Usage: %s <pickled-psm-filename> <hmm-training-data-prefix> <file-list> 
  """

  if len(sys.argv) < 4:
    raise SystemExit, usage % (sys.argv[0])
    
  if not sys.argv[1].endswith(".pickledpsm"):
    print >>sys.stderr, "pickled-psm-filename must end in pickledpsm!"
    raise SystemExit, usage % (sys.argv[0])

  # Input PSMs
  print >>sys.stderr, "Loading PSM file %s" % sys.argv[1]
  psmFile = open(sys.argv[1])
  psmColl = cPickle.load(psmFile)
  psmFile.close()


  print >>sys.stderr, "Analyzing %i PSMs" % len(psmColl["matches"])
  idx=0
  dirName = sys.argv[2] + ".hmmtrain"
  if not os.path.isdir(dirName): os.mkdir(dirName)
  fileList = open(sys.argv[3], 'w')
  for match in psmColl["matches"]:
    idx+=1
    if idx/10.0 == idx/10: print >>sys.stderr, "Reached idx %i" % idx
    if not Mass.canonicalOnly(match.peptide): 
      print >>sys.stderr, "Skipping non-canonical aa at idx %i" % idx
      continue

    if not match.spectrum["parsePeaks"]: match.spectrum.parsePeaks()

    # convert the spectrum into a richSpectrum
    r =Spectrum.RichSpectrum() 
    r.addSpectrumInfo(match.spectrum)
    r.addPeaksDirectly(match.spectrum)
    r.rankRelativizeIntensity()
    # 1.0 is hard coded so that the integer representations work
    r.sumNormalizeIntensity(ESTIMATED_TIC_TO_BASE_PEAK_RATIO, max=1.0)
    match.spectrum = r

    hmmTrainDataFilename = "%s-%s-%i-%i.hmmtrain" % (sys.argv[2], \
      match.peptide, int(match.spectrum["charge"]), \
      int(match.spectrum["first scan"]))

    fileList.write(os.getcwd() + os.sep + dirName + os.sep + hmmTrainDataFilename + "\n")

    hmmTrainData = open(dirName + os.sep + hmmTrainDataFilename, 'w')
    writePepHmmData(match, hmmTrainData)
    hmmTrainData.close()

  fileList.close()
