#$Revision: 1.2 $
import Numeric
import misc
import datafunc

__docformat__ = "restructuredtext en"


usage = """USAGE: pyml-learning-curve.py [options] <data> <labels>
  Produces ROC scores as a function of training set size.

  Options:
    -num-folds <int>
    -num-reps <int>
    -c-values <int>,<int>,...
    -negatives-only
    -sizes <int>,<int>,... (default is 16, 32, 64, ..., max)
"""

def learningCurve(classifier, data, **args) :

    """
    :Returns:

    - a list of result objects (if the number of repetitions per size is 1)
      or a list of dictionaries that provide the average of the various accuracy
      measures (rocArea, successRate etc.) for each dataset size.
    - 
    
    :Parameters:

    - `classifier` - a classifier object
    - `data` - a dataset object
    
    :Keywords:

    - `sizes` - a list of dataset sizes.  default: [16, 32, 64,..., max]
    - `classLabel` - compute the learning curve with respect to a particular
      class
    - `numFolds` - number of CV folds to use [default: 5]
    - `numReps` - number of samples for each dataset size [default: 1]
    """

    if 'sizes' in args :
        sizes = args['sizes']
    else :
        sizes = []
        size = 32
        while size < len(data) :
            sizes.append(size)
            size*=2
        sizes.append(len(data))
    numFolds = 5
    if 'numFolds' in args :
        numFolds = args['numFolds']
    numReps = 1
    if 'numReps' in args :
        numReps = args['numReps']
    classLabel = None
    if 'classLabel' in args :
        classLabel = args['classLabel']

    results = []
    print sizes

    for i in range(len(sizes)) :
        res = []
        for rep in range(numReps) :
            sample = datafunc.sample(data, sizes[i], stratified = True)
            res.append(classifier.stratifiedCV(sample))
        if numReps == 1 :
            results.extend(res)
        else :
            results.append(averageResults(res))

    return results, sizes

def averageResults(resultsList) :

    attributes = ['rocArea', 'rocNarea', 'successRate', 'balancedSuccessRate']
    results = {}

    for attribute in attributes :
        results[attribute] = Numeric.average(misc.extractAttribute(resultsList,
                                                                   attribute))

    return results

        
