
"""
"""

__author__ = "Sheila M. Reynolds (sheila@ee.washington.edu)"

XMIN = 0.0
XMAX = 1.0
XDELTA = 0.01
#------------------------------------------------------------------------------

def gaussDist ( x, mu, var ):

  fx = (1./sqrt(2.*pi*var)) * exp ( -1.*(x-mu)*(x-mu)/(2.*var) )

  return ( fx )

#------------------------------------------------------------------------------
# main program is here
#

from math import *
import sys

#------------------------------------------------------------------------------
def getParamInfo(parms):

  lpFilename = parms[1]
  if ( len(parms) == 4 ): 
    refFilename = parms[3]
    useRefFile = 1
  else:
    useRefFile = 0

  CARD_possCount = 40
  CARD_pepRegion =  5
  CARD_BINARY  =  2

  calcGM = 0
  writeCPT = 0

  # print parms[2]

  if ( parms[2].endswith('_CPT') ):

    print ' allocating CPT matrices ... '

    cptName = parms[2]
    if ( cptName == 'bState_2_CPT' ):
      bObs2 = [0] * CARD_possCount
      for iPossCount in range(CARD_possCount):
        bObs2[iPossCount] = [0] * CARD_pepRegion
        for iRegion in range(CARD_pepRegion):
          bObs2[iPossCount][iRegion] = [0] * CARD_BINARY
      if ( useRefFile ):
        bObs2Ref = [0] * CARD_possCount
        for iPossCount in range(CARD_possCount):
          bObs2Ref[iPossCount] = [0] * CARD_pepRegion
          for iRegion in range(CARD_pepRegion):
            bObs2Ref[iPossCount][iRegion] = [0] * CARD_BINARY

    elif ( cptName == 'bState_4_CPT' ):
      bObs4 = [0] * CARD_possCount
      for iPossCount in range(CARD_possCount):
        bObs4[iPossCount] = [0] * CARD_BINARY
        for iObs in range(CARD_BINARY):
          bObs4[iPossCount][iObs] = [0] * CARD_BINARY
          for iBit in range(CARD_BINARY):
            bObs4[iPossCount][iObs][iBit] = [0] * CARD_pepRegion
            for iRegion in range(CARD_pepRegion):
              bObs4[iPossCount][iObs][iBit][iRegion] = [0] * CARD_BINARY
      if ( useRefFile ):
        bObs4Ref = [0] * CARD_possCount
        for iPossCount in range(CARD_possCount):
          bObs4Ref[iPossCount] = [0] * CARD_BINARY
          for iObs in range(CARD_BINARY):
            bObs4Ref[iPossCount][iObs] = [0] * CARD_BINARY
            for iBit in range(CARD_BINARY):
              bObs4Ref[iPossCount][iObs][iBit] = [0] * CARD_pepRegion
              for iRegion in range(CARD_pepRegion):
                bObs4Ref[iPossCount][iObs][iBit][iRegion] = [0] * CARD_BINARY

    else:
      print ' unknown CPT ??? '
      sys.exit(-1)

    gmwName  = 'not looking for a Gaussian mixture'
    meanName = 'not looking for a Gaussian mixture'
    cvarName = 'not looking for a Gaussian mixture'

    writeCPT = 1

  else:

    mixName  = parms[2]
    # print ' Gaussian Mixture : ', mixName
    gmwName  = 'gmw_'  + mixName
    meanName = 'mean_' + mixName
    cvarName = 'cvar_' + mixName

    cptName = 'not looking for a DenseCPT'

    calcGM = 1

  # print ' opening input <%s> ' % lpFilename
  lpFile = file ( lpFilename )


  foundMixture = 0
  done = 0
  while not done:

    aLine = lpFile.readline()
    if ( len(aLine) == 0 ): done = 1

    if ( aLine.startswith(gmwName) ):
      tokenList = aLine.split()
      foundMixture = 1
      numComponents = int ( tokenList[1] )
      weightVals = []
      meanVals = []
      cvarVals = []
      for iC in range(numComponents):
        weightVals.append( float ( tokenList[2+iC] ) )
        

    elif ( aLine.startswith(meanName) ):
      # if aLine.startswith("mean_iCaaId_8"):
      tokenList = aLine.split()
      if ( tokenList[0][len(meanName)] != '_' ): 
        continue
      # print tokenList[0], meanName, len(meanName), tokenList[0][len(meanName)]
      try:
        numMean = int(tokenList[1])
        if (numMean > 1):
          print ' not ready to handle more than 1-D mixtures ... '
        else:
          meanVals.append( float(tokenList[2]) )
      except ValueError:
        continue

    elif ( aLine.startswith(cvarName) ):
      tokenList = aLine.split()
      if ( tokenList[0][len(cvarName)] != '_' ): 
        continue
      try:
        numCvar = int ( tokenList[1] )
        if ( numCvar > 1 ):
          print ' not ready to handle more than 1-D mixtures ... '
        else:
          cvarVals.append( float ( tokenList[2] ) )
      except ValueError:
        continue

    elif ( aLine.startswith(cptName) ):

      # bState_2_CPT only applies in the very first frame ... ie Region 0
      if ( cptName == 'bState_2_CPT' ):
        print ' reading in ', cptName

        bLine = lpFile.readline()
        bLine = lpFile.readline()
        bLine = lpFile.readline()

        for iPossCount in range(CARD_possCount):
          for iRegion in range(CARD_pepRegion):
            bLine = lpFile.readline()
            tokenList = bLine.split()
            for iObs in range(CARD_BINARY):
              fVal = float ( tokenList[iObs] )
              bObs2[iPossCount][iRegion][iObs] = fVal

      # bState_4_CPT applies for the rest of the peptide (except Region 0)
      if ( cptName == 'bState_4_CPT' ):
        print ' reading in ', cptName

        bLine = lpFile.readline()
        bLine = lpFile.readline()
        bLine = lpFile.readline()

        for iPossCount in range(CARD_possCount):
          for iObs in range(CARD_BINARY):
            for iBit in range(CARD_BINARY):
              for iRegion in range(CARD_pepRegion):
                bLine = lpFile.readline()
                tokenList = bLine.split()
                for jObs in range(CARD_BINARY):
                  fVal = float ( tokenList[jObs] )
                  bObs4[iPossCount][iObs][iBit][iRegion][jObs] = fVal
  ## end of reading lpFile ...

  if ( useRefFile ):

    #print '   opening input reference file %s ' % refFilename
    refFile = file ( refFilename )

    done = 0
    while not done:

      aLine = refFile.readline()
      if ( len(aLine) == 0 ): done = 1

      if ( aLine.startswith(cptName) ):

        # bState_2_CPT only applies in the very first frame ... ie Region 0
        if ( cptName == 'bState_2_CPT' ):

          bLine = refFile.readline()
          bLine = refFile.readline()
          bLine = refFile.readline()

          for iPossCount in range(CARD_possCount):
            for iRegion in range(CARD_pepRegion):
              bLine = refFile.readline()
              tokenList = bLine.split()
              for iObs in range(CARD_BINARY):
                fVal = float ( tokenList[iObs] )
                bObs2Ref[iPossCount][iRegion][iObs] = fVal

        # bState_4_CPT applies for the rest of the peptide (except Region 0)
        if ( cptName == 'bState_4_CPT' ):

          bLine = refFile.readline()
          bLine = refFile.readline()
          bLine = refFile.readline()

          for iPossCount in range(CARD_possCount):
            for iObs in range(CARD_BINARY):
              for iBit in range(CARD_BINARY):
                for iRegion in range(CARD_pepRegion):
                  bLine = refFile.readline()
                  tokenList = bLine.split()
                  for jObs in range(CARD_BINARY):
                    fVal = float ( tokenList[jObs] )
                    bObs4Ref[iPossCount][iObs][iBit][iRegion][jObs] = fVal
  ## end of reading refFile ...



  if ( calcGM ):

    if (foundMixture == 0):
      print ' failed to find mixture ... '
      sys.exit(-1)

    # print ' calculating Gaussian Mixture density : '
    # print ' weightVals : ', weightVals
    # print ' meanVals   : ', meanVals
    # print ' cvarVals   : ', cvarVals
    
    xMin = XMIN
    xMax = XMAX
    dX = XDELTA
    numX = int ((xMax - xMin)/dX)  +  1

    mean = 0.
    meanSq = 0.
    modeX = 0.
    modeY = 0.

    totalProb = 0.
    sumfx = 0.

    for iX in range(numX):
      x = xMin + (iX*dX)
      fx = 0.
      # for iC in range(numComponents):
      for iC in range(len(meanVals)): # FIX
          fx = fx  +  weightVals[iC] * gaussDist ( x, meanVals[iC], cvarVals[iC] )
      totalProb = totalProb + ( fx * dX )

    # print ' totalProb = %9.6f ' % totalProb
    halfProb = totalProb / 2.

    points = []

    median = None
    for iX in range(numX):
      x = xMin + (iX*dX)
      fx = 0.
      # for iC in range(numComponents):  
      for iC in range(len(meanVals)): # FIX
          fx = fx  +  weightVals[iC] * gaussDist ( x, meanVals[iC], cvarVals[iC] )
      # logfx = log10(fx)
      # print ' %9.6f   %15.6f   %15.6f ' % ( x, fx, logfx )
      # print ' %9.6f   %15.6f ' % ( x, fx )
      points.append(fx)
      sumfx = sumfx + ( fx * dX )
      if ( sumfx <= halfProb ):
        median = x
      mean = mean + ( fx * x * dX )
      meanSq = meanSq + ( fx * x * x * dX )
      if ( fx > modeY ):
        modeY = fx
        modeX = x

    # print meanSq, mean, mean*mean, (meanSq - mean*mean)
    var = meanSq - (mean*mean)
    if ( var > 0. ):
      sigma = sqrt ( var )
    else:
      sigma = 0.

    output = { 'points' : points, 'mean' : mean, 'median' : median, 'sigma' : sigma , 'modeX' : modeX, 'modeY':modeY}
    return output

  if ( writeCPT and useRefFile ):

      if ( cptName == 'bState_2_CPT' ):
        print ' writing out details for ', cptName

        for iPossCount in range(CARD_possCount):
          for iRegion in range(CARD_pepRegion):
            delta = bObs2[iPossCount][iRegion][1] - bObs2Ref[iPossCount][iRegion][1]
            if ( abs(delta) > 0.001 ):
              print '   iPossCount=%d  iRegion=%d  Pr[iObs=1] = %7.4f ' %\
                  ( iPossCount, iRegion, bObs2[iPossCount][iRegion][1] )

      if ( cptName == 'bState_4_CPT' ):
        print ' writing out details for ', cptName

        print ' first looking at deltas over entire table ... '
        for iPossCount in range(CARD_possCount):
          for iObs in range(CARD_BINARY):
            for iBit in range(CARD_BINARY):
              for iRegion in range(CARD_pepRegion):
                delta = bObs4[iPossCount][iObs][iBit][iRegion][1] \
                    - bObs4Ref[iPossCount][iObs][iBit][iRegion][1] 
                if ( abs(delta) > 0.001 ):
                  print '   iPossCount=%d  iPrevObs=%d  iPrevBit=%d  iRegion=%d   Pr[iObs=1] = %7.4f   (delta = %7.4f) ' %\
                      ( iPossCount, iObs, iBit, iRegion, \
                        bObs4[iPossCount][iObs][iBit][iRegion][1], delta )

        print ' next looking at part of the table : iPossCount=[0,1] ... '
        for iPossCount in range(2):
          for iObs in range(CARD_BINARY):
            for iBit in range(CARD_BINARY):
              for iRegion in range(CARD_pepRegion):
                print '   iPossCount=%d  iPrevObs=%d  iPrevBit=%d  iRegion=%d  old Pr[iObs=1] = %7.4f  new = %7.4f ' %\
                      ( iPossCount, iObs, iBit, iRegion, \
                        bObs4Ref[iPossCount][iObs][iBit][iRegion][1], \
                        bObs4[iPossCount][iObs][iBit][iRegion][1] )


  
#------------------------------------------------------------------------------
if __name__ == '__main__':
  if ( len(sys.argv) < 3 ):
    print ' Usage : %s <learned-params-filename> <mixture-name> ' % sys.argv[0]
    # print '     there is an optional third argument which will compare the learned-params to the starting parameters '
    print '       the mixture-name should be something like iNaaId_12 or iCaaId_5 or iRegion_2 '
    sys.exit(-1)

  y = getParamInfo(sys.argv)


  
#------------------------------------------------------------------------------
