#---------------------------------------------------------------------------
# File: hmmTestSpectraSion.py
#---------------------------------------------------------------------------
# Author: $Author: aklammer $
#---------------------------------------------------------------------------
# Revision: $Revision: 1.6 $
#---------------------------------------------------------------------------
# Inputs:
# 
#  - a <collection-file> which contains the spectra 
#  - the output of generatePeptides.py, <prefix>.peps, which maps spectra 
#    to candidate peptides:
#
#     SPECTRUM_ID \t HMM_CAND_FILE \n
#     - SPECTRUM_ID is the identifier of the spectrum
#     - HMM_CAND_FILE is the file containing the candidate peptides
#  - a trained GMTK model specifier  
#  - a trained GMTK model params file prefix specifier  
#---------------------------------------------------------------------------
# Outputs:
#
#  - a directory <collection>,<GMTK-model>,<GMTK-params>.scoredir, containing the output files of 
#    hmmTestSpectrum.py run on each spectrum in the <collection-prefix>.psm
#    peptide-spectrum match:
# 
#     spectrum id \t peptide \t prob \n
# 
#---------------------------------------------------------------------------
import os, sys

#---------------------------------------------------------------------------
if __name__ == "__main__":
  if len(sys.argv) < 4:
    print >>sys.stderr, """
Usage: %s <collection-file> <GMTK-model-name> <GMTK-params-file> <svm-prefix>
    <collection-file> - a pickledpsm file
    <GMTK-model-name> - A GMTK model name, to use to test GMTK
    <GMTK-params-file> - A GMTK params fname, to use to test GMTK
    <svm-prefix> - The prefix to an .svm and .rescale object.

    output the scores for each spectrum to <collection-prefix>.scoredir
  """ % os.path.basename(sys.argv[0])
    sys.exit(1)

#---------------------------------------------------------------------------
from hmmParams import *
import os.path, cPickle, tempfile
import Mass
import HmmData
import math
import Numeric
import MatchCollection
import generatePeptides
import GMTKmodel 

#---------------------------------------------------------------------------
# Returns directory with ".scores" files in it
#---------------------------------------------------------------------------
def main(collectionFile=None, model=None, gmtkParms=None, \
    kernel='sion', svmPrefix='prefix'):

  # parse the spectrum collection
  coll = cPickle.load(open(collectionFile))

  runDir = ','.join(map(lambda x: os.path.basename(x).split('.')[0], 
      [ collectionFile, svmPrefix ])) + '.scoredir'

  if not os.path.isdir(runDir):  os.mkdir(runDir)


  cmdLines = []
  copyCmdLines = []

  positiveSuids = []
  if os.path.isdir(runDir) and REDO == False:
    positiveFnames = map(lambda x: os.path.abspath(runDir+ "/" + x), \
      filter( lambda x: x.endswith(".scores"), os.listdir(runDir) ) ) 
    positiveFnames = filter(lambda x: os.path.getsize(x) > 0, positiveFnames)
    positiveSuids = map(lambda x: os.path.basename(x).split('.')[0], positiveFnames)

  for idx in range(int(len(coll["matches"])/NUM_JOBS_IN_CLUSTER)):

    # create the small match collection, with several matches
    smallMatchCollection = MatchCollection.PsmMatchCollection()
    startIdx = idx * NUM_JOBS_IN_CLUSTER 
    endIdx = (idx +1 ) * NUM_JOBS_IN_CLUSTER 
    matches = coll["matches"][ startIdx : endIdx ]
    smallMatchCollection["matches"] = matches
    smallCollectionFile = "%s/%i,%i.pickledpsm" % (runDir, startIdx, endIdx)

    fh = open(smallCollectionFile, 'w')
    cPickle.dump(smallMatchCollection, fh)
    fh.close()

    # create the cmdlines
    
    for match in matches:
      suid = str(match.spectrum.uid())
      # if suid in positiveSuids: continue

      copyCmdLine = "cp %s %s.svm %s.rescale %s/aklammer/ ; "  \
        % (os.path.abspath(smallCollectionFile), os.path.abspath(svmPrefix),
           os.path.abspath(svmPrefix), S_DIR)

      copyCmdLines.append(copyCmdLine)


      cmdLine = "python %s %s %s/aklammer/%s %s %s %s ; " % \
          (HMM_SCORE_SPECTRUM, suid, S_DIR, os.path.basename(smallCollectionFile),\
            model.name, gmtkParms, svmPrefix)
      cmdLines.append(cmdLine)


  # stop if there aren't any commands
  if len(cmdLines) == 0: 
    print >>sys.stderr, "No commands, so no migrate"
    return runDir

  # output to migrate file
  tmpTuple = tempfile.mkstemp()
  tmpFileFd, tmpFile = tmpTuple[0], tmpTuple[1]
  migrateFh = os.fdopen(tmpFileFd, "w")

  count = lineCount = 0
  for cmdLineIdx in range(len(cmdLines)): 
    count += 1

    if (count % NUM_JOBS_IN_CLUSTER == 1): 
      lineCount += 1
      # only sleep for the first several jobs submitted
      if lineCount < SLEEP_MAX: 
        sleepTime = lineCount * SLEEP_TIME
        migrateFh.write("sleep %i;" % sleepTime)

      linePrefix  = "find %s/aklammer/ -maxdepth 1 -mtime +1 | grep -v '%s/aklammer/$' | xargs rm -fr ; " % (S_DIR, S_DIR)
      linePrefix += "space_req /%s >& /dev/null; %s"  % (S_DIR, copyCmdLines[cmdLineIdx])

      migrateFh.write(linePrefix)

    cmdLine = cmdLines[cmdLineIdx]
    migrateFh.write(cmdLine)

    if (count % NUM_JOBS_IN_CLUSTER == 0): migrateFh.write("\n")

  if (count % NUM_JOBS_IN_CLUSTER != 0): migrateFh.write("\n")
  migrateFh.close()

  cwd = os.getcwd()
  # create and cd to rundir
  os.chdir(runDir)

  svm = svmPrefix + ".svmobject"
  if not os.path.isfile(svm):
    try:
      os.link("../%s" % svm, "%s" % svm)
    except:
      print >>sys.stderr, "Strange error linking files"

  migrateCmdLine = "migrate %s --restart -f %s" % (MIGRATE_OPTIONS, tmpFile)

  print >>sys.stderr, "Running %s" % migrateCmdLine
  pvec = os.popen(migrateCmdLine)
  ret = pvec.close()
  print >>sys.stderr, "Finished migrate"
  os.chdir(cwd)
  return runDir

#---------------------------------------------------------------------------
if __name__ == "__main__":
  model = GMTKmodel.gmtkModelFactory(sys.argv[2])
  main(collectionFile=sys.argv[1], model=model, \
      gmtkParms=sys.argv[3], svmPrefix=sys.argv[4])



