#---------------------------------------------------------------------------
# File: hmmAccuracy.py
#---------------------------------------------------------------------------
# Author: $Author: aklammer $
#---------------------------------------------------------------------------
# Revision: $Revision: 1.5 $
#---------------------------------------------------------------------------
# Inputs:
# 
#  - the output of hmmTest.py, file <prefix>.zscore, 
#    containing the Z scores for each peptide-spectrum match:
# 
#     spectrum id \t peptide \t zscore \n
# 
#  - <positives>.txt to compare with
# 
#---------------------------------------------------------------------------
import os, sys
#------------------------------------------------------------------------------
from hmmParams import *
from hmmAccuracySionSvm import *
import os.path, random, cPickle, re 
import Mass
import HmmData
import math
try: import Numeric; from pylab import *;
except: print >>sys.stderr, "Could not import Numeric or pylab!"
import Spectrum
import bestLor

#------------------------------------------------------------------------------
import preproc, assess, datafunc, svm

#------------------------------------------------------------------------------
# single SVM analysis
#------------------------------------------------------------------------------
def singleSvmAccuracy(svmInputFile=None, svmFilePrefix=None):
 
  svmFname = svmFilePrefix + ".svm"
  rescaleFname = svmFilePrefix + ".rescale"

  rs = cPickle.load(open(rescaleFname))
  s = svm.loadSVM(svmFname)

  labels = []
  newInputFile = fixFile(svmInputFile, labels)

  DataSet = datafunc.SparseDataSet
  try:
    testData = DataSet(newInputFile, idColumn = 0)
  except:
    print >>sys.stderr, "Failed to parse %ith test data set %s " \
        % (idx, newInputFile)
    return None

  rs.test(testData)

  pep2score = {}
  for idx in range(len(testData)):
    pep = labels[idx].split('.')[1] 
    pep2score[pep]=float(s.decisionFunc(testData, idx)) 
  
  os.unlink(newInputFile)
  return pep2score

#------------------------------------------------------------------------------
if __name__ == "__main__":
  if len(sys.argv) < 2:
    print >>sys.stderr, """
Usage: %s <svm-input-file> <svm-file-prefix>
  """ % os.path.basename(sys.argv[0])
    sys.exit(1)

  svmInputFile  = sys.argv[1]
  svmFilePrefix = sys.argv[2]
  outputFileName = '.'.join(svmInputFile.split('.')[0:-1])
  outputFileName += ".disc"
  pep2score = singleSvmAccuracy(svmInputFile=svmInputFile, svmFilePrefix=svmFilePrefix)
  outputFile = open(outputFileName, 'w')
  for pep in pep2score.keys(): outputFile.write("%s\t%s\n" % (pep, pep2score[pep]))
  outputFile.close()

#------------------------------
#  
#
#
#  posLors = []
#  negLors = []
#  topNegLors = []
#  for suid in lor.keys():
#    thisSuidNegLors = []
#    match = psm.getMatch(suid)
#    truePeptide = match.peptide
#    for peptide in lor[suid].keys():
#      if peptide == truePeptide:
#        posLors.append(lor[suid][peptide])
#      else:
#        negLors.append(lor[suid][peptide])
#        thisSuidNegLors.append(lor[suid][peptide])
#    thisSuidNegLors.sort()
#    topNegLors.append(thisSuidNegLors[-1])
#
#  #cPickle.dump(negLors, open("neg", "w"))
#  #cPickle.dump(posLors, open("pos", "w"))
#
#  negLors = filter(lambda x: x > -40, negLors)
#  negHist = hist(negLors, bins = 20, normed = True, hold = False) 
#  posHist = hist(posLors, bins = 20, normed = True, hold = False) 
#  hold(False)
#  plot(negHist[1], negHist[0], label="Neg")
#  hold(True)
#  xlabel("LOR")
#  ylabel("Prob")
#  plot(posHist[1], posHist[0], label="Pos")
#  legend()
#  savefig("all-prob.png")
#  show()
#
#  hold(False)
#  negLors = filter(lambda x: x > 20, negLors)
#  negHist = hist(negLors, bins = 20,  hold = False) 
#  posHist = hist(posLors, bins = 20,  hold = False) 
#  plot(negHist[1], negHist[0], label="Neg")
#  hold(True)
#  xlabel("LOR")
#  ylabel("Freq")
#  plot(posHist[1], posHist[0], label="Pos")
#  legend()
#  savefig("all-freq.png")
#  show()
#
#  hold(False)
#  negHist = hist(topNegLors, bins = 20,  hold = False) 
#  posHist = hist(posLors, bins = 20,  hold = False) 
#  plot(negHist[1], negHist[0], label="Neg")
#  hold(True)
#  xlabel("LOR")
#  ylabel("Freq")
#  plot(posHist[1], posHist[0], label="Pos")
#  legend()
#  savefig("topneg-freq.png")
#  show()
#

#------------------------------------------------------------------------------
#def annotSpectrum(match = None, outFile = None):
#  match.spectrum.parsePeaks()
#
#  r = Spectrum.RichSpectrum()
#  r.addPeptideIons(match.peptide, neutrals=True)
#
#  r2 = Spectrum.RichSpectrum()
#  r2.name = match.spectrum.uid() + "-" + "raw"
#  r2.addPeaksDirectly(match.spectrum)
#
#  r.addNearestIntensitiesFromSpectrum(r2)
#
#  r.basePeakNormalizeIntensity()
#  r["richPeaks"].append(Spectrum.RichPeak(mz=2000, i=0))
#  r["richPeaks"].insert(0,Spectrum.RichPeak(mz=-500, i=0))
#  r.name = match.spectrum.uid() + "-" + match.peptide
#  outFile.write(str(r) + "\n")
#

