#---------------------------------------------------------------------------
# File: spFilter.py
#---------------------------------------------------------------------------
# Author: $Author: aklammer $
#---------------------------------------------------------------------------
# Revision: $Revision: 1.17 $
#---------------------------------------------------------------------------
# Inputs:
# 
#  - a <collection-file> which contains the spectra 
#  - the output of generatePeptides.py, <prefix>.peps, which maps spectra 
#    to candidate peptides:
#
#     SPECTRUM_ID \t HMM_CAND_FILE \n
#     - SPECTRUM_ID is the identifier of the spectrum
#     - HMM_CAND_FILE is the file containing the candidate peptides
#---------------------------------------------------------------------------
# Outputs:
#
#  - another <prefix>,sp.peps file, mapping spectra to candidate peptides
#---------------------------------------------------------------------------
import sys, os

#-------------------------------------------------------------------------------
if __name__ == "__main__":
  if len(sys.argv) < 3:
    print >>sys.stderr, """
Usage: %s <collection-file> <peps-file> 
    <collection-file> - a pickledpsm file
    <peps> - The .peps file.

    output probs and zscores for each spectrum to <peps-prefix>,sp.peps file
  """ % os.path.basename(sys.argv[0])
    sys.exit(1)

#-------------------------------------------------------------------------------
import os.path, cPickle
from Mass import mono_mass
import MatchCollection
from hmmTestSpectrum import parsePeps
from Sp import *

#-------------------------------------------------------------------------------
# Returns outputPepsFile
#-------------------------------------------------------------------------------
def main(collectionFile=None, pepsFile=None):

  pepOutFile = '.'.join(pepsFile.split('.')[0:-1]) + ",sp.peps"

  # don't redo if not necessary
  if os.path.isfile(pepOutFile) and not ALL_REDO:
    if os.path.getsize(pepOutFile) > 0:
      print >>sys.stderr, "Using old %s" % pepOutFile
      return pepOutFile
    else:
      print >>sys.stderr, "Ignoring zero size %s" % pepOutFile

  print >>sys.stderr, "Reading spectrum collection %s" % collectionFile
  coll = cPickle.load(open(collectionFile))

  #---- TEST_SP -----
  if TEST_SP == True:
    suid2pep = {}
    for match in coll["matches"]:
      suid2pep[ match.spectrum.uid() ] = match.peptide
    accuracy = 0

  # parse the .peps file
  spectrumCount = 0 
  suid2spCand = {}
  pepsFh = open(pepsFile)
  for line in pepsFh:
    line = line.rstrip('\n')

    suid, candFile = line.split()

    spectrum = coll.getSpectrum(suid)
    if spectrum == None: continue
    spectrum.parsePeaks()

    spCandFile = '.'.join(candFile.split('.')[0:-1]) + ",sp.cand"
    suid2spCand[suid] = spCandFile

    #print >>sys.stderr, "Analyzing spectrum %s" % suid
    spectrumCount += 1
    spFilterFromFiles(spectrum, candFile, spCandFile)

    #---- TEST_SP -----
    if TEST_SP == True:
      peps = parsePeps(spCandFile)
      for pep in peps:
        if (suid2pep[suid] == pep): accuracy += 1

  pepsFh.close()

  fh = open(pepOutFile, 'w')
  print >>sys.stderr, "Outputting to %s" % pepOutFile
  for suid, spFile in suid2spCand.items():
    fh.write( "\t".join( [ suid, spFile ] ) + "\n") 
  fh.close()

  #---- TEST_SP -----
  if TEST_SP == True:
    print >>sys.stderr, "Accuracy = %.3f" % (float(accuracy)/spectrumCount)

  print >>sys.stderr, "Finished Sp filtering"

  return pepOutFile

#-------------------------------------------------------------------------------
if __name__ == "__main__":
  pepOutFile = main(collectionFile=sys.argv[1], pepsFile=sys.argv[2])


