#!/usr/bin/python
#-----------------
# $Revision: 1.2 $
#-----------------
import cPickle, re, sys
import compile_aa_stats  
import Spectrum
import SpectrumCollection
import MatchCollection
import Model
from pylab import *
subset = False
NUM_COMPARE_PEAKS = 10
TYPE = 'by'
#BINS = 10
BINS = [0.9,0.91,0.92,0.93,0.94,0.95,0.96,0.97,0.98,0.99]
MIN_X = 0.90

def plotDistributions(allObservedIons, observedMaxIntensity,\
    predictedMaxIntensity):

  for idx in range(len(allObservedIons)): 
    if allObservedIons[idx] < MIN_X: 
      allObservedIons[idx] = MIN_X
  for idx in range(len(observedMaxIntensity)): 
    if observedMaxIntensity[idx] < MIN_X: 
      observedMaxIntensity[idx] = MIN_X
  for idx in range(len(predictedMaxIntensity)): 
    if predictedMaxIntensity[idx] < MIN_X: 
      print predictedMaxIntensity[idx]
      predictedMaxIntensity[idx] = MIN_X

  allObservedHist = \
      hist(allObservedIons, bins = BINS, normed = True, hold = False)
  observedHist = \
      hist(observedMaxIntensity, bins = BINS, normed = True, hold = False)
  predictedHist = \
      hist(predictedMaxIntensity, bins = BINS, normed = True, hold = False)

  hold(False)
  plot(allObservedHist[1], allObservedHist[0], 'b-', label = 'All')
  hold(True)
  plot(observedHist[1], observedHist[0], 'r-', label = 'Observed Max')
  plot(predictedHist[1], predictedHist[0], 'g-', label = 'Predicted Max')
  xlabel('Percentile')
  ylabel('Number')
  title("%s-ion" % TYPE)
  legend()
  savefig("%s-ion.png" % TYPE)
  savefig("%s-ion.eps" % TYPE)
  show()

#-------
# Main
#-------
if __name__ == '__main__':

  usage = """Usage: %s <pickled-psm-filename> <pickled-model-filename>
  For each spectrum in the <pickled-psm-filename>, predict a spectrum
  using the model in <pickled-model-filename>
  plot the distribution of the y-ions"""

  if len(sys.argv) < 2:
    raise SystemExit, usage % (sys.argv[0])
    
  if not sys.argv[1].endswith(".pickledpsm"):
    print >>sys.stderr, "pickled-psm-filename must end in pickledpsm!"
    raise SystemExit, usage % (sys.argv[0])

  # Input PSMs
  print >>sys.stderr, "Loading PSM file %s" % sys.argv[1]
  psmFile = open(sys.argv[1])
  psmColl = cPickle.load(psmFile)
  psmFile.close()

  # Input Model
  print >>sys.stderr, "Loading model file %s" % sys.argv[2]
  modelFile = open(sys.argv[2])
  model = cPickle.load(modelFile)
  modelFile.close()

  # we will plot three distributions, all b-ions, the max b-ion, and the
  # max-predicted b-ion
  allObservedIons = []
  observedMaxIntensities = []
  predictedMaxIntensities = []

  idx = 0
  tol = 0.1
  for match in psmColl["matches"]:
    idx+=1
    if idx/10.0 == idx/10: print >>sys.stderr, "Reached idx %i" % idx
 
    observed = match.spectrum
    observed.rankRelativizeIntensity()
    temp = Spectrum.RichSpectrum()
    temp.addSpectrumInfo(observed)
    temp.addPeaksDirectly(observed)

    richObserved = Spectrum.RichSpectrum()
    richObserved.addPeptideIons(match.peptide, neutrals=False, multiCharge=False)
    richObserved.addNearestIntensitiesFromSpectrum(\
      temp, doubleCounting=False)

    # use model to determine the "theoretical" ion spectra
    theoretical = Spectrum.RichSpectrum()
    theoretical.addPeptideIons(match.peptide, neutrals=False, multiCharge=False)
    theoretical.addSpectrumInfo(match.spectrum)
    model.addIntensityInfo(theoretical)

    # find the max predicted peak
    maxMz = None
    maxIntensity = -1.0
    for peak in theoretical["richPeaks"]: 
      if (peak.type in TYPE) and (peak.i > maxIntensity) : 
        maxIntensity = peak.i
        maxMz = peak.mz

    maxIntensity = -1.0
    predictedMaxIntensity = -1.0
    for peak in richObserved["richPeaks"]: 
      if peak.type in TYPE: 
        allObservedIons.append(peak.i)
        if maxMz - tol < peak.mz and peak.mz < maxMz + tol:
          if peak.i > predictedMaxIntensity:
            predictedMaxIntensity = peak.i
            if peak.i == 0.0:
              print str(theoretical)
              print maxMz, peak.i
        if peak.i > maxIntensity:
          maxIntensity = peak.i
   
    predictedMaxIntensities.append(predictedMaxIntensity)
    observedMaxIntensities.append(maxIntensity)

  plotDistributions(allObservedIons, observedMaxIntensities, \
      predictedMaxIntensities)

