#!/usr/bin/python
#-----------------
# $Revision: 1.7 $
#-----------------
import cPickle, re, sys
from Model import * 
from pylab import *
import Spectrum
import SpectrumCollection
import MatchCollection

# NUMBER_PREDICTED_PEAKS = range(10)
NUMBER_PREDICTED_PEAKS = range(1,11)
# NUMBER_OBSERVED_PEAKS = range(20)
NUMBER_OBSERVED_PEAKS = range(1,11)
TOLERANCE = 0.5

#---------
def initData(data):
  for numPickedPeaks in NUMBER_PREDICTED_PEAKS:
    for numObservedPeaks in NUMBER_OBSERVED_PEAKS:
      data[numPickedPeaks,numObservedPeaks] = 0

#---------
def dataString(data):
  string = ""
  for numPickedPeaks in NUMBER_PREDICTED_PEAKS:
    for numObservedPeaks in NUMBER_OBSERVED_PEAKS:
      string += "%s\t%s\t%s\n" % (numPickedPeaks, numObservedPeaks, \
        data[numPickedPeaks,numObservedPeaks]) 
  return string

#---------
def mzOverlapPeaks(peaks1, peaks2, tol):
  for peak1 in peaks1:
    for peak2 in peaks2:
      if (peak1.mz - tol <= peak2.mz and peak2.mz <= peak1.mz + tol):
        return True
  return False

#---------
def countOverlap(observed, predicted, data):
  #---- Assumes observed and predicted are sorted by intensity
  assert(observed["richPeaks"][0].i >= observed["richPeaks"][-1].i)
  assert(predicted["richPeaks"][0].i >= predicted["richPeaks"][-1].i)

  for numberObservedPeaks in NUMBER_OBSERVED_PEAKS:
    observedPeaks = observed.getFirstNpeaks(numberObservedPeaks)
    for numberPredictedPeaks in NUMBER_PREDICTED_PEAKS:
      predictedPeaks = predicted.getFirstNpeaks(numberPredictedPeaks)

      if mzOverlapPeaks(predictedPeaks, observedPeaks, TOLERANCE):
        data[numberPredictedPeaks, numberObservedPeaks] += 1

#---------
def plotData(real, straw, maxY=100):
  print "<html>"
  for numPickedPeaks in NUMBER_PREDICTED_PEAKS:
    #--- need an origin
    xs = [0] + NUMBER_OBSERVED_PEAKS
    realYs = [0]; strawYs = [0];
    for numObservedPeaks in NUMBER_OBSERVED_PEAKS:
      realYs.append(real[numPickedPeaks, numObservedPeaks])
      strawYs.append(straw[numPickedPeaks,numObservedPeaks])

    plot(xs, realYs, label="Real")
    hold(True)
    title("Predicted %i" % numPickedPeaks)
    xlabel("Number matched peaks")
    ylabel("Number spectra")
    plot(xs, strawYs, label="Straw")
    xlim(0, max(xs)+1)
    ylim(0, maxY)
    legend()
    prefix = "predicted-%i" % numPickedPeaks
    savefig("%s.png" % prefix)
    savefig("%s.eps" % prefix)
    print "<img src='%s.png'><br><br>" % prefix
    hold(False)
  print "</html>"

#-------
# Main
#-------
if __name__ == '__main__':

  usage = """Usage: %s <pickled-psm-filename> <pickled-model-filename>
  For each spectrum in the <pickled-psm-filename>, predict a spectrum
  using the model in <pickled-model-filename> """

  if len(sys.argv) < 2:
    raise SystemExit, usage % (sys.argv[0])
    
  if not sys.argv[1].endswith(".pickledpsm"):
    print >>sys.stderr, "pickled-psm-filename must end in pickledpsm!"
    raise SystemExit, usage % (sys.argv[0])

  # Input PSMs
  print >>sys.stderr, "Loading PSM file %s" % sys.argv[1]
  psmFile = open(sys.argv[1])
  psmColl = cPickle.load(psmFile)
  psmFile.close()

  # Input Model
  print >>sys.stderr, "Loading model file %s" % sys.argv[2]
  modelFile = open(sys.argv[2])
  model = cPickle.load(modelFile)
  modelFile.close()

  # init out main data store
  strawPeakData = {}
  predictedPeakData = {}
  initData(strawPeakData)
  initData(predictedPeakData)

  # we will plot three distributions, all b-ions, the max b-ion, and the
  idx = 0
  for match in psmColl["matches"]:
    idx+=1
    if idx % 10 == 0: print >>sys.stderr, "Reached idx %i" % idx
 
    observed = match.spectrum
    observed.parsePeaks()
    observed.rankRelativizeIntensity()
    temp = Spectrum.RichSpectrum()
    temp.addSpectrumInfo(observed)
    temp.addPeaksDirectly(observed)

    # create a RichSpectrum from the observed
    richObserved = Spectrum.RichSpectrum()
    richObserved.addPeptideIons(match.peptide, neutrals=False, multiCharge=False)
    richObserved.addNearestIntensitiesFromSpectrum(\
      temp, doubleCounting=False)
    richObserved.sortDescendingIntensity()

    # use model to determine the "predicted" ion spectra
    predicted = Spectrum.RichSpectrum()
    predicted.addPeptideIons(match.peptide, neutrals=False, multiCharge=False)
    predicted.addSpectrumInfo(match.spectrum)
    model.addIntensityInfo(predicted)
    predicted.sortDescendingIntensity()

    # use straw man to determine the "straw" ion spectra
    straw = Spectrum.RichSpectrum()
    straw.addPeptideIons(match.peptide, neutrals=False, multiCharge=False)
    straw.addSpectrumInfo(match.spectrum)
    strawModel = StrawModel()
    strawModel.addIntensityInfo(straw)
    straw.sortDescendingIntensity()

    # count the overlapping peaks
    countOverlap(richObserved, straw, strawPeakData) 
    countOverlap(richObserved, predicted, predictedPeakData) 

  # plot the resulting data
  plotData(predictedPeakData, strawPeakData, maxY=len(psmColl["matches"]))
    
    
