#!/usr/bin/python
#-----------------
# $Revision: 1.24 $
#-----------------
import cPickle, re, sys
import compile_aa_stats  
import Spectrum
import SpectrumCollection
import MatchCollection
import Model
subset = False
NUM_COMPARE_PEAKS = 20

#-------
# Main
#-------
if __name__ == '__main__':

  usage = """Usage: %s <pickled-psm-filename> <pickled-model-filename> <sim-score>
  For each spectrum in the <pickled-psm-filename>, predict a spectrum
  using the model in <pickled-model-filename> and calculate a similarity
  score. Output this score in <prefix1>-<model-shortname>.sim where
  <prefix> is the prefix of the <pickled-psm-filename>."""

  if len(sys.argv) < 3:
    raise SystemExit, usage % (sys.argv[0])
    
  if not sys.argv[1].endswith(".pickledpsm"):
    print >>sys.stderr, "pickled-psm-filename must end in pickledpsm!"
    raise SystemExit, usage % (sys.argv[0])

  # Input PSMs
  print >>sys.stderr, "Loading PSM file %s" % sys.argv[1]
  psmFile = open(sys.argv[1])
  psmColl = cPickle.load(psmFile)
  psmFile.close()

  # Input Model
  print >>sys.stderr, "Loading model file %s" % sys.argv[2]
  modelFile = open(sys.argv[2])
  model = cPickle.load(modelFile)
  modelFile.close()

  simScore = 1
  if len(sys.argv) == 4 : 
    simScore = int(sys.argv[3])

  if (simScore != 1) and (simScore != 2) and (simScore != 3):
    raise SystemExit, "Invalid simScore %i" % simScore

  # Calculate similarity scores
  idx = 0
  scores = []

  # TODO add other models that this could be
  subsetModel = None
  realByModel = Model.RealByModel()
  realByModel.train(psmColl)
  subsetModel = realByModel

  for match in psmColl["matches"]:
    idx+=1
    if idx/10.0 == idx/10: print >>sys.stderr, "Reached idx %i" % idx
  
    observed = match.spectrum
    observed.sumNormalizeIntensity()

    # create a subsetted spectrum with only those ions 
    # that we are predicting
    observedSubset = Spectrum.RichSpectrum()
    if subset == True:
      observedSubset.addPeptideIons(match.peptide)
      observedSubset.addSpectrumInfo(observed)
      subsetModel.addIntensityInfo(observedSubset)
      observedSubset.sumNormalizeIntensity()

    # use model to determine the "theoretical" ion spectra
    theoretical = Spectrum.RichSpectrum()
    if subset == False:
      theoretical.addPeptideIons(match.peptide,\
        neutrals=True, multiCharge=True, isotopes=False)
    else:
      theoretical.addPeptideIons(match.peptide)

    theoretical.addSpectrumInfo(match.spectrum)
    s = 0.0
    model.addIntensityInfo(theoretical)
    theoretical.sumNormalizeIntensity()

    compare = None
    if subset == True: compare = observedSubset
    else: compare = observed

    if simScore == 1: s = theoretical.cosineAngle(compare)
    elif simScore == 2: s = theoretical.sqrtCosineAngle(compare)
    else: s = theoretical.matchedNormTopPeaks(compare, NUM_COMPARE_PEAKS)

    scores.append(s)

  # Output
  outFileName = sys.argv[2].split(".")[0] + ".sim" + str(simScore)
  print >>sys.stderr, "Outputting similarity scores to %s"  % outFileName
  outFile = open(outFileName, 'w')
  for score in scores: outFile.write(str(score) + "\n")
  outFile.close()
