#---------------------------------------------------------------------------
# File: hmmTestSpectraSion.py
#---------------------------------------------------------------------------
# Author: $Author: aklammer $
#---------------------------------------------------------------------------
# Revision: $Revision: 1.6 $
#---------------------------------------------------------------------------
# Inputs:
# 
#  - a <collection-file> which contains the spectra 
#  - the output of generatePeptides.py, <prefix>.peps, which maps spectra 
#    to candidate peptides:
#
#     SPECTRUM_ID \t HMM_CAND_FILE \n
#     - SPECTRUM_ID is the identifier of the spectrum
#     - HMM_CAND_FILE is the file containing the candidate peptides
#  - a trained GMTK model specifier  
#  - a trained GMTK model params file prefix specifier  
#---------------------------------------------------------------------------
# Outputs:
#
#  - a directory <collection>,<GMTK-model>,<GMTK-params>.scoredir, containing the output files of 
#    hmmTestSpectrum.py run on each spectrum in the <collection-prefix>.psm
#    peptide-spectrum match:
# 
#     spectrum id \t peptide \t prob \n
# 
#---------------------------------------------------------------------------
import os, sys
import Utils
import signal

#---------------------------------------------------------------------------
if __name__ == "__main__":
  if len(sys.argv) < 4:
    print >>sys.stderr, """
Usage: %s <collection-file> <indexed-fasta-file> <model-name> <param-prefix> <svm-prefix>
   <collection-file>   - a pickled file (.pickledscoll | .pickledpsm) 
                         containing the spectrum
   <indexed-fasta-file> - an indexed fasta file
   <model-name> - name of the model
   <param-prefix> - parameter prefix
   <svm-prefix> - svm prefix

    output the scores for each spectrum to <colleciton-prefix>.scoredir
  """ % os.path.basename(sys.argv[0])
    sys.exit(1)

#---------------------------------------------------------------------------
from hmmParams import *
import os.path, cPickle, tempfile
import Mass
import HmmData
import math
import Numeric
import MatchCollection
import generatePeptides
import GMTKmodel 

#---------------------------------------------------------------------------
# Returns directory with ".scores" files in it
#---------------------------------------------------------------------------
def main(collectionFile, fastaFname, modelName, paramPrefix, svmPrefix):

  # parse the spectrum collection
  coll = cPickle.load(open(collectionFile))

  runDir = ','.join(map(lambda x: os.path.basename(x).split('.')[0], 
      [ collectionFile, fastaFname, modelName, paramPrefix, svmPrefix ])) + '.scoredir'

  if not os.path.isdir(runDir):  os.mkdir(runDir)
  fastaFname = os.path.abspath(fastaFname)

  cmdLines = []

  positiveSuids = []
  if os.path.isdir(runDir) and REDO == False:
    positiveFnames = map(lambda x: os.path.abspath(runDir+ "/" + x), \
      filter( lambda x: x.endswith(".scores"), os.listdir(runDir) ) ) 
    positiveFnames = filter(lambda x: os.path.getsize(x) > 0, positiveFnames)
    positiveSuids = map(lambda x: os.path.basename(x).split('.')[0], positiveFnames)

  for idx in range(len(coll["matches"])):

    match = coll["matches"][idx]
    suid = str(match.spectrum.uid())
    if suid in positiveSuids: continue

    # create the small match collection, with only one match
    smallMatchCollection = MatchCollection.PsmMatchCollection()
    smallMatchCollection["matches"] = [ match ]
    smallCollectionFile = runDir + "/" + suid + ".pickledpsm"

    cPickle.dump(smallMatchCollection,open(smallCollectionFile, 'w'))

    # remove all old dirs
    # create the cmdlines
    cmd = "python %s %s %s %s %s %s %s; " % (HMM_TEST_SPECTRUM, suid, \
          os.path.abspath(smallCollectionFile), fastaFname, modelName, \
          paramPrefix, svmPrefix)
    cmdLines.append(cmd)

  # stop if there aren't any commands
  if len(cmdLines) == 0: 
    print >>sys.stderr, "No commands, so no migrate"
    return runDir

  # output to migrate file
  tmpTuple = tempfile.mkstemp("", "", os.getcwd())
  tmpFileFd, tmpFile = tmpTuple[0], tmpTuple[1]
  migrateFh = os.fdopen(tmpFileFd, "w")
  count = 0
  prefixCmd = "find %s/aklammer/ -maxdepth 1 -mtime +1 | grep -v
  '%s/aklammer/$' | xargs rm -fr ; " % (S_DIR, S_DIR)
  prefixCmd += "space_req %s >& /dev/null ; "  % S_DIR

  for cmdLine in cmdLines: 
    count += 1

    if (count % NUM_JOBS_IN_CLUSTER == 1): 
      migrateFh.write(prefixCmd)
    
    migrateFh.write(cmdLine)

    if (count % NUM_JOBS_IN_CLUSTER == 0): 
      migrateFh.write("\n")

  if (count % NUM_JOBS_IN_CLUSTER != 0): 
    migrateFh.write("\n")

  migrateFh.close()

  cwd = os.getcwd()
  # create and cd to rundir
  os.chdir(runDir)
  svm = svmPrefix + ".svmobject"
  if not os.path.isfile(svm):
    os.link("../%s" % svm, "%s" % svm) 

  # sys.exit(1)
  # run migrate
  migrateCmdLine = "migrate %s --restart -f %s >& %s.error ; %s" \
      % (MIGRATE_OPTIONS, tmpFile, tmpFile, Utils.alertCommand(errorFile=tmpFile))

  print >>sys.stderr, "Running %s" % migrateCmdLine
  pvec = os.popen3(migrateCmdLine)
  # ret = pvec.close()
  # print >>sys.stderr, "Finished migrate"
  os.chdir(cwd)
  return runDir

#---------------------------------------------------------------------------
if __name__ == "__main__":
  main(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5])
  sys.exit(0)
