#---------------------------------------------------------------------------
# File: hmmTestSpectraSion.py
#---------------------------------------------------------------------------
# Author: $Author: aklammer $
#---------------------------------------------------------------------------
# Revision: $Revision: 1.17 $
#---------------------------------------------------------------------------
# Inputs:
# 
#  - a <collection-file> which contains the spectra 
#  - the output of generatePeptides.py, <prefix>.peps, which maps spectra 
#    to candidate peptides:
#
#     SPECTRUM_ID \t HMM_CAND_FILE \n
#     - SPECTRUM_ID is the identifier of the spectrum
#     - HMM_CAND_FILE is the file containing the candidate peptides
#  - a trained GMTK model specifier  
#  - a trained GMTK model params file prefix specifier  
#---------------------------------------------------------------------------
# Outputs:
#
#  - a directory <collection>,<GMTK-model>,<GMTK-params>.scoredir, containing the output files of 
#    hmmTestSpectrum.py run on each spectrum in the <collection-prefix>.psm
#    peptide-spectrum match:
# 
#     spectrum id \t peptide \t prob \n
# 
#---------------------------------------------------------------------------
import os, sys
import Utils
import popen2

#---------------------------------------------------------------------------
if __name__ == "__main__":
  if len(sys.argv) < 4:
    print >>sys.stderr, """
Usage: %s <suid-file> <ms2-file> <indexed-fasta-file> <param-prefix>
    <suid-file> - a list of spectrum uids
    <ms2-file> - an ms2 file
    <indexed-fasta-file> - The indexed fasta file.
    <param-prefix> - parameter prefix

    output the scores for each spectrum to <colleciton-prefix>.scoredir
  """ % os.path.basename(sys.argv[0])
    sys.exit(1)

#---------------------------------------------------------------------------
from hmmParams import *
import os.path, cPickle, tempfile
import Mass
import HmmData
import math
import Numeric
import MatchCollection
import generatePeptides
import GMTKmodel 

#---------------------------------------------------------------------------
# Returns directory with ".scores" files in it
#---------------------------------------------------------------------------
def main(suidFile=None, ms2File=None, fastaFname=None, paramPrefix=""):

  # parse the spectrum collection
  runDir = ','.join(map(lambda x: os.path.basename(x).split('.')[0], 
      [ fastaFname, suidFile, 'merged'])) + '.scoredir'

  if not os.path.isdir(runDir):  os.mkdir(runDir)

  cmdLines = []

  positiveSuids = []
  if os.path.isdir(runDir) and REDO == False:
    positiveFnames = map(lambda x: os.path.abspath(runDir+ "/" + x), \
      filter( lambda x: x.endswith(SUFFIX_STRING), os.listdir(runDir) ) ) 
    positiveFnames = filter(lambda x: os.path.getsize(x) > 0, positiveFnames)
    positiveSuids = map(lambda x: os.path.basename(x).split('.')[0], positiveFnames)
  
  # read in suids and create commands
  fh = open(suidFile)
  for line in fh:
    suid = line.rstrip("\n")

    if suid in positiveSuids: 
      continue
    suidMerge = suid + "-merged"
    if suidMerge in positiveSuids: 
      continue

    # create the small match collection, with only one match
    smallMatchCollection = MatchCollection.PsmMatchCollection()
    # create the cmdlines
    cmd = "python %s %s %s %s %s ; " % (HMM_TEST_MERGE_SPECTRUM, suid, \
          os.path.abspath(ms2File), fastaFname, paramPrefix)
    cmdLines.append(cmd)
  fh.close()

  # stop if there aren't any commands
  if len(cmdLines) == 0: 
    print >>sys.stderr, "No commands, so no migrate"
    return runDir

  for cmd in cmdLines:
    print >>sys.stderr, "Running: %s" % cmd
    os.system(cmd)


  # 2010-04 AAK Commented out to allow local execution
  # output to migrate file

#  tmpTuple = tempfile.mkstemp("",".cmds", os.getcwd())
#  tmpFileFd, tmpFile = tmpTuple[0], tmpTuple[1]
#  migrateFh = os.fdopen(tmpFileFd, "w")
#  count = 0
#  prefixCmd = "find %s/aklammer/ -maxdepth 1 -mtime +1 | grep -v '%s/aklammer/$' | xargs rm -fr ; " % (S_DIR, S_DIR)
#  prefixCmd += "space_req /%s >& /dev/null ; " % S_DIR 
#
#  for cmdLine in cmdLines: 
#    count += 1
#
#    if (count % NUM_JOBS_IN_CLUSTER == 1): 
#      migrateFh.write(prefixCmd)
#    
#    migrateFh.write(cmdLine)
#
#    if (count % NUM_JOBS_IN_CLUSTER == 0): 
#      migrateFh.write("\n")
#
#  if (count % NUM_JOBS_IN_CLUSTER != 0): 
#    migrateFh.write("\n")
#
#  migrateFh.close()
#
  cwd = os.getcwd()
  # create and cd to rundir
  os.chdir(runDir)
  svm = paramPrefix + ".svmobject"
  if not os.path.isfile(svm):
    try:
      os.link("../%s" % svm, "%s" % svm) 
    except:
      print >>sys.stderr, "Strange error linking files"

  # sys.exit(1)
  # run migrate
  # 2010-04 Commented out to allow local execution
#  migrateCmdLine = "migrate %s --restart -f %s >& %s.error ; %s" \
#      % (MIGRATE_OPTIONS, tmpFile, tmpFile, Utils.alertCommand())
#
#  print >>sys.stderr, "Running %s" % migrateCmdLine
#  popper = popen2.Popen3(migrateCmdLine, True)
#
#  stdout = popper.fromchild.read()
#  popper.fromchild.close()
#
#  stderr = popper.childerr.read()
#  popper.childerr.close()
#
#  for line in stderr:
#    print >>sys.stderr, line,
#
#  popper.wait()

  os.chdir(cwd)
  return runDir

#---------------------------------------------------------------------------
if __name__ == "__main__":
  main(suidFile=sys.argv[1], ms2File=sys.argv[2], fastaFname=sys.argv[3], paramPrefix=sys.argv[4])

