#--------------------------------------------------------
# $Revision: 1.2 $
#--------------------------------------------------------
import getopt, sys
import MLab, Numeric

#--------------------------------------------------------
if __name__ == '__main__':
  if len(sys.argv) < 2: 
    print """Usage: python %s 
  <-r train-file> 
  <-d score-dir>
  <-s suffix>
""" % (sys.argv[0])
    sys.exit(-1)

#--------------------------------------------------------
import cPickle, composite, preproc, time, datafunc, svm, assess
import modelSelection
from hmmParams import *

global G_row_norm
G_row_std = 1.0
G_row_norm = []

global G_column_std
global G_column_mean
G_column_std = 1.0
G_column_mean = 0.0

#--------------------------------------------------------
def varianceOneColumns(X, train):
  '''returns X / std(X) '''

  global G_column_std
  if train == True:
    G_column_std = MLab.std(X)

  # from Asa's code
  eps = 1e-5
  I = Numeric.nonzero(Numeric.less(G_column_std, eps))
  Numeric.put(G_column_std, I, 1)

  for i in range(len(G_column_std)):
    if G_column_std[i] == 0 and G_column_std[i] == 1:
      G_column_std[i] = 1

  n = Numeric.shape(X)[0]
  return X/Numeric.resize(G_column_std, (n,len(G_column_std))) 
  
#--------------------------------------------------------
def varianceOneRows(X, train) :
  '''returns X / std(X) '''
  transpose_X = Numeric.transpose(X)

  G_row_std = MLab.std(transpose_X)
  # from Asa's code
  eps = 1e-5
  I = Numeric.nonzero(Numeric.less(G_row_std, eps))
  Numeric.put(G_row_std, I, 1)

  for i in range(len(G_row_std)):
    if G_row_std[i] == 0 and G_row_std[i] == 1:
      G_row_std[i] = 1
  
  n = Numeric.shape(transpose_X)[0]
  std = Numeric.resize(G_row_std, (n,len(G_row_std))) 
  scaled_X = transpose_X/std
  new_transpose_X = Numeric.transpose(scaled_X)
  return new_transpose_X

#--------------------------------------------------------
def centerColumns(X, train) :
    '''returns X - mean(X), where the mean is taken over the columns of X'''
    
    global G_column_mean 
    if train == True:
      G_column_mean = MLab.mean(X)

    n = Numeric.shape(X)[0]
    return X - Numeric.resize(G_column_mean, (n,len(G_column_mean)))

#--------------------------------------------------------
def normalizeNorm(X, train) :
    '''normalize each row of X to unit vectors'''

    (numRows, numCols) = Numeric.shape(X)
    Xnorm = Numeric.zeros((numRows, numCols), Numeric.Float)
    
    global G_row_norm
    for i in range(numRows) :
      if train == True:
        G_row_norm.append(preproc.norm2(X[i]))

    for i in range(numRows) :
        Xnorm[i] = X[i] / G_row_norm[i]

    return Xnorm

#--------------------------------------------------------
def processData(testData, variance, lnorm, train):
  # testData.X = testData.X

  print "Before"
  print testData.X
  if variance == 'r':
    testData.X = varianceOneRows(testData.X, train)
  elif variance == 'c':
    testData.X = varianceOneColumns(testData.X, train)
  elif variance == 'rc':
    testData.X = varianceOneRows(testData.X, train)
    testData.X = varianceOneColumns(testData.X, train)
  elif variance == 'cr':
    testData.X = varianceOneColumns(testData.X, train)
    testData.X = varianceOneRows(testData.X, train)
  else:
    raise SystemExit, "Unknown variance %s" % variance

  print "After variance"
  print testData.X

  if lnorm == 0:
    pass
  elif lnorm == 2:
    testData.X = centerColumns(testData.X, train)
    testData.X = normalizeNorm(testData.X, train)
  else:
    raise SystemExit, "Unknown lnorm %i" % lnorm

  print "After norm"
  print testData.X

#--------------------------------------------------------
if __name__ == '__main__':
  optlist, args = getopt.getopt(sys.argv[1:],'r:d:p:v:l:s:')
  train = suffix = scoreDir = None
  for option_name, option_value in optlist:
    if option_name == '-r': train=option_value
    if option_name == '-d': scoreDir=option_value
    if option_name == '-s': suffix=option_value
    if option_name == '-v': variance=option_value
    if option_name == '-l': lnorm=int(option_value)

  print >>sys.stderr, "Using variance %s and lnorm %i" % \
      (variance, lnorm)

  DataSet = datafunc.DataSet
  trainData = DataSet(train, idColumn = 0, labelsColumn = 1) 
  processData(trainData, variance, lnorm, train=True)
  sys.exit(1)



  svmSelect = modelSelection.SVMselect()
  svmSelect.train(trainData)
  os.chdir(scoreDir)
  scoreFiles = filter(lambda x: x.endswith(".scores"), os.listdir('.'))
  for file in scoreFiles:
    testData = DataSet(file, idColumn = 0, labelsColumn = 1)
    try:
      processData(testData, variance, lnorm, train=False)
    except Exception, e:
      print >>sys.stderr, "Trouble processing %s %s" % (file, e)
      continue
    outputFh = open(file + "-" + suffix , "w")
    for idx in range(len(testData)):
      label = testData.labels.patternID[idx]
      peptide = label.split('.')[1]
      print >>outputFh, "%s\t%s\n" % (peptide, float(svmSelect.decisionFunc(testData, idx))),
    outputFh.close()
     
#---------------------------
#if matrix2png != None:
#    dataFields = []
#    data = open(train)
#    for line in data:
#      line = line.rstrip()
#      dataFields.append(line.split(','))
#    data.close()
#    outputLines = []
#
#    for idx in range(len(trainData)):
#      outputFields = []
#      for field in dataFields[idx]:
#        outputFields.append(field)
#      outputFields[0] = outputFields[0].split('.')[1]
#      outputFields.append(s.decisionFunc(trainData, idx))
#      outputLines.append(outputFields)
#
#    outputLines.sort(lambda x,y: cmp(float(x[-1]), float(y[-1])))
#
#    fh = open(matrix2png, "w")
#    header = "svm label b y a b-h2o y-h2o b-nh3 y-nh3 logtic numions disc".split(" ")
#    fh.write("\t".join(header) + "\n")
#    for idx in range(len(outputLines)):
#      fh.write("\t".join(map(str, outputLines[idx])) + "\n")
#    fh.close()
#---------------------------
