#!/usr/bin/python
#-----------------
# $Revision: 1.14 $
#-----------------
import cPickle, re, sys
import compile_aa_stats  
import Spectrum
import SpectrumCollection
import MatchCollection
import Model

#-------
# Main
#-------
if __name__ == '__main__':

	usage = """Usage: %s <pickled-psm-filename> <model-shortname>
  Places a pickled, trained model in <model-shortname>.model where"""
	if len(sys.argv) < 3:
		raise SystemExit, usage % (sys.argv[0])

	if not sys.argv[1].endswith(".pickledpsm"):
		print >>sys.stderr, "pickled-psm-filename must end in pickledpsm!"
		raise SystemExit, usage % (sys.argv[0])

	modelName = sys.argv[2]

	# If this is a non-trainable model, don't bother with the PSMs
	psmColl = None
	if modelName not in [ "RAND", "RAND-HT", "UNI", "REAL-BY-UNI", "REAL-BY", "MPROTON", "MPROTON-BY" ]:
		print >>sys.stderr, "Loading PSM file %s" % sys.argv[1]
		psmFile = open(sys.argv[1])
		psmColl = cPickle.load(psmFile)
		psmFile.close()

	model = None
	if modelName == "RAND": 				model = Model.RandomModel()
	elif modelName == "RAND-HT": 		model = Model.RandomHeightModel()

	elif modelName == "UNI": 				model = Model.UniformModel()

	elif modelName == "REAL-BY-UNI": 	model = Model.RealByUniformModel()
	elif modelName == "REAL-BY": 			model = Model.RealByModel()
	elif modelName == "REAL-BY-DIFF":	model = Model.RealByDiffModel()

	elif modelName == "ADJ": 		model = Model.PrecursorMzModel()

	elif modelName == "AVG-N": 			model = Model.AverageNtermModel()
	elif modelName == "AVG-N-ADJ": 	model = Model.AverageNtermAdjusterModel()

	elif modelName == "AVG-C": 			model = Model.AverageCtermModel()
	elif modelName == "AVG-C-ADJ": 	model = Model.AverageCtermAdjusterModel()

	elif modelName == "AVG": 				model = Model.AverageModel()
	elif modelName == "AVG-BY": 		model = Model.AverageChargeAllocationModel()
	elif modelName == "AVG-ADJ": 		model = Model.AverageAdjusterModel()

	elif modelName == "PAIR-N": 		model = Model.PairNtermModel()
	elif modelName == "PAIR-N-ADJ":	model = Model.PairNtermAdjusterModel()

	elif modelName == "MPROTON":		model = Model.MobileProtonModel()
	elif modelName == "MPROTON-BY":	model = Model.MobileProtonByModel()

	elif modelName == "PAIR-C": 		model = Model.PairCtermModel()
	elif modelName == "PAIR-C-ADJ": model = Model.PairCtermAdjusterModel()

	elif modelName == "PAIR": 			model = Model.PairModel()
	elif modelName == "PAIR-ADJ": 	model = Model.PairAdjusterModel()
	elif modelName == "PAIR-PR-ADJ":model = Model.PairProductAdjusterModel()
	elif modelName == "PAIR-MP": 		model = Model.PairAdjusterMprotonModel()

	elif modelName == "PAIR-CHR":		model = Model.PairChargeModel()

	elif modelName == "PAIR-BY": 		model = Model.PairChargeAllocationModel()

	elif modelName == "PAIR-NEU":		model = Model.PairNeutralLossModel()
	elif modelName == "ASMS":		model = Model.AsmsModel()



	else: raise SystemExit, "Unsupported model %s" % modelName

	print >>sys.stderr, "Training model %s" % modelName
	model.train(psmColl)

	outFileName = modelName + ".model"
	print >>sys.stderr, "Pickling model to %s"  % outFileName
	outFile = open(outFileName, 'w')
	cPickle.dump(model, outFile)
	outFile.close()

