#---------------------------------------------------------------------------
# File: zscoreMargin.py
#---------------------------------------------------------------------------
# Author: $Author: aklammer $
#---------------------------------------------------------------------------
# Revision: $Revision: 1.2 $
#---------------------------------------------------------------------------
# Inputs:
# 
#  - the output of hmmTest.py, file <prefix>.allzscore, 
#    containing the Z scores for each peptide-spectrum match:
# 
#     spectrum id \t peptide \t zscore \n
# 
#  - <positives>.txt to compare with
# 
#---------------------------------------------------------------------------
import os, os.path, sys, random, cPickle
import Mass
import HmmData
import math
import Numeric
import Spectrum


#------------------------------------------------------------------------------
def dtaIdToSuid(dtaId):
  groupId, firstScan, lastScan, charge = dtaId.split('/')[1].split('.')
  suid = ",".join([ firstScan, groupId, charge])
  return suid

#------------------------------------------------------------------------------
if __name__ == "__main__":
  if len(sys.argv) < 3:
    print >>sys.stderr, """
Usage: %s <prefix> <positives>
    <prefix> - A prefix to the <prefix>.allzscores file.
    <positives> - A .txt filename (for OMICS data) or .psm filename (for in
    house data), which contains the list of positive PSMs
  """ % os.path.basename(sys.argv[0])
    sys.exit(1)

  zscorePrefix = sys.argv[1]
  positivesFile = sys.argv[2]

  fh = open(zscorePrefix + ".allzscores")
  suid2peptideScore = {}
  for line in fh:
    line = line.rstrip()
    suid, peptide, zscore = line.split()
    if suid2peptideScore.has_key(suid):
      suid2peptideScore[suid][peptide] = zscore
    else:
      suid2peptideScore[suid] = {peptide:zscore}

  fh.close()

  suid2positivePeptide = {}
  if positivesFile.endswith(".txt"):
    fh = open(positivesFile)
    for line in fh:
      line = line.rstrip()
      fields = line.split()
      dtaId, peptide = fields[0], fields[3]
      peptide = peptide.split('.')[1]
      suid = dtaIdToSuid(dtaId)
      suid2positivePeptide[suid] = peptide
    fh.close()
  elif positivesFile.endswith(".psm"):
    fh = open(positivesFile)
    for line in fh:
      line = line.rstrip()
      fields = line.split()
      groupId = fields[1]
      scan = fields[2]
      peptide = fields[3]
      charge = fields[4]
      suid = Spectrum.spectrumUID(scan, groupId, charge)
      suid2positivePeptide[suid] = peptide
    fh.close()
  else:
    raise SystemExit, "Input file %s must end in .txt or .psm" % positivesFile

  # print suid2positivePeptide
  # print suid2peptideScore

  problems = []
  margins = []
  for suid in suid2peptideScore.keys():
    realPeptide = suid2positivePeptide[suid]
    peptide2score = suid2peptideScore[suid]
    if peptide2score.has_key(realPeptide):
      realPeptideScore = peptide2score[realPeptide]
    else:
      print >>sys.stderr, "No peptide %s anywhere for %s" % (realPeptide, suid)
      continue
    nonRealPeptides = filter(lambda x: x != realPeptide,\
        peptide2score.keys())
    nonRealScores   = map(lambda x: float(peptide2score[x]), nonRealPeptides)
    nonRealScores.sort()
    # print nonRealScores
    margin = float(peptide2score[realPeptide]) - float(nonRealScores[-1])
    margins.append(margin)
    print "%s\t%s\t%s" % (suid, realPeptide, margin)
 
  print "Mean: %.2f" % Numeric.average(margins)

