#---------------------------------------------------------------------------
# File: hmmTrainSion.py
#---------------------------------------------------------------------------
# Author: $Author: aklammer $
#---------------------------------------------------------------------------
# Revision: $Revision: 1.68 $
#---------------------------------------------------------------------------
# hmmTrainSion.py 
#   - <pos-psm-collection> 
#   - <neg-psm-collection> 
#   - <sion-model> 
#   - <integrator-model> 
#   - <param-prefix> 
#---------------------------------------------------------------------------
# Outputs:
#
#---------------------------------------------------------------------------
import os, sys

#-------------------------------------------------------------------------
if __name__ == "__main__":
  if len(sys.argv) < 4:
    print >>sys.stderr, """
Usage: %s 
  <pos-psm-collection> 
  <neg-psm-collection> 
  <cand-psm-collection> 
  <model> 
  <param-prefix> 
  """ % os.path.basename(sys.argv[0])
    sys.exit(1)

#-------------------------------------------------------------------------
from hmmParams import *
import os.path, random, cPickle, tempfile, shutil, signal
import os
import Mass
import HmmData
import math
import Match
import Spectrum
from GMTK import *
import GMTKmodel
import MatchCollection

#---------------------------------------------------------------------------
def paramString(paramPrefix, ionType, positive = True):
  if positive == True: positivityString = "pos"
  else: positivityString = "neg"
  return "%s,%s,%s.params" % (paramPrefix, ionType, positivityString)

#---------------------------------------------------------------------------
def migratify(cmdLines):
  tmpTuple = tempfile.mkstemp()
  tmpFileFd, tmpFile = tmpTuple[0], tmpTuple[1]
  migrateFh = os.fdopen(tmpFileFd, "w")
  for cmdLine in cmdLines:
    migrateFh.write(cmdLine)
  migrateFh.close()

  # run migrate
  migrateCmdLine = "migrate %s --restart -f %s" % (MIGRATE_OPTIONS, tmpFile)

  print >>sys.stderr, "Running %s" % migrateCmdLine
  stdin, stdout, stderr = os.popen3(migrateCmdLine)
  stderrLine = stderr.read()
  if len(stderrLine) != 0: 
    print >>sys.stderr, "Problem with migrate %s" % stderrLine
  print >>sys.stderr, "Finished migrate"

#---------------------------------------------------------------------------
def createGmtkFiles(psms, paramPrefix, model, positive=True):
  posGmtkFiles = {}
  negGmtkFiles = {}
  collectionName = psms["file"]
  prefix = os.path.basename(collectionName).split('.')[0] + "," + paramPrefix
  outDir = os.path.abspath(prefix) + ".hmmtraindirs"
  if ((not os.path.isdir(outDir)) and (not os.path.islink(outDir))) \
      or OVERWRITE: 
    print >>sys.stderr, "Writing ion files"
    if not os.path.isdir(outDir): os.mkdir(outDir)
    # GAUSSIAN possibly add in parameter for training/testing issue
    model.writeIonFilesC(psms, outDir, posGmtkFiles, positive=True) 
    model.writeIonFilesC(psms, outDir, negGmtkFiles, positive=False) 

  else:
    print >>sys.stderr, "Naming ion files"
    model.nameIonFiles(outDir, posGmtkFiles, positive=True)
    model.nameIonFiles(outDir, negGmtkFiles, positive=False)
  return posGmtkFiles, negGmtkFiles

#---------------------------------------------------------------------------
def subsetPsms(psmToKeep, psmToSubset):
  uidsToKeep = {}
  for match in psmToKeep["matches"]:
    uidsToKeep[match.spectrum.uid()] = CANDIDATE_PEPTIDE_NUMBER

  subsetPsms = []
  random.shuffle(psmToSubset["matches"])
  for match in psmToSubset["matches"]:
    uid = match.spectrum.uid()
    if uidsToKeep.has_key(uid):
      subsetPsms.append(match)
      uidsToKeep[uid] -= 1
      if uidsToKeep[uid] == 0: uidsToKeep.pop(uid)
  psmToSubset["matches"] = subsetPsms

#---------------------------------------------------------------------------
def main(posPsmCollectionName=None, randomPsmCollectionName=None, \
    candidatePsmCollectionName=None, model=None, paramPrefix=None):

  svmOutput = model.name + "," + paramPrefix + ".svminput"
  if looksGood(svmOutput): return svmOutput

  posPsmCollection = cPickle.load(open(posPsmCollectionName))
  posPsmCollection['file'] = posPsmCollectionName
  # random.shuffle(posPsmCollection["matches"])
  # halfLength = int(len(posPsmCollection["matches"])/2.0)

  # svmPsmCollection = MatchCollection.PsmMatchCollection()
  # svmPsmCollection["matches"] = posPsmCollection["matches"][halfLength:]
  # svmPsmCollection["file"] = "svm-" + posPsmCollectionName

  candPsmCollection = cPickle.load(open(candidatePsmCollectionName))
  candPsmCollection["file"] = candidatePsmCollectionName
  # subsetPsms(svmPsmCollection, candPsmCollection)

  # create the training files for the PSM collections
  # NOTE if uncomment svmGmtkFiles, put in positive or negative
  # svmGmtkFiles =  createGmtkFiles(svmPsmCollection, paramPrefix, model) 
  posGmtkFilesPosnorm, posGmtkFilesNegnorm, = \
    createGmtkFiles(posPsmCollection, paramPrefix, model)
  candGmtkFilesPosnorm, candGmtkFilesNegnorm = \
    createGmtkFiles(candPsmCollection, paramPrefix, model)

  # svmGmtkFiles = dbnGmtkFiles = posGmtkFiles
  # svmPsmCollection = dbnPsmCollection = posPsmCollection

  #----------
  # train a model for each ion type
  #----------
  if TRAIN == True:

    # create the training files for the DBN
    # dbnPsmCollection = MatchCollection.PsmMatchCollection()
    # dbnPsmCollection["matches"] = posPsmCollection["matches"][:halfLength]
    # dbnPsmCollection["file"] = "dbn-" + posPsmCollectionName
    # dbnGmtkFiles =  createGmtkFiles(dbnPsmCollection, paramPrefix, model)

    # GAUSSIAN make the model a non-training model now and use the gaussian parameters learned before
    randomPsmCollection = cPickle.load(open(randomPsmCollectionName))
    randomPsmCollection["file"] = randomPsmCollectionName
    # subsetPsms(dbnPsmCollection, negPsmCollection)
    randomGmtkFilesPosnorm, randomGmtkFilesNegnorm =\
        createGmtkFiles(randomPsmCollection, paramPrefix, model)

    migrateCmds = ""
    print >>sys.stderr, "Constructing positive model commands"
    for ionType in posGmtkFilesPosnorm.keys():
      if ionType == 'p': continue 
      paramFile = paramString(paramPrefix, ionType, positive = True)
      cmd = constructTrainCmd(posGmtkFilesPosnorm[ionType], model, paramFile)
      migrateCmds += (cmd + "\n")

    # TODO make consistent with constructTestCmds
    print >>sys.stderr, "Constructing negative model commands"
    for ionType in randomGmtkFilesNegnorm.keys(): 
      if ionType == 'p': continue
      paramFile = paramString(paramPrefix, ionType, positive = False)
      cmd = constructTrainCmd(randomGmtkFilesNegnorm[ionType], model, paramFile)
      migrateCmds += (cmd + "\n")
    
    # TRAIN!
    migratify(migrateCmds)

  #----------
  # Create svm file
  #----------
  svmOutputFh = open(svmOutput, "w")
  # GAUSSIAN need to create two more sets of files for positive and negative PSMs 
  # normed with correct params and to evaluate under the positive and negative models
  # is there any way we can just put sionLor in here??

  if KERNEL == 'lor':
    print >>sys.stderr, "Using kernel %s" % KERNEL

    # put in pos and neg GmtkFiles
    svmCommands, svmFiles = constructTestCmds(posPsmCollection, \
        model, paramPrefix, posGmtkFilesPosnorm, posGmtkFilesNegnorm)
    candCommands, candFiles = constructTestCmds(candPsmCollection, \
        model, paramPrefix, candGmtkFilesPosnorm, candGmtkFilesNegnorm)

    # TEST!
    migratify(svmCommands + candCommands)

    svmScores = calculateScores(posPsmCollection, svmFiles, model)
    candScores = calculateScores(candPsmCollection, candFiles, model)
    
    svmScores.outputSvmString(svmOutputFh, positive = True),
    candScores.outputSvmString(svmOutputFh, positive = False),
  elif KERNEL == 'accumulator' or KERNEL == 'fisher':
    print >>sys.stderr, "Using kernel %s" % KERNEL
    # print >>sys.stderr, "Accumulator kernel does not yet support the Gaussian normalization"
    # sys.exit(1)
    vectors = sionFisherKernel(posPsmCollection, model, paramPrefix) 
    vectors.outputSvmString(svmOutputFh, positive = True),
    vectors = sionFisherKernel(candPsmCollection, model, paramPrefix)
    vectors.outputSvmString(svmOutputFh, positive = False),
  else: 
    print >>sys.stderr, "Unsupported kernel %s" % KERNEL; sys.exit(1);

  svmOutputFh.close()

  return svmOutput


if __name__ == "__main__":
  paramPrefix=sys.argv[5]
  model = GMTKmodel.gmtkModelFactory(sys.argv[4], training=True, \
      paramPrefix=paramPrefix)
  # GAUSSIAN add a param prefix to the model
  svmOutput = main(posPsmCollectionName=sys.argv[1], \
      randomPsmCollectionName=sys.argv[2], \
      candidatePsmCollectionName=sys.argv[3], \
      model=model, \
      paramPrefix=paramPrefix)


