#---------------------------------------------------------------------------
def sionLor(psmCollection, modelName, paramPrefix, gmtkFiles=None):
  """
  calculate and return a IonScoreCollection that contains LORs, pos, and
  neg scores for the matches in psmCollection under the pos and neg trained
  versions of modelName for each ion type. paramPrefix is the prefix to the
  various parameter files for each ion type in the modelName params dir. 
  will create gmtkFiles from scratch, or optionally use the files in
  gmtkFiles, which *must* be in the same order as the matches in
  psmCollection
  """
  
  assert(len(psmCollection["matches"])!=0)
  # instantiate our output --- the collection of scores foreach PSM and ionType
  scores = IonScoreCollection()

  # create the GMTK files if necessary
#  if gmtkFiles is None:
#    gmtkFiles = {}
#    prefix = os.path.basename(psmCollection["file"]).split('.')[0] + "," + paramPrefix
#    outDir = os.path.abspath(prefix) + ".hmmtraindirs"
#    if not os.path.isdir(outDir): os.mkdir(outDir)
#    HmmData.writeIonFiles(psmCollection, outDir, gmtkFiles) 
#
  # iterate through each ion type
  for ionType in Spectrum.ION_TYPES:

    if ionType == 'p': continue  # FIX

    # run testGMTK for the pos model ...
    paramFile = paramString(paramPrefix, ionType, positive = True)
    posProbs = testGMTK(gmtkFiles[ionType], modelName, paramFile)

    # put the pos probs into our IonScoreCollection
    scores.populate(psmCollection, ionType, posProbs, positive = True)


    # ... and the neg model
    paramFile = paramString(paramPrefix, ionType, positive = False)
    negProbs = testGMTK(gmtkFiles[ionType], modelName, paramFile)

    # put the neg probs into our IonScoreCollection
    scores.populate(psmCollection, ionType, negProbs, positive = False)

  # calculate our lors
  scores.lors()

  # return the IonScoreCollection
  return scores

